package gov.va.med.fee.claim;

import gov.va.med.domain.fee.AdjustmentReason;
import gov.va.med.domain.fee.ClaimQueue;
import gov.va.med.domain.fee.WsFacStatus;
import gov.va.med.fee.DataPage;
import gov.va.med.fee.Page;
import gov.va.med.fee.PageControls;
import gov.va.med.fee.PagedListDataModel;
import gov.va.med.fee.sec.HomePageUserData;

import java.math.BigDecimal;
import java.util.LinkedList;
import java.util.List;
import java.util.Observable;

import javax.ejb.Remove;
import javax.faces.context.FacesContext;
import javax.faces.event.ValueChangeEvent;
import javax.faces.model.DataModel;
import javax.persistence.EntityManager;
import javax.persistence.Query;

import org.hibernate.Session;
import org.jboss.seam.log.Log;


public class RejectedClaimsList extends Observable implements RejectedClaims {

   private int currentStartRow = -1;  //on start set to a non-valid value

   private int currentPageSize = -1; //on start set to a non-valid value

   // since this is not managed by seam created log from factory method.
   private Log log = org.jboss.seam.log.Logging.getLog(RejectedClaimsList.class);

   private EntityManager em;

   private HomePageUserData homePageUserData;

   private String sortCol = "";

   private String sortDir = "";

   private int numberOfResults = 0;

   private List<RejectedClaimsTo> rejectedClaims = new LinkedList<RejectedClaimsTo>();

   private Page<?> page;

   private PageControls pageControls;

   private transient LocalDataModel dataModel;

   public String stationsHolder = "";

   public RejectClaims sessionObserver;





   public RejectedClaimsList() {
      page = new Page<Object>();
      pageControls = new PageControls();
   }

   @Override
   public List<?> buildClaimsListByType(int startRow, int pageSize ) {
      return buildClaimsListByType(startRow, pageSize, false);
   }
   
   void rebuildClaimsListByType() {
      buildClaimsListByType(this.currentStartRow, this.currentPageSize, true);
   }

   @SuppressWarnings("unchecked")
   List<?> buildClaimsListByType(int startRow, int pageSize, boolean force) {
      String stations = this.getActiveStations();
      String privacyRestrictions =  homePageUserData.getPrivacyRestrictions();
      String visns = homePageUserData.getVisnSelector();

      if (startRow == currentStartRow && pageSize == currentPageSize && !isNumResultChanged() &&  !force) {
         return rejectedClaims;
      }
 
      currentStartRow = startRow;
      currentPageSize = pageSize;

      rejectedClaims.clear();
      try {        
         StringBuilder qstrb = new StringBuilder("select c.* from ws_fac_status c, claim_queue q")
                                       .append(" where c.claim_status_cd = 'REJECTED' ")
                                       .append(" and c.claim_index = q.claim_index ")
                                       .append(" and q.claim_queue_type_cd = 'REJECTION_APPROVAL'")
                                       .append(" and q.dequeue_date is null ");
         //qstrb.append(" and ws.va_facility_cd in ( ").append(stations).append(" ) ");
         if (!"".equalsIgnoreCase(visns)) {

            qstrb.append("and ( ")
                 .append("  c.va_facility_cd in ( ").append(stations).append(" ) ")
                 .append(visns)
//                 .append("  or ( ws.visn_id_cd in ( ").append(visns).append(" )")
//                 .append("       and (ws.va_facility_cd is null or ws.va_facility_cd='')")
//                 .append("      )")
                 .append("    )");
                 
         } else {
            qstrb.append(" and c.va_facility_cd in ( ").append(stations).append(" ) ");
         }
         if (!"".equals(privacyRestrictions)) {
            qstrb.append(" and c.person_id not in ( ").append(privacyRestrictions).append(" ) ");
         }   
         if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortClaimIndex")) {
            qstrb.append(" order by c.claim_index ").append(this.sortDir);
         } else if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortClaimType")) {
            qstrb.append(" order by c.service_type_cd ").append(this.sortDir).append(", c.claim_index ").append(this.sortDir);
         } else if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortProjectHero")) {
            qstrb.append(" order by c.hero_flag ").append(this.sortDir).append(", c.claim_index ").append(this.sortDir);
         } else if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortPatientId")) {
            qstrb.append(" order by c.person_id ").append(this.sortDir).append(", c.claim_index ").append(this.sortDir); 
         } else if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortClaimDate")) {  
            qstrb.append(" order by c.date_created ").append(this.sortDir).append(", c.claim_index ").append(this.sortDir); 
         } else if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortClaimAmount")) {  
            qstrb.append(" order by c.total_billed_charge ").append(this.sortDir).append(", c.claim_index ").append(this.sortDir);
         } else if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortProvider")) {  
            qstrb.append(" order by c.provider_name ").append(this.sortDir).append(", c.claim_index ").append(this.sortDir);
         } else if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortDisapprovedBy")) {  
            qstrb.append(" order by q.created_by ").append(this.sortDir).append(", c.claim_index ").append(this.sortDir);   
         } else if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortDisapprovalDate")) {  
            qstrb.append(" order by q.date_created ").append(this.sortDir).append(", c.claim_index ").append(this.sortDir);  
         } else if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortReason")) {  
            qstrb.append(" order by lpad(regexp_replace(q.reason, '([0-9]+)', '\\1'),3, 0) ").append(this.sortDir).append(", c.claim_index ").append(this.sortDir);   
         } else if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortExplanation")) {  
            qstrb.append(" order by q.enqueue_comments ").append(this.sortDir).append(", c.claim_index ").append(this.sortDir);             
         } else {
            qstrb.append(" order by c.claim_index ").append(this.sortDir);
         }

       // Used hibernate specific API to take advantage of class marshalling with native queries
       // native queries used because there is no DB association between the ws_fac_status and claim_queue tables. 
       Session session = (Session) em.getDelegate();
       List<WsFacStatus> results = session.createSQLQuery(qstrb.toString())
                                          .addEntity(WsFacStatus.class)
                                          .setFirstResult(startRow)
                                          .setMaxResults(startRow + pageSize).list();
        
         
         /* loop through the results. */
         for (WsFacStatus fac : results) {
            RejectedClaimsTo rct = new RejectedClaimsTo();
            rct.setClaimId(fac.getClaimIndex());
            rct.setType(fac.getServiceTypeCd());
            rct.setHero(fac.getHeroFlag().toString());
            rct.setSsn(fac.getPersonId());
            rct.setClaimDate(fac.getDateCreated());
            rct.setProvider(fac.getProviderName());
            rct.setAmount(fac.getTotalBilledCharge());
            ClaimQueue disapproval = this.getClaimDisapprovalReason(fac.getClaimIndex());
            rct.setDisapprover(disapproval.getCreatedBy());
            rct.setDisapprovedDate(disapproval.getDateCreated());
            rct.setReason(disapproval.getReason());
            rct.setReasonDesc(getReasonCodeDesc(disapproval.getReason()));
            rct.setDisapproversComments(disapproval.getEnqueueComments());
            
            this.attachRejectionHistory(rct);

            rejectedClaims.add(rct);
         }
         
         
         
         

         BigDecimal recordCount = getRecordCount();

         /*
          * set the numberOfResults property and the pageControls numberOfResults.
          */
         this.numberOfResults = recordCount.intValue();
         this.pageControls.setNumberOfResults(numberOfResults);

      } catch (Exception ex) {
         throw new RuntimeException(ex);
      }

      //notify the session bean that the page contents have changed.
      this.sessionObserver.update(rejectedClaims);

      log.debug("exiting buildClaimsListByType with {0} claims ", rejectedClaims.size());
      return rejectedClaims;
   }
   
   private String getReasonCodeDesc(String cd) {
      Query q = em.createQuery("select rc from fAdjustmentReason rc where rc.adjustmentReasonCd = :code ");
      q.setParameter("code", cd);
      AdjustmentReason rc = (AdjustmentReason) q.getSingleResult();
      return rc.getDescription();
   }

   private BigDecimal getRecordCount() {
      log.debug("entering getRecordCount()");
      String stations = this.getActiveStations();
      String privacyRestrictions = homePageUserData.getPrivacyRestrictions();
      String visns = homePageUserData.getVisnSelector();
      StringBuilder cntSql = new StringBuilder("select count(*) ")
                             .append(" from fpps_owner.ws_fac_status c ")
                             .append(" where c.claim_status_cd = 'REJECTED' ");
      if (!"".equalsIgnoreCase(visns)) {
         cntSql.append("and ( ")
               .append("  c.va_facility_cd in ( ").append(stations).append(" ) ")
               .append(visns) 
//               .append("  or ( c.visn_id_cd in ( ").append(visns).append(" )")
//               .append("       and (c.va_facility_cd is null or c.va_facility_cd='')")
//               .append("      )")
               .append("    )");
      } else {
         cntSql.append(" and c.va_facility_cd in ( ").append(stations).append(" ) ");
      }

      if (!"".equalsIgnoreCase(privacyRestrictions)) {
         cntSql.append(" and c.person_id not in ( ").append(privacyRestrictions).append(" ) ");
      }

      Query qcnt = em.createNativeQuery(cntSql.toString());

      /* query the database. */
      BigDecimal recordCount = (BigDecimal) qcnt.getSingleResult();
      log.debug("exiting getRecordCount");
      return recordCount;
   }

   @Override
   public void changePageSize(ValueChangeEvent event) {
      log.debug("entering changePageSize(event)");
      if (event == null || event.getNewValue() == null) {
         FacesContext.getCurrentInstance().renderResponse();
         return;
      }
      int newVal = ((Integer) event.getNewValue()).intValue();
      this.pageControls.setPageSize(newVal);
      FacesContext.getCurrentInstance().renderResponse();
      log.debug("exiting changePageSize");
   }

   @Remove
   @Override
   public void destroy() {
      log.debug("object destroyed.");
   }

   public final void doSort(String col) {
      log.debug("entering doSort({0})", col);
      if (this.sortCol.equalsIgnoreCase(col)) {
         this.sortDir = this.sortDir.equalsIgnoreCase("asc") ? "desc" : "asc";
      } else {
         this.sortCol = col;
         this.sortDir = "asc";
      }
      log.debug("exiting doSort");
   }

   @Override
   public DataModel getDataModel() {
      log.debug("entering getDataModel()");

      if (dataModel == null || !stationsHolder.equalsIgnoreCase(getActiveStations())) {
         log.debug("new data model created");
         dataModel = new LocalDataModel(this.pageControls.getPageSize());
         this.stationsHolder = getActiveStations();
      } else if (dataModel.getPageSize() != this.pageControls.getPageSize()) {
         log.debug("old data model used");
         dataModel.setPageSize(this.pageControls.getPageSize());
      }

      log.debug("exiting getDataModel");
      return dataModel;
   }


   public int getPageSize() {
      return dataModel.getPageSize();
   }

   private String getActiveStations() {
      log.debug("entering getActiveStations()");
      String stations = homePageUserData.getActiveStations().toString();
      stations = stations.replace('[', '\'');
      stations = stations.replace(']', '\'');
      stations = stations.replace(", ", "\', \'");
      log.debug("exiting getActiveStations");
      return stations;
   }

   @SuppressWarnings("unchecked")
   @Override
   public DataPage<RejectedClaimsTo> getDataPage(int startRow, int pageSize) {
      log.debug("entering getDataPage({0}, {1})", startRow, pageSize);
      List dp = this.buildClaimsListByType(startRow, pageSize);
      log.debug("exiting getDataPage");
      return new DataPage(numberOfResults, startRow, dp);
   }

   @Override
   public int getNumberOfResults() {
      log.debug("entering / exiting getNumberOfResults()");
      return getRecordCount().intValue();
   }

   @Override
   public Page<?> getPage() {
      log.debug("entering / exiting getPage()");
      return this.page;
   }

   @Override
   public PageControls getPageControls() {
      log.debug("entering / exiting getPageControls()");
      return this.pageControls;
   }

   @Override
   public String getSortCol() {
      log.debug("entering / exiting getSortCol()");
      return this.sortCol;
   }

   @Override
   public String getSortDir() {
      log.debug("entering / exiting getSortDir()");
      return this.sortDir;
   }

   @Override
   public void setNewPage(int newPage) {
      log.debug("entering / exiting setNewPage({0})", newPage);
      this.pageControls.setPage(newPage);
   }

   @Override
   public void setSortCol(String sortCol) {
      log.debug("entering / exiting setSortCol({0})", sortCol);
      this.sortCol = sortCol;
   }

   @Override
   public void setSortDir(String sortDir) {
      log.debug("entering / exiting setSortDir({0})", sortDir);
      this.sortDir = sortDir;
   }

   @SuppressWarnings("unchecked")
   ClaimQueue getClaimDisapprovalReason(long index) {
      log.debug("entering getClaimDisapprovalReason({0})", index);
      ClaimQueue retVal = null;
      Query q = em.createQuery("select q from fClaimQueue q "
                               + " where q.claim.claimIndex = :index "
                               + " and q.claimQueueTypeCd = 'REJECTION_APPROVAL' "
                               + " and q.dequeueDate is null "
                               + " order by q.enqueueDate desc ");
      q.setParameter("index", index);
      List<ClaimQueue> r = q.getResultList();
      if (r.size() > 0) {
         retVal = r.get(0);
      }
      log.debug("exiting getClaimDisappprovalReason");
      return retVal;
   }

   @SuppressWarnings({"unused", "unchecked"})
   private class LocalDataModel extends PagedListDataModel {

      public LocalDataModel() {
         super(1);
         log.debug("entering / exiting LocalDataModel()");
      }

      public LocalDataModel(int pageSize) {
         super(pageSize);
         log.debug("entering / exiting LocalDataModel({0})", pageSize);
      }

      private int getPageSize() {
         log.debug("entering / exiting getPageSize({0})", super.pageSize);
         return super.pageSize;
      }

      private void setPageSize(int pageSize) {
         log.debug("entering / exiting setPageSize({0})", pageSize);
         super.pageSize = pageSize;
      }

      @Override
      public DataPage fetchPage(int startRow, int pagesize) {
         log.debug("entering / exiting fetchPage({0}, {0})", startRow, pagesize);
         return getDataPage(startRow, pageSize);
      }
   }

   /**
    * @return the homePageUserData
    */
   public HomePageUserData getHomePageUserData() {
      log.debug("entering / exiting getHomePageUserData()");
      return homePageUserData;
   }



   /**
    * @param homePageUserData the homePageUserData to set
    */
   public void setHomePageUserData(HomePageUserData homePageUserData) {
      log.debug("entering / exiting setHomePageUserData({0})", homePageUserData);
      this.homePageUserData = homePageUserData;
   }



   /**
    * @return the em
    */
   public EntityManager getEm() {
      log.debug("entering / exiting getEm()");
      return em;
   }



   /**
    * @param em the em to set
    */
   public void setEm(EntityManager em) {
      log.debug("entering / exiting setEm({0})", em);
      this.em = em;
   }

   public void addObserver(RejectClaims rc) {
      log.debug("entering / exiting addObserver({0})", rc);
      this.sessionObserver = rc;
   }



   /**
    * @return the numResultChanged
    */
   public boolean isNumResultChanged() {
      boolean retVal = this.numberOfResults != getRecordCount().intValue();
      log.debug("isNumbResultChanged() returning: {0} ", retVal);
      return retVal;
   }

   @SuppressWarnings("unchecked")
   void attachRejectionHistory(RejectedClaimsTo rct) {
      Session session = (Session) em.getDelegate();
      Query qcq = em.createQuery("select q from  fClaimQueue q where q.claim.claimIndex = :index "
                                + " and q.claimQueueTypeCd = 'REJECTION_APPROVAL' "
                                + " order by q.enqueueDate desc");
      qcq.setParameter("index", rct.getClaimId());
      List<ClaimQueue> l = (List<ClaimQueue>) qcq.getResultList();
      for(ClaimQueue claimQueue : l) {
         //detach object, read only use
         session.evict(claimQueue);
      }
      rct.setRejHist(l);
   }
     
}
