package gov.va.med.fee.claim;

import gov.va.med.domain.fee.FeeClaimRoutingsReason;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import javax.ejb.Stateless;
import javax.faces.model.SelectItem;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.Factory;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Logger;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Out;
import org.jboss.seam.core.SeamResourceBundle;
import org.jboss.seam.log.Log;

@Stateless
@Name("rerouteSelections")
public class RerouteSelectionsBean implements RerouteSelections{
	
	 private static final int LINE_LENGTH_LIMIT = 120; 
   
	/** The log. */
    @Logger
    private Log log;

    /** The em. */
    @PersistenceContext
    private EntityManager em;

    @In(value = "rerouteReasonMap", required = false, scope = ScopeType.SESSION)
    @Out(value = "rerouteReasonMap", required = false, scope = ScopeType.SESSION)
    private Map<String, FeeClaimRoutingsReason> rerouteReasonMap;
	
    /**
     * Gets the reroute reason.
     *
     * @param reason the reason
     *
     * @return the reroute reason
     */
    public final FeeClaimRoutingsReason getFeeClaimRoutingsReason(String reason) {
        return rerouteReasonMap.get(reason);
    }
	
	/**
	 * Builds the list of reroute reasons from database
	 * 
	 */
		
	@SuppressWarnings("unchecked")
    private void setupRerouteReasonMap() {
        Query q = em.createQuery("from fFeeClaimRoutingsReason ");
        List<FeeClaimRoutingsReason> rr = q.getResultList();

        rerouteReasonMap = new LinkedHashMap<String, FeeClaimRoutingsReason>(rr.size());
        Map<String, FeeClaimRoutingsReason> sortedNumber = new TreeMap<String, FeeClaimRoutingsReason>();

        for (FeeClaimRoutingsReason reason : rr) {
             sortedNumber.put(reason.getTransferReasonCd(), reason);
        }

        // Put back in the list without the padding.
        for (FeeClaimRoutingsReason reason : sortedNumber.values()) {
            rerouteReasonMap.put(reason.getTransferReasonCd().trim(), reason);
        }

    }
	

    /**
     * Gets the reroute reasons.
     *
     * @return the reroute reasons
     */
    @Factory(value = "rerouteReasons", scope = ScopeType.PAGE)
    public final SelectItem[] getRerouteReasons() {
        if (rerouteReasonMap == null) {
            setupRerouteReasonMap();
        }
        int size = rerouteReasonMap.size();
        String[][] items = new String[size][2];
        int index = 0;
        for (FeeClaimRoutingsReason reason : rerouteReasonMap.values()) {
            items[index][0] = reason.getTransferReasonCd();
            items[index][1] = reason.getDescription();
            index++;
        }

        return convertToSelectItemArray(items,
                SeamResourceBundle.getBundle().getString("gov.va.med.fee.ManualRecon.SELECT_REASON"));
    }
    
    
    
    /**
     * Convert to select item array.
     *
     * @param items the items
     * @param selectText the select text
     *
     * @return the select item[]
     */
    private SelectItem[] convertToSelectItemArray(String[][] items,
            String selectText) {
        SelectItem[] ops = null;
        if (selectText != null) {
            if (items != null && items.length != 0) {
                ops = new SelectItem[items.length + 1];
                ops[0] = new SelectItem(null, selectText);
                for (int i = 0; i < items.length; i++) {
                    String key = items[i][0];
                    String value = items[i][1];
                    ops[i + 1] = new SelectItem(key, getDisplayValue(key, value));
                }
            } else {
                ops = new SelectItem[1];
                ops[0] = new SelectItem("0", selectText);
            }
        } else {
            if (items != null && items.length != 0) {
                ops = new SelectItem[items.length];
                for (int i = 0; i < items.length; i++) {
                    String key = items[i][0];
                    String value = items[i][1];
                    ops[i] = new SelectItem(key, getDisplayValue(key, value));
                }
            }
        }

        return ops;
    }
	
    /**
     * Gets the display value.
     *
     * @param code the code
     * @param value the value
     *
     * @return the display value
     */
    private String getDisplayValue(String code, String value) {
        StringBuilder sb = new StringBuilder(code);
        sb.append(" - ").append(value);
        String ret = sb.length() > LINE_LENGTH_LIMIT ? sb.substring(0,
                LINE_LENGTH_LIMIT)
                + "..." : sb.toString();
        return ret;
    }
	
}