package gov.va.med.fee.claim;

import gov.va.med.domain.fee.Claim;
import gov.va.med.domain.fee.FeeClaimPayments;
import gov.va.med.domain.fee.FeeClaimPaymentLines;
import gov.va.med.domain.fee.ServiceType;
import gov.va.med.domain.fee.PersonInfo;
import gov.va.med.domain.fee.ProviderInfo;
import gov.va.med.domain.fee.FeeClaimRouting;
import gov.va.med.domain.fee.AppUser;
import gov.va.med.fee.Page;
import gov.va.med.fee.PageControls;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Out;
import org.jboss.seam.annotations.Transactional;
import org.jboss.seam.annotations.web.RequestParameter;
import org.jboss.seam.annotations.datamodel.DataModel;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import java.util.List;
import java.util.HashSet;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.Comparator;
import static org.jboss.seam.ScopeType.SESSION;
import javax.ejb.Remove;
import javax.ejb.Stateful;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.ScopeType;
import org.jboss.seam.log.Log;
import org.jboss.seam.annotations.Logger;

import gov.va.med.fee.Constants;
import gov.va.med.fee.sec.HomePageUserData;

import java.io.Serializable;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.TreeSet;
import java.util.Iterator;
import java.util.Set;
import java.math.BigDecimal;
import java.util.Date;
import javax.faces.event.ValueChangeEvent;
import javax.faces.context.FacesContext;
import gov.va.med.domain.fee.ClaimReconciliation;
import java.util.UUID;



/**
 * ClaimSearchBean class.
 */

@Stateful
@Name("unmatchedPayments")
@Scope(SESSION)

public class UnmatchedPaymentsBean implements UnmatchedPayments, Serializable  {

	   /** The app user. */
	   @In(value = "appUser", required = true, scope = ScopeType.SESSION)
	   private AppUser appUser;	  

	   /** The homePageUserData bean. */
	   @In(value = "homePageUserData", required = false, scope = ScopeType.SESSION)
	   private HomePageUserData homePageUserData;	  
	   
	   /** The em. */
	   @PersistenceContext(unitName = "FPPS_Owner")
	   private EntityManager em;

       /** The log. */
       @Logger
       private Log log;
       
       private int numResults = 0;
       
       private List<FeeClaimPayments> payments;
       
 	   private FeeClaimPayments searchLine;       
       
       private String totalNumResults;
	
  	   public void clear(){
  		   log.debug("Clearing unmatched bean");
	   }
  	   
  	   @Remove
	   public final void destroy(){
  		   log.debug("destroying: unmatchedPayments ");
	   }

  	   public List<FeeClaimPayments> getPayments(){
  		   return payments;
  	   }
  	   
  	   public int getNumResults(){
  		   if(payments != null){
  			   return payments.size();
  		   }
  		   return 0;
  	   }

	   public FeeClaimPayments getSearchLine(){
		   return searchLine;
	   }

  	   public void search(){
  		    String sqlStr = "Select fcp from fFeeClaimPayments fcp where fcp.claimIndex not in (select cl.claimIndex from fClaim cl) and rownum < 101";
  	   		 if(searchLine.getTransactionTypeCode() != null && searchLine.getTransactionTypeCode() != ""
    			 	&& !searchLine.getTransactionTypeCode().equals("%") ){
    			 sqlStr += " and fcp.transactionTypeCode = '" + searchLine.getTransactionTypeCode() + "'";
    		 }
    		 if(searchLine.getBilledAmount() != null && searchLine.getBilledAmount().toString() != "" 
    			 	&& !searchLine.getBilledAmount().equals("%") ){
    			 sqlStr += " and fcp.billedAmount = " + searchLine.getBilledAmount().toString();
    		 } 
    		 if(searchLine.getFeeInvoiceNumber() != null && searchLine.getFeeInvoiceNumber().toString() != ""
    			    && !searchLine.getFeeInvoiceNumber().toString().equals("%") ){
    			 sqlStr += " and fcp.feeInvoiceNumber = '" + searchLine.getFeeInvoiceNumber().toString() + "' ";   			 
    		 }
    		 if(searchLine.getDiagnosisRelatedGroupCode() != null && searchLine.getDiagnosisRelatedGroupCode() != ""
    			 	&& !searchLine.getDiagnosisRelatedGroupCode().equals("%") ){
    			 sqlStr +=" and fcp.diagnosisRelatedGroupCode = '" + searchLine.getDiagnosisRelatedGroupCode().toString() + "' ";   			 
    		 }
    		 if(searchLine.getResolutionStatusCode() != null && searchLine.getResolutionStatusCode() != ""
    			 	&& !searchLine.getResolutionStatusCode().equals("%") ){
    			 sqlStr += " and fcp.resolutionStatusCode = '" + searchLine.getResolutionStatusCode() + "' ";
    		 }
    		    		    		   		
    		 if(searchLine.getFeeInvoiceDate() != null && searchLine.getFeeInvoiceDate().toString() != ""
    			 	&& !searchLine.getFeeInvoiceDate().toString().equals("%") ){
    	         /* Append the lower claim date range to the SQL statement */
    	         sqlStr += " AND fcp.feeInvoiceDate = TO_DATE('" + String.format("%1$tm/%1$td/%1$tY", searchLine.getFeeInvoiceDate()) 
                        + "', 'MM/dd/yyyy')"; 
    		 }
  		    
  		    
	  	    Query q = em.createQuery(sqlStr);
	   		payments = (List<FeeClaimPayments>) q.getResultList();
	   		
  	   }

 	  public void setSearchLine(FeeClaimPayments fcpl){
		  searchLine = fcpl;
   }


   public UnmatchedPaymentsBean(){
      if (em != null) {
    	 String activeUserStations = homePageUserData.getActiveStations().toString();
	     activeUserStations = activeUserStations.replace('[', '\'');
	     activeUserStations = activeUserStations.replace(']', '\'');
	     activeUserStations = activeUserStations.replace(", ", "\', \'");
         String sqlStr = "Select fcp from fFeeClaimPayments fcp where fcp.claimIndex " +
         				  "not in (select cl.claimIndex from fClaim cl) AND (cl.vaFacility IN ("
         				  + activeUserStations + ") ";    	   
    	  
         Query q = em.createQuery(sqlStr);
         payments = (List<FeeClaimPayments>) q.getResultList();
         numResults = payments.size();             
      }
      searchLine = new FeeClaimPayments();          
   }
  	   
}