/*
 * $Id: ViewClaim.java,v 1.11.4.1 2012/04/19 20:33:05 vhahacnguyes Exp $
 */
package gov.va.med.fee.claim;

import javax.ejb.Local;
import gov.va.med.domain.fee.Claim;
import gov.va.med.domain.fee.PersonInfo;
import gov.va.med.domain.fee.ProviderInfo;
import gov.va.med.domain.fee.PayerInfo;
import java.util.List;
import java.math.BigDecimal;

/**
 * The Interface ViewClaim.
 */
@Local
public interface ViewClaim {

	
	/**
	 * Get the claimPayment
	 */
	ClaimPaymentBean getClaimPaymentBean();
	
   /**
    * Gets the claimIndex.
    * 
    * @return the claimIndex.
    */
   long getViewClaimIndex();

   /**
    * Sets the claimIndex.
    * 
    * @param claimIndex
    *           the claimIndex to set.
    */
   void setViewClaimIndex(long viewClaimIndex);

   /**
    * Searches for the claim and sets bean properties to the claim fields (if
    * found).
    */
   String findClaimByClaimIndex(long ci);

   /**
    * Retrieves the Claim from the search.
    * 
    * @return the claim searched for.
    */
   Claim getClaim();

   /**
    * Retrieves the personInfo from the search.
    * 
    * @return the personInfo object.
    */
   PersonInfo getPerson();

   /**
    * Retrieves the service providerInfo from the search.
    * 
    * @return the service providerInfo object.
    */
   ProviderInfo getServiceProvider();

   /**
    * Retrieves the billing providerInfo from the search.
    * 
    * @return the billing providerInfo object.
    */
   ProviderInfo getBillingProvider();

   /**
    * Retrieves the payto providerInfo from the search.
    * 
    * @return the payto providerInfo object.
    */
   ProviderInfo getPaytoProvider();

   /**
    * Retrieves the AttendingPhysician providerInfo from the search.
    * 
    * @return the AttendingPhysician providerInfo object.
    */
   ProviderInfo getAttendingPhysician();

   /**
    * Retrieves the rerouting history of the claim.
    * 
    * @return List of rerouting history of the claim.
    */
   List<?> getReroutingHistory();

   /**
    * Retrieves the rejection history of the claim.
    * 
    * @return List of rejection history of the claim.
    */
   List<?> getRejectionHistory();
   
   /**
    * Retrieves the diagnosis codes of the PROF or INST claim.
    * 
    * @return List of diagnosis codes of the PROF or INST claim.
    */
   List<?> getDiagnosisCodes();

   /**
    * Retrieves the rerouting history of the claim.
    * 
    * @return List of rerouting history of the claim.
    */
   String getCoveredDays();

   /**
    * Retrieves the claim line items of the claim.
    * 
    * @return claimLineItems List.
    */
   List<?> getClaimLineItems();

   /**
    * Retrieves the totalClaimAmountPaid of the claim.
    * 
    * @return totalClaimAmountPaid of the claim.
    */
   BigDecimal getTotalClaimAmountPaid();

   /**
    * Retrieves the Provider DRG of the claim.
    * 
    * @return providerDrg
    */
   String getProviderDrg();

   /**
    * Retrieves the Secondary Payer Info of the claim.
    * 
    * @return secondaryPayerInfo
    */
   PayerInfo getSecondaryPayerInfo();

   /**
    * Retrieves the Tertiary Payer Info of the claim.
    * 
    * @return tertiaryPayerInfo
    */
   PayerInfo getTertiaryPayerInfo();

   /**
    * Print the single claim.
    */
   void printClaim();

   String processWithoutPrinting();

   String doSearch();

   String displayReroute(Long index);
   
   /**
    * Destroy.
    */
   void destroy();

   String displayDisapproval(Long index);

    boolean getNotReconciled();
}
