/*
 * $Id: FeeOptionsBean.java,v 1.8 2010/01/07 14:50:31 vhahacnguyes Exp $
 */

package gov.va.med.fee.report;

import static org.jboss.seam.ScopeType.SESSION;
import gov.va.med.domain.fee.FeeClaimRoutingsReason;
import gov.va.med.domain.fee.VaFacility;
import gov.va.med.fee.report.util.SelectableItem;
import gov.va.med.fee.sec.HomePageUserData;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.ejb.Remove;
import javax.ejb.Stateless;
import javax.faces.model.SelectItem;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.Factory;
import org.jboss.seam.annotations.Logger;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.log.Log;

/**
 * Retrieves options from database per username.
 */
@SuppressWarnings("serial")
@Stateless
@Name("feeOptions")
@Scope(SESSION)
public class FeeOptionsBean implements FeeOptions, Serializable {

    /** The log. */
    @Logger
    private Log log;

    /** The facilities. */
    private SelectItem[] facilities;

    /** The reasons. */
    private SelectItem[] reasons;

    /** The app user.
    @In(value = "appUser", required = true, scope = ScopeType.SESSION)
    private AppUser appUser;*/

    /** The em. */
    @PersistenceContext(unitName = "FPPS_Owner")
    private EntityManager em;

    /**
    * Creates a new instance of FeeOptions.
    */
    public FeeOptionsBean() {
    }

    /**
    * Gets the facility options.
    *
    * @return the facility options
    */
    @SuppressWarnings("unchecked")
    @Factory(value = "facilities", scope = ScopeType.SESSION)
    public final SelectItem[] getFacilityOptions() {
        List<VaFacility> s = em
            .createQuery(
                "select distinct fac from fVaFacility fac, fAppUser user,"
                    + " fUserRoleUsages usage where usage.id.vaFacilityCd ="
                    + " fac.vaFacilityCd and usage.id.appUserId ="
                    + " #{appUser.appUserId}"
                    + " order by fac.vaFacilityCd").getResultList();

        List<SelectableItem> list = new ArrayList<SelectableItem>();
        if (!s.isEmpty()) {
            for (VaFacility fac : s) {
                list.add(new SelectableItem(fac.getVaFacilityCd(), fac
                        .getVaFacilityName()));
            }
        }

        facilities = createOptions(list);

        return facilities;
    }

    /**
    * Gets the reason options.
    *
    * @return the reason options
    */
    @SuppressWarnings("unchecked")
    @Factory(value = "reasons", scope = ScopeType.SESSION)
    public final SelectItem[] getReasonOptions() {
        List<FeeClaimRoutingsReason> results = em.createQuery(
                "from fFeeClaimRoutingsReason").getResultList();

        List<SelectableItem> list = new ArrayList<SelectableItem>();
        if (!results.isEmpty()) {
            for (FeeClaimRoutingsReason u : results) {
                list.add(new SelectableItem(u.getTransferReasonCd(), u
                        .getDescription()));
            }
        }

        reasons = createOptions(list);

        return reasons;
    }

    /**
    * Transforms model objects to display objects.
    *
    * @param list
    *            the list
    *
    * @return the select item[]
    */
    private SelectItem[] createOptions(final List<SelectableItem> list) {

        if (list != null && list.size() != 0) {
            SelectItem[] ops = new SelectItem[list.size()];
            for (int i = 0; i < list.size(); i++) {
                SelectableItem item = list.get(i);
                String key = item.getKey();
                String value = item.getValue();
                ops[i] = new SelectItem(key, formatForDisplay(key, value));
            }
            return ops;
        }
        return null;
    }

    /**
    * Format for display.
    *
    * @param code
    *            the code
    * @param value
    *            the value
    *
    * @return the string
    */
    private String formatForDisplay(final String code, final String value) {
        return new StringBuffer(code).append(" ").append(value).toString();
    }

    /** Holds value of property username. */
    private String username;

    /**
    * Getter for property username.
    *
    * @return Value of property username.
    */
    public final String getUsername() {
        return this.username;
    }

    /**
    * Setter for property username.
    *
    * @param username
    *            New value of property username.
    */
    public final void setUsername(final String username) {
        this.username = username;
    }

    /**
    * {@inheritDoc}
    */
    @Remove
    public final void destroy() {
        log.debug("destroyed:" + this);
    }
}
