/*
 * $Id: FeeReportBean.java,v 1.4 2009/09/18 22:03:31 vhahacnguyes Exp $
 */
package gov.va.med.fee.report;

import gov.va.med.fee.FeeMessages;
import gov.va.med.fee.report.jasper.ReportGenerator;
import gov.va.med.fee.report.jasper.ReportPrepFactory;
import gov.va.med.fee.report.jasper.ReportPreparer;
import gov.va.med.fee.sec.HomePageUserData;

import java.io.Serializable;
import java.util.Map;
import java.util.Set;

import javax.ejb.Remove;
import javax.ejb.Stateful;

import org.jboss.seam.Component;
import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.Destroy;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.annotations.Synchronized;
import org.jboss.seam.international.StatusMessages;
import org.jboss.seam.international.StatusMessage.Severity;

/**
 * Backing bean for the report pages.
 */
@Stateful
@SuppressWarnings("serial")
@Name("feeReport")
@Scope(ScopeType.SESSION)
@Synchronized(timeout=300000L)
public class FeeReportBean implements FeeReport, Serializable {

    /** The criteria. */
    private Criteria criteria = new Criteria();

    /** The Constant NEXT. */
    public static final String NEXT = "next";

    /** The Constant ERROR. */
    public static final String ERROR = "error";

    /** The Constant SAME. */
    public static final String SAME = "same";

    /** Path to report template. */
    public static final String REPORT_PATH = "/WEB-INF/reports/";

    /** Report template suffix. */
    public static final String TEMPLATE_EXTENSION = ".jrxml";

    /** The show print popup. */
    private boolean showPrintPopup = false;

    /**
     * <p>
     * Construct a new Page bean instance.
     * </p>
     */
    public FeeReportBean() {
    }

    /**
     * Verifies user's input.
     *
     * @return navigation result code
     */
    public final String submitJasper() {
        if (criteria.isValid()) {
            showPrintPopup = true;
            StatusMessages.instance().addFromResourceBundle(
                    Severity.INFO,
                    "gov.va.med.fee.Reports.SUCCSESS"
                    );
            return NEXT;
        } else {
            FeeMessages.addFromResourceBundle(
                    "reportForm:dateTypeDc:dateAgeRangeTab",
                    javax.faces.application.FacesMessage.SEVERITY_ERROR,
                    "gov.va.med.fee.Reports.MISSING_DATES");
        }
        return ERROR;
    }

    /**
     * Assumes that all user's entries are valid and generates jasperreport.
     *
     * @return navigation result code
     */
    @SuppressWarnings("unchecked")
    public final String displayJasper() {
        showPrintPopup = false;
        ReportPreparer prep = new ReportPrepFactory().getPreparer(criteria);

        String template = prep.getTemplateName();
        Map p = prep.getParams();

        ReportGenerator gen = new ReportGenerator();
        if (gen.printReport(
                criteria.getFormat(),
                p,
                new StringBuffer(REPORT_PATH).append(template)
                    .append(TEMPLATE_EXTENSION).toString(),
                template)) {
            return NEXT;
        }

        return ERROR;
    }

    /**
     * Reset.
     */
    public final void reset() {
        criteria.reset();
        HomePageUserData homePageUserData = (HomePageUserData) Component.getInstance("homePageUserData");
        if (homePageUserData != null) {
            Set<String> activeStations = homePageUserData.getActiveStations();
            String[] facilities = new String[activeStations.size()];
            facilities = activeStations.toArray(facilities);
            criteria.setFacilities(facilities);
            criteria.setFromFacilities(facilities);
            criteria.setToFacilities(facilities);
        }
    }

    /**
     * Gets the criteria.
     *
     * @return the criteria
     */
    public final Criteria getCriteria() {
        return criteria;
    }

    /**
     * Sets the criteria.
     *
     * @param criteria the new criteria
     */
    public final void setCriteria(final Criteria criteria) {
        this.criteria = criteria;
    }

    /**
     * {@inheritDoc}
     */
    public boolean isShowPrintPopup() {
        return showPrintPopup;
    }

    /**
     * {@inheritDoc}
     */
    @Destroy
    @Remove
    public final void destroy() {
    }
}
