/*
 * $Id: ReportDateValidator.java,v 1.4 2009/07/08 15:07:53 vhahacnguyes Exp $
 */
package gov.va.med.fee.report;

import gov.va.med.fee.report.DateValue.FieldName;

import java.util.Date;

import org.hibernate.validator.Validator;
import org.jboss.seam.Component;

/**
 * Tests the date entries. Validators are called before the values are
 * applied to the bean.
 */
public class ReportDateValidator implements Validator<ReportDate> {

    /** The field. */
    private FieldName field;

    /**
    * {@inheritDoc}
    */
    public final void initialize(final ReportDate rd) {
        field = rd.field();
    }

    /**
    * {@inheritDoc}
    */
    public final boolean isValid(final Object value) {
        if (value == null) {
            return true;
        }

        if (value instanceof DateValue) {
            DateValue dv = (DateValue) value;
            return dv.isValid();
        } else if (value instanceof Integer && field == FieldName.DAYS) {
            Integer days = (Integer) value;
            DateValue dv = (DateValue) Component.getInstance("dateValue");
            dv.ageTab();
            dv.setDays(days);
            return dv.isValid();
        } else if (value instanceof Date) {
            // This is request scope so we can compare two dates
            DateValue dv = (DateValue) Component.getInstance("dateValue");
            dv.rangeTab();

            // This test is executed only when both values are not null.
            if (field == FieldName.FROM) {
                dv.setFromDate((Date) value);
                if (dv.getToDate() != null) {
                    return dv.isValid();
                }
            }
            if (field == FieldName.TO) {
                dv.setToDate((Date) value);
                if (dv.getFromDate() != null) {
                    return dv.isValid();
                }
            }
            return true;
        }
        return false;
    }
}
