/*
 * $Id: ReportGenerator.java,v 1.10 2009/09/02 20:16:21 vhahacnguyes Exp $
 */
package gov.va.med.fee.report.jasper;

import java.io.InputStream;
import java.sql.Connection;
import java.util.Map;

import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.persistence.EntityManager;
import javax.servlet.http.HttpServletResponse;

import net.sf.jasperreports.engine.JRExporterParameter;
import net.sf.jasperreports.engine.JRParameter;
import net.sf.jasperreports.engine.JasperCompileManager;
import net.sf.jasperreports.engine.JasperExportManager;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.JasperReport;
import net.sf.jasperreports.engine.export.JExcelApiExporter;
import net.sf.jasperreports.engine.export.JRCsvExporter;
import net.sf.jasperreports.engine.export.JRCsvExporterParameter;
import net.sf.jasperreports.engine.export.JRXlsExporterParameter;

import org.jboss.seam.Component;
import org.jboss.seam.faces.FacesMessages;
import org.jboss.seam.international.StatusMessage.Severity;
import org.jboss.seam.log.Log;
import org.jboss.seam.log.Logging;

/**
 * Generates report using Jasper Report.
 */
public class ReportGenerator {

    /** The log. */
    private final Log log = Logging.getLog(this.getClass());

    /** The faces messages. */
    private final FacesMessages facesMessages = FacesMessages.instance();

    /**
     * Instantiates a new report generator.
     */
    public ReportGenerator() {
    }

    /**
     * Prints report in PDF format.
     *
     * @param params the params
     * @param template the template
     * @param attachmentName the attachment name
     *
     * @return true, if prints the report
     */
    @SuppressWarnings("unchecked")
    public final boolean printReport(Map params, String template,
            String attachmentName) {
        return printReport("P", params, template, attachmentName);
    }

    /**
     * Prints report in specified format. P=PDF, H=HTML, C=CSV, E=Excel
     *
     * @param format the format
     * @param params the params
     * @param template the template
     * @param attachmentName the attachment name
     *
     * @return true, if prints the report
     */
    @SuppressWarnings("unchecked")
    public final boolean printReport(String format, Map params,
            String template, String attachmentName) {

        Connection c = getConnection();

        try {
            FacesContext context = FacesContext.getCurrentInstance();
            ExternalContext ext = context.getExternalContext();
            InputStream fis = ext.getResourceAsStream(template);

            JasperReport jasperReport = JasperCompileManager.compileReport(fis);

            if ("P".equals(format)) {
                params.put(JRParameter.IS_IGNORE_PAGINATION, Boolean.FALSE);

                JasperPrint jasperPrint = JasperFillManager.fillReport(
                        jasperReport, params, c);
                HttpServletResponse resp = (HttpServletResponse) ext
                        .getResponse();
                resp.setContentType("application/pdf");
                String filename = new StringBuffer(attachmentName).append(
                        ".pdf").toString();
                resp.addHeader("Content-Disposition", "inline; filename="
                        + filename);

                JasperExportManager.exportReportToPdfStream(jasperPrint, resp
                        .getOutputStream());

            } else if ("C".equals(format)) {
                params.put(JRParameter.IS_IGNORE_PAGINATION, Boolean.TRUE);

                JasperPrint jasperPrint = JasperFillManager.fillReport(
                        jasperReport, params, c);
                HttpServletResponse resp = (HttpServletResponse) ext
                        .getResponse();

                resp.setContentType("text/plain");
                String filename = new StringBuffer(attachmentName).append(
                        ".csv").toString();
                resp.setHeader("Content-Disposition", "inline;filename="
                        + filename);

                JRCsvExporter exporter = new JRCsvExporter();
                exporter.setParameter(JRCsvExporterParameter.JASPER_PRINT,
                        jasperPrint);
                exporter.setParameter(JRCsvExporterParameter.OUTPUT_STREAM,
                        resp.getOutputStream());
                exporter.setParameter(JRExporterParameter.IGNORE_PAGE_MARGINS,
                        Boolean.TRUE);
                // exporter.setParameter(JRParameter.IS_IGNORE_PAGINATION,
                // Boolean.TRUE);
                exporter.exportReport();
            } else if ("E".equals(format)) {
                JasperPrint jasperPrint = JasperFillManager.fillReport(
                        jasperReport, params, c);
                HttpServletResponse resp = (HttpServletResponse) ext
                        .getResponse();

                resp.setContentType("application/vnd.ms-excel");
                String filename = new StringBuffer(attachmentName).append(
                        ".xls").toString();
                resp.setHeader("Content-Disposition", "inline;filename="
                        + filename);

                JExcelApiExporter exporterXLS = new JExcelApiExporter();

                exporterXLS.setParameter(
                    JRXlsExporterParameter.JASPER_PRINT,
                    jasperPrint);
                exporterXLS.setParameter(
                    JRXlsExporterParameter.IS_DETECT_CELL_TYPE,
                    Boolean.TRUE);
                exporterXLS.setParameter(
                    JRXlsExporterParameter.IS_WHITE_PAGE_BACKGROUND,
                    Boolean.FALSE);
                exporterXLS.setParameter(
                    JRXlsExporterParameter.IS_REMOVE_EMPTY_SPACE_BETWEEN_ROWS,
                    Boolean.TRUE);
                exporterXLS.setParameter(
                    JRXlsExporterParameter
                        .IS_REMOVE_EMPTY_SPACE_BETWEEN_COLUMNS,
                    Boolean.TRUE);
                // exporterXLS.setParameter(
                // JRXlsExporterParameter.IS_IGNORE_CELL_BORDER,
                // Boolean.TRUE);
                exporterXLS.setParameter(JRXlsExporterParameter.OUTPUT_STREAM,
                        resp.getOutputStream());
                exporterXLS.exportReport();
            }

            context.getApplication().getStateManager().saveView(context);
            context.responseComplete();

            return true;
        } catch (Exception e) {
            log.error(e);
            facesMessages.addFromResourceBundle(
                    Severity.ERROR,
                    "gov.va.med.fee.Reports.ERROR",
                    e);
        } finally {
            try {
                c.close();
            } catch (Exception e1) {
                log.error(e1);
            }
        }
        return false;
    }

    /**
     * Returns the connection for data.
     *
     * @return jdbc connection
     */
    private Connection getConnection() {
        // AppUser appUser = (AppUser) Component.getInstance("appUser");
        EntityManager em = (EntityManager) Component
                .getInstance("FPPSEntityManager");

        Connection c = null;
        Object del = em.getDelegate();

        if (del instanceof org.jboss.seam.persistence.HibernateSessionProxy) {
            c = ((org.jboss.seam.persistence.HibernateSessionProxy) del)
                    .connection();
        }

        return c;
    }

}
