/*
 * $Id: AuthenticatorBean.java,v 1.10 2009/08/18 21:09:25 vhahacnguyes Exp $
 */
package gov.va.med.fee.sec;

import gov.va.med.domain.fppsowner.AppUser;
import gov.va.med.domain.fppsowner.SsoAuthentication;
import gov.va.med.domain.fppsowner.UserRoleUsages;

import java.util.Date;
import java.util.List;
import java.util.Set;

import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Logger;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Observer;
import org.jboss.seam.annotations.Out;
import org.jboss.seam.faces.Redirect;
import org.jboss.seam.international.StatusMessages;
import org.jboss.seam.log.Log;
import org.jboss.seam.security.Credentials;
import org.jboss.seam.security.Identity;
import org.jboss.seam.web.Session;

/**
 * The Class AuthenticatorBean.
 */
@Stateless
@Name("authenticator")
public class AuthenticatorBean implements Authenticator {

    /** The log. */
    @Logger
    private Log log;

    /** The em. */
    @PersistenceContext(unitName = "FPPS_Owner")
    private EntityManager em;

    /** The identity. */
    @In
    private Identity identity;

    /** The credentials. */
    @In
    private Credentials credentials;

    /** The app user. */
    @Out(value = "appUser", required = false, scope = ScopeType.SESSION)
    private AppUser appUser;

    /** The last login time. */
    @Out(value = "lastLoginDate", required = false, scope = ScopeType.SESSION)
    private Date lastLoginDate;

    /** The redirect. */
    @SuppressWarnings("unused")
    @In
    private Redirect redirect;

    /** The http request. */
    @SuppressWarnings("unused")
    @In
    private HttpServletRequest httpRequest;

    /** The http session. */
    @In
    private HttpSession httpSession;

    /**
    * Gets the username from asp auth.
    *
    * @return the username from asp auth
    */
    @SuppressWarnings("unchecked")
    private String getUsernameFromAspAuth() {
    	
        String username = null;
        
        log.info(httpSession.getAttribute(IDCheckFilter.GUID));
        
String guid = (String) httpSession.getAttribute(IDCheckFilter.GUID);        
System.out.println("getUsernameFromAspAuth() - guid is: " + guid);   


        Query q = em.createQuery("select t from fSsoAuthentication t "
                + "where t.guid = :guid");
        
        q.setParameter("guid", httpSession.getAttribute(IDCheckFilter.GUID));
 
      
		List<SsoAuthentication> results = q.getResultList();
		        
		        
		System.out.println("getUsernamefromAspAuth*( - resultset size returned is: " + results.size());        
		
		        if (results.size() != 0) {
		            SsoAuthentication t = results.get(0);
		            
		            
		            username = t.getNtname();
		            
		System.out.println("getUsernameFromAspAuth() - username from fSsoAuthentication db is: " + username);            
		
		            if (t != null) {
		                em.remove(t);
		                em.flush();
		            }
		        } else {
		        	System.out.println("getUsernameFromAspAuth() - error getting username from fSsoAuthentication table");
		        }
       
        
        log.info("Retrieved Username: {0} from Asp Auth", username);

        return username;
    }

    /**
    * {@inheritDoc}
    */
    public final boolean autoLogin() {
        log.info("Performing Auto Login");

        boolean isAuthenticated = false;
        String username = getUsernameFromAspAuth();

        if (username != null) {
            credentials.setUsername(username);
            credentials.setPassword("jibberish");
            isAuthenticated = true;
        } else {
            Session.instance().invalidate();
            log.fatal("Could not obtain user name from ASP.  Authentication module error.");
        }

        log.info("Auto Login {0}", isAuthenticated ? "passed" : "failed");

        return isAuthenticated;
    }

    /**
    * {@inheritDoc}
    */
    @SuppressWarnings("unchecked")
    public final boolean authenticate() {
        if (credentials.getUsername() == null) {
            return false;
        }

        System.out.println("authenticate() - credentials username is: " + credentials.getUsername());
        
        log.info("Authenticating {0}", credentials.getUsername());
        boolean retval = false;

        List<AppUser> results = em.createQuery(
                "select u from fAppUser u where upper(u.userName)="
                        + "upper(#{credentials.username})").getResultList();
        
        System.out.println("authenticate() - AppUser results size is: " + results.size());

        if (results.size() != 0) {
            appUser = results.get(0);
            
            System.out.println("authenticate() - AppUser result is: " + appUser);

            if (appUser.getEnabled().equalsIgnoreCase("Y")) {
            	
           	
            	
                Set<UserRoleUsages> userRoleUsages = appUser.getUserRoleUsageses();
                
                if (userRoleUsages.size() > 0) {
                    for (UserRoleUsages u : userRoleUsages) {
                        // load these values
                        identity.addRole(u.getAppRole().getRoleName());
                        u.getVaFacility().getVaFacilityCd();
                        retval = true;
                    }
                }
                
                lastLoginDate = appUser.getLastLoginDate();
                appUser.setLastLoginDate(new Date());
                em.merge(appUser);
                em.flush();
            } else {
            	System.out.println("authenticate() - appUser is NOT enabled"); 
            }
        }

        log.info("User: #{identity.username} {0} authentication",
                retval ? "passed" : "failed");
        
        System.out.println("authenticate() - returning authentication value of: " + retval);
        
        
        return retval;
    }

    /**
    * {@inheritDoc}
    */
    @Observer("org.jboss.seam.security.postAuthenticate")
    public final void clearLoginMsg() {
        StatusMessages.instance().clear();
    }
}
