/*
 * $Id: HomePageBean.java,v 1.8 2010/01/06 15:03:04 vhahacnguyes Exp $
 */
package gov.va.med.fee.sec;

import gov.va.med.domain.fee.AppUser;
import gov.va.med.domain.fee.UserRoleUsages;

import java.util.Date;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;

import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.Factory;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Logger;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Out;
import org.jboss.seam.core.Events;
import org.jboss.seam.log.Log;

/**
 * The Class HomePageBean.
 */
@Stateless
@Name("homePage")
public class HomePageBean implements HomePage {

    /** The log. */
    @Logger
    private Log log;

    /** The em. */
    @PersistenceContext(unitName = "FPPS_Owner")
    private EntityManager em;

    /** The events. */
    @In
    private Events events;

    /** The app user. */
    @In(value = "appUser", required = false, scope = ScopeType.SESSION)
    private AppUser appUser;

    /** The last login time. */
    @In(value = "lastLoginDate", required = false, scope = ScopeType.SESSION)
    private Date lastLoginDate;

    /** The home page user data. */
    @In(value = "homePageUserData", required = false, scope = ScopeType.SESSION)
    @Out(value = "homePageUserData", required = false, scope = ScopeType.SESSION)
    private HomePageUserData homePageUserData;

    /**
     * {@inheritDoc}
     */
    public final void submit() {
        refresh();
    }

    /**
     * {@inheritDoc}
     */
    public final void refresh() {
        getHomePageUserData();
    }

    /**
     * {@inheritDoc}
     */
    @Factory
    public final HomePageUserData getHomePageUserData() {
        Set<String> activeStations = null;
        if (homePageUserData != null) {
            activeStations = homePageUserData.getActiveStations();
        }

        homePageUserData = new HomePageUserData();
        homePageUserData.setUserName(appUser.getUserName());
        homePageUserData.setLastLogin(lastLoginDate);

        Map<String, UserFacilityData> userFacilities = new TreeMap<String, UserFacilityData>();
        Set<UserRoleUsages> userRoleUsages = appUser.getUserRoleUsages();
        if (userRoleUsages.size() > 0) {
            for (UserRoleUsages u : userRoleUsages) {
                String facility = u.getVaFacility().getVaFacilityCd();
                String role = u.getAppRole().getRoleName();

                UserFacilityData facilityData = userFacilities.get(facility);
                if (facilityData == null) {
                    facilityData = new UserFacilityData();
                }

                if (activeStations != null) {
                    // retain selected stations
                    if (activeStations.contains(facility)) {
                        facilityData.setActive(true);
                    } else {
                        facilityData.setActive(false);
                    }
                } else {
                    // on login all stations are selected by default
                    facilityData.setActive(true);
                }

                facilityData.addRole(role);
                facilityData.setFacilityCode(facility);

                userFacilities.put(facility, facilityData);
            }
        }

        Map<String, UserFacilityData> homePageFacilities = homePageUserData.getUserFacilities();
        homePageFacilities.putAll(userFacilities);

        homePageUserData.setActiveVisns(em);

        // add a visn level at top
        if (homePageUserData.hasVisnLevelAccess()) {
            homePageFacilities.clear();
            homePageFacilities.put(null, new UserFacilityData());
            homePageFacilities.putAll(userFacilities);
        }

        homePageUserData.getClaimCounts(em);
        homePageUserData.setPrivacyRestrictions(em);

        return homePageUserData;
    }
}
