package gov.va.med.fee.tool;

import gov.va.med.domain.fee.IcdCode;

import java.util.Arrays;

import org.jboss.seam.annotations.Name;
import org.jboss.seam.framework.EntityQuery;


/**
 * The IcdCodeList class.
 */
@Name("icdCodeList")
public class IcdCodeList extends EntityQuery<IcdCode> {
    /** sets the maximum number of results to be returned. */
    private static final int MAX_RESULTS = 15;

    /** the query string used to get the icdCode. */
    private static final String EJBQL = "select icdCode from fIcdCode icdCode";

    /** string query to restrict the icd codes returned. */
    private static final String[] RESTRICTIONS = {
            "lower(icdCode.codeType) like concat(lower(#{icdCodeList.icdCode.codeType}),'%')",
            "lower(icdCode.icdCode) like concat(lower(#{icdCodeList.icdCode.icdCode}),'%')",
            "lower(icdCode.icdDesc) like concat(lower(#{icdCodeList.icdCode.icdDesc}),'%')",
        };

    /** instance of the icd code. */
    private final IcdCode icdCode = new IcdCode();

    /**
     * Default constructor.
     */
    public IcdCodeList() {
        setEjbql(EJBQL);
        setRestrictionExpressionStrings(Arrays.asList(RESTRICTIONS));
        setMaxResults(MAX_RESULTS);
    }

    /**
     * Overrides to return the correct persistent context name for this app.
     *
     * @return persistence context name
     */
    @Override
    protected final String getPersistenceContextName() {
       return "FPPSEntityManager";
    }

    /**
     * Gets the ICD code.
     *
     * @return ICD CODE.
     */
    // !!!DO NOT MAKE THIS METHOD FINAL!!!  IT WILL BREAK THE SEARCH.
    public IcdCode getIcdCode() {
        return icdCode;
    }

}
