/** $Id: UserAdmin.java,v 1.13 2010/07/19 19:04:19 vhahacdoansc Exp $ */
package gov.va.med.fee.tool;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import gov.va.med.domain.fee.AppUser;
import gov.va.med.domain.fee.AppUserRequest;

import javax.ejb.Local;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import javax.faces.model.SelectItem;

/**
 * This class acts as a transport object for User search form for
 * the user administration beans.
 * 
 * @author vhahacmagdua
 *
 */
@Local
public interface UserAdmin {
   

	
   /**
    * This method grabs the roles to popuate the Roles in the 
    * selection box in the User Admin page.
    */
   void populateRoles();
   
   /**
    * This method grabs the available facilities to populate the
    * selection box in the User Admin page. 
    */
   void populateStations();
   
   /**
    * Performs a search of Users.
    * 
    * @param user User search criteria.
    */
   void searchUsers();
   
   /**
    * Updates the Screens backing model.
    * 
    * @param ev The action event 
    */
   void update(ActionEvent ev);
   
   
   /**
    * Selects the user given the users nt name.  The population
    * of the user is to be done by populating an instance variable 
    * in the implemeting class.
    * 
    * @param user the application user's NT name.
    */
   void selectUser(String user);
   
   
   
   void setUserToCreate(AppUser userToCreate);   
   AppUser getUserToCreate();
   void setUserToSearch(AppUser userToSearch);
   AppUser getUserToSearch();
   void setUserToEdit(AppUser userToEdit);
   AppUser getUserToEdit();
   
   void setUserFacilitiesToSearch(String userFacilitiesToSearch);   
   String getUserFacilitiesToSearch();
   
   void setUserToCreateEnabled(boolean userToCreateEnabled);
   boolean getUserToCreateEnabled();
   String createUser();
   List<AppUser> getUserSearchResults();
   String findUserToEditByUserId(long appUserId);
   
   void setUserToEditEnabled(boolean userToEditEnabled);
   boolean getUserToEditEnabled();
   String updateUserDetails();
   String updateUserAssignments();
   
   List<SelectItem> getRolesAvailableToUser();
   void setRolesAvailableToUser(List<SelectItem> rolesAvailableToUser);
   List<String> getUserRoles();
   void setUserRoles(List<String> userRoles);
   List<String> getUserRolesForNewUser();
   void setUserRolesForNewUser(List<String> userRolesForNewUser);
   
   List<SelectItem> getFacilitiesAvailableToUser();
   void setFacilitiesAvailableToUser(List<SelectItem> facilitiesAvailableToUser);
   List<String> getUserFacilities();
   void setUserFacilities(List<String> userFacilities);
   List<String> getUserFacilitiesForNewUser();
   void setUserFacilitiesForNewUser(List<String> userFacilitiesForNewUser);
   
   String cancelModify();
   
   
   AppUserRequest getAppUserRequest();
   void setAppUserRequest(AppUserRequest aur);
   List<AppUserRequest> getUserAccountRequests();
   String openUserAccountRequest(Long id); 
   //String markRequestComplete(Long id); // changed method to updateRequestStatus
   String updateRequestStatus(Long id, int status);
   int getRequestCount();
   
   int getUSER_REQUEST_STATUS_NEW();
   int getUSER_REQUEST_STATUS_COMPLETE();	       
   int getUSER_REQUEST_STATUS_PENDING_CAAP();
   
   
   boolean userRolesContains(String role);
   
   /**
    * Gets the sort column associated with the claim search.
    * 
    * @return the sort column associated with the claim search.
    */
   String getSortCol();
   
   /**
    * Sets the sort column associated with the claim search.
    * 
    * @param sortCol
    *             the sort column.
    */
   void setSortCol(String sortCol);
   
   /**
    * Gets the sort direction associated with the claim search.
    * 
    * @return the sort direction associated with the claim search.
    */
   String getSortDir();
   
   /**
    * Sets the sort direction associated with the claim search.
    * @param sortDir
    *            the sort direction.
    */
   void setSortDir(String sortDir);
   
   
   
   
   void validateTextAreaLength(FacesContext context, UIComponent component, Object obj);
   void destroy();
   
}
