/** $Id: UserAdminSessionBean.java,v 1.21 2010/08/16 21:26:07 vhahacmagdua Exp $ */
package gov.va.med.fee.tool;

import static org.jboss.seam.ScopeType.SESSION;
import gov.va.med.domain.fee.AppUser;
import gov.va.med.domain.fee.AppUserRequest;
import gov.va.med.domain.fee.Claim;
import gov.va.med.fee.sec.UserFacilityData;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.ejb.Remove;
import javax.ejb.Stateless;
import javax.ejb.Stateful;
import javax.faces.application.FacesMessage;
import javax.faces.component.UIComponent;
import javax.faces.component.UIViewRoot;
import javax.faces.component.html.HtmlInputTextarea;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import javax.faces.model.SelectItem;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.Destroy;
import org.jboss.seam.annotations.Factory;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Logger;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Out;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.annotations.datamodel.DataModel;
import org.jboss.seam.annotations.security.Restrict;
import org.jboss.seam.annotations.web.RequestParameter;
import org.jboss.seam.log.Log;
import org.jboss.seam.security.Identity;
import org.jboss.seam.international.StatusMessages;
import org.jboss.seam.international.StatusMessage.Severity;

/**
 *  This class User Administration requests.
 */
@SuppressWarnings("serial")
@Stateful
@Name("userAdmin")
@Scope(SESSION)
@Restrict("#{s:hasRole('ADMIN') or s:hasRole('FPPS_HELP_DESK')}")
public class UserAdminSessionBean implements UserAdmin, Serializable {
	
	
	private static final int USER_REQUEST_STATUS_NEW = 0;
	private static final int USER_REQUEST_STATUS_COMPLETE = 1;
	private static final int USER_REQUEST_STATUS_PENDING_CAAP = 2;
	
	
	
   /** The log. */
   @Logger
   private Log log;

   /** The Entity Manager. */
   @PersistenceContext(unitName = "FPPS_Owner")
   private EntityManager em;
   
   /** The identity. */
   @In
   private Identity identity;

   /** list to populate the roles selection box. */
   @Out(required = false, scope = ScopeType.SESSION)
   List<String> rolesToSelect;

   /** list to populate the station selection box. */
   @Out(required = false, scope = ScopeType.SESSION)
   List<String> stationsToSelect;
   
   /** list to populate users found. */
   @Out(required = false, scope = ScopeType.SESSION)
   List<String> userMatches = new ArrayList<String>();
  
   /** list of roles and facility for a given user. */
   @Out(required = false, scope = ScopeType.SESSION)
   List<String> rolesFacilities = new ArrayList<String>();
   
   /** The collection of results. */
   @DataModel
   private List<AppUser> userSearchResults = new LinkedList<AppUser>();
   
   /** the user backing bean for performing user searches. */
   @In(value = "search", create = true)
   User search;
   
   /** the user backing bean for editing user info. */
   @In(value = "edit", create = true)
   User edit;
   
   /** the AppUser object to for creating a new user */
   AppUser userToCreate;
   
   /** the AppUser object to for searching for a user */
   private AppUser userToSearch;
   
   /** the User facilities to search for a user */
   String userFacilitiesToSearch;
   
   /** the AppUser object to for searching for a user */
   private AppUser userToEdit;
   
   /** The app user. */
   @In(value = "appUser", required = true, scope = ScopeType.SESSION)
   private AppUser appUser;
   
   
   
   
   
   
   
   /**
    * Default constructor. 
    */
   public UserAdminSessionBean() {
      rolesFacilities.add("");
      userToCreate = new AppUser();
      userToSearch = new AppUser();
      userToEdit = new AppUser();
   }
   
   
   /**
    * {@inheritDoc}
    */
   @SuppressWarnings("unchecked")
   @Factory("rolesToSelect")
   @Override
   public final void populateRoles() {
      Query q = em.createNativeQuery("select role_name from app_role order by role_name");
      rolesToSelect = new ArrayList<String>();
      List<String> l = q.getResultList();
      java.util.Iterator<String> i = l.iterator();
      while (i.hasNext()) {
         String str = i.next();
         rolesToSelect.add(str.replaceAll("_", " "));
      }
   }

   /**
    * {@inheritDoc}
    */
   @SuppressWarnings("unchecked")
   @Factory("stationsToSelect")
   @Override
   public final void populateStations() {
      Query q = em
            .createNativeQuery("select upper(va_facility_name) from va_facility where va_type_cd = 'FAC' order by va_facility_name");
      stationsToSelect = q.getResultList();
   }
   
   
   
   
   private HashMap<String, String> searchParametersToSet = new HashMap<String, String>();
   private int classCounter=0;
   
   /**
    * A method to build narrowing clauses in the SQL statement. Determines if a
    * database string field is set by the user. If so, append it appropriately
    * using 'LIKE' or '=' depending on ' whether or not there is an Oracle
    * wildcard in the user defined criteria.
    * 
    * @return a SQL clause
    * @param searchParam
    *           the search parameter to evaluate.
    * @param sqlColumn
    *           the database column to apply the search parameters to.
    * 
    */
   private String appendSqlStringParameters(String searchParam, String prefix, String suffix) {
      String ret = "";
      String param = "p" + classCounter++;
      log.debug("searchParam: " + searchParam);
      if (searchParam != null && !searchParam.trim().equals("") && !searchParam.trim().equals("%")
            && !searchParam.trim().equals("_")) {
         if (searchParam.trim().indexOf("%") >= 0 || searchParam.trim().indexOf("_") >= 0) {
            ret += " " + prefix + " LIKE :" + param + " " + suffix;
            searchParametersToSet.put(param, searchParam.toUpperCase());
         } 
         else {
            ret += " " + prefix + " = :" + param + " " + suffix;
            searchParametersToSet.put(param, searchParam.toUpperCase());
         }
      }
      log.debug("ret: " + ret);
      return ret;
   }

   
   
   

   /**
    * {@inheritDoc}
    */
   @SuppressWarnings("unchecked")
   @Override
   public final void searchUsers() {
      searchParametersToSet = new HashMap<String, String>();
      this.userSearchResults.clear();
      //resetUser(this.edit);      
      List<?> l;
      Iterator it;
      String sql;
      Query q;
      
      if (this.sc != null && !this.sc.equalsIgnoreCase("")) {
          this.sortCol = this.sc;
       }
       if (this.sd != null && !this.sd.equalsIgnoreCase("")) {
          this.sortDir = this.sd;
       }

         
        sql = " select a.app_user_id, " 
                 + " UPPER(a.user_name), "
                 + " UPPER(a.last_name), "
                 + " UPPER(a.first_name), "
                 + " UPPER(a.enabled), "
                 + " a.last_login_date, "
                 + " UPPER(a.phone_number), "
                 + " UPPER(a.email_address) "
                 + " from app_user a "
                 + " WHERE a.app_user_id is not null ";
         
         sql += appendSqlStringParameters(this.userToSearch.getUserName().trim().toUpperCase(), "AND UPPER(a.user_name)", "");
         sql += appendSqlStringParameters(this.userToSearch.getLastName().trim().toUpperCase(), "AND UPPER(a.last_name)", "");
         sql += appendSqlStringParameters(this.userToSearch.getFirstName().trim().toUpperCase(), "AND UPPER(a.first_name)", "");
         sql += appendSqlStringParameters(this.userToSearch.getPhoneNumber().trim().toUpperCase(), "AND UPPER(a.phone_number)", "");
         sql += appendSqlStringParameters(this.userToSearch.getEmailAddress().trim().toUpperCase(), "AND UPPER(a.email_address)", "");
         sql += appendSqlStringParameters(this.userFacilitiesToSearch.trim().toUpperCase(), "AND a.app_user_id IN (SELECT u.app_user_id FROM user_role_usages u WHERE u.va_facility_cd", ")");
                 
         
         
         /* Build the SQL ORDER BY clause. */
         String orderBy = "";

         if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortusername")) {
             orderBy = " ORDER BY UPPER(a.user_name) " + this.sortDir;
          }
         if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortlastname")) {
             orderBy = " ORDER BY UPPER(a.last_name) " + this.sortDir;
          }
         if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortfirstname")) {
             orderBy = " ORDER BY UPPER(a.first_name) " + this.sortDir;
          }
         if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortactive")) {
             orderBy = " ORDER BY UPPER(a.enabled) " + this.sortDir;
          }
         if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortlastlogin")) {
             orderBy = " ORDER BY a.last_login_date " + this.sortDir;
          }
         if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortphone")) {
             orderBy = " ORDER BY UPPER(a.phone_number) " + this.sortDir;
          }
         if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortemail")) {
             orderBy = " ORDER BY UPPER(a.email_address) " + this.sortDir;
          }
         
         // add default columns to ORDER BY if they haven't been specified in the
         // clause already         
         if (orderBy.indexOf("ORDER BY ") <= 0) {
            orderBy += " ORDER BY a.user_name ASC ";
            this.sortCol = "sortusername";
            this.sortDir = "asc";
         }
  
         sql+=orderBy;
         
         q = em.createNativeQuery(sql);
         
         for(String key : searchParametersToSet.keySet()) {
            q.setParameter(key, searchParametersToSet.get(key));
         }
         
         l = q.getResultList();
         it = l.iterator();

         
         /* loop through the results. */
         while (it.hasNext()) {
             AppUser au = new AppUser();
             Object[] row = (Object[]) it.next();
             BigDecimal aid = (BigDecimal) row[0];
             au.setAppUserId(aid.longValue());
             au.setUserName((String) row[1]);
             au.setLastName((String) row[2]);
             au.setFirstName((String) row[3]);
             au.setEnabled((String) row[4]);             
             au.setLastLoginDate((Date) row[5]);
             au.setPhoneNumber((String) row[6]);
             au.setEmailAddress((String) row[7]);
             
             userSearchResults.add(au);
         }
   
   }
   
   /**
    * {@inheritDoc}
    */
   @Override
   public final void selectUser(String username) {
      log.info("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<AJM: " + username);
      edit.setUserName(username);
      Query query = em.createQuery("select u from fAppUser u where userName = :username").setParameter("username", username);
      AppUser user = (AppUser) query.getSingleResult();
      edit.setUserName(user.getUserName());
      edit.setFirstName(user.getFirstName());
      edit.setLastName(user.getLastName());
      edit.setEmailAddress(user.getEmailAddress());
   }

   
   /**
    * Creates a case sensitive where clause.
    * 
    * @param wherecls previously partially generated where clause.
    * @param column the column's name.
    * @param value the value to check for.
    * @return the updated where clause.
    */
   final StringBuilder genWhereCls(StringBuilder wherecls, String column, String value) {
      if (wherecls == null) {
        wherecls =  new StringBuilder().append("WHERE "); 
      } else {
        wherecls.append("AND ");
      }
      wherecls.append(column).append(" = '").append(value).append("' ");
      log.debug("in gen where clause: {0}", wherecls);
      return wherecls;
   }
   
   /**
    * Creates a case insensitive where clause (Oracle specific).
    * 
    * @param wherecls previously partially generated where clause.
    * @param column the columns name.
    * @param value the value to check for.
    * @return the updated where clause.
    */
   final StringBuilder genCiWhereCls(StringBuilder wherecls, String column, String value) {
      if (wherecls == null) {
         wherecls =  new StringBuilder().append("WHERE "); 
       } else {
         wherecls.append("AND ");
       }
       wherecls.append(" REGEXP_LIKE( ").append(column).append(",'^").append(value).append("$', 'i')  ");
       log.debug("in gen where clause: {0}", wherecls);
       return wherecls;     
   }
   
   /**
    * {@inheritDoc}
    */
   @Override
   public final void update(ActionEvent event) {
      FacesContext context = FacesContext.getCurrentInstance();
      UIViewRoot root = context.getViewRoot();
      root.processUpdates(context);
   }
   
   
   /**
    * Clears the data from the user object passed in.
    * 
    * @param u <code>User</code> object to clear.
    */
   final void resetUser(User u) {
      u.setUserName(null);
      u.setAppUserId(0);
      u.setCreatedBy(null);
      u.setDateCreated(null);
      u.setDateModified(null);
      u.setEmailAddress(null);
      u.setEnabled(null);
      u.setFirstName(null);
      u.setLastName(null);
      u.setMiddleName(null);
      u.setModifiedBy(null);
      u.setPhoneNumber(null);
      u.setUserName(null);
      u.setUserRoleUsages(null);
   }
   
   
   
   
   public void setUserToCreate(AppUser userToCreate){
       this.userToCreate = userToCreate;
   }
   
   public AppUser getUserToCreate(){
       return this.userToCreate;
   }
   
   public void setUserToSearch(AppUser userToSearch){
       this.userToSearch = userToSearch;
   }
   
   public AppUser getUserToSearch(){
       return this.userToSearch;
   }
   
   public void setUserToEdit(AppUser userToEdit){
       this.userToEdit = userToEdit;
   }
   
   public AppUser getUserToEdit(){
       return this.userToEdit;
   }
   
   public void setUserFacilitiesToSearch(String userFacilitiesToSearch){
       this.userFacilitiesToSearch = userFacilitiesToSearch;
   }
   
   public String getUserFacilitiesToSearch(){
       return this.userFacilitiesToSearch;
   }
   
   
   @RequestParameter
   private String sd;

   public final String getSd() {
      return this.sd;
   }

   public final void setSd(String sd) {
      this.sd = sd;
   }

   @RequestParameter
   private String sc;

   public final String getSc() {
      return this.sc;
   }

   public final void setSc(String sc) {
      this.sc = sc;
   }
   
   /** The column to sort by. */
   private String sortCol;

   /**
    * Gets the sort column.
    * 
    * @return the sortCol.
    */
   public final String getSortCol() {
      return this.sortCol;
   }

   /**
    * Sets the sort column.
    * 
    * @param sortCol
    *           the sort column.
    */
   public final void setSortCol(String sortCol) {
      this.sortCol = sortCol;
   }

   /** The direction to sort. */
   private String sortDir;

   /**
    * Gets the sort direction.
    * 
    * @return the sortDir.
    */
   public final String getSortDir() {
      return this.sortDir;
   }

   /**
    * Sets the sort direction.
    * 
    * @param sortDir
    *           the sort direction.
    */
   public final void setSortDir(String sortDir) {
      this.sortDir = sortDir;
   }
   
   /**
    * A boolean to hold the value from the search page. Can't map directly to
    * the hibernate object because the ENABLED field is not boolean, but varchar2(1)
    * */
   private boolean userToCreateEnabled = true;

   /**
    * Sets the userToCreateEnabled.
    * 
    * @param userToCreateEnabled
    *           the userToCreateEnabled flag.
    */
   public final void setUserToCreateEnabled(boolean userToCreateEnabled) {
      this.userToCreateEnabled = userToCreateEnabled;
   }

   /**
    * Gets the userToCreateEnabled flag.
    * 
    * @return the userToCreateEnabled.
    */
   public final boolean getUserToCreateEnabled() {
      return this.userToCreateEnabled;
   }
   
   /**
    * A boolean to hold the value from the search page. Can't map directly to
    * the hibernate object because the ENABLED field is not boolean, but varchar2(1)
    * */
   private boolean userToEditEnabled = true;

   /**
    * Sets the userToEditEnabled.
    * 
    * @param userToEditEnabled
    *           the userToEditEnabled flag.
    */
   public final void setUserToEditEnabled(boolean userToEditEnabled) {
      this.userToEditEnabled = userToEditEnabled;
   }

   /**
    * Gets the userToEditEnabled flag.
    * 
    * @return the userToEditEnabled.
    */
   public final boolean getUserToEditEnabled() {
      return this.userToEditEnabled;
   }
   
   public final List<AppUser> getUserSearchResults() {       
       return this.userSearchResults;
   }
   
   
   
   public int getUSER_REQUEST_STATUS_NEW(){
		return USER_REQUEST_STATUS_NEW;
	}
	public int getUSER_REQUEST_STATUS_COMPLETE(){
		return USER_REQUEST_STATUS_COMPLETE;
	}
	public int getUSER_REQUEST_STATUS_PENDING_CAAP(){
		return USER_REQUEST_STATUS_PENDING_CAAP;
	}
	
   
   public String createUser(){
       
           String ret = "createUserFAIL";
           boolean success = false;
           String enabled = "N"; 
           String sql;
           Query q;

           sql = " SELECT COUNT(*) FROM fpps_owner.app_user p "
                   + " WHERE p.user_name=UPPER(:upperntname)";
           q = em.createNativeQuery(sql);
           q.setParameter("upperntname", this.userToCreate.getUserName().trim().toUpperCase());         
           BigDecimal recordCount = (BigDecimal) q.getSingleResult();

           if (recordCount.intValue() > 0) {
               StatusMessages.instance().addFromResourceBundle(Severity.ERROR,
                       "gov.va.med.fee.userAdmin.CREATE_USER_ERROR_ON_EXISTING",
                       "");
               return ret;
           } else {
        	   sql = "SELECT fpps_owner.mig.nextval FROM dual";
        	   
               q = em.createNativeQuery(sql);                 
               BigDecimal seqId = (BigDecimal)q.getSingleResult();
                    	   
               enabled = this.userToCreateEnabled ? "Y" : "N";
               sql = " INSERT INTO fpps_owner.app_user(app_user_id, " 
               		    + " user_name, " 
                   		+ " enabled, " 
                   		+ " last_name, " 
                   		+ " first_name, " 
                   		+ " middle_name, " 
                   		+ " phone_number, " 
                   		+ " email_address, " 
                   		+ " created_by, " 
                   		+ " date_created) "
                        + " VALUES("+seqId+", " 
                    	+ ":ntname, "
                    	+ ":enabled, " 
                    	+ ":lastname, "
                    	+ ":firstname, "
                    	+ ":middlename, "
                    	+ ":phone, "
                    	+ ":email, "
                    	+ ":createdby, "
                    	+ "sysdate) ";
               q = em.createNativeQuery(sql);
               q.setParameter("ntname", this.userToCreate.getUserName().trim().toUpperCase());
               q.setParameter("enabled", enabled);
               q.setParameter("lastname", this.userToCreate.getLastName().trim().toUpperCase());
               q.setParameter("firstname", this.userToCreate.getFirstName().trim().toUpperCase());
               q.setParameter("middlename", this.userToCreate.getMiddleName().trim().toUpperCase());
               q.setParameter("phone", this.userToCreate.getPhoneNumber().trim().toUpperCase());
               q.setParameter("email", this.userToCreate.getEmailAddress().trim().toUpperCase());
               q.setParameter("createdby", appUser.getUserName().trim().toUpperCase());

               q.executeUpdate();
               q = em.createNativeQuery("COMMIT");
               q.executeUpdate();
               
               
               
               for(String role : this.userRolesForNewUser){           
                   for(String fac : this.userFacilitiesForNewUser){                 
                        sql = " INSERT INTO fpps_owner.user_role_usages(app_role_id, app_user_id, va_facility_cd, primary_indicator, created_by, date_created) "
                            + " VALUES(:role, :userid, :facility, 'Y', :createdby, sysdate)";
                            
                            q = em.createNativeQuery(sql);
                            q.setParameter("role", role);       
                            q.setParameter("userid", seqId);  
                            q.setParameter("facility", fac);  
                            q.setParameter("createdby", appUser.getUserName());  
                            
                            q.executeUpdate();
                            q = em.createNativeQuery("COMMIT");
                            q.executeUpdate();              
                   }
               }
               
               this.setUserRolesForNewUser(new ArrayList());               
               this.setUserFacilitiesForNewUser(new ArrayList());
               
               
               ret = "createUserSUCCESS";
               success = true;

              this.userToCreate = new AppUser();
           }

           if (success) {
               StatusMessages.instance().addFromResourceBundle(Severity.INFO,
                       "gov.va.med.fee.userAdmin.CREATE_USER_SUCCESS", "");
           } else {
               StatusMessages.instance().addFromResourceBundle(Severity.ERROR,                       
                       "gov.va.med.fee.userAdmin.CREATE_USER_ERROR", "");
           }

           return ret;
       
   }
   
   
   public String updateUserDetails(){
       String ret = "updateUserDetailsFAIL";
       boolean success = false;
       String enabled = "N"; 
       String sql;
       Query q;
             
       
       sql = " SELECT COUNT(*) FROM fpps_owner.app_user p "
           + " WHERE p.user_name=UPPER(:upperntname) "
           + " AND p.app_user_id <> :userid";
       q = em.createNativeQuery(sql);
       q.setParameter("upperntname", this.userToEdit.getUserName().trim().toUpperCase());         
       q.setParameter("userid", this.userToEdit.getAppUserId());
       BigDecimal recordCount = (BigDecimal) q.getSingleResult();
        
       if (recordCount.intValue() > 0) {
           StatusMessages.instance().addFromResourceBundle(Severity.ERROR,
                   "gov.va.med.fee.userAdmin.UPDATE_USER_ERROR_ON_EXISTING",
                   "");
           return ret;
       } else {
           enabled = this.userToEditEnabled ? "Y" : "N";
           sql = " UPDATE fpps_owner.app_user SET " 
                    + " user_name=:ntname, " 
                    + " enabled=:enabled, " 
                    + " last_name=:lastname, " 
                    + " first_name=:firstname, " 
                    + " middle_name=:middlename, " 
                    + " phone_number=:phone, " 
                    + " email_address=:email, " 
                    + " deactivation_comments=:deactivationcomments, "
                    + " modified_by=:modifiedby, " 
                    + " date_modified=sysdate "
                    + " WHERE app_user_id=:appuserid";
                
           q = em.createNativeQuery(sql);
           q.setParameter("ntname", this.userToEdit.getUserName().trim().toUpperCase());
           q.setParameter("enabled", enabled);
           
           //only allow disable comments if user is disabled
           if(!enabled.equalsIgnoreCase("Y")){
               q.setParameter("deactivationcomments", this.userToEdit.getDeactivationComments().trim().toUpperCase());
           }
           else{
               q.setParameter("deactivationcomments", "");
           }
           
           q.setParameter("lastname", this.userToEdit.getLastName().trim().toUpperCase());
           q.setParameter("firstname", this.userToEdit.getFirstName().trim().toUpperCase());
           q.setParameter("middlename", this.userToEdit.getMiddleName().trim().toUpperCase());
           q.setParameter("phone", this.userToEdit.getPhoneNumber().trim().toUpperCase());
           q.setParameter("email", this.userToEdit.getEmailAddress().trim().toUpperCase());
           q.setParameter("modifiedby", appUser.getUserName().trim().toUpperCase());
           q.setParameter("appuserid",this.userToEdit.getAppUserId());
    
           q.executeUpdate();
           q = em.createNativeQuery("COMMIT");
           q.executeUpdate();
           ret = "updateUserDetailsSUCCESS";
           success = true;             
       }
    
       if (success) {
           StatusMessages.instance().addFromResourceBundle(Severity.INFO,
                   "gov.va.med.fee.userAdmin.UPDATE_USER_SUCCESS", "");
       } else {
           StatusMessages.instance().addFromResourceBundle(Severity.ERROR,                       
                   "gov.va.med.fee.userAdmin.UPDATE_USER_ERROR", "");
       }
       
       
       searchUsers();
       findUserToEditByUserId(this.userToEdit.getAppUserId());
       
       return ret;
   }
   
   
   public String updateUserAssignments(){
       String ret = "updateUserAssignmentsFAIL";
       boolean success = false;
       String enabled = "N"; 
       String sql;
       Query q;
             
       
       sql = " DELETE " 
           + " FROM fpps_owner.user_role_usages uru "
           + " WHERE uru.app_user_id=:userid";           
      
       q = em.createNativeQuery(sql);
       q.setParameter("userid",this.userToEdit.getAppUserId());       
       q.executeUpdate();
       q = em.createNativeQuery("COMMIT");
       q.executeUpdate();
       
       for(String role : this.userRoles){           
           for(String fac : this.userFacilities){                 
                sql = " INSERT INTO fpps_owner.user_role_usages(app_role_id, app_user_id, va_facility_cd, primary_indicator, created_by, date_created) "
                    + " VALUES(:role, :userid, :facility, 'Y', :createdby, sysdate)";
                    
                    q = em.createNativeQuery(sql);
                    q.setParameter("role", role);       
                    q.setParameter("userid",this.userToEdit.getAppUserId());  
                    q.setParameter("facility", fac);  
                    q.setParameter("createdby", appUser.getUserName());  
                    
                    q.executeUpdate();
                    q = em.createNativeQuery("COMMIT");
                    q.executeUpdate();              
           }
       }
       
       success = true;
       ret = "updateUserAssignmentsSUCCESS";
              

       if (success) {
           StatusMessages.instance().addFromResourceBundle(Severity.INFO,
                   "gov.va.med.fee.userAdmin.UPDATE_USER_SUCCESS", "");
       } else {
           StatusMessages.instance().addFromResourceBundle(Severity.ERROR,                       
                   "gov.va.med.fee.userAdmin.UPDATE_USER_ERROR", "");
       }
       
       searchUsers();
       findUserToEditByUserId(this.userToEdit.getAppUserId());
       
       return ret;
   }
   
     
   
   public boolean userRolesContains(String role){
	   
	   boolean ret = false;
	   
	   //get the role ID of the specified role 
	   String sql = "SELECT DISTINCT(a.app_role_id) "          
           + " FROM "
           + "   app_role a "
           + " WHERE "
           + " a.role_name = :rolename "
           + " and rownum=1 ";
	   
       Query q = em.createNativeQuery(sql);       
       q.setParameter("rolename", role);
       BigDecimal rid = (BigDecimal) q.getSingleResult();

       //if the role exists, compare with the userRoles collection
       if(rid!=null){
    	   for(String roleId : userRoles){
    		   //if a match is found, the user has that role
    		   if(roleId.equalsIgnoreCase(rid.toString()))
    			   ret = true;
    	   }
       }
       else{
    	   //that role doesn't exist at all
    	   ret = false;
       }
       
     
	   
       
	   return ret;
   }
   
   public final String findUserToEditByUserId(long id){
       log.debug("******************* findUserToEditByUserId ID = " + id);
       this.userToEdit = new AppUser();
       String ret = "findUserByUserIdFAIL";       
       boolean success = false;     
       String sql;
       Query q;
       List<?> l;
       Iterator<?> it;
       
       sql = " SELECT "
             + " UPPER(a.user_name), "
             + " UPPER(a.last_name), "
             + " UPPER(a.first_name), "
             + " UPPER(a.middle_name), "
             + " UPPER(a.enabled), "
             + " a.last_login_date, "
             + " UPPER(a.phone_number), "
             + " UPPER(a.email_address), "
             + " UPPER(a.deactivation_comments)"
             + " from app_user a "
             + " WHERE a.app_user_id=:userid";
       
       q = em.createNativeQuery(sql);      
       q.setParameter("userid", id);         
       l = q.getResultList();
       it = l.iterator();

       if (!it.hasNext()) {
          success = false;
          return ret;
       }
       
       while (it.hasNext()) {
           ret = "findUserByUserIdSUCCESS";
           Object[] row = (Object[]) it.next();
           userToEdit.setAppUserId(id);
           userToEdit.setUserName((String) row[0]);
           userToEdit.setLastName((String) row[1]);
           userToEdit.setFirstName((String) row[2]);
           userToEdit.setMiddleName((String) row[3]);
           userToEdit.setEnabled((String) row[4]);             
           this.userToEditEnabled=userToEdit.getEnabled().equalsIgnoreCase("Y") ? true : false;
           userToEdit.setLastLoginDate((Date) row[5]);
           userToEdit.setPhoneNumber((String) row[6]);
           userToEdit.setEmailAddress((String) row[7]);
           userToEdit.setDeactivationComments((String) row[8]);
          
       }
       
       
       
       //userRoles.clear();          
       userRoles = new ArrayList<String>();
       
       sql = "SELECT DISTINCT(a.app_role_id), "
             + " a.role_name, " 
             + " a.description "
             + " FROM "
             + "   app_role a, user_role_usages uru "
             + " WHERE "
             + " a.app_role_id = uru.app_role_id "
             + " and uru.app_user_id=:userid "
             + " ORDER BY a.role_name ASC "; 
       
       q = em.createNativeQuery(sql);
       q.setParameter("userid", id);         
       l = q.getResultList();
       it = l.iterator();
       
       while(it.hasNext()){
           try {
               Object[] row = (Object[]) it.next();
               BigDecimal rid = (BigDecimal)row[0];
               String role = (String) row[1];
               userRoles.add(rid.toString());
               //userRoles.add(new Long(rid.longValue()));
              
           }catch (NumberFormatException ne){
               ne.printStackTrace();
           }
           catch (NoResultException e ) {
                   e.printStackTrace();
                   //log.error("unable to find index {0}", index);
           }
       }
       
       
       
       
       rolesAvailableToUser.clear();       
       
        sql = "SELECT DISTINCT(ar.app_role_id), "
            + "     ar.role_name, "
            + "     ar.description "
            + " FROM app_role ar " 
            + " ORDER BY ar.role_name ASC";        
       
       q = em.createNativeQuery(sql);                      
       l = q.getResultList();
       it = l.iterator();
       
       while(it.hasNext()){
           try {
               Object[] row = (Object[]) it.next();
               BigDecimal rid = (BigDecimal)row[0];
               String role = (String) row[1];
               rolesAvailableToUser.add(new SelectItem(rid.toString(), role));
           }catch (NumberFormatException ne){
               ne.printStackTrace();
           }
           catch (NoResultException e ) {
                   e.printStackTrace();
                   //log.error("unable to find index {0}", index);
           }
       }
       
       
       
       
       
       
       
       
       //userFacilities.clear();            
       userFacilities = new ArrayList<String>();
       
       sql = "SELECT DISTINCT(f.va_facility_cd), "
             + " f.visn_id_cd, " 
             + " f.short_name "
             + " FROM "
             + "   va_facility f, user_role_usages uru "
             + " WHERE "
             + " f.va_facility_cd = uru.va_facility_cd "
             + " and uru.app_user_id=:userid "
             + " ORDER BY f.visn_id_cd, f.va_facility_cd, f.short_name ASC "; 
       
       q = em.createNativeQuery(sql);
       q.setParameter("userid", id);         
       l = q.getResultList();
       it = l.iterator();
       
       while(it.hasNext()){
           try {
               Object[] row = (Object[]) it.next();                            
               userFacilities.add((String) row[0]);
   
           }catch (NumberFormatException ne){
               ne.printStackTrace();
           }
           catch (NoResultException e ) {
                   e.printStackTrace();
                   //log.error("unable to find index {0}", index);
           }
       }
       
       
       
       
       facilitiesAvailableToUser.clear();       
             
        sql = "SELECT DISTINCT(f.va_facility_cd), "
            + " f.visn_id_cd, " 
            + " f.short_name "
            + " FROM "
            + "   va_facility f "         
            + " ORDER BY f.visn_id_cd, f.va_facility_cd, f.short_name ASC ";
        
       q = em.createNativeQuery(sql);                      
       l = q.getResultList();
       it = l.iterator();
       
       while(it.hasNext()){
           try {
               Object[] row = (Object[]) it.next();
               
               BigDecimal vid = (BigDecimal)row[1];
               String shortName = row[2]!=null ? " ("  + row[2] + ")" : "";
               
               String facLabel = "Visn: " + vid.toString() + " / Facility: " + ((String) row[0]) + shortName;
               facilitiesAvailableToUser.add(new SelectItem(((String) row[0]), facLabel));
               
           }catch (NumberFormatException ne){
               ne.printStackTrace();
           }
           catch (NoResultException e ) {
                   e.printStackTrace();
                   //log.error("unable to find index {0}", index);
           }
       }
       
       
       
       return ret;
   }
   
   
   
   
   private List<SelectItem> rolesAvailableToUser = new ArrayList<SelectItem>();  
   public List<SelectItem> getRolesAvailableToUser(){
       
       Query q;
       List l;
       Iterator it;
       rolesAvailableToUser.clear();       
       
       String sql = "SELECT DISTINCT(ar.app_role_id), "
           + "     ar.role_name, "
           + "     ar.description "
           + " FROM app_role ar ";
       
	      if(!identity.hasRole("ADMIN")){
	    	   sql+=" WHERE ar.role_name <> 'ADMIN' ";
	       }
	       sql += " ORDER BY ar.role_name ASC ";
      
      q = em.createNativeQuery(sql);                      
      l = q.getResultList();
      it = l.iterator();
      
      while(it.hasNext()){
          try {
              Object[] row = (Object[]) it.next();
              BigDecimal rid = (BigDecimal)row[0];
              String role = (String) row[1];
              rolesAvailableToUser.add(new SelectItem(rid.toString(), role));
          }catch (NumberFormatException ne){
              ne.printStackTrace();
          }
          catch (NoResultException e ) {
                  e.printStackTrace();
                  //log.error("unable to find index {0}", index);
          }
      }
      
      return this.rolesAvailableToUser;

   }   
   public void setRolesAvailableToUser(List<SelectItem> rolesAvailableToUser){
       this.rolesAvailableToUser = rolesAvailableToUser;
   }
 
   private List<String> userRoles = new ArrayList<String>();
   public List<String> getUserRoles(){
       return this.userRoles;
   }
   public void setUserRoles(List<String> userRoles){
       this.userRoles = userRoles;
   }
   
   private List<String> userRolesForNewUser = new ArrayList<String>();
   public List<String> getUserRolesForNewUser(){
       return this.userRolesForNewUser;
   }
   public void setUserRolesForNewUser(List<String> userRolesForNewUser){
       this.userRolesForNewUser = userRolesForNewUser;
   }
   
   
   private List<SelectItem> facilitiesAvailableToUser = new ArrayList<SelectItem>();  
   public List<SelectItem> getFacilitiesAvailableToUser(){
       String sql;
       Query q;
       List l;
       Iterator it;
       
	   facilitiesAvailableToUser.clear();       
       
       sql = "SELECT DISTINCT(f.va_facility_cd), "
           + " f.visn_id_cd, " 
           + " f.short_name "
           + " FROM "
           + "   va_facility f "         
           + " ORDER BY f.visn_id_cd, f.va_facility_cd, f.short_name ASC ";
       
      q = em.createNativeQuery(sql);                      
      l = q.getResultList();
      it = l.iterator();
      
      while(it.hasNext()){
          try {
              Object[] row = (Object[]) it.next();
              
              BigDecimal vid = (BigDecimal)row[1];
              String shortName = row[2]!=null ? " ("  + row[2] + ")" : "";
              
              String facLabel = "Visn: " + vid.toString() + " / Facility: " + ((String) row[0]) + shortName;
              facilitiesAvailableToUser.add(new SelectItem(((String) row[0]), facLabel));
              
          }catch (NumberFormatException ne){
              ne.printStackTrace();
          }
          catch (NoResultException e ) {
                  e.printStackTrace();
                  //log.error("unable to find index {0}", index);
          }
      }
	   
	   return this.facilitiesAvailableToUser;
   }   
   public void setFacilitiesAvailableToUser(List<SelectItem> facilitiesAvailableToUser){
       this.facilitiesAvailableToUser = facilitiesAvailableToUser;
   }

   private List<String> userFacilities = new ArrayList<String>();
   public List<String> getUserFacilities(){
       return this.userFacilities;
   }
   public void setUserFacilities(List<String> userFacilities){
       this.userFacilities = userFacilities;
   }
   
   private List<String> userFacilitiesForNewUser = new ArrayList<String>();
   public List<String> getUserFacilitiesForNewUser(){
       return this.userFacilitiesForNewUser;
   }
   public void setUserFacilitiesForNewUser(List<String> userFacilitiesForNewUser){
       this.userFacilitiesForNewUser = userFacilitiesForNewUser;
   }
   
   public String cancelModify(){
       String ret = "userAdminModifyCANCEL";       
       this.userToEdit = null;
       searchUsers();
       return ret;
   }
   
   
   
   public final void validateTextAreaLength(FacesContext context,
           UIComponent component, Object obj) {
       String value = obj.toString();
       HtmlInputTextarea t = (HtmlInputTextarea) component;
       if (value.length() > 200) {
           t.setValid(false);
           FacesMessage message = new FacesMessage("200 characters max");
           context.addMessage(component.getClientId(context), message);
       }
   }
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   
   AppUserRequest appUserRequest = new AppUserRequest();
   public AppUserRequest getAppUserRequest(){
       return this.appUserRequest;
   }
   public void setAppUserRequest(AppUserRequest aur){
       this.appUserRequest = aur;
   }
   

   public final int getRequestCount(){

       String sql = " select count(*) " 
           + " from fpps_owner.app_user_request t "
           + " where t.request_status<>1";
       
       Query q = em.createNativeQuery(sql);       
       BigDecimal recordCount = (BigDecimal) q.getSingleResult();

       return recordCount.intValue();          
   }
   
private List<AppUserRequest> userAccountRequests = new LinkedList<AppUserRequest>();
   
   public final List<AppUserRequest> getUserAccountRequests() {
             
       searchParametersToSet = new HashMap<String, String>();
       this.userAccountRequests.clear();
       //resetUser(this.edit);      
       List<?> l;
       Iterator it;
       String sql;
       Query q;
       
       if (this.sc != null && !this.sc.equalsIgnoreCase("")) {
           this.sortCol = this.sc;
        }
        if (this.sd != null && !this.sd.equalsIgnoreCase("")) {
           this.sortDir = this.sd;
        }

          
         sql = " select " 
               + " t.app_user_request_id, "
               + " t.user_name, "
               + " t.enabled, "
               + " t.last_name, "
               + " t.first_name, "
               + " t.middle_name, "
               + " t.phone_number, "
               + " t.email_address, "
               + " t.deactivation_comments, "
               + " t.request_roles, "
               + " t.request_facilities, "
               + " t.request_type, "
               + " t.request_status, "
               + " t.created_by, "
               + " t.date_created "
               + " from fpps_owner.app_user_request t "
               + " where t.request_status=0 " 
               + " or t.request_status=2 ";
          
             
          
          /* Build the SQL ORDER BY clause. */
          String orderBy = "";
      
          if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortusername")) {
              orderBy = " ORDER BY UPPER(t.user_name) " + this.sortDir;
           }
          if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortfirstname")) {
              orderBy = " ORDER BY UPPER(t.first_name) " + this.sortDir;
           }
          if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortmiddlename")) {
              orderBy = " ORDER BY UPPER(t.middle_name) " + this.sortDir;
           }
          if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortlastname")) {
              orderBy = " ORDER BY UPPER(t.last_name) " + this.sortDir 
                      + ", UPPER(t.first_name) " + this.sortDir
                      + ", UPPER(t.middle_name) " + this.sortDir;
           }
          if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortphone")) {
              orderBy = " ORDER BY UPPER(t.phone_number) " + this.sortDir;
           }
          if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortemail")) {
              orderBy = " ORDER BY UPPER(t.email_address) " + this.sortDir;
           }          
          if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortactive")) {
              orderBy = " ORDER BY UPPER(t.enabled) " + this.sortDir;
           }                    
          if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sorttype")) {
              orderBy = " ORDER BY t.request_type " + this.sortDir;
           }
          if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortstatus")) {
              orderBy = " ORDER BY t.request_status " + this.sortDir;
           }
          if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortcreatedby")) {
              orderBy = " ORDER BY t.created_by " + this.sortDir;
           }
          if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortdatecreated")) {
              orderBy = " ORDER BY t.date_created " + this.sortDir;
           }
          
          
          // add default columns to ORDER BY if they haven't been specified in the
          // clause already         
          if (orderBy.indexOf("ORDER BY ") <= 0) {
             orderBy += " ORDER BY t.date_created ASC ";
             this.sortCol = "sortdatecreated";
             this.sortDir = "asc";
          }
   
          sql+=orderBy;
          
          q = em.createNativeQuery(sql);          
          l = q.getResultList();
          it = l.iterator();

          /* loop through the results. */
          while (it.hasNext()) {
              AppUserRequest aur = new AppUserRequest();
              Object[] row = (Object[]) it.next();
              BigDecimal aid = (BigDecimal) row[0];
              aur.setAppUserRequestId(aid.longValue());
              aur.setUserName((String) row[1]);
              aur.setEnabled((String) row[2]);              
              aur.setLastName((String) row[3]);
              aur.setFirstName((String) row[4]);
              aur.setMiddleName((String) row[5]);
              aur.setPhoneNumber((String) row[6]);
              aur.setEmailAddress((String) row[7]);
              aur.setDeactivationComments((String) row[8]);
             
              String roles = (String) row[9];
              roles = roles==null ? "" : roles;
              roles = roles.replace(",","<br />");
              aur.setRequestRoles(roles);
              
              
              
              String facs = (String) row[10];
              if(facs!=null){
                  facs = "'" + facs + "'";
                  facs = facs.replace(", ", "','");
                  log.debug("----------------" + facs + "----------------");
              } else{
                  log.debug("----------------facs is null----------------");
              }
              sql = " select 'Visn:' || f.visn_id_cd || ' / Facility:' || " +
                    " f.va_facility_cd as facstring " +
                    " from va_facility f " +
                    " where f.va_facility_cd in ("+facs+") " +
                    " order by f.visn_id_cd ASC, f.va_facility_cd ASC";
              q = em.createNativeQuery(sql);
              
              List<String> list2 = q.getResultList();
              String facslist = "";
              for(String fac : list2) {
                  facslist+=fac + "<br />";
              }
              //facs = facs==null ? "" : facs;
              //facslist = facslist.replace(",","<br />");
              aur.setRequestFacilities(facslist);
              
              
              
              
              
              BigDecimal rt = (BigDecimal) row[11];
              boolean rtb = rt.intValue()==0 ? false : true;
              aur.setRequestType(rtb);
              BigDecimal rs = (BigDecimal) row[12];             
              aur.setRequestStatus(rs.intValue());
              aur.setCreatedBy((String) row[13]);
              aur.setDateCreated((Date) row[14]);
              
              userAccountRequests.add(aur);
          }
       
   
       return this.userAccountRequests;
   }
   
   
   public final String openUserAccountRequest(Long id){       
       log.debug("openUserAccountRequest ID=" + id);    
       appUserRequest = new AppUserRequest();
           
       Query q = em.createNativeQuery(" SELECT "
                                    + " app_user_request_id, "
                                    + " user_name,  "
                                    + " enabled,     "
                                    + " last_name,  "
                                    + " first_name,  "
                                    + " middle_name,  "
                                    + " phone_number,  "
                                    + " email_address,  "                                                                        
                                    + " deactivation_comments,  "
                                    + " request_roles,  "
                                    + " request_facilities, " 
                                    + " request_type,  "
                                    + " request_status,  "
                                    + " created_by,  "
                                    + " date_created  "
                                    + " FROM fpps_owner.app_user_request a where a.app_user_request_id=:id");
       q.setParameter("id", id);
       AppUserRequest aur= new AppUserRequest();
     
       Object[] row = (Object[]) q.getSingleResult();
       
       BigDecimal aid = (BigDecimal) row[0];
       aur.setAppUserRequestId(aid.longValue());
       aur.setUserName((String) row[1]);
       aur.setEnabled((String) row[2]);              
       aur.setLastName((String) row[3]);
       aur.setFirstName((String) row[4]);
       aur.setMiddleName((String) row[5]);
       aur.setPhoneNumber((String) row[6]);
       aur.setEmailAddress((String) row[7]);
       aur.setDeactivationComments((String) row[8]);
       
       String roles = (String) row[9];
       roles = roles==null ? "" : roles;
       roles = roles.replace(",","<br />");
       aur.setRequestRoles(roles);
       
       String facs = (String) row[10];
       if(facs!=null){
           facs = "'" + facs + "'";
           facs = facs.replace(", ", "','");
           log.debug("----------------" + facs + "----------------");
       } else{
           log.debug("----------------facs is null----------------");
       }
       String sql = " select 'Visn:' || f.visn_id_cd || ' / Facility:' || " +
             " f.va_facility_cd as facstring " +
             " from va_facility f " +
             " where f.va_facility_cd in ("+facs+") " +
             " order by f.visn_id_cd ASC, f.va_facility_cd ASC";
       q = em.createNativeQuery(sql);
       
       List<String> list2 = q.getResultList();
       String facslist = "";
       for(String fac : list2) {
           facslist+=fac + "<br />";
       }
       //facs = facs==null ? "" : facs;
       //facslist = facslist.replace(",","<br />");
       aur.setRequestFacilities(facslist);
       
       BigDecimal rt = (BigDecimal) row[11];
       boolean rtb = rt.intValue()==0 ? false : true;
       aur.setRequestType(rtb);
       BigDecimal rs = (BigDecimal) row[12];      
       aur.setRequestStatus(rs.intValue());
       aur.setCreatedBy((String) row[13]);
       aur.setDateCreated((Date) row[14]);
       
       appUserRequest = aur;
       
       String ret = row==null ? "openUserRequestFAIL" : "openUserRequestSUCCESS";
       
       return ret;
   }
   
   
   //public final String markRequestComplete(Long id){   
   public final String updateRequestStatus(Long id, int status){  
	   log.debug("================================updateRequestStatus======================================");
	   log.debug("id: " + id);
	   log.debug("status: " +  status);
	   //String ret = "markRequestCompleteFAIL";
	   String ret = "updateRequestStatusFAIL";
       boolean success = false;
       
       Query q = em.createNativeQuery("UPDATE fpps_owner.app_user_request a " 
                            + " SET "
                            + " a.request_status=:status, "
                            + " a.modified_by=:modifiedby "
                            + " WHERE a.app_user_request_id=:id");
                       
       q.setParameter("status", status);
       q.setParameter("modifiedby", appUser.getUserName().trim().toUpperCase());
       q.setParameter("id", id);
       q.executeUpdate();
       q = em.createNativeQuery("COMMIT");
       q.executeUpdate();
       
       
       success = true;
       //ret = "markRequestCompleteSUCCESS";
       ret = "updateRequestStatusSUCCESS";
              

       if (success) {
           StatusMessages.instance().addFromResourceBundle(Severity.INFO,
                   "gov.va.med.fee.userAdmin.REQUEST_COMPLETED_SUCCESS", "");
       } else {
           StatusMessages.instance().addFromResourceBundle(Severity.ERROR,                       
                   "gov.va.med.fee.userAdmin.REQUEST_COMPLETED_ERROR", "");
       }
       
       return ret;
   }
   
   
   
   
   
   
   
   
   
   
   
   
   /**
    * {@inheritDoc}
    */
   @Destroy
   @Remove
   public final void destroy() {
      // log.info("destroyed:" + this);
   }
}
