/** $Id: UserRequestSessionBean.java,v 1.7 2010/07/19 20:16:31 vhahacdoansc Exp $ */
package gov.va.med.fee.tool;

import static org.jboss.seam.ScopeType.SESSION;
import gov.va.med.domain.fee.AppUser;
import gov.va.med.domain.fee.AppUserRequest;
import gov.va.med.domain.fee.Claim;
import gov.va.med.fee.sec.UserFacilityData;
import gov.va.med.fee.Email; 

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.ejb.Remove;
import javax.ejb.Stateless;
import javax.ejb.Stateful;
import javax.faces.application.FacesMessage;
import javax.faces.component.UIComponent;
import javax.faces.component.UIViewRoot;
import javax.faces.component.html.HtmlInputTextarea;
import javax.faces.context.FacesContext;
import javax.faces.event.ActionEvent;
import javax.faces.model.SelectItem;
import javax.mail.MessagingException;
import javax.mail.internet.AddressException;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.Destroy;
import org.jboss.seam.annotations.Factory;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Logger;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Out;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.annotations.datamodel.DataModel;
import org.jboss.seam.annotations.security.Restrict;
import org.jboss.seam.annotations.web.RequestParameter;
import org.jboss.seam.log.Log;
import org.jboss.seam.international.StatusMessages;
import org.jboss.seam.international.StatusMessage.Severity;

/**
 *  This class User Administration requests.
 */
@SuppressWarnings("serial")
@Stateful
@Name("userRequest")
@Scope(SESSION)
@Restrict("#{s:hasRole('FEE_VISN_PGM_MANAGER') or s:hasRole('FEE_SUPERVISOR')  or s:hasRole('FPPS_HELP_DESK')}")
public class UserRequestSessionBean implements UserRequest, Serializable {
   
   /** The log. */
   @Logger
   private Log log;

   /** The Entity Manager. */
   @PersistenceContext(unitName = "FPPS_Owner")
   private EntityManager em;

   /** list to populate the roles selection box. */
   @Out(required = false, scope = ScopeType.SESSION)
   List<String> rolesToSelectForRequest;

   /** list to populate the station selection box. */
   @Out(required = false, scope = ScopeType.SESSION)
   List<SelectItem> stationsToSelectForRequest;
   
   /** list to populate users found. */
   @Out(required = false, scope = ScopeType.SESSION)
   List<String> userMatches = new ArrayList<String>();
  
   /** list of roles and facility for a given user. */
   @Out(required = false, scope = ScopeType.SESSION)
   List<String> rolesFacilities = new ArrayList<String>();
   
   /** The collection of results. */
   @DataModel
   private List<AppUser> userSearchResults = new LinkedList<AppUser>();
   
   /** the user backing bean for performing user searches. */
   @In(value = "search", create = true)
   User search;
   
   /** the user backing bean for editing user info. */
   @In(value = "edit", create = true)
   User edit;
   
   /** the AppUser object to for creating a new user */
   AppUser userToCreate;
   
   /** the AppUser object to for searching for a user */
   private AppUser userToSearch;
   
   /** the User facilities to search for a user */
   String userFacilitiesToSearch;
   
   /** the AppUser object to for searching for a user */
   private AppUser userToEdit;
   
   /** the AppUser object to for searching for a user */
   private AppUser userToEditForRequest;
   
   /** The app user. */
   @In(value = "appUser", required = true, scope = ScopeType.SESSION)
   private AppUser appUser;
   
   /**
    * Default constructor. 
    */
   public UserRequestSessionBean() {
      rolesFacilities.add("");
      userToCreate = new AppUser();
      userToSearch = new AppUser();
      userToEdit = new AppUser();
     
      //userRequest = new AppUserRequest();
   }
   
   
   /**
    * {@inheritDoc}
    */
   @SuppressWarnings("unchecked")
   @Factory("rolesToSelectForRequest")
   @Override
   public final void populateRoles() {
      Query q = em.createNativeQuery("select a.role_name from app_role a where a.role_name <> 'ADMIN' order by a.role_name");
      rolesToSelectForRequest = new ArrayList<String>();
      List<String> l = q.getResultList();
      java.util.Iterator<String> i = l.iterator();
      while (i.hasNext()) {
         String str = i.next();
         rolesToSelectForRequest.add(str.replaceAll("_", " "));
      }
   }

   public List<SelectItem> getRolesForRequest(){
       List<SelectItem> list = new ArrayList<SelectItem>();
       if(rolesToSelectForRequest==null){
           populateRoles();
       }
       for(String role : rolesToSelectForRequest){
           list.add(new SelectItem(role, role));
       }
       return list;
   }
   
   /**
    * {@inheritDoc}
    */
   @SuppressWarnings("unchecked")
   @Factory("stationsToSelectForRequest")
   @Override
   public final void populateStations() {    
      /* 
       String sql = "SELECT   'Visn: ' "
           + " || f.visn_id_cd "
           + " || ' / Facility: ' " 
           + " || f.va_facility_cd  " 
           + " || CASE  "
           + " WHEN f.short_name IS NOT NULL " 
           + " THEN ' (' || replace(f.short_name, ',','') || ')' " 
           + " ELSE NULL  "
           + " END AS shortname " 
           + " FROM va_facility f " 
           + " WHERE f.va_type_cd = 'FAC' " 
           + " AND f.visn_id_cd <> 0  "
           + " ORDER BY f.visn_id_cd  "
           + " ,f.va_facility_cd  "
           + " ,f.va_facility_name";
                            		
      Query q = em.createNativeQuery(sql);
      stationsToSelect = q.getResultList();      
      */
       
      stationsToSelectForRequest = new LinkedList<SelectItem>();
      String sql;
      Query q;
      List l;
      Iterator it;
      
      
      
      
      sql = "SELECT DISTINCT(f.va_facility_cd), "
          + " f.visn_id_cd, " 
          + " f.short_name "
          + " FROM va_facility f " 
          + " WHERE f.va_type_cd = 'FAC' "         
          + " ORDER BY f.visn_id_cd, f.va_facility_cd, f.short_name ASC ";
      
     q = em.createNativeQuery(sql);                      
     l = q.getResultList();
     it = l.iterator();
     
     while(it.hasNext()){
         try {
             Object[] row = (Object[]) it.next();
             
             BigDecimal vid = (BigDecimal)row[1];
             String shortName = row[2]!=null ? " ("  + row[2] + ")" : "";
             
             String facLabel = "Visn: " + vid.toString() + " / Facility: " + ((String) row[0]) + shortName;
             stationsToSelectForRequest.add(new SelectItem(((String) row[0]), facLabel));
             
         }catch (NumberFormatException ne){
             ne.printStackTrace();
         }
         catch (NoResultException e ) {
                 e.printStackTrace();
                 //log.error("unable to find index {0}", index);
         }
     }
      
      
   }
   
   public List<SelectItem> getFacilitiesForRequest(){
       List<SelectItem> list = new ArrayList<SelectItem>();    
       if(stationsToSelectForRequest==null){
           populateStations();
       }
       for(SelectItem fac : stationsToSelectForRequest){
           list.add(fac);
       }
       return list;

   }
   
   
   
   private HashMap<String, String> searchParametersToSet = new HashMap<String, String>();
   private int classCounter=0;
   
   /**
    * A method to build narrowing clauses in the SQL statement. Determines if a
    * database string field is set by the user. If so, append it appropriately
    * using 'LIKE' or '=' depending on ' whether or not there is an Oracle
    * wildcard in the user defined criteria.
    * 
    * @return a SQL clause
    * @param searchParam
    *           the search parameter to evaluate.
    * @param sqlColumn
    *           the database column to apply the search parameters to.
    * 
    */
   private String appendSqlStringParameters(String searchParam, String prefix, String suffix) {
      String ret = "";
      String param = "p" + classCounter++;
      log.debug("searchParam: " + searchParam);
      if (searchParam != null && !searchParam.trim().equals("") && !searchParam.trim().equals("%")
            && !searchParam.trim().equals("_")) {
         if (searchParam.trim().indexOf("%") >= 0 || searchParam.trim().indexOf("_") >= 0) {
            ret += " " + prefix + " LIKE :" + param + " " + suffix;
            searchParametersToSet.put(param, searchParam.toUpperCase());
         } 
         else {
            ret += " " + prefix + " = :" + param + " " + suffix;
            searchParametersToSet.put(param, searchParam.toUpperCase());
         }
      }
      log.debug("ret: " + ret);
      return ret;
   }

   
   
   

   /**
    * {@inheritDoc}
    */
   @SuppressWarnings("unchecked")
   @Override
   public final void searchUsers() {
      searchParametersToSet = new HashMap<String, String>();
      this.userSearchResults.clear();
      List<?> l;
      Iterator it;
      String sql;
      Query q;
      
      if (this.sc != null && !this.sc.equalsIgnoreCase("")) {
          this.sortCol = this.sc;
       }
       if (this.sd != null && !this.sd.equalsIgnoreCase("")) {
          this.sortDir = this.sd;
       }

         
        sql = " select a.app_user_id, " 
                 + " UPPER(a.user_name), "
                 + " UPPER(a.last_name), "
                 + " UPPER(a.first_name), "
                 + " UPPER(a.enabled), "
                 + " a.last_login_date, "
                 + " UPPER(a.phone_number), "
                 + " UPPER(a.email_address) "
                 + " from app_user a "
                 + " WHERE a.app_user_id is not null ";
         
         sql += appendSqlStringParameters(this.userToSearch.getUserName().trim().toUpperCase(), "AND UPPER(a.user_name)", "");
         sql += appendSqlStringParameters(this.userToSearch.getLastName().trim().toUpperCase(), "AND UPPER(a.last_name)", "");
         sql += appendSqlStringParameters(this.userToSearch.getFirstName().trim().toUpperCase(), "AND UPPER(a.first_name)", "");
         sql += appendSqlStringParameters(this.userToSearch.getPhoneNumber().trim().toUpperCase(), "AND UPPER(a.phone_number)", "");
         sql += appendSqlStringParameters(this.userToSearch.getEmailAddress().trim().toUpperCase(), "AND UPPER(a.email_address)", "");
         sql += appendSqlStringParameters(this.userFacilitiesToSearch.trim().toUpperCase(), "AND a.app_user_id IN (SELECT u.app_user_id FROM user_role_usages u WHERE u.va_facility_cd", ")");
                 
         
         
         /* Build the SQL ORDER BY clause. */
         String orderBy = "";

         if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortusername")) {
             orderBy = " ORDER BY UPPER(a.user_name) " + this.sortDir;
          }
         if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortlastname")) {
             orderBy = " ORDER BY UPPER(a.last_name) " + this.sortDir;
          }
         if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortfirstname")) {
             orderBy = " ORDER BY UPPER(a.first_name) " + this.sortDir;
          }
         if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortactive")) {
             orderBy = " ORDER BY UPPER(a.enabled) " + this.sortDir;
          }
         if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortlastlogin")) {
             orderBy = " ORDER BY a.last_login_date " + this.sortDir;
          }
         if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortphone")) {
             orderBy = " ORDER BY UPPER(a.phone_number) " + this.sortDir;
          }
         if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortemail")) {
             orderBy = " ORDER BY UPPER(a.email_address) " + this.sortDir;
          }
         
         // add default columns to ORDER BY if they haven't been specified in the
         // clause already         
         if (orderBy.indexOf("ORDER BY ") <= 0) {
            orderBy += " ORDER BY a.user_name ASC ";
            this.sortCol = "sortusername";
            this.sortDir = "asc";
         }
  
         sql+=orderBy;
         
         q = em.createNativeQuery(sql);
         
         for(String key : searchParametersToSet.keySet()) {
            q.setParameter(key, searchParametersToSet.get(key));
         }
         
         l = q.getResultList();
         it = l.iterator();

         
         /* loop through the results. */
         while (it.hasNext()) {
             AppUser au = new AppUser();
             Object[] row = (Object[]) it.next();
             BigDecimal aid = (BigDecimal) row[0];
             au.setAppUserId(aid.longValue());
             au.setUserName((String) row[1]);
             au.setLastName((String) row[2]);
             au.setFirstName((String) row[3]);
             au.setEnabled((String) row[4]);             
             au.setLastLoginDate((Date) row[5]);
             au.setPhoneNumber((String) row[6]);
             au.setEmailAddress((String) row[7]);
             
             userSearchResults.add(au);
         }
   
   }
   
   /**
    * {@inheritDoc}
    */
   @Override
   public final void selectUser(String username) {
      log.info("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<AJM: " + username);
      edit.setUserName(username);
      Query query = em.createQuery("select u from fAppUser u where userName = :username").setParameter("username", username);
      AppUser user = (AppUser) query.getSingleResult();
      edit.setUserName(user.getUserName());
      edit.setFirstName(user.getFirstName());
      edit.setLastName(user.getLastName());
      edit.setEmailAddress(user.getEmailAddress());
   }

   
   /**
    * Creates a case sensitive where clause.
    * 
    * @param wherecls previously partially generated where clause.
    * @param column the column's name.
    * @param value the value to check for.
    * @return the updated where clause.
    */
   final StringBuilder genWhereCls(StringBuilder wherecls, String column, String value) {
      if (wherecls == null) {
        wherecls =  new StringBuilder().append("WHERE "); 
      } else {
        wherecls.append("AND ");
      }
      wherecls.append(column).append(" = '").append(value).append("' ");
      log.debug("in gen where clause: {0}", wherecls);
      return wherecls;
   }
   
   /**
    * Creates a case insensitive where clause (Oracle specific).
    * 
    * @param wherecls previously partially generated where clause.
    * @param column the columns name.
    * @param value the value to check for.
    * @return the updated where clause.
    */
   final StringBuilder genCiWhereCls(StringBuilder wherecls, String column, String value) {
      if (wherecls == null) {
         wherecls =  new StringBuilder().append("WHERE "); 
       } else {
         wherecls.append("AND ");
       }
       wherecls.append(" REGEXP_LIKE( ").append(column).append(",'^").append(value).append("$', 'i')  ");
       log.debug("in gen where clause: {0}", wherecls);
       return wherecls;     
   }
   
   /**
    * {@inheritDoc}
    */
   @Override
   public final void update(ActionEvent event) {
      FacesContext context = FacesContext.getCurrentInstance();
      UIViewRoot root = context.getViewRoot();
      root.processUpdates(context);
   }
   
   
   /**
    * Clears the data from the user object passed in.
    * 
    * @param u <code>User</code> object to clear.
    */
   final void resetUser(User u) {
      u.setUserName(null);
      u.setAppUserId(0);
      u.setCreatedBy(null);
      u.setDateCreated(null);
      u.setDateModified(null);
      u.setEmailAddress(null);
      u.setEnabled(null);
      u.setFirstName(null);
      u.setLastName(null);
      u.setMiddleName(null);
      u.setModifiedBy(null);
      u.setPhoneNumber(null);
      u.setUserName(null);
      u.setUserRoleUsages(null);
   }
   
   
   
   
   public void setUserToCreate(AppUser userToCreate){
       this.userToCreate = userToCreate;
   }
   
   public AppUser getUserToCreate(){
       return this.userToCreate;
   }
   
   public void setUserToSearch(AppUser userToSearch){
       this.userToSearch = userToSearch;
   }
   
   public AppUser getUserToSearch(){
       return this.userToSearch;
   }
   
   public void setUserToEdit(AppUser userToEdit){
       this.userToEdit = userToEdit;
   }
   
   public AppUser getUserToEdit(){
       return this.userToEdit;
   }
   
   public void setUserFacilitiesToSearch(String userFacilitiesToSearch){
       this.userFacilitiesToSearch = userFacilitiesToSearch;
   }
   
   public String getUserFacilitiesToSearch(){
       return this.userFacilitiesToSearch;
   }
   
   
   @RequestParameter
   private String sd;

   public final String getSd() {
      return this.sd;
   }

   public final void setSd(String sd) {
      this.sd = sd;
   }

   @RequestParameter
   private String sc;

   public final String getSc() {
      return this.sc;
   }

   public final void setSc(String sc) {
      this.sc = sc;
   }
   
   /** The column to sort by. */
   private String sortCol;

   /**
    * Gets the sort column.
    * 
    * @return the sortCol.
    */
   public final String getSortCol() {
      return this.sortCol;
   }

   /**
    * Sets the sort column.
    * 
    * @param sortCol
    *           the sort column.
    */
   public final void setSortCol(String sortCol) {
      this.sortCol = sortCol;
   }

   /** The direction to sort. */
   private String sortDir;

   /**
    * Gets the sort direction.
    * 
    * @return the sortDir.
    */
   public final String getSortDir() {
      return this.sortDir;
   }

   /**
    * Sets the sort direction.
    * 
    * @param sortDir
    *           the sort direction.
    */
   public final void setSortDir(String sortDir) {
      this.sortDir = sortDir;
   }
   
   /**
    * A boolean to hold the value from the search page. Can't map directly to
    * the hibernate object because the ENABLED field is not boolean, but varchar2(1)
    * */
   private boolean userToCreateEnabled = true;

   /**
    * Sets the userToCreateEnabled.
    * 
    * @param userToCreateEnabled
    *           the userToCreateEnabled flag.
    */
   public final void setUserToCreateEnabled(boolean userToCreateEnabled) {
      this.userToCreateEnabled = userToCreateEnabled;
   }

   /**
    * Gets the userToCreateEnabled flag.
    * 
    * @return the userToCreateEnabled.
    */
   public final boolean getUserToCreateEnabled() {
      return this.userToCreateEnabled;
   }
   
   /**
    * A boolean to hold the value from the search page. Can't map directly to
    * the hibernate object because the ENABLED field is not boolean, but varchar2(1)
    * */
   private boolean userToEditEnabled = true;

   /**
    * Sets the userToEditEnabled.
    * 
    * @param userToEditEnabled
    *           the userToEditEnabled flag.
    */
   public final void setUserToEditEnabled(boolean userToEditEnabled) {
      this.userToEditEnabled = userToEditEnabled;
   }

   /**
    * Gets the userToEditEnabled flag.
    * 
    * @return the userToEditEnabled.
    */
   public final boolean getUserToEditEnabled() {
      return this.userToEditEnabled;
   }
   
   public final List<AppUser> getUserSearchResults() {       
       return this.userSearchResults;
   }
   
   
   public String createUser(){
       
           String ret = "createUserFAIL";
           boolean success = false;
           String enabled = "N"; 
           String sql;
           Query q;

           sql = " SELECT COUNT(*) FROM fpps_owner.app_user p "
                   + " WHERE p.user_name=UPPER(:upperntname)";
           q = em.createNativeQuery(sql);
           q.setParameter("upperntname", this.userToCreate.getUserName().trim().toUpperCase());         
           BigDecimal recordCount = (BigDecimal) q.getSingleResult();

           if (recordCount.intValue() > 0) {
               StatusMessages.instance().addFromResourceBundle(Severity.ERROR,
                       "gov.va.med.fee.userAdmin.CREATE_USER_ERROR_ON_EXISTING",
                       "");
               return ret;
           } else {
               enabled = this.userToCreateEnabled ? "Y" : "N";
               sql = " INSERT INTO fpps_owner.app_user(app_user_id, " 
               		    + " user_name, " 
                   		+ " enabled, " 
                   		+ " last_name, " 
                   		+ " first_name, " 
                   		+ " middle_name, " 
                   		+ " phone_number, " 
                   		+ " email_address, " 
                   		+ " created_by, " 
                   		+ " date_created) "
                        + " VALUES(fpps_owner.mig.nextval, " 
                    	+ ":ntname, "
                    	+ ":enabled, " 
                    	+ ":lastname, "
                    	+ ":firstname, "
                    	+ ":middlename, "
                    	+ ":phone, "
                    	+ ":email, "
                    	+ ":createdby, "
                    	+ "sysdate) ";
               q = em.createNativeQuery(sql);
               q.setParameter("ntname", this.userToCreate.getUserName().trim().toUpperCase());
               q.setParameter("enabled", enabled);
               q.setParameter("lastname", this.userToCreate.getLastName().trim().toUpperCase());
               q.setParameter("firstname", this.userToCreate.getFirstName().trim().toUpperCase());
               q.setParameter("middlename", this.userToCreate.getMiddleName().trim().toUpperCase());
               q.setParameter("phone", this.userToCreate.getPhoneNumber().trim().toUpperCase());
               q.setParameter("email", this.userToCreate.getEmailAddress().trim().toUpperCase());
               q.setParameter("createdby", appUser.getUserName().trim().toUpperCase());

               q.executeUpdate();
               q = em.createNativeQuery("COMMIT");
               q.executeUpdate();
               ret = "createUserSUCCESS";
               success = true;

              this.userToCreate = new AppUser();
           }

           if (success) {
               StatusMessages.instance().addFromResourceBundle(Severity.INFO,
                       "gov.va.med.fee.userAdmin.CREATE_USER_SUCCESS", "");
           } else {
               StatusMessages.instance().addFromResourceBundle(Severity.ERROR,                       
                       "gov.va.med.fee.userAdmin.CREATE_USER_ERROR", "");
           }

           return ret;
       
   }
   
   
   public String updateUserDetails(){
       String ret = "updateUserDetailsFAIL";
       boolean success = false;
       String enabled = "N"; 
       String sql;
       Query q;
             
       
       sql = " SELECT COUNT(*) FROM fpps_owner.app_user p "
           + " WHERE p.user_name=UPPER(:upperntname) "
           + " AND p.app_user_id <> :userid";
       q = em.createNativeQuery(sql);
       q.setParameter("upperntname", this.userToEdit.getUserName().trim().toUpperCase());         
       q.setParameter("userid", this.userToEdit.getAppUserId());
       BigDecimal recordCount = (BigDecimal) q.getSingleResult();
        
       if (recordCount.intValue() > 0) {
           StatusMessages.instance().addFromResourceBundle(Severity.ERROR,
                   "gov.va.med.fee.userAdmin.UPDATE_USER_ERROR_ON_EXISTING",
                   "");
           return ret;
       } else {
           enabled = this.userToEditEnabled ? "Y" : "N";
           sql = " UPDATE fpps_owner.app_user SET " 
                    + " user_name=:ntname, " 
                    + " enabled=:enabled, " 
                    + " last_name=:lastname, " 
                    + " first_name=:firstname, " 
                    + " middle_name=:middlename, " 
                    + " phone_number=:phone, " 
                    + " email_address=:email, " 
                    + " deactivation_comments=:deactivationcomments, "
                    + " modified_by=:modifiedby, " 
                    + " date_modified=sysdate "
                    + " WHERE app_user_id=:appuserid";
                
           q = em.createNativeQuery(sql);
           q.setParameter("ntname", this.userToEdit.getUserName().trim().toUpperCase());
           q.setParameter("enabled", enabled);
           
           //only allow disable comments if user is disabled
           if(!enabled.equalsIgnoreCase("Y")){
               q.setParameter("deactivationcomments", this.userToEdit.getDeactivationComments().trim().toUpperCase());
           }
           else{
               q.setParameter("deactivationcomments", "");
           }
           
           q.setParameter("lastname", this.userToEdit.getLastName().trim().toUpperCase());
           q.setParameter("firstname", this.userToEdit.getFirstName().trim().toUpperCase());
           q.setParameter("middlename", this.userToEdit.getMiddleName().trim().toUpperCase());
           q.setParameter("phone", this.userToEdit.getPhoneNumber().trim().toUpperCase());
           q.setParameter("email", this.userToEdit.getEmailAddress().trim().toUpperCase());
           q.setParameter("modifiedby", appUser.getUserName().trim().toUpperCase());
           q.setParameter("appuserid",this.userToEdit.getAppUserId());
    
           q.executeUpdate();
           q = em.createNativeQuery("COMMIT");
           q.executeUpdate();
           ret = "updateUserDetailsSUCCESS";
           success = true;             
       }
    
       if (success) {
           StatusMessages.instance().addFromResourceBundle(Severity.INFO,
                   "gov.va.med.fee.userAdmin.UPDATE_USER_SUCCESS", "");
       } else {
           StatusMessages.instance().addFromResourceBundle(Severity.ERROR,                       
                   "gov.va.med.fee.userAdmin.UPDATE_USER_ERROR", "");
       }
       
       
       searchUsers();
       findUserToEditByUserId(this.userToEdit.getAppUserId());
       
       return ret;
   }
   
   
   public String createModifyUserRequest(){
       String ret = "updateUserAssignmentsForRequestFAIL";
       boolean success = false;
       String enabled = "N"; 
       String sql;
       Query q;
             

  
       
       
       
       
       
       String requestedRoles = "";
       String comma = "";
       for(String role : this.userRolesForModify){
           requestedRoles+=comma+role;
           comma = ", ";
       }
       
       String requestedFacilities = "";
       comma = "";
       for(String fac : this.userFacilities){
           requestedFacilities+=comma+fac;
           comma = ", ";        
       }
       
       log.debug("============================================================================================================");
       log.debug("requested roles: " + requestedRoles);
       log.debug("requested facs: " + requestedFacilities);
       log.debug("============================================================================================================");
       enabled = this.userToEditEnabled ? "Y" : "N";
       sql = " INSERT INTO fpps_owner.app_user_request(app_user_request_id, " 
                + " user_name, " 
                + " enabled, " 
                + " last_name, " 
                + " first_name, " 
                + " middle_name, " 
                + " phone_number, " 
                + " email_address, " 
                + " request_roles, "
                + " request_facilities, "
                + " request_type, "
                + " request_status, "
                + " created_by, " 
                + " date_created) "
                + " VALUES(fpps_owner.mig.nextval, " 
                + ":ntname, "
                + ":enabled, " 
                + ":lastname, "
                + ":firstname, "
                + ":middlename, "
                + ":phone, "
                + ":email, "
                + ":roles, "
                + ":facs, "
                + ":type, "
                + ":status, "
                + ":createdby, "
                + "sysdate) ";
       q = em.createNativeQuery(sql);
       q.setParameter("ntname", this.userToEdit.getUserName().trim().toUpperCase());
       q.setParameter("enabled", enabled);
       q.setParameter("lastname", this.userToEdit.getLastName().trim().toUpperCase());
       q.setParameter("firstname", this.userToEdit.getFirstName().trim().toUpperCase());
       q.setParameter("middlename", this.userToEdit.getMiddleName().trim().toUpperCase());
       q.setParameter("phone", this.userToEdit.getPhoneNumber().trim().toUpperCase());
       q.setParameter("roles", requestedRoles);
       q.setParameter("facs", requestedFacilities);
       q.setParameter("type", 1); //type 0 = create user request, type 1 = modify user request
       q.setParameter("status", 0); //status 0 = open, status 1 = closed 
       q.setParameter("email", this.userToEdit.getEmailAddress().trim().toUpperCase());
       q.setParameter("createdby", appUser.getUserName().trim().toUpperCase());
       
       q.executeUpdate();
       q = em.createNativeQuery("COMMIT");
       q.executeUpdate();
       
       
   
    
       success = true;
       ret = "updateUserAssignmentsForRequestSUCCESS";
              

       if (success) {
           StatusMessages.instance().addFromResourceBundle(Severity.INFO,
                   "gov.va.med.fee.userAdmin.REQUEST_MODIFY_USER_SUCCESS", "");
           
           Email e = new Email();
           String recipients[] = gov.va.med.fee.Constants.USER_REQUEST_NOTIFICATION_RECIPIENTS;
           String fromAddress = gov.va.med.fee.Constants.USER_REQUEST_NOTIFICATION_FROM;
           String emailstatus = "";
           try {
        	   

        	   String emailMessage = "There is a new request to create an FPPS user:<br />"
        		   + "Type: Modify <br />"
        		   + "User logon ID: " + (this.userToEdit.getUserName()!=null?this.userToEdit.getUserName().trim().toUpperCase():"") + "<br />"        		   
	               + "First Name: " + (this.userToEdit.getFirstName()!=null?this.userToEdit.getFirstName().trim().toUpperCase():"") + "<br />"
	               + "Middle Name: " + (this.userToEdit.getMiddleName()!=null?this.userToEdit.getMiddleName().trim().toUpperCase():"") + "<br />"
	               + "Last Name: " + (this.userToEdit.getLastName()!=null?this.userToEdit.getLastName().trim().toUpperCase():"") + "<br />"
	               + "Phone: " + (this.userToEdit.getPhoneNumber()!=null?this.userToEdit.getPhoneNumber().trim().toUpperCase():"") + "<br />"
	               + "Email: " + (this.userToEdit.getEmailAddress()!=null?this.userToEdit.getEmailAddress().trim().toUpperCase():"") + "<br />"            		  
	               + "Requestor: " + (appUser.getFirstName()!=null?appUser.getFirstName().trim().toUpperCase():"") + " " 
	               + (appUser.getLastName()!=null?appUser.getLastName().trim().toUpperCase():"") 
	               + " ("+ (appUser.getUserName()!=null?appUser.getUserName().trim().toUpperCase():"") + ")<br />"               
	               + "Requestor Phone: " + (appUser.getPhoneNumber()!=null?appUser.getPhoneNumber().trim().toUpperCase():"") + "<br />"
	               + "Requestor Email: " + (appUser.getEmailAddress()!=null?appUser.getEmailAddress().trim().toUpperCase():"") + "<br />"
	               + "Requested Role(s): " + (requestedRoles!=null?requestedRoles:"") + "<br />"
	               + "Requested Station(s): " + (requestedFacilities!=null?requestedFacilities:"") + "<br />";
               
        	   
                emailstatus = e.postMail(recipients, 
                             "FPPS Modify User Request",                                                                   
                             emailMessage,   
                             fromAddress,
                             false,
                             "");
                } catch (AddressException e1) {                
                    e1.printStackTrace();
                } catch (MessagingException e1) {
                    e1.printStackTrace();
                }
                
                log.debug("--------------------------------- email status: " + emailstatus);
                
       } else {
           StatusMessages.instance().addFromResourceBundle(Severity.ERROR,                       
                   "gov.va.med.fee.userAdmin.REQUEST_MODIFY_USER_ERROR", "");
       }
       
       userRoles = new ArrayList<String>();
       userRolesForModify = new ArrayList<String>();
       userFacilities = new ArrayList<String>();
  
       
       searchUsers();
       findUserToEditByUserId(this.userToEdit.getAppUserId());
       
       return ret;
   }
   
     
   
   public boolean userRolesContains(String role){
	   
	   boolean ret = false;
	   
	   //get the role ID of the specified role 
	   String sql = "SELECT DISTINCT(a.app_role_id) "          
           + " FROM "
           + "   app_role a "
           + " WHERE "
           + " a.role_name = :rolename "
           + " and rownum=1 ";
	   
       Query q = em.createNativeQuery(sql);       
       q.setParameter("rolename", role);
       BigDecimal rid = (BigDecimal) q.getSingleResult();

       //if the role exists, compare with the userRoles collection
       if(rid!=null){
    	   for(String roleId : userRoles){
    		   //if a match is found, the user has that role
    		   if(roleId.equalsIgnoreCase(rid.toString()))
    			   ret = true;
    	   }
       }
       else{
    	   //that role doesn't exist at all
    	   ret = false;
       }
       
     
	   
       
	   return ret;
   }
   
   public final String findUserToEditByUserId(long id){
       log.debug("******************* findUserToEditByUserId ID = " + id);
       this.userToEdit = new AppUser();
       String ret = "findUserByUserIdForRequestFAIL";       
       boolean success = false;     
       String sql;
       Query q;
       List<?> l;
       Iterator<?> it;
       
       sql = " SELECT "
             + " UPPER(a.user_name), "
             + " UPPER(a.last_name), "
             + " UPPER(a.first_name), "
             + " UPPER(a.middle_name), "
             + " UPPER(a.enabled), "
             + " a.last_login_date, "
             + " UPPER(a.phone_number), "
             + " UPPER(a.email_address), "
             + " UPPER(a.deactivation_comments)"
             + " from app_user a "
             + " WHERE a.app_user_id=:userid";
       
       q = em.createNativeQuery(sql);      
       q.setParameter("userid", id);         
       l = q.getResultList();
       it = l.iterator();

       if (!it.hasNext()) {
          success = false;
          return ret;
       }
       
       while (it.hasNext()) {
           ret = "findUserByUserIdForRequestSUCCESS";
           Object[] row = (Object[]) it.next();
           userToEdit.setAppUserId(id);
           userToEdit.setUserName((String) row[0]);
           userToEdit.setLastName((String) row[1]);
           userToEdit.setFirstName((String) row[2]);
           userToEdit.setMiddleName((String) row[3]);
           userToEdit.setEnabled((String) row[4]);             
           this.userToEditEnabled=userToEdit.getEnabled().equalsIgnoreCase("Y") ? true : false;
           userToEdit.setLastLoginDate((Date) row[5]);
           userToEdit.setPhoneNumber((String) row[6]);
           userToEdit.setEmailAddress((String) row[7]);
           userToEdit.setDeactivationComments((String) row[8]);
          
       }
       
       
       
       //userRoles.clear();          
       userRoles = new ArrayList<String>();
       userRolesForModify = new ArrayList<String>();
       
       sql = "SELECT DISTINCT(a.app_role_id), "
             + " a.role_name, " 
             + " a.description "
             + " FROM "
             + "   app_role a, user_role_usages uru "
             + " WHERE "
             + " a.app_role_id = uru.app_role_id "
             + " and uru.app_user_id=:userid "
             + " ORDER BY a.role_name ASC "; 
       
       q = em.createNativeQuery(sql);
       q.setParameter("userid", id);         
       l = q.getResultList();
       it = l.iterator();
       
       while(it.hasNext()){
           try {
               Object[] row = (Object[]) it.next();
               BigDecimal rid = (BigDecimal)row[0];
               String role = (String) row[1];
               userRoles.add(rid.toString());
               userRolesForModify.add(role);
               //userRoles.add(new Long(rid.longValue()));
              
           }catch (NumberFormatException ne){
               ne.printStackTrace();
           }
           catch (NoResultException e ) {
                   e.printStackTrace();
                   //log.error("unable to find index {0}", index);
           }
       }
       
       
       
       
       rolesAvailableToUser.clear();       
       rolesAvailableToUserForUserModifyRequest.clear();
       
        sql = "SELECT DISTINCT(ar.app_role_id), "
            + "     ar.role_name, "
            + "     ar.description "
            + " FROM app_role ar "
            + " WHERE ar.role_name <> 'ADMIN' "
            + " ORDER BY ar.role_name ASC";        
       
       q = em.createNativeQuery(sql);                      
       l = q.getResultList();
       it = l.iterator();
       
       while(it.hasNext()){
           try {
               Object[] row = (Object[]) it.next();
               BigDecimal rid = (BigDecimal)row[0];
               String role = (String) row[1];
               rolesAvailableToUser.add(new SelectItem(rid.toString(), role));
               rolesAvailableToUserForUserModifyRequest.add(new SelectItem(role, role));
               
           }catch (NumberFormatException ne){
               ne.printStackTrace();
           }
           catch (NoResultException e ) {
                   e.printStackTrace();
                   //log.error("unable to find index {0}", index);
           }
       }
       
       
       
       
       
       
       
       
       //userFacilities.clear();            
       userFacilities = new ArrayList<String>();
       
       sql = "SELECT DISTINCT(f.va_facility_cd), "
             + " f.visn_id_cd, " 
             + " f.short_name "
             + " FROM "
             + "   va_facility f, user_role_usages uru "
             + " WHERE "
             + " f.va_facility_cd = uru.va_facility_cd "
             + " and uru.app_user_id=:userid "
             + " ORDER BY f.visn_id_cd, f.va_facility_cd, f.short_name ASC "; 
       
       q = em.createNativeQuery(sql);
       q.setParameter("userid", id);         
       l = q.getResultList();
       it = l.iterator();
       
       
       while(it.hasNext()){
           try {
               Object[] row = (Object[]) it.next();               
               userFacilities.add((String) row[0]);
   
           }catch (NumberFormatException ne){
               ne.printStackTrace();
           }
           catch (NoResultException e ) {
                   e.printStackTrace();
                   //log.error("unable to find index {0}", index);
           }
       }
       
       
       
       
       facilitiesAvailableToUser.clear();       
        
       /*
        sql = "SELECT DISTINCT(f.va_facility_cd), "
            + " f.visn_id_cd, " 
            + " f.short_name "
            + " FROM va_facility f " 
            + " WHERE f.va_type_cd = 'FAC' "         
            + " ORDER BY f.visn_id_cd, f.va_facility_cd, f.short_name ASC ";
        */
        
        sql = "SELECT DISTINCT(f.va_facility_cd), "
            + " f.visn_id_cd, " 
            + " f.short_name "
            + " FROM va_facility f "     
            + " ORDER BY f.visn_id_cd, f.va_facility_cd, f.short_name ASC ";
        
        
       q = em.createNativeQuery(sql);                      
       l = q.getResultList();
       it = l.iterator();
       
       while(it.hasNext()){
           try {
               Object[] row = (Object[]) it.next();
               
               BigDecimal vid = (BigDecimal)row[1];
               //String shortName = row[2]!=null ? " ("  + row[2] + ")" : "";
               
               String facLabel = "Visn: " + vid.toString() + " / Facility: " + ((String) row[0]);
               facilitiesAvailableToUser.add(new SelectItem(((String) row[0]), facLabel));
               
           }catch (NumberFormatException ne){
               ne.printStackTrace();
           }
           catch (NoResultException e ) {
                   e.printStackTrace();
                   //log.error("unable to find index {0}", index);
           }
       }
       
       
       
       return ret;
   }
   
   

   
   private List<SelectItem> rolesAvailableToUser = new ArrayList<SelectItem>();  
   public List<SelectItem> getRolesAvailableToUser(){
       return this.rolesAvailableToUser;
   }   
   public void setRolesAvailableToUser(List<SelectItem> rolesAvailableToUser){
       this.rolesAvailableToUser = rolesAvailableToUser;
   }
   private List<SelectItem> rolesAvailableToUserForUserModifyRequest = new ArrayList<SelectItem>();  
   public List<SelectItem> getRolesAvailableToUserForUserModifyRequest(){
       return this.rolesAvailableToUserForUserModifyRequest;
   }   
   public void setRolesAvailableToUserForUserModifyRequest(List<SelectItem> rolesAvailableToUserForUserModifyRequest){
       this.rolesAvailableToUserForUserModifyRequest = rolesAvailableToUserForUserModifyRequest;
   }
 
   private List<String> userRoles = new ArrayList<String>();
   public List<String> getUserRoles(){
       return this.userRoles;
   }
   public void setUserRoles(List<String> userRoles){
       this.userRoles = userRoles;
   }
   
   private List<String> userRolesForModify = new ArrayList<String>();
   public List<String> getUserRolesForModify(){
       return this.userRolesForModify;
   }
   public void setUserRolesForModify(List<String> userRolesForModify){
       this.userRolesForModify = userRolesForModify;
   }
   
   
   private List<SelectItem> facilitiesAvailableToUser = new ArrayList<SelectItem>();  
   public List<SelectItem> getFacilitiesAvailableToUser(){
       return this.facilitiesAvailableToUser;
   }   
   public void setFacilitiesAvailableToUser(List<SelectItem> facilitiesAvailableToUser){
       this.facilitiesAvailableToUser = facilitiesAvailableToUser;
   }

   private List<String> userFacilities = new ArrayList<String>();
   public List<String> getUserFacilities(){
       return this.userFacilities;
   }
   public void setUserFacilities(List<String> userFacilities){
       this.userFacilities = userFacilities;
   }
   
  
   
   
   
   
   
   
   
   
   
   
   public String cancelModify(){
       String ret = "userRequestModifyCANCEL";       
       this.userToEdit = null;
       searchUsers();
       return ret;
   }
   
   
   
   public final void validateTextAreaLength(FacesContext context,
           UIComponent component, Object obj) {
       String value = obj.toString();
       HtmlInputTextarea t = (HtmlInputTextarea) component;
       if (value.length() > 200) {
           t.setValid(false);
           FacesMessage message = new FacesMessage("200 characters max");
           context.addMessage(component.getClientId(context), message);
       }
   }
   
   
   public final String requestAddUser(){
       String ret = "requestAddUserFAIL";
       boolean success = false;
       String enabled = "N"; 
       String sql;
       Query q;
       
    // "gov.va.med.fee.userAdmin.REQUEST_ADD_USER_ERROR_ON_EXISTING";
              
       String requestedRoles = "";
       String comma = "";
       for(String role : this.userRoles){
           requestedRoles+=comma+role;
           comma = ", ";
       }
       
       String requestedFacilities = "";
       comma = "";
       for(String fac : this.userFacilities){
           requestedFacilities+=comma+fac;
           comma = ", ";        
       }
       
       log.debug("============================================================================================================");
       log.debug("requested roles: " + requestedRoles);
       log.debug("requested facs: " + requestedFacilities);
       log.debug("============================================================================================================");
       enabled = this.userToCreateEnabled ? "Y" : "N";
       sql = " INSERT INTO fpps_owner.app_user_request(app_user_request_id, " 
                + " user_name, " 
                + " enabled, " 
                + " last_name, " 
                + " first_name, " 
                + " middle_name, " 
                + " phone_number, " 
                + " email_address, " 
                + " request_roles, "
                + " request_facilities, "
                + " request_type, "
                + " request_status, "
                + " created_by, " 
                + " date_created) "
                + " VALUES(fpps_owner.mig.nextval, " 
                + ":ntname, "
                + ":enabled, " 
                + ":lastname, "
                + ":firstname, "
                + ":middlename, "
                + ":phone, "
                + ":email, "
                + ":roles, "
                + ":facs, "
                + ":type, "
                + ":status, "
                + ":createdby, "
                + "sysdate) ";
       q = em.createNativeQuery(sql);
       q.setParameter("ntname", this.userToCreate.getUserName().trim().toUpperCase());
       q.setParameter("enabled", enabled);
       q.setParameter("lastname", this.userToCreate.getLastName().trim().toUpperCase());
       q.setParameter("firstname", this.userToCreate.getFirstName().trim().toUpperCase());
       q.setParameter("middlename", this.userToCreate.getMiddleName().trim().toUpperCase());
       q.setParameter("phone", this.userToCreate.getPhoneNumber().trim().toUpperCase());
       q.setParameter("roles", requestedRoles);
       q.setParameter("facs", requestedFacilities);
       q.setParameter("type", 0); //type 0 = create user request, type 1 = modify user request
       q.setParameter("status", 0); //status 0 = open, status 1 = closed 
       q.setParameter("email", this.userToCreate.getEmailAddress().trim().toUpperCase());
       q.setParameter("createdby", appUser.getUserName().trim().toUpperCase());
      
       
       q.executeUpdate();
       q = em.createNativeQuery("COMMIT");
       q.executeUpdate();
       
       
      
       
       success = true;
       ret = "requestAddUserSUCCESS";
              

      
       
       if (success) {
           StatusMessages.instance().addFromResourceBundle(Severity.INFO,
                   "gov.va.med.fee.userAdmin.REQUEST_ADD_USER_SUCCESS", "");

           Email e = new Email();
           String recipients[] = gov.va.med.fee.Constants.USER_REQUEST_NOTIFICATION_RECIPIENTS;
           String fromEmail = gov.va.med.fee.Constants.USER_REQUEST_NOTIFICATION_FROM;
           String emailstatus = "";
           try {
        	   
        	   String emailMessage = "There is a new request to create an FPPS user:<br />"
        		   + "Type: Create <br />"
        		   + "User logon ID: " + (this.userToCreate.getUserName()!=null?this.userToCreate.getUserName().trim().toUpperCase():"") + "<br />"        		   
               + "First Name: " + (this.userToCreate.getFirstName()!=null?this.userToCreate.getFirstName().trim().toUpperCase():"") + "<br />"
               + "Middle Name: " + (this.userToCreate.getMiddleName()!=null?this.userToCreate.getMiddleName().trim().toUpperCase():"") + "<br />"
               + "Last Name: " + (this.userToCreate.getLastName()!=null?this.userToCreate.getLastName().trim().toUpperCase():"") + "<br />"
               + "Phone: " + (this.userToCreate.getPhoneNumber()!=null?this.userToCreate.getPhoneNumber().trim().toUpperCase():"") + "<br />"
               + "Email: " + (this.userToCreate.getEmailAddress()!=null?this.userToCreate.getEmailAddress().trim().toUpperCase():"") + "<br />"               
               + "Requestor: " + (appUser.getFirstName()!=null?appUser.getFirstName().trim().toUpperCase():"") + " " 
               + (appUser.getLastName()!=null?appUser.getLastName().trim().toUpperCase():"") 
               + " ("+ (appUser.getUserName()!=null?appUser.getUserName().trim().toUpperCase():"") + ")<br />"               
               + "Requestor Phone: " + (appUser.getPhoneNumber()!=null?appUser.getPhoneNumber().trim().toUpperCase():"") + "<br />"
               + "Requestor Email: " + (appUser.getEmailAddress()!=null?appUser.getEmailAddress().trim().toUpperCase():"") + "<br />"
               + "Requested Role(s): " + (requestedRoles!=null?requestedRoles:"") + "<br />"
               + "Requested Station(s): " + (requestedFacilities!=null?requestedFacilities:"") + "<br />";
               
        	           	  
                emailstatus = e.postMail(recipients, 
                             "FPPS Create User Request",                                                                   
                             emailMessage,   
                             fromEmail,
                             false,
                             "");
                } catch (AddressException e1) {                
                   log.error(e1.getMessage(), e1);
                } catch (MessagingException e1) {
                   log.error(e1.getMessage(), e1);
                }
                
            log.debug("--------------------------------- email status: " + emailstatus);
   
       } else {
           StatusMessages.instance().addFromResourceBundle(Severity.ERROR,                       
                   "gov.va.med.fee.userAdmin.REQUEST_ADD_USER_ERROR", "");
       }
       
       
       userRoles = new ArrayList<String>();
       userFacilities = new ArrayList<String>();
       userToCreate = new AppUser();
       
       return ret;
   }
   
   
   
   
   
   private List<AppUserRequest> userAccountRequests = new LinkedList<AppUserRequest>();
   
   public final List<AppUserRequest> getUserAccountRequests() {
             
       searchParametersToSet = new HashMap<String, String>();
       this.userAccountRequests.clear();
       //resetUser(this.edit);      
       List<?> l;
       Iterator it;
       String sql;
       Query q;
       
       if (this.sc != null && !this.sc.equalsIgnoreCase("")) {
           this.sortCol = this.sc;
        }
        if (this.sd != null && !this.sd.equalsIgnoreCase("")) {
           this.sortDir = this.sd;
        }

          
         sql = " select " 
               + " t.app_user_request_id, "
               + " t.user_name, "
               + " t.enabled, "
               + " t.last_name, "
               + " t.first_name, "
               + " t.middle_name, "
               + " t.phone_number, "
               + " t.email_address, "
               + " t.deactivation_comments, "
               + " t.request_roles, "
               + " t.request_facilities, "
               + " t.request_type, "
               + " t.request_status, "
               + " t.created_by, "
               + " t.date_created "
               + " from fpps_owner.app_user_request t "
               + " where t.request_status=0";
          
             
          
          /* Build the SQL ORDER BY clause. */
          String orderBy = "";
      
          if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortusername")) {
              orderBy = " ORDER BY UPPER(t.user_name) " + this.sortDir;
           }
          if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortfirstname")) {
              orderBy = " ORDER BY UPPER(t.first_name) " + this.sortDir;
           }
          if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortmiddlename")) {
              orderBy = " ORDER BY UPPER(t.middle_name) " + this.sortDir;
           }
          if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortlastname")) {
              orderBy = " ORDER BY UPPER(t.last_name) " + this.sortDir 
                      + ", UPPER(t.first_name) " + this.sortDir
                      + ", UPPER(t.middle_name) " + this.sortDir;
           }
          if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortphone")) {
              orderBy = " ORDER BY UPPER(t.phone_number) " + this.sortDir;
           }
          if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortemail")) {
              orderBy = " ORDER BY UPPER(t.email_address) " + this.sortDir;
           }          
          if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortactive")) {
              orderBy = " ORDER BY UPPER(t.enabled) " + this.sortDir;
           }                    
          if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sorttype")) {
              orderBy = " ORDER BY t.request_type " + this.sortDir;
           }
          if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortstatus")) {
              orderBy = " ORDER BY t.request_status " + this.sortDir;
           }
          if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortcreatedby")) {
              orderBy = " ORDER BY t.created_by " + this.sortDir;
           }
          if (this.sortCol != null && this.sortCol.equalsIgnoreCase("sortdatecreated")) {
              orderBy = " ORDER BY t.date_created " + this.sortDir;
           }
          
          
          // add default columns to ORDER BY if they haven't been specified in the
          // clause already         
          if (orderBy.indexOf("ORDER BY ") <= 0) {
             orderBy += " ORDER BY t.date_created ASC ";
             this.sortCol = "sortdatecreated";
             this.sortDir = "asc";
          }
   
          sql+=orderBy;
          
          q = em.createNativeQuery(sql);          
          l = q.getResultList();
          it = l.iterator();

          /* loop through the results. */
          while (it.hasNext()) {
              AppUserRequest aur = new AppUserRequest();
              Object[] row = (Object[]) it.next();
              BigDecimal aid = (BigDecimal) row[0];
              aur.setAppUserRequestId(aid.longValue());
              aur.setUserName((String) row[1]);
              aur.setEnabled((String) row[2]);              
              aur.setLastName((String) row[3]);
              aur.setFirstName((String) row[4]);
              aur.setMiddleName((String) row[5]);
              aur.setPhoneNumber((String) row[6]);
              aur.setEmailAddress((String) row[7]);
              aur.setDeactivationComments((String) row[8]);
             
              String roles = (String) row[9];
              roles = roles==null ? "" : roles;
              roles = roles.replace(",","<br />");
              aur.setRequestRoles(roles);
              
              
              
              String facs = (String) row[10];
              if(facs!=null){
                  facs = "'" + facs + "'";
                  facs = facs.replace(", ", "','");
              } 
              
              sql = " select 'Visn:' || f.visn_id_cd || ' / Facility:' || " +
              		" f.va_facility_cd as facstring " +
              		" from va_facility f " +
              		" where f.va_facility_cd in ("+facs+") " +
              		" order by f.visn_id_cd ASC, f.va_facility_cd ASC";
              q = em.createNativeQuery(sql);
              
              List<String> list2 = q.getResultList();
              String facslist = "";
              for(String fac : list2) {
                  facslist+=fac + "<br />";
              }
              //facs = facs==null ? "" : facs;
              //facslist = facslist.replace(",","<br />");
              aur.setRequestFacilities(facslist);
              
              
              
              
              
              BigDecimal rt = (BigDecimal) row[11];
              boolean rtb = rt.intValue()==0 ? false : true;
              aur.setRequestType(rtb);
              BigDecimal rs = (BigDecimal) row[12];
             
              aur.setRequestStatus(rs.intValue());
              aur.setCreatedBy((String) row[13]);
              aur.setDateCreated((Date) row[14]);
              
              userAccountRequests.add(aur);
          }
       
   
       return this.userAccountRequests;
   }
   
   
   
   
   AppUserRequest appUserRequest = new AppUserRequest();
   public AppUserRequest getAppUserRequest(){
       return this.appUserRequest;
   }
   public void setAppUserRequest(AppUserRequest aur){
       this.appUserRequest = aur;
   }
   
   public final String openUserAccountRequest(Long id){       
       appUserRequest = new AppUserRequest();
           
       Query q = em.createNativeQuery(" SELECT "
       		                        + " app_user_request_id, "
                                    + " user_name,  "
                                    + " enabled,     "
                                    + " last_name,  "
                                    + " first_name,  "
                                    + " middle_name,  "
                                    + " phone_number,  "
                                    + " email_address,  "                                                                        
                                    + " deactivation_comments,  "
                                    + " request_roles,  "
                                    + " request_facilities, " 
                                    + " request_type,  "
                                    + " request_status,  "
                                    + " created_by,  "
                                    + " date_created  "
                                    + " FROM fpps_owner.app_user_request a where a.app_user_request_id=:id");
       q.setParameter("id", id);
       AppUserRequest aur= new AppUserRequest();
     
       Object[] row = (Object[]) q.getSingleResult();
       
       BigDecimal aid = (BigDecimal) row[0];
       aur.setAppUserRequestId(aid.longValue());
       aur.setUserName((String) row[1]);
       aur.setEnabled((String) row[2]);              
       aur.setLastName((String) row[3]);
       aur.setFirstName((String) row[4]);
       aur.setMiddleName((String) row[5]);
       aur.setPhoneNumber((String) row[6]);
       aur.setEmailAddress((String) row[7]);
       aur.setDeactivationComments((String) row[8]);
       
       String roles = (String) row[9];
       roles = roles==null ? "" : roles;
       roles = roles.replace(",","<br />");
       aur.setRequestRoles(roles);
       
       String facs = (String) row[10];
       if(facs!=null){
           facs = "'" + facs + "'";
           facs = facs.replace(", ", "','");
       }
       String sql = " select 'Visn:' || f.visn_id_cd || ' / Facility:' || " +
             " f.va_facility_cd as facstring " +
             " from va_facility f " +
             " where f.va_facility_cd in ("+facs+") " +
             " order by f.visn_id_cd ASC, f.va_facility_cd ASC";
       q = em.createNativeQuery(sql);
       
       List<String> list2 = q.getResultList();
       String facslist = "";
       for(String fac : list2) {
           facslist+=fac + "<br />";
       }
       //facs = facs==null ? "" : facs;
       //facslist = facslist.replace(",","<br />");
       aur.setRequestFacilities(facslist);
       
       BigDecimal rt = (BigDecimal) row[11];
       boolean rtb = rt.intValue()==0 ? false : true;
       aur.setRequestType(rtb);
       BigDecimal rs = (BigDecimal) row[12];
       //boolean rsb = rs.intValue()==0 ? false : true;
       aur.setRequestStatus(rs.intValue());
       aur.setCreatedBy((String) row[13]);
       aur.setDateCreated((Date) row[14]);
       
       appUserRequest = aur;
       
       String ret = row==null ? "openUserRequestFAIL" : "openUserRequestSUCCESS";
       
       return ret;
   }
   
   
   
   public final String markRequestComplete(Long id){   
       String ret = "markRequestCompleteFAIL";
       boolean success = false;
       
       Query q = em.createNativeQuery("UPDATE fpps_owner.app_user_request a " 
                       		+ " SET "
                       		+ " a.request_status=1, "
                       		+ " a.modified_by=:modifiedby, "
                       		+ " a.date_modified=sysdate "
                       		+ " WHERE a.app_user_request_id=:id");
                       
       q.setParameter("id", id);
       q.setParameter("modifiedby", appUser.getUserName().trim().toUpperCase());
       
       q.executeUpdate();
       q = em.createNativeQuery("COMMIT");
       q.executeUpdate();
       
       
       success = true;
       ret = "markRequestCompleteSUCCESS";
              

       if (success) {
           StatusMessages.instance().addFromResourceBundle(Severity.INFO,
                   "gov.va.med.fee.userAdmin.REQUEST_COMPLETED_SUCCESS", "");
       } else {
           StatusMessages.instance().addFromResourceBundle(Severity.ERROR,                       
                   "gov.va.med.fee.userAdmin.REQUEST_COMPLETED_ERROR", "");
       }
       
       return ret;
   }
   
   /**
    * {@inheritDoc}
    */
   @Destroy
   @Remove
   public final void destroy() {
      // log.info("destroyed:" + this);
   }
}
