package gov.va.med.hac.edi.era.jpa;

import gov.va.med.hac.edi.utils.*;
import gov.va.med.hac.edi.utils.jpa.JPAUtil;
import gov.va.med.hac.edi.utils.jpa.LogUtils;

import java.io.*;
import java.util.logging.*;

import javax.naming.*;
import javax.persistence.EntityManager;
import javax.persistence.FlushModeType;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.GnuParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public final class ERepos835Load {    

	static final Logger summaryLOGGER = LoggerFactory.getLogger(ERepos835Load.class);
	static final Logger traceLOGGER = LoggerFactory.getLogger("trace." + ERepos835Load.class);
   /**
	   * This is the main program to call to move data from the hac staging tables 
	   * [FPPS_OWNER schema]
	   * into the E_REPOS schema.  
	   * 
	   * Normally this program is run without passing it 
	   * any arguments although, it allows the caller to override the database 
	   * configuration (run the program with the -h command line pararmeter for more 
	   * information).  
	   * 
	   * This class sets up two entity managers - one that normally reads data
	   * from the staging tables and one that writes to the erepos schema. It 
	   * delegates the responsibility of moving the data from the stage tables to the
	   * eRepository tables to ProcessERepos835Load.
	   * 
	   * It handles any exceptions that occur during the processing by rolling back the
	   * active transaction and logging the error to the log.
	   * 
	   * @param args
	   */
		public static void main(final String[] args) {
			// Local variables
			boolean processSuccess = false;
			
			boolean DEBUG = true;
	
			CommandLine line = null;
			Options opts = getCliOptions();
			try {
				CommandLineParser parser = new GnuParser();
				line = parser.parse(opts, args);
				// If the "help" option is specified, just show usage options and exit
				if (line.hasOption("h")) {
					printUsage(opts, args, false);
					System.exit(1);
				}
				// If any required command line parameter is missing, show usage options and exit
			} catch (MissingOptionException moe) {
				printUsage(opts, args, true);
				System.exit(1);
			} catch (ParseException pe) {
				System.out.println("Problem parsing the command line");
				summaryLOGGER.error("Problem parsing the command line");
				traceLOGGER.error("Problem parsing the command line");
				printUsage(opts, args, false);
				System.exit(1);
			}
	
			// Catch any exception (outside of setting up for logging)
			try {
				LogUtils.stampStartingTime("eRepos 835 Start", summaryLOGGER);
				summaryLOGGER.info("eRepos 835 Start");
				summaryLOGGER.info("Code Version: 5010 - a");
				traceLOGGER.info("eRepos 835 Start");
				traceLOGGER.info("Code Version: 5010");
	
				File configFile = null;
				String configFilePath = line.getOptionValue("f");
				if (configFilePath != null && configFilePath.length() > 0) {
					configFile = JPAUtil.getJpaConfigFile(configFilePath.trim());
				} else {
					// .properties file found in resources/ directory
					configFile = JPAUtil.getJpaConfigFile("dbhibernate.properties");
					// configFile = JPAUtil.getJpaConfigFile("ereposrt.properties");
				}
	
				// Carry out the loading
				delegateLoad( null, configFile);
	
				// Put a completion message into the log file
				summaryLOGGER.info("Process completed successfully");
				traceLOGGER.info("Process completed successfully");
				LogUtils.stampEndingTime("eRepos 835", summaryLOGGER);
				processSuccess = true;
	
			} catch (OutOfMemoryError e) {
				
				summaryLOGGER.error("!!!!! Out Of Memory Error !!!!!", e);
				traceLOGGER.error("!!!!! Out Of Memory Error !!!!!", e);
			} catch (Throwable e) {
				
				if(DEBUG) {
					System.out.println("ERepos835Load.main() - error: " + e.getMessage());
				}
				
				
				
				summaryLOGGER.error("Exception in processing", e);
				traceLOGGER.error("Exception in processing", e);
			} finally {
				if (!processSuccess) {
					
					if(DEBUG) {
						System.out.println("ERepos835Load.main() - process failed!");
					}
					
					
					summaryLOGGER.error("!!!!! Process seems to have failed !!!!!");
					traceLOGGER.error("!!!!! Process seems to have failed !!!!!");
				}
			}
		}

/**
   * This is the main program to call to move data from the hac staging tables 
   * [FPPS_OWNER schema]
   * into the eRepos schema.  
   * 
   * Normally this program is run without passing it 
   * any arguments although, it allows the caller to override the database 
   * configuration (run the program with the -h command line pararmeter for more 
   * information).  
   * 
   * This class sets up two entity managers - one that normally reads data
   * from the staging tables and one that writes to the erepos schema. It 
   * delegates the responsibility of moving the data from the stage tables to the
   * eRepository tables to ProcessERepos835Load.
   * 
   * It handles any exceptions that occur during the processing by rolling back the
   * active transaction and logging the error to the log.
   * 
   * @param args
   */
		
		/**
	public static void main(final String[] args) {
		// Local variables
		boolean processSuccess = false;
		
		
		boolean DEBUG = true;

		CommandLine line = null;
		Options opts = getCliOptions();
		try {
			CommandLineParser parser = new GnuParser();
			line = parser.parse(opts, args);
			// If the "help" option is specified, just show usage options and exit
			if (line.hasOption("h")) {
				printUsage(opts, args, false);
				System.exit(1);
			}
			// If any required command line parameter is missing, show usage options and exit
		} catch (MissingOptionException moe) {
			printUsage(opts, args, true);
			System.exit(1);
		} catch (ParseException pe) {
			System.out.println("Problem parsing the command line");
			summaryLOGGER.error("Problem parsing the command line");
			traceLOGGER.error("Problem parsing the command line");
			printUsage(opts, args, false);
			System.exit(1);
		}

		// Catch any exception (outside of setting up for logging)
		try {
			LogUtils.stampStartingTime("eRepos 835 Start", summaryLOGGER);
			summaryLOGGER.info("eRepos 835 Start");
			summaryLOGGER.info("Code Version: 5010 - a");
			traceLOGGER.info("eRepos 835 Start");
			traceLOGGER.info("Code Version: 5010");

			File configFile = null;
			String configFilePath = line.getOptionValue("f");
			if (configFilePath != null && configFilePath.length() > 0) {
				configFile = JPAUtil.getJpaConfigFile(configFilePath.trim());
			} else {
				// .properties file found in resources/ directory
				configFile = JPAUtil.getJpaConfigFile("dbhibernate.properties");
			}

			// Carry out the loading
			delegateLoad( null, configFile);

			// Put a completion message into the log file
			summaryLOGGER.info("Process completed successfully");
			traceLOGGER.info("Process completed successfully");
			LogUtils.stampEndingTime("eRepos 835", summaryLOGGER);
			processSuccess = true;

		} catch (OutOfMemoryError e) {
			
			summaryLOGGER.error("!!!!! Out Of Memory Error !!!!!", e);
			traceLOGGER.error("!!!!! Out Of Memory Error !!!!!", e);
		} catch (Throwable e) {
			
			if(DEBUG) {
				System.out.println("ERepos835Load.main() - error: " + e.getMessage());
			}
			
			
			
			summaryLOGGER.error("Exception in processing", e);
			traceLOGGER.error("Exception in processing", e);
		} finally {
			if (!processSuccess) {
				
				if(DEBUG) {
					System.out.println("ERepos835Load.main() - process failed!");
				}
				
				
				summaryLOGGER.error("!!!!! Process seems to have failed !!!!!");
				traceLOGGER.error("!!!!! Process seems to have failed !!!!!");
			}
		}
	}
**/
	private static void delegateLoad(final PrintWriter lFile, File configFile)
	throws Throwable
	{
		String DATE_FORMAT = "HH:mm:ss:SSS";
		final ProcessERepos835Load p835Load = new ProcessERepos835Load();

		EntityManager dbEM835Stg = null;  
		EntityManager dbEMErepos = null;

		// Set up an entity manager using the hibernate properties in
			// dbhibernate.properties
			// You may change the properties set in this file to point at a
			// different schema, log sql statements, etc.
			// This code also uses the persistence unit named codeConversion,
			// this is where JPA looks to see if it
			// should persist a class or not. JPA also uses this info to see how
			// it should map a class to the database
			// see ormidsmapped.xml for more details
			// For JPA to connect to the database "dbhibernate.properties"
			// should contain the following properties
			// hibernate.connection.username, hibernate.connection.password, &
			// hibernate.connection.url

			dbEM835Stg =  JPAUtil.getEntityManager(configFile, 
			"stgToEreposReader");

			dbEMErepos = JPAUtil.getEntityManager(configFile,
			"stgToEreposWriter");

			dbEM835Stg.setFlushMode(FlushModeType.COMMIT);
			dbEMErepos.setFlushMode(FlushModeType.COMMIT);
			

			p835Load.process835Load(dbEM835Stg, dbEMErepos);
			// Log the ending of this file number
	}

	private static void printUsage(Options opts, String[] inArgs, boolean missingArg) {
		final int helpwidth = 80;
		HelpFormatter fmtr = new HelpFormatter();
		System.out.println();
		if (missingArg) {
			System.out.println("Some required commandline argument was missing");
			summaryLOGGER.error("Some required commandline argument was missing");
		}
		fmtr.printHelp(helpwidth, "ERepos835Load", "", opts, "", true);
		if (missingArg) {
			System.out.println(inArgs.length + " command line arguments were received: ");
			summaryLOGGER.error(inArgs.length + " command line arguments were received: ");
			for (int i = 0; i < inArgs.length; i++) {
				System.out.println(inArgs[i]);
				summaryLOGGER.error(inArgs[i]);
			}
		}
	}

	/**
	 * Helper method to populate command line options objects.
	 * 
	 * @return the <code>Options</code> object containing the set of command line option.
	 */
	@SuppressWarnings("static-access")
	private static Options getCliOptions() {
		Options options = new Options();

		Option configfile = OptionBuilder.withArgName("jpaFile").hasArg().withLongOpt("jpaFile")
		.withDescription("The JPA Configuration file, such as hibernate.properties.").create("f");
		Option help = OptionBuilder.withLongOpt("help").withDescription("Print this message.").create(
		"h");
		options.addOption(help);
		options.addOption(configfile);
		return options;
	}

}