package gov.va.med.hac.edi.era.jpa;

import gov.va.med.domain.erepos.*;
import gov.va.med.domain.erepos.batchload.ClaimDtDAO;
import gov.va.med.domain.erepos.batchload.EntityAddressDAO;
import gov.va.med.domain.erepos.batchload.EntityNameDAO;
import gov.va.med.hac.edi.utils.*;

import java.math.BigDecimal;
import java.sql.*;
import java.util.*;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

// Copy data from the 835 staging tables into the regular ERepository tables
public class ProcessERepos835Load {
  static final Logger summaryLOGGER = LoggerFactory.getLogger(ProcessERepos835Load.class);
  static final Logger traceLOGGER = LoggerFactory.getLogger("trace." + ProcessERepos835Load.class);
  // Constants
  final private static String DATE_FORMAT = "HH:mm:ss:SSS";
  final private static String INPROCESS_STATUS = "INPROCESS";
  final private static String ERROR_STATUS = "ERROR";
  final private static BigDecimal zeroBD = new BigDecimal("0");

  ///////////////////// Testing and debugging instrumentation ///////////
  // filesetRestrictionString is used during testing to create a run
  //   over a subset of file_numbers in a file
  // Set this to a blank string when not testing
  //private static String testFilesetRestrictionClause
  //    = "";
      // = " AND file_number = 'W84146_9999'";
 
  
  // test with actual input file here............file in path of this class
  private static String testFilesetRestrictionClause = " AND file_number = '835_input_test'";
  
  

  // testTransactionRestrictionString is used during testing to create a run
  //   over a subset of transactions in a fileset
  // Set this to a blank string when not testing
  private static String testTransactionRestrictionClause
      = "";
      // = " AND TO_NUMBER(ST02TXN_CTL_NBR) < 101";
  ///////////////////// End of Testing and debugging instrumentation ///////////

  // Facades for inserting and updating
  InsertionMethods insertFacade = new InsertionMethods();
  StagingFacade stagingFacade = new StagingFacade();

  // Top-level control field; there are few file numbers in a run
  private String vFileNumber = "";

  // Control field, level 2 : a file_number has many transaction control numbers
  // Transaction control number is plain in staging, but left-padded with zeros
  //   in the ERepository
  private String vSt02txnCtlNbrPadded = "";
  private String vSt02txnCtlNumberNoPad = "";

  // Claim key is for querying from the 837 data
  private String vClaimKey = "";
  private String v837FileNumber = "";
  private String v837St02txnCtlNbr = "";
  private String v837SbsrId = "";
  private String v837BillHrchlKey = "";
  private String v837SbsrHrchlKey = "";

  // The Detail Head Info Key stores the GUID of the DETAIL_HEAD_INFO record
  //   for when it is needed in table CLAIM_PYMT_INFO
  private String vDtlHeadInfoKey = "";

  // Control field, level 3: A transaction has a small number of claim payments
  private String vClpKey = "";

  // Control field, level 4: A claim payment has a variable number of service lines
  private String vSvcKey = "";

  // The staging versions of claim payment key and service line key are used in
  //   queries against the staging area
  private String vStgClpKey = "";
  private String vStgSvcKey = "";

  // vTransactionCount counts the number of transactions (ie. headers)
  //   in a fileset, for updating the functional group record
  private int vTransactionCount = 0;

  // vSegmentCount counts the number of segments in a header, that is, the
  //   number of records inserted for a transaction.
  private int vSegmentCount = 0;

  // vQueryCount is used for naming dynamically-created queries
  // Perhaps this is used if more than one file are picked up for the run
  private int vQueryCount = 0;

  private String vClearinghouseId = "";
  private String vPayerId    = "";

  private String vFacilityId = "";
  private String vProviderId = "";
  private String vProviderNpi= null;
  private String vPayeeName  = "";

  // The production date will be set in insertTraceNumbers, then get its own record
  //   in table CLAIM_DT
  private String vProdDate   = "";
  private String vClaimDate  = "";

  // We have no mechanism to set vIntAmt at present
  private BigDecimal vIntAmt = zeroBD;

  // List of the filesets that are being processed
  final private ArrayList fileNumbersList  = new ArrayList(3);

  // hasXCodeError marks whether an error occurred in XCODE conversion, for
  //   the current fileset
  private boolean hasXCodeError = false;

  // This method carries most of the logic
  final protected void process835Load(EntityManager dbEMStg, EntityManager dbEMErepos)
  throws Throwable
  {
	  
	  boolean DEBUG = true;
	 
if(DEBUG) {
	System.out.println("process835Load() dbEMStg is: " + dbEMStg);
}
	  
	  
	// no records returned here - all stg_*.* are EMPTY - even where there is a single record with correct status ! 
    Query query = dbEMStg.createNativeQuery("SELECT DISTINCT FILE_NUMBER FileNumber FROM E_REPOS.STG_835_HDR WHERE STSE_STATUS = 'NEW' "); // + testFilesetRestrictionClause);
    ;

    List fileNumberList = null;
    Iterator fileNumberIterator = null;
    
    	try {
    		// NO RECORDS IN FPPS_OWNER.stg_835_HDR table.
				  fileNumberList = query.getResultList();	
					  
				  fileNumberIterator = fileNumberList.iterator();
				  
			    // Put the fileset identifiers into a vector
				 while (fileNumberIterator.hasNext()) {
				      final String nextFileNumber = (String) fileNumberIterator.next();
				      
				      if(DEBUG) {
				    	  System.out.println("process835Load() - nextFileNumber is: " + nextFileNumber);
				      }
				      fileNumbersList.add(nextFileNumber);
			    }	// end while
			    
			    if(DEBUG) {
			    	  System.out.println("process835Load() - size of fileNumbersList is: " + fileNumbersList.size());
			     }  
    	}catch(Exception e) {
    		System.out.println("process835Load() ERROR: " + e.getMessage());
    		e.printStackTrace();
    	}
    
    // Loop through the filesets, processing each one
    final int numFileSets = fileNumbersList.size();
    System.out.println("process835Load(): " + numFileSets + " Filesets are available in 835 Staging Tables for Processing");
    
    
 //   System.out.println(numFileSets + " Filesets are available in 835 Staging Tables for Processing");
    for (int i=0; i < numFileSets; i++) {
    	
      dbEMStg.getTransaction().begin();
  	  dbEMErepos.getTransaction().begin();
  	  
   	  // what is the significance of this?
      dbEMStg.flush();
      dbEMStg.clear(); 
      dbEMErepos.flush();
      dbEMErepos.clear();
      
      // Log and rollback if this doesn't work
      try {
        // Count the transactions (ie. headers) for this file number
        vTransactionCount = 0;
  
        // Start processing this fileset
        vFileNumber = (String) fileNumbersList.get(i);
        
        if(DEBUG) {
        	System.out.println("process835Load() - Start Processing For FileNumber : " + vFileNumber );
        }
        
        System.out.println("process835Load() - Start Processing For FileNumber : " + vFileNumber);
   
        // For filesets that come from the HAC system, rather
        //   than the FEE system, we need to convert from the
        //   old XCODE procedure codes
        // Ticket: DEF004652 - Changed 4 to 5 below.
        if (vFileNumber.substring(1,5).equalsIgnoreCase("8414")) {
          // Assume no XCODE errors in this fileset
          hasXCodeError = false;
  
          // Perform XCODE conversion as necessary, while updating the
          //   status of HDR, CLP, and SVC records on the way, for all the
          //   records in this fileset
          convertProcCode(dbEMStg, dbEMErepos);
        } else {
          // Update the HDR, CLP, SVC table statuses from 'NEW' to 'INPROCESS'
           stagingFacade.updateStatusColumns(vFileNumber, dbEMStg);
           System.out.println("process835Load() - Statuses changed from 'NEW' to 'INPROCESS' for FileNumber : " + vFileNumber);
        }
        
        dbEMStg.flush();
        dbEMErepos.flush();
        dbEMErepos.clear();
        
        // Insert into the two X12 envelope tables
        // Note: At this point, we have not yet counted the transactions,
        //   whose count goes into table FUNCTIONAL_GROUP.  But, we have to
        //   insert FUNCTIONAL_GROUP first because TXN_SET has a referential
        //   integrity constraint to it.  The movement of data from
        //   the cache to the database goes by default in the same order in
        //   which records were inserted in the code.
        System.out.println("process835Load() - Inserting envelope records at " + DateUtil.getCurrentTimeFormatted(DATE_FORMAT));
        
        
        FunctionalGroup functionalGroup = insertEnvelopeInfo(dbEMStg, dbEMErepos);
        
        dbEMStg.flush();
        dbEMErepos.flush();
        dbEMErepos.clear();
        
        //Don't clear eRepos objects
        
        // Insert records for most of the tables
        System.out.println("process835Load() - Starting to insert transactions at " + DateUtil.getCurrentTimeFormatted(DATE_FORMAT));
        
        insertHeaderSegments_STSE(dbEMStg, dbEMErepos);
        dbEMStg.flush();        
        dbEMErepos.flush();
        dbEMErepos.clear();
        
        System.out.println("process835Load() - DONE inserting transactions at " + DateUtil.getCurrentTimeFormatted(DATE_FORMAT));
       
        // Update the transaction count in the record we earlier created
        //   for table FUNCTIONAL_GROUP
        
        System.out.println("process835Load() - updateTransactionSetCount() at " + DateUtil.getCurrentTimeFormatted(DATE_FORMAT));
        
        updateTransactionSetCount(dbEMErepos);
        
        
        System.out.println("process835Load() - DONE updateTransactionSetCount() at " + DateUtil.getCurrentTimeFormatted(DATE_FORMAT));
        
        
        
        
        System.out.println("Transactions inserted:" + vTransactionCount);
        
        dbEMStg.flush();
        dbEMErepos.flush();
        dbEMErepos.clear();
        
        // Insert a record of this process having been run
        System.out.println("process835Load() - Inserting an 835 process insertFacade.insertEdi835Process() records at "
              + DateUtil.getCurrentTimeFormatted(DATE_FORMAT));
        
        insertFacade.insertEdi835Process(vFileNumber, vClearinghouseId, vPayerId, vSt02txnCtlNbrPadded, dbEMErepos);
        
        
        System.out.println("process835Load() - DONE Inserting an 835 process insertFacade.insertEdi835Process() records at "
                + DateUtil.getCurrentTimeFormatted(DATE_FORMAT));

        
        
        
        dbEMStg.flush();
        dbEMErepos.flush();
        dbEMErepos.clear();
                  
	        // For File Numbers that encountered an error with XCodes, add
	        //   a suffix to distinguish them the next time they are processed
	        // Note: If there was an XCode error for a given header, that header
	        //   would not have been copied to the eRepos tables.  We need to update
	        //   the file number for those headers, giving it a new suffix, so that
	        //   it will in future be treated as a different fileset
	        if (hasXCodeError) {
	          System.out.println("process835Load() - Error occurred. Updating file number at "
	              + DateUtil.getCurrentTimeFormatted(DATE_FORMAT));
	          System.out.println("process835Load() - Error occurred. Updating file number at "
	                  + DateUtil.getCurrentTimeFormatted(DATE_FORMAT));
	          stagingFacade.updateFileNumber( vFileNumber, dbEMStg);
	        }
  
  			System.out.println(("process835Load() - Committing the ERepository transaction at "
  					+ DateUtil.getCurrentTimeFormatted(DATE_FORMAT)));
  
  			dbEMErepos.getTransaction().commit();
  			System.out.println("process835Load() - Commit successful");
  
  			System.out.println("process835Load() - Committing the staging transaction at "
  					+ DateUtil.getCurrentTimeFormatted(DATE_FORMAT));
  
  			dbEMStg.getTransaction().commit();
  			System.out.println("process835Load() - ALL DONE Commit successful");	       
  			
  		}catch(Exception e) {
  			handleException(dbEMStg, "staging", e);
  			handleException(dbEMErepos, "erepos", e);
  			
  			if(DEBUG) {
  				System.out.println("process835Load() error: " + e.getMessage());
  				e.printStackTrace();
  			}
  		}
    } // end of loop through the filesets
      
  } // end method process835Load

  private static void handleException(EntityManager dbEMErepos, String identifier, Throwable e) throws Throwable {
  	
  	// System.out.println("Rolling back the database transaction " + identifier + "\n" , e);
  	// System.out.println("Rolling back the database transaction " + identifier + "\n" , e);
  	if (dbEMErepos != null) {
  		if (dbEMErepos.getTransaction().isActive()) {
  			dbEMErepos.getTransaction().rollback();
  		}
  	}
  	System.out.println("Rollback successful");
  	System.out.println("Rollback successful");
  	throw e;
  }
  
  private void updateTransactionSetCount(EntityManager dbEM)
  {
	  Query getFunctionalGroupToUpdate = dbEM.createQuery("SELECT f FROM gov.va.med.domain.erepos.FunctionalGroup f "
                        + "  WHERE f.fileNumber=?1" );
	  getFunctionalGroupToUpdate.setParameter(1, vFileNumber);
	  FunctionalGroup functionalGroupToUpdate = (FunctionalGroup) getFunctionalGroupToUpdate.getSingleResult();
	  functionalGroupToUpdate.setGe01nbrTranSets(Integer.toString(vTransactionCount));
	  dbEM.persist(functionalGroupToUpdate);
    
  } // end method u
  
  // Insert a record into each of the "envelope" tables:
  //   INTCHG_CONTROL (Interchange control) and FUNCTIONAL_GROUP
  private FunctionalGroup insertEnvelopeInfo(EntityManager dbEMStg, EntityManager dbEMErepos)
  throws SQLException
  {
	  
	  boolean DEBUG = true;
	  
	  FunctionalGroup functionalGroupToReturn = null;
      // Call up the first header record from the staging tables
    Query getHeaderRecord = dbEMStg.createQuery("SELECT h"
                        + "  FROM gov.va.med.domain.erepos.Stg835Hdr h "
                        + "  WHERE h.fileNumber=?1 AND ROWNUM=?2");
    getHeaderRecord.setParameter(1, vFileNumber);
    getHeaderRecord.setParameter(2, new Long(1));
    
    List headers = getHeaderRecord.getResultList();    
   
      Stg835Hdr crStg835Hdr = null;
      if(headers.size()>0)
      {
    	  crStg835Hdr = (Stg835Hdr) headers.get(0);
      }
      
      // Only do this if a header record was found
      if (crStg835Hdr != null) {
          // Pull a few pieces of data out of the header record, into globals
          vFileNumber = crStg835Hdr.getFileNumber();
          vClearinghouseId = crStg835Hdr.getClearinghouseId();
          vPayerId = crStg835Hdr.getPayerId();

          final String transactionDate = DateUtil.getCurrentDateFormatted("yyyyMMdd");
          final String transactionTime = DateUtil.getCurrentTimeFormatted("HHmm");

          // Insert a row into INTCHG_CONTROL
          // hangs here.....
          
          
if(DEBUG) {
	System.out.println("insertEnvelopeInfo() - calling insertInterchangeControl() ");
}
          insertFacade.insertInterchangeControl(vFileNumber, vPayerId, vClearinghouseId, crStg835Hdr, transactionDate, transactionTime, dbEMErepos);
          
if(DEBUG) {
	System.out.println("insertEnvelopeInfo() - DONE calling insertInterchangeControl() - calling insertFunctionalGroup() ");
}

          // Insert a row into FUNCTIONAL_GROUP
          functionalGroupToReturn = insertFacade.insertFunctionalGroup(vFileNumber, vPayerId, vClearinghouseId, transactionDate, transactionTime, dbEMErepos);
      }
      
      
      
  if(DEBUG) {
		System.out.println("insertEnvelopeInfo() - DONE calling insertFunctionalGroup() ");
}      
      
      
      
      return functionalGroupToReturn;
  } // end method insertEnvelopeInfo

  // For those headers in the current fileset that were marked as INPROCESS
  //   by the proc code conversion, insert rows into some tables (like TXN_SET,
  //   FINANCIAL_INFO, CLAIM_PYMT_INFO, and SVC_PYMT_INFO), and call methods
  //   to insert rows into most of the other tables.
  private void insertHeaderSegments_STSE(EntityManager dbEMStg, EntityManager dbEMErepos)
  throws SQLException, BadInputDataException
  {
    
	  
	  boolean DEBUG = true;
	Query getInProcessHeadersQuery = dbEMStg.createNativeQuery("SELECT * FROM STG_835_HDR " +
			 " WHERE STSE_STATUS = ?1 AND FILE_NUMBER = ?2 " + 
			 testTransactionRestrictionClause +
			 " ORDER BY TO_NUMBER(ST02TXN_CTL_NBR)", Stg835Hdr.class );				
	getInProcessHeadersQuery.setParameter(1,"INPROCESS");
	getInProcessHeadersQuery.setParameter(2,vFileNumber);
	
	List inProcessHeaders = getInProcessHeadersQuery.getResultList();
	Iterator inProcessHeaderIterator = inProcessHeaders.iterator();
	
	//Hack so that I can clear the JPA cache
	while(inProcessHeaderIterator.hasNext()) { inProcessHeaderIterator.next(); }
	inProcessHeaderIterator = inProcessHeaders.iterator();
	
	int headerCounter = 0;
    // Loop through the headers, processing each by inserting into several tables
    while (inProcessHeaderIterator.hasNext()) {
    	headerCounter++;
    	
    	if(headerCounter % 50 == 0)
    	{
    		dbEMStg.flush();
    		dbEMErepos.flush();    		
    		dbEMErepos.clear();
    	}
    	
      // Get the next header
      final Stg835Hdr crStg835Hdr = (Stg835Hdr) inProcessHeaderIterator.next();

      // Insert that header
      
      if(DEBUG) {
    		System.out.println("insertHeaderSegments_STSE() - calling insertHeaderSegment_STSE() ");
    	}      
      
      
      
      insertHeaderSegment_STSE(crStg835Hdr, dbEMStg, dbEMErepos);

      
      if(DEBUG) {
  		System.out.println("insertHeaderSegments_STSE() - DONE calling insertHeaderSegment_STSE() ");
  	}      
    
      
      
      
      // Show a little output in case this is being run manually
      if (vTransactionCount % 100 == 0)
      {
        System.out.println("insertHeaderSegments_STSE() - Finished transaction count: " + vTransactionCount);
      }
      
      
      
      
    }
  } // end method insertHeaderSegments_STSE

  // Insert information for one header, including its claim payments
  //   and service lines
  private void insertHeaderSegment_STSE(final Stg835Hdr crStg835Hdr, EntityManager dbEMStg, EntityManager dbEMErepos)
  throws SQLException, BadInputDataException
  {
	  
	  
	boolean DEBUG = true;
    // count the number of segments inserted in this transaction
    vSegmentCount = 0;

    // Reset the instance variables that vary for each header
    initializeHeaderVariables();

    // TODO: File number and clearinghouse and payerId
    //   should have already been set, and should
    //   not change from header to header.  Maybe assert this?
    vFileNumber = crStg835Hdr.getFileNumber();
    vClearinghouseId = crStg835Hdr.getClearinghouseId();
    vPayerId = crStg835Hdr.getPayerId();

    // Get variable pieces of info from the header record
    vFacilityId = crStg835Hdr.getFacilityId();
    vProviderId = crStg835Hdr.getProviderId();
    vPayeeName  = crStg835Hdr.getProviderName();
    vProviderNpi = crStg835Hdr.getNpi();

    // Use both the unpadded and padded versions of the
    //   transaction control number
    vSt02txnCtlNumberNoPad = crStg835Hdr.getSt02txnCtlNbr();
    vSt02txnCtlNbrPadded = StringUtils.lPadZeros(vSt02txnCtlNumberNoPad, 9);

    // Create a new record representing the transaction, in TXN_SET
    
    TxnSet trTxnSet = (TxnSet) new TxnSet();
    TxnSetId trTxnSetId = (TxnSetId) new TxnSetId();
    trTxnSetId.setFileNumber(vFileNumber);
    trTxnSetId.setSt02txnCtlNbr(vSt02txnCtlNbrPadded);    
    trTxnSet.setId(trTxnSetId);
    trTxnSet.setTxType("835");
    trTxnSet.setSt01txnId("835");
    vSegmentCount++;
    

    // TWP: REMOVE Forgein Key Order Hacking !!! Insert the new transaction record
    trTxnSet.setSe02txnCtlNbr(vSt02txnCtlNbrPadded);
    trTxnSet.setSe01txnSgmtCnt(new BigDecimal(Integer.toString(vSegmentCount)));
    dbEMErepos.persist(trTxnSet);
    
    // Get a record from the provider information table, used mostly
    //   for setting fields in the financial info record   
    Query getProviderInformationQuery = dbEMStg.createQuery("SELECT bpr"
            + "  FROM gov.va.med.domain.erepos.Stg835HdrBpr bpr "
            + "   WHERE bpr.fileNumber=?1 AND bpr.st02txnCtlNbr=?2");
    getProviderInformationQuery.setParameter(1, vFileNumber);
    getProviderInformationQuery.setParameter(2, vSt02txnCtlNumberNoPad);
    
    List providerInformations = getProviderInformationQuery.getResultList();    
        
    Stg835HdrBpr crStg835HdrBpr = null;
    if(providerInformations.size() > 0)
    {
    	crStg835HdrBpr = (Stg835HdrBpr) providerInformations.get(0);
    }
    
    // Create a new record for table financial_info
    insertFacade.insertFinancialInfo(crStg835HdrBpr
            , vFileNumber, vSt02txnCtlNbrPadded, dbEMErepos);
    vSegmentCount++;

    // Insert a single record into table TRACE_NUMBER, which identifies
    //   this transaction for future reference
    insertTraceNumbers_TRN(dbEMStg, dbEMErepos);

    // Insert a record indicating the identity of the receiver of this 835,
    //   but only if the clearinghouse is not SXC
    if (!vClearinghouseId.equals("752578509")) {
      insertFacade.insertSegmentREF(vFileNumber, vSt02txnCtlNbrPadded,
          vClpKey, vSvcKey, "EV", vClearinghouseId, "HEADER", dbEMErepos);
      vSegmentCount++;
    }

    // Insert the production date, that is, the end date of the
    //   adjudication production cycle for included claims
    insertFacade.insertDate(vFileNumber, vSt02txnCtlNbrPadded,
        vClpKey, vSvcKey, "405", vProdDate, dbEMErepos);
    vSegmentCount++;

    // Insert records into several tables to identify the payer
    insertPayerIdentification(dbEMStg, dbEMErepos);

    // Insert a record into table ENTITY_ID to identify the payee
    insertPayeeIdentification_N1(dbEMErepos);

    // Insert a record into table ENTITY_ADTL_ID identifying the
    insertPayeeIdentification_REF(dbEMErepos);

    // Insert a skeleton record into DETAL_HEAD_INFO, setting the
    //   key on the way
    vDtlHeadInfoKey = insertFacade.insertProviderSummaryLX(
        vFileNumber, vSt02txnCtlNbrPadded, dbEMErepos);
    vSegmentCount++;

    // Get all the claim payment records for this transaction  
    Query getClaimPaymentsQuery = dbEMStg.createQuery("SELECT cp"
            + "  FROM gov.va.med.domain.erepos.Stg835Clp cp "
            + "   WHERE cp.fileNumber=?1 AND cp.st02txnCtlNbr=?2");
    getClaimPaymentsQuery.setParameter(1, vFileNumber);
    getClaimPaymentsQuery.setParameter(2, vSt02txnCtlNumberNoPad);
    
    List claimPayments = getClaimPaymentsQuery.getResultList();    
    Iterator claimPaymentIterator = claimPayments.iterator();

    // Loop through the claim payment records
    while (claimPaymentIterator.hasNext()) {
      final Stg835Clp crStg835Clp1 = (Stg835Clp) claimPaymentIterator.next();

      insertClaimPayment_CLP(crStg835Clp1,dbEMStg, dbEMErepos);

    }  // end loop for each claim

    insertPayeeIdentification_N3N4(dbEMErepos);
    
    //----Provider Adjustment (PLB)
    insertProviderAdjustment_PLB(dbEMStg, dbEMErepos);

    // On the new transaction record (table TXN_SET), set the count
    //   of segments we have been accumulating, and the control number
    trTxnSet.setSe01txnSgmtCnt(new BigDecimal(Integer.toString(vSegmentCount)));
    trTxnSet.setSe02txnCtlNbr(vSt02txnCtlNbrPadded);

    // Insert the new transaction record
    dbEMErepos.persist(trTxnSet);
    vTransactionCount++;

    // Mark the header record for deletion from staging
    crStg835Hdr.setStseStatus("DELETE");
    dbEMStg.persist(crStg835Hdr);
    
    
    if(DEBUG) {
    	System.out.println("insertHeaderSegment_STSE() - DONE");
    }
    
    
    
    
  } // end method insertHeaderSegmentSTSE

  // Insert information for one claim payment, including service lines
  private void insertClaimPayment_CLP(final Stg835Clp crStg835Clp1, EntityManager dbEMStg, EntityManager dbEMErepos)
  throws SQLException
  {

    // Service line records are linked to claim payments over column clp_key
    vStgClpKey = crStg835Clp1.getClpKey();
    vClaimKey  = crStg835Clp1.getClaimKey();
    v837FileNumber = "";
    v837St02txnCtlNbr = "";
    v837SbsrId = "";
    v837BillHrchlKey = "";
    v837SbsrHrchlKey = "";
    
    
    Query getClaimQuery = 
		dbEMErepos.createNativeQuery("SELECT * FROM CLAIM WHERE CLAIM_KEY=?1 ", Claim.class);
    	getClaimQuery.setParameter(1, vClaimKey);
    	//System.out.println("E_REPOS Claim Query");
		List claim = getClaimQuery.getResultList();
		Iterator claimIterator = claim.iterator();
		Claim rowClaim = null;
		if (claimIterator.hasNext()) {
			rowClaim = (Claim) claimIterator.next();			
		}
		
		if (rowClaim != null) {
			v837FileNumber = rowClaim.getFileNumber();
			v837St02txnCtlNbr = rowClaim.getSt02txnCtlNbr();
			v837SbsrId = rowClaim.getSubscriberId();
			v837BillHrchlKey = rowClaim.getBillingHrchlLevelKey();
			v837SbsrHrchlKey = rowClaim.getSubscriberHrchlLevelKey();
		}
    /* 
	Query getEntityNameQuery = 
		dbEMErepos.createNativeQuery("SELECT *"
	            + "  FROM ENTITY_NAME "
	            + "   WHERE  CLAIM_KEY=?1 AND LOOP_ID=?2 AND NM101ENTY_ID_CD IN (?3)", EntityName.class);
		getEntityNameQuery.setParameter(1, claimKey);
		getEntityNameQuery.setParameter(2, "2310B");
		String [] codes = {"72","82"};
		getEntityNameQuery.setParameter(3, codes);		
		List entityNames = getEntityNameQuery.getResultList();
		Iterator entityNameIterator = entityNames.iterator();
		String vDate = null;
		EntityName rowEntityName = null;
		if (entityNameIterator.hasNext()) {
			rowEntityName = (EntityName) entityNameIterator.next();
		}*/		

	    // Get all the claim adjustment records at the claim payment level,
	    //   for the current claim payment
		//Sreedhar - Changes made for CR 13902
	    Query getClaimAdjSegQuery = dbEMStg.createQuery("SELECT adj"
	            + "  FROM gov.va.med.domain.erepos.Stg835ClpCas adj "
	            + "   WHERE adj.clpKey=?1 ");
	    getClaimAdjSegQuery.setParameter(1, vStgClpKey);
	    
	    List claimAdjSeg = getClaimAdjSegQuery.getResultList();   
	    Iterator claimAdjSegIterator = claimAdjSeg.iterator();

	    String vPayerClmCode = "1";

	    // Loop through the claim adjustments to the current claim payment
	    while (claimAdjSegIterator.hasNext()) {
	      final Stg835ClpCas vStg835ClpCas = (Stg835ClpCas) claimAdjSegIterator.next();
	      
	      if (vStg835ClpCas.getCas02clmAdjRsn1().equalsIgnoreCase("23")) 
	      {
	    	  vPayerClmCode = "2";
	      }

	      
	      if ((vStg835ClpCas.getCas05clmAdjRsn2() != null) && (vStg835ClpCas.getCas05clmAdjRsn2().equalsIgnoreCase("23")))
	      {
	    	  vPayerClmCode = "2";
	      }
	      
	      if ((vStg835ClpCas.getCas08clmAdjRsn3() != null) && (vStg835ClpCas.getCas08clmAdjRsn3().equalsIgnoreCase("23")))
	      {
	    	  vPayerClmCode = "2";
	      }
	      
	      if ((vStg835ClpCas.getCas11clmAdjRsn4() != null) && (vStg835ClpCas.getCas11clmAdjRsn4().equalsIgnoreCase("23")))
	      {
	    	  vPayerClmCode = "2";
	      }
	      
	      if ((vStg835ClpCas.getCas14clmAdjRsn5() != null) && (vStg835ClpCas.getCas14clmAdjRsn5().equalsIgnoreCase("23")))
	      {
	    	  vPayerClmCode = "2";
	      }
	      
	      if ((vStg835ClpCas.getCas17clmAdjRsn6() != null) && (vStg835ClpCas.getCas17clmAdjRsn6().equalsIgnoreCase("23")))
	      {
	    	  vPayerClmCode = "2";
	      }
	      
	      
	    }
		
    // Insert a record for the claim payment, setting the key on the way
    vClpKey = insertFacade.insertClaimPymtInfo(crStg835Clp1, rowClaim
          , vFileNumber, vSt02txnCtlNbrPadded, vDtlHeadInfoKey, vPayerClmCode, dbEMErepos);// end changes for CR 13902
    vClaimDate = crStg835Clp1.getClaimDate();
    vSegmentCount++;

    // Insert adjustment records at the claim payment level
    insertClaimAdjustmants_CAS(dbEMStg,dbEMErepos);

    // Insert a record in table ENTITY_NAME to hold the Patient Name
    insertNames_NM1("QC", dbEMStg,dbEMErepos);

    // Service Provider (NM1):  HAC and FEE have different
    //    methods due to a difficiency in how the service
    //    provider is set in FEE.
    // If the payer is FEE, we need a more complicated method
    //   of setting the provider name
    /*if ("12115".equals(vPayerId) || "12116".equals(vPayerId)) {
        insertFeeProviderName_NM1(crStg835Clp1.getClaimKey(), dbEMStg,dbEMErepos);
    } else {
        insertNames_NM1("82", dbEMStg,dbEMErepos);
    }*/
    
    insertServiceProviderName_NM1(crStg835Clp1.getClaimKey(), dbEMStg,dbEMErepos);
    
    //Sreedhar - Changes made for CR 13904

    String v837ClaimType = "";
    String vBillType = "";
    BigDecimal v837Clm02Amt = new BigDecimal("0");
    BigDecimal v835Clp03Amt = new BigDecimal("0");
    
    if (rowClaim != null) {
        v837ClaimType = rowClaim.getClaimType();
        vBillType = rowClaim.getClm05fclyTypeCd();
        v837Clm02Amt = rowClaim.getClm02clmChrgAmt();
        v835Clp03Amt = crStg835Clp1.getClp03clmChrgAmt();
    }
    
    if (v835Clp03Amt.longValue() < v837Clm02Amt.longValue()) {    
	    if (((v837ClaimType.substring(0, 10).equalsIgnoreCase("005010X223")) ||
	       (v837ClaimType.equalsIgnoreCase("004010X096A1"))) &&
	       ((vBillType.equalsIgnoreCase("11")) || (vBillType.equalsIgnoreCase("21")) ||
	        (vBillType.equalsIgnoreCase("28")) || (vBillType.equalsIgnoreCase("41")) ||
	    	(vBillType.equalsIgnoreCase("65")) || (vBillType.equalsIgnoreCase("66")) ||
	    	(vBillType.equalsIgnoreCase("86")))) {
	    	
	    	insertFacade.insertInpatientAdjId_MIA(vClpKey, dbEMErepos);
	     } else {
	    	insertFacade.insertOutpatientAdjId_MOA(vClpKey, dbEMErepos); 
	     }
     }
    
    // Claim Date
    insertFacade.insertDate
        (null, null, vClpKey, vSvcKey, "050", vClaimDate, dbEMErepos);
    vSegmentCount++;

    // Claim Supplemental Amount (AMT) is only inserted if there is interest,
    //   which there never is as of this writing
    final int numClaimAmountsInserted = insertFacade.insertClaimSuplAmountAMT
        (vClpKey, vIntAmt, dbEMErepos);
    vSegmentCount += numClaimAmountsInserted;

    // Get records in STG_835_SVC for this claim payment    
    Query getSvcRecordsQuery = dbEMStg.createQuery("SELECT svc"
            + "  FROM gov.va.med.domain.erepos.Stg835Svc svc "
            + "   WHERE svc.clpKey=?1 ");
    getSvcRecordsQuery.setParameter(1, vStgClpKey);
    
    List svcRecords = getSvcRecordsQuery.getResultList();   
    Iterator svcRecordsIterator = svcRecords.iterator();

    // Loop through the service lines, inserting into several tables
    while (svcRecordsIterator.hasNext()) {
      final Stg835Svc crStg835Svc = (Stg835Svc) svcRecordsIterator.next();

      insertServiceLine_SVC(crStg835Svc, dbEMStg, dbEMErepos);

    } // end loop for each service line
    vClpKey = "";
    vSvcKey = "";

    // Mark the claim payment record for deletion from staging
    //TWP:NEED TO CONVERT & SAVE BC$J figures this out not JPA
    crStg835Clp1.setClpStatus("DELETE");
    dbEMStg.persist(crStg835Clp1);
  } // end method insertClaimPayment_CLP

// Insert information for one service line
  private void insertServiceLine_SVC(final Stg835Svc crStg835Svc, EntityManager dbEMStg, EntityManager dbEMErepos)
  throws SQLException
  {
    vStgSvcKey = crStg835Svc.getSvcKey();

    // Insert a record for the service, while setting the key for the service line
    vSvcKey = insertFacade.insertSvcPymtInfo(crStg835Svc , vClpKey, dbEMErepos);
    vSegmentCount++;

    // Insert a record for the service date (type 472) or the start and
    //   end of the service period
    final String vSvcDtQual = crStg835Svc.getDtm01ddtmQual();
    if ("472".equals(vSvcDtQual)) {
      insertFacade.insertDate
          (null, null, vClpKey, vSvcKey,
            vSvcDtQual, crStg835Svc.getDtm01dtFld(), dbEMErepos);
      vSegmentCount++;
    } else {
      insertFacade.insertDate
          (null, null, vClpKey, vSvcKey,
            vSvcDtQual, crStg835Svc.getDtm01dtFld(), dbEMErepos);
      vSegmentCount++;

      insertFacade.insertDate
          (null, null, vClpKey, vSvcKey,
            crStg835Svc.getDtm02ddtmQual(), crStg835Svc.getDtm02dtFld(), dbEMErepos);
      vSegmentCount++;
    }

    // Insert the line control number used by the provider, if we have one
    if (crStg835Svc.getRef02adtlId() != null) {
      insertFacade.insertSegmentREF(vFileNumber,
          vSt02txnCtlNbrPadded, vClpKey, vSvcKey, "6R",
          crStg835Svc.getRef02adtlId(), "2110", dbEMErepos);
      vSegmentCount++;
    }

    // Insert the Service Level Rendering Provider ID
    if (crStg835Svc.getSvcRefRenderingProvId() != null) {
      insertFacade.insertSegmentREF(vFileNumber,
          vSt02txnCtlNbrPadded, vClpKey, vSvcKey, "HPI",
          crStg835Svc.getSvcRefRenderingProvId(), "2110", dbEMErepos);
      vSegmentCount++;
    }

    // Insert Service Supplemental Amount
    if (crStg835Svc.getSvcAllowedAmt() != null) {
      insertFacade.insertServiceSuplAmountAMT(vClpKey, vSvcKey, "B6",
          crStg835Svc.getSvcAllowedAmt(), "2110", dbEMErepos);
      vSegmentCount++;
    }

    // Insert adjustments at the service level
    insertServiceLevelAdjustments_CAS(dbEMStg,dbEMErepos);

    // Insert Health Care Remark Codes (LQ)
    insertHealthCareRemarkCodes_LQ(dbEMStg,dbEMErepos);

    vSvcKey = "";

    // Mark the service line record for deletion from staging
    crStg835Svc.setSvcStatus("DELETE");
    dbEMStg.persist(crStg835Svc);
  } // end method insertServiceLine_SVC

  // Insert a record into ERepos table TRACE_NUMBER for the current transaction
  private void insertTraceNumbers_TRN(EntityManager dbEMStg, EntityManager dbEMErepos)
  //throws Exception
  {
      // Get the trace number records from staging for the current transaction      
      Query getTraceNumberQuery = dbEMStg.createQuery("SELECT trn"
              + "  FROM gov.va.med.domain.erepos.Stg835HdrTrn trn "
              + "   WHERE trn.fileNumber=?1 AND trn.st02txnCtlNbr=?2");
      getTraceNumberQuery.setParameter(1, vFileNumber);
      getTraceNumberQuery.setParameter(2, vSt02txnCtlNumberNoPad);
     
      List traceNumbers = getTraceNumberQuery.getResultList(); 
      Iterator traceNumberIterator = traceNumbers.iterator();

      // Loop through the staged trace number records
      // TODO: There seem to never be more than one, which makes sense
      //   Maybe we should check this as a precondition?
      while (traceNumberIterator.hasNext()) {
          final Stg835HdrTrn crStg835HdrTrn = (Stg835HdrTrn) traceNumberIterator.next();
          vSegmentCount++;

          insertFacade.insertTraceNumber(crStg835HdrTrn,
                vFileNumber, vSt02txnCtlNbrPadded, vPayerId, dbEMErepos);
          vProdDate = crStg835HdrTrn.getProdDate();

      }  // -- end loop while
  }

  // Create rows in several tables that identify the payer
  private void insertPayerIdentification(EntityManager dbEMStg, EntityManager dbEMErepos)
  throws SQLException, BadInputDataException
  {
    // Get the applicable row from table STG_835_PAYER_INFO
    Query getPayerInfoQuery = dbEMStg.createQuery("SELECT inf "
            + "  FROM gov.va.med.domain.erepos.Stg835PayerInfo inf "
            + "   WHERE inf.id.payerId=?1 AND inf.id.facilityId=?2");
    getPayerInfoQuery.setParameter(1, vPayerId);
    getPayerInfoQuery.setParameter(2, vFacilityId);   
    List payerInfos = getPayerInfoQuery.getResultList(); 
 
    Query getPayerInfoQueryBus = dbEMStg.createQuery("SELECT inf "
            + "  FROM gov.va.med.domain.erepos.Stg835PayerInfo inf "
            + "   WHERE inf.id.payerId=?1 AND inf.id.facilityId=?2");    
    getPayerInfoQueryBus.setParameter(1, "12345");
    getPayerInfoQueryBus.setParameter(2, "0");   
    List payerInfosBus = getPayerInfoQueryBus.getResultList(); 
    
    // Note: As of this writing, there are no multiples in the data
    Stg835PayerInfo crStg835PayerInfo = null;
    Stg835PayerInfo crStg835PayerInfoBus = null;
    if(payerInfos.size() > 0)
    {
    	crStg835PayerInfo = (Stg835PayerInfo) payerInfos.get(0);
    }
    // If no payer info records were found in staging, log it and error out
    if (crStg835PayerInfo == null) {
      System.out.println("Couldn't find the Payer Information in STG_835_PAYER_INFO table for the following Visin Id: " + vFacilityId);
      System.out.println("Couldn't find the Payer Information in STG_835_PAYER_INFO table for the following Visin Id: " + vFacilityId);
      
      throw new BadInputDataException();

    // If we found a payer info record in staging, insert rows in several tables
    } else {
      // Insert a row into ENTITY_ID
      insertFacade.insertEntityId(vFileNumber, vSt02txnCtlNbrPadded,
          crStg835PayerInfo.getPayerName(), "1000A", "PR", null, null, dbEMErepos);
      vSegmentCount++;

      // Insert a row into ENTITY_ADDRESS
      insertFacade.insertEntityAddress(crStg835PayerInfo, vFileNumber, vSt02txnCtlNbrPadded, dbEMErepos);
      vSegmentCount++;

      // Business Contact Information
      // In order to write a contact record for this payer, we need at least one
      //   of contact name, phone number, or fax number

      if(payerInfosBus.size() > 0)
      {
      	crStg835PayerInfoBus = (Stg835PayerInfo) payerInfosBus.get(0);
      }
      
      String cntcName = crStg835PayerInfoBus.getPayerContact();
      String phoneNum = crStg835PayerInfoBus.getPayerPhone();
      String eMail   = crStg835PayerInfoBus.getPayerEmail();

      if ((!((cntcName == null) || (cntcName.equals("")))) ||
              (!((phoneNum == null) || (phoneNum.equals("")))) ||
              (!((eMail == null)   || (eMail.equals(""))))) {

// If we got one of those, create a new row for table CNTC_INFO
        insertFacade.insertContactInfoBus(vFileNumber, vSt02txnCtlNbrPadded
          , cntcName, phoneNum, eMail, dbEMErepos);
        vSegmentCount++;
      }

	    //Technical Contact Information
	    // In order to write a contact record for this payer, we need at least one
	    //   of contact name, phone number, or fax number
	    cntcName = crStg835PayerInfo.getPayerContact();
	    phoneNum = crStg835PayerInfo.getPayerPhone();
	    eMail   = crStg835PayerInfo.getPayerEmail();
	
	    if ((!((cntcName == null) || (cntcName.equals("")))) ||
	            (!((phoneNum == null) || (phoneNum.equals("")))) ||
	            (!((eMail == null)   || (eMail.equals(""))))) {
	
	      // If we got one of those, create a new row for table CNTC_INFO
	      insertFacade.insertContactInfo(vFileNumber, vSt02txnCtlNbrPadded
	        , cntcName, phoneNum, eMail, dbEMErepos);
	      vSegmentCount++;               
      }
    }
  } // end method insertPayerIdentification

  // Insert a row in table ENTITY_ID to identify the payee
  private void insertPayeeIdentification_N1(EntityManager dbEMErepos)
  throws SQLException
  {
    // Decide on the code and qualifier before calling the insert method
    String n103IdCdQual = null;
    String n104IdCd = null;

    // Currently we require the tax id to process claims.  So
    // during the NPI transition if we get only one id then it has
    // to be the TIN stored in vProviderId (atypical providers such as
    // HUMANA which are not eligible for NPI's). IF They send an NPI
    // then that goes in the N104 segment and the TIN goes in the REF segment
    if(vProviderNpi == null) {
      // check because some of the pre NPI era providerId columns can contain
      // both NPI or TIN
      if (vProviderId.length() > 9) {
        n103IdCdQual = "XX";
      } else {
        n103IdCdQual = "FI";
      }
      n104IdCd = vProviderId;
    } else {
      n103IdCdQual = "XX";
      n104IdCd = vProviderNpi;
    }

    // Insert the row
    insertFacade.insertEntityId(vFileNumber, vSt02txnCtlNbrPadded
        ,vPayeeName, "1000B", "PE", n103IdCdQual, n104IdCd, dbEMErepos);
    vSegmentCount++;
    
  } // end method insertPayeeIdentificationN1

  //
  private void insertPayeeIdentification_REF(EntityManager dbEMErepos)
  throws SQLException
  {
    // TODO: What does this comment mean?
    // If both the NPI and Tin were sent then the TIN goes
    // in the REF segment of the 1000B (ENTITY_ADTL_ID table).
     if (vProviderNpi != null) {
       insertFacade.insertSegmentREF(vFileNumber, vSt02txnCtlNbrPadded,
            vClpKey, vSvcKey, "TJ", vProviderId, "1000B", dbEMErepos);
       vSegmentCount++;
     }
  } // end method insertPayeeIdentificationRef

  //
  private void insertPayeeIdentification_N3N4(EntityManager dbEMErepos)
  throws SQLException
  {  
	    Query getEntityAddressQuery = 
		dbEMErepos.createNativeQuery("SELECT *"
	            + "  FROM ENTITY_ADDRESS "
	            + "  WHERE (FILE_NUMBER = ?1 "
	            + "  AND ST02TXN_CTL_NBR = ?2 "
	            + "  AND NM109BILL_PRVD_ID = ?3 "
	            + "  AND LOOP_ID = ?4) "
	            + "  OR (HRCHL_LEVEL_KEY = ?5)", EntityAddressDAO.class);
		getEntityAddressQuery.setParameter(1, v837FileNumber);
		getEntityAddressQuery.setParameter(2, v837St02txnCtlNbr);
		if (vProviderNpi != null){
			getEntityAddressQuery.setParameter(3, vProviderNpi);
		} else {
			getEntityAddressQuery.setParameter(3, vProviderId);
		}
		
		getEntityAddressQuery.setParameter(4, "2010AA");
		getEntityAddressQuery.setParameter(5, v837BillHrchlKey);
		List entityAddresss = getEntityAddressQuery.getResultList();
		Iterator entityAddressIterator = entityAddresss.iterator();

		EntityAddressDAO rowEntityAddress837 = null;
		if (entityAddressIterator.hasNext()) {
			rowEntityAddress837 = (EntityAddressDAO) entityAddressIterator.next();			
		}
	  	
		// in the N3 and N4 segments for 1000B (ENTITY_ADDRESS table).
		if (rowEntityAddress837 != null) {
		     insertFacade.insertEntityAddress(rowEntityAddress837, vFileNumber, vSt02txnCtlNbrPadded, dbEMErepos);
		     vSegmentCount++; //N3 Count
		     vSegmentCount++; //N4 Count
		} 
		else if (vFileNumber.substring(0, 3).equalsIgnoreCase("SXC")) {
			Query getTPQuery = 
			dbEMErepos.createNativeQuery("SELECT *"
		            + "  FROM TRADING_PARTNERS "
		            + "  WHERE ABBREV_TAG = ?1 ", TradingPartners.class);
		    getTPQuery.setParameter(1, "SXC");
		
			List tradingPartners = getTPQuery.getResultList();
			Iterator tradingPartnersIterator = tradingPartners.iterator();
		
			TradingPartners rowTradingPartners = null;
			if (tradingPartnersIterator.hasNext()) {
				rowTradingPartners = (TradingPartners) tradingPartnersIterator.next();			
			}
			
			if (rowTradingPartners != null) {
			     insertFacade.insertEntityAddress(rowTradingPartners, vFileNumber, vSt02txnCtlNbrPadded, dbEMErepos);
			     vSegmentCount++; //N3 Count
			     vSegmentCount++; //N4 Count
			} 			
			
			
		 }
     
  } // end method insertPayeeIdentificationRef
  
  // Insert adjustment records at the claim payment level
  private void insertClaimAdjustmants_CAS(EntityManager dbEMStg, EntityManager dbEMErepos)
  throws SQLException
  {
    // Get all the claim adjustment records at the claim payment level,
    //   for the current claim payment
    Query getClaimAdjustmentsQuery = dbEMStg.createQuery("SELECT adj"
            + "  FROM gov.va.med.domain.erepos.Stg835ClpCas adj "
            + "   WHERE adj.clpKey=?1 ");
    getClaimAdjustmentsQuery.setParameter(1, vStgClpKey);
    
    List claimAdjustments = getClaimAdjustmentsQuery.getResultList();   
    Iterator claimAdjustmentIterator = claimAdjustments.iterator();

    int claimCasesSeqNbr = 0;

    // Loop through the claim adjustments to the current claim payment
    while (claimAdjustmentIterator.hasNext()) {
      vSegmentCount++;
      claimCasesSeqNbr++;
      final Stg835ClpCas crStg835ClpCas = (Stg835ClpCas) claimAdjustmentIterator.next();

      // Only do this if a claim adjustment group is present
      // TODO: This seems to never be null, why do we check?
      if (crStg835ClpCas != null && crStg835ClpCas.getCas01clmAdjGrp() != null &&
    		  !(crStg835ClpCas.getCas03adjAmt1().equals(new BigDecimal(0)))) {
          insertFacade.insertClaimAdjustment(crStg835ClpCas
              , vClpKey, claimCasesSeqNbr, dbEMErepos);
      }
    }

  } // end method insertClaimAdjustmantCAS

  // Insert a new record into table ENTITY_NAME, coming from table STG_835_NAME,
  //   with the type of name coming in as a parameter
  private void insertNames_NM1(final String vQual, EntityManager dbEMStg, EntityManager dbEMErepos)
  throws SQLException
  {
    // Get the record from STG_835_CLP_NAME for this claim payment and the
    //   given qualifier  
    Query getNameQuery = dbEMStg.createQuery("SELECT name"
            + "  FROM gov.va.med.domain.erepos.Stg835ClpName name "
            + "   WHERE name.clpKey=?1 AND name.nm101entyIdCd=?2");
    getNameQuery.setParameter(1, vStgClpKey);
    getNameQuery.setParameter(2, vQual);
    
    List names = getNameQuery.getResultList();   
    Iterator nameIterator = names.iterator();    
    // Loop through the results, creating a record in the target for each one
    while (nameIterator.hasNext()) {
        final Stg835ClpName crStg835ClpName = (Stg835ClpName) nameIterator.next();
        insertFacade.insertEntityNameFromStaging(crStg835ClpName
            , vFileNumber, vClpKey, vSt02txnCtlNbrPadded, dbEMErepos);
        vSegmentCount++;
    }

  } // end method insertNameNM1

  /**
   *  This method, like the inserNameNM1, method populates the EntityName
   *  table for 835 Service Provider Name segment.  But for FEE, and unlike
   *  the HAC CPS, the information returned from the FEE CPS is incorrect
   *  and needs to be gathered from the information sent in the 837 instead
   *  of the information sent in from the CPS.
   *
   *  @param claimKey the claim key used to look up the Service Provider
   *         name.
   */
  // This method reads info from ENTITY_NAME that came in on the 837, and
  //   creates another record in the same table, linked to claim_pymt_info,
  //   that is, used in creating the 835 going out.
  private void insertServiceProviderName_NM1(final String claimKey, EntityManager dbEMStg, EntityManager dbEMErepos)
  throws SQLException
  {
    // Get the record from table ENTITY_NAME, filled by the 837 process,
    //   that holds physician name information for the claim with the given key
		//EntityName entityNameToReturn = null;
		
		//TWP:REMEDIATED FOR PERFORMANCE
//		Query getEntityNameQuery = 
//		dbEMErepos.createQuery("SELECT en"
//	            + "  FROM gov.va.med.domain.erepos.batchload.EntityNameDAO en "
//	            + "   WHERE  en.claim.claimKey=?1 AND en.loopId=?2 AND en.nm101entyIdCd IN (?3)");
	
	    Query getEntityNameQuery = 
		dbEMErepos.createNativeQuery("SELECT *"
	            + "  FROM ENTITY_NAME "
	            + "   WHERE  CLAIM_KEY=?1 AND LOOP_ID IN (?2,?3) AND NM101ENTY_ID_CD=?4", EntityNameDAO.class);
		getEntityNameQuery.setParameter(1, claimKey);
		getEntityNameQuery.setParameter(2, "2310B");
		getEntityNameQuery.setParameter(3, "2310D");
		//String [] codes = {"72","82"};
		//getEntityNameQuery.setParameter(3, codes);
		getEntityNameQuery.setParameter(4, "82");
		List entityNames = getEntityNameQuery.getResultList();
		Iterator entityNameIterator = entityNames.iterator();
		//String vDate = null;
		EntityNameDAO rowEntityName = null;
		if (entityNameIterator.hasNext()) {
			rowEntityName = (EntityNameDAO) entityNameIterator.next();
		}

    // If one was found, insert a new record into the same table, linked to
    //   the new claim payment record
    // Manual merge: 9-23-08 head change
    //if (rowEntityName != null) {
    if (rowEntityName != null
      && validRenderingProviderId(rowEntityName.getNm108idCdQual(), rowEntityName.getNm109idCd())) {
    // End manual merge
      insertFacade.insertEntityNameFromERepos(rowEntityName,
          vFileNumber, vClpKey, vSt02txnCtlNbrPadded, dbEMErepos);
      vSegmentCount++;
    }
    
    
	Query getClaimDtQuery = 
		dbEMErepos.createNativeQuery("SELECT * "
	            + "  FROM CLAIM_DT "
	            + "   WHERE  CLAIM_KEY=?1 AND DTP01DTTM_QUAL=?2 ", ClaimDtDAO.class);
	getClaimDtQuery.setParameter(1, claimKey);
	getClaimDtQuery.setParameter(2, "434");

	List claimDtl = getClaimDtQuery.getResultList();
	Iterator claimDtIterator = claimDtl.iterator();
	//String vDate = null;
	ClaimDtDAO rowClaimDt = null;
	if (claimDtIterator.hasNext()) {
		rowClaimDt = (ClaimDtDAO) claimDtIterator.next();
	}    
 
    if (rowClaimDt != null) {
    	String vStmtDate = rowClaimDt.getDtp03dtFld();
    	
    	if (rowClaimDt.getDtp02dttmFmtQual().equalsIgnoreCase("D8"))
    	{
    	    // Claim Date
    	    insertFacade.insertDate
    	        (null, null, vClpKey, vSvcKey, "232", vStmtDate, dbEMErepos);    		
    	}  
    	else if (rowClaimDt.getDtp02dttmFmtQual().equalsIgnoreCase("RD8"))
    	{
    	    insertFacade.insertDate
	        (null, null, vClpKey, vSvcKey, "232", vStmtDate.substring(0,8), dbEMErepos);
    	    vSegmentCount++;
    	    
    	    insertFacade.insertDate
	        (null, null, vClpKey, vSvcKey, "233", vStmtDate.substring(9), dbEMErepos);  
    	}

      vSegmentCount++;
    }
    
  } // end method insertFeeProvNameNM1

  
  // Insert claim adjustments at the service line level
  private void insertServiceLevelAdjustments_CAS(EntityManager dbEMStg, EntityManager dbEMErepos)
  throws SQLException
  {
    // Get the claim adjustments for the current service line
    Query getCalimAdjustmentsQuery = dbEMStg.createQuery("SELECT adj"
            + "  FROM gov.va.med.domain.erepos.Stg835SvcCas adj "
            + "   WHERE adj.svcKey=?1 AND adj.clpKey=?2 ");
    getCalimAdjustmentsQuery.setParameter(1, vStgSvcKey);
    getCalimAdjustmentsQuery.setParameter(2, vStgClpKey);
    
    List claimAdjustments = getCalimAdjustmentsQuery.getResultList();   
    Iterator claimAdjustmentIterator = claimAdjustments.iterator();
    
    int serviceCasesSeqNbr = 0;

    // Loop through the claim adjustments
    while (claimAdjustmentIterator.hasNext()) {
      vSegmentCount++;
      serviceCasesSeqNbr++;
      final Stg835SvcCas crStg835SvcCas = (Stg835SvcCas) claimAdjustmentIterator.next();
      // Only do this if a claim adjustment group is present
      if ( crStg835SvcCas.getCas01clmAdjGrp() != null &&
        !(crStg835SvcCas.getCas03adjAmt1().equals(new BigDecimal(0)))) {
        insertFacade.insertServiceLevelAdjustment(crStg835SvcCas,
            vClpKey, vSvcKey, serviceCasesSeqNbr, dbEMErepos);
      }
    }
  } // end method insertServiceLevelAdjustmentsCAS

  // Insert records into table HCARE_REMARK_CD
  private void insertHealthCareRemarkCodes_LQ(EntityManager dbEMStg, EntityManager dbEMErepos)
  throws SQLException
  {
    // Get the records from STG_835_SVC_LQ for this service line
    Query getSvcRecordsQuery = dbEMStg.createQuery("SELECT r"
            + "  FROM gov.va.med.domain.erepos.Stg835SvcLq r "
            + "   WHERE r.svcKey=?1 AND r.clpKey=?2 ");
    getSvcRecordsQuery.setParameter(1, vStgSvcKey);
    getSvcRecordsQuery.setParameter(2, vStgClpKey);
    
    List svcRecords = getSvcRecordsQuery.getResultList();   
    Iterator svcRecordsIterator = svcRecords.iterator();
    int remarkCodesSeqNbr = 0;

    // Loop through the results
    while (svcRecordsIterator.hasNext()) {
      vSegmentCount++;
      remarkCodesSeqNbr++;
      final Stg835SvcLq crStg835SvcLq = (Stg835SvcLq) svcRecordsIterator.next();
      // Only do this if both a qualifier and remark code are present
      if (  ( crStg835SvcLq.getLq01cdQual() != null ) &&
              ( crStg835SvcLq.getLq02rmkCd() != null )  ) {

        // Insert a row into HCARE_REMARK_CD
        insertFacade.insertHcareRemarkCd(crStg835SvcLq,
              vSvcKey, remarkCodesSeqNbr, dbEMErepos);
      }
    }
  } // end method insertHealthCareRemarkCodesLQ

  // Insert a single row into table PRV_ADJ, which represents an adjustment
  //   at the provider level
  private void insertProviderAdjustment_PLB(EntityManager dbEMStg, EntityManager dbEMErepos)
  throws SQLException
  {
      // Get the record from STG_835_PLB for this transaction
      // (There should always be exactly one.)      
      Query getAdjInfoQuery = dbEMStg.createQuery("SELECT plb"
              + "  FROM gov.va.med.domain.erepos.Stg835Plb plb "
              + "   WHERE plb.fileNumber=?1 AND plb.st02txnCtlNbr=?2 AND plb.plb04prvdAdjAmt<>?3");
      getAdjInfoQuery.setParameter(1, vFileNumber);
      getAdjInfoQuery.setParameter(2, vSt02txnCtlNumberNoPad);
      getAdjInfoQuery.setParameter(3, new BigDecimal(0));
     
      List adjInfos = getAdjInfoQuery.getResultList(); 
      Stg835Plb crStg835Plb = null;
    
      if(adjInfos.size()>0)
      {
    	  crStg835Plb = (Stg835Plb) adjInfos.get(0);

    	  // Insert a row into table PRV_ADJ 
    	  //5010: This is now a conditional segment
    	  //we are not inserting rows with 0 in them.
          insertFacade.insertProviderAdjustment(vFileNumber,
              vSt02txnCtlNbrPadded, crStg835Plb, dbEMErepos);
          vSegmentCount++;
      }
           
      

  } // end method insertProviderAdjustmentPLB

  /**
   * For the file number currently stored in vFileNumber, convert all the
   *   procedure codes from the old XCODE standard
   * The strategy is to match back to the Claim table (filled before the data
   *   passed through Cache/Mumps) according to the charge amount on the service line.
   * If a single match is found, use the procedure code in one of the three
   *   subsidiary tables for service lines.
   * If unable to match because a claim has multiple lines with the same charge
   *   amount, try to convert the XCODE through our table.
   * If that doesn't work, give up and mark it as an error.
   *
   * @throws java.lang.Exception
   */
  private void convertProcCode(EntityManager dbEMStg, EntityManager dbEMErepos)
  throws BadInputDataException, DbTransactionEnderException
  {
    try {
      String tProcCode = "";
      int numConvertedNormally = 0;
      int numConvertedThroughXCodeTable = 0;
      int numErrors = 0;

      // Were all the service lines precessable for the current claim payment?
      boolean allSvcClear = true;

      // Were all the claim payments processable for the current header?
      boolean allClpClear = true;

      // Get all the header records for this fileset that are in NEW status
		Query getNewHeadersQuery = dbEMStg.createNativeQuery("SELECT * FROM STG_835_HDR " +
				 " WHERE STSE_STATUS = ?1 AND FILE_NUMBER = ?2 " + 
				 testTransactionRestrictionClause +
				 " ORDER BY TO_NUMBER(ST02TXN_CTL_NBR)", Stg835Hdr.class );				
		getNewHeadersQuery.setParameter(1,"NEW");
		getNewHeadersQuery.setParameter(2,vFileNumber);
		List newHeaders = getNewHeadersQuery.getResultList();
		Iterator newHeaderIterator = newHeaders.iterator();

	  int headerCounter = 0;
      while (newHeaderIterator.hasNext()) {
    	  headerCounter++;          
          if(headerCounter % 200 == 0)
          {
        	  
        	  dbEMStg.flush();
        	  dbEMErepos.flush();
        	  dbEMErepos.clear();
          }
          
        final Stg835Hdr  crStg835HdrA = (Stg835Hdr) newHeaderIterator.next();
        
        // Get all the new claim payment records for the current fileset
        //   and transaction
        vSt02txnCtlNbrPadded = crStg835HdrA.getSt02txnCtlNbr();
        
        Query getNewClaimPaymentsQuery = dbEMStg.createQuery("SELECT cp"
                + "  FROM gov.va.med.domain.erepos.Stg835Clp cp "
                + "   WHERE cp.fileNumber=?1 AND cp.st02txnCtlNbr=?2 AND cp.clpStatus=?3");
        getNewClaimPaymentsQuery.setParameter(1, vFileNumber);
        getNewClaimPaymentsQuery.setParameter(2, vSt02txnCtlNbrPadded);
        getNewClaimPaymentsQuery.setParameter(3, "NEW");
        
        List newClaimPayments = getNewClaimPaymentsQuery.getResultList();    
        Iterator newClaimPaymentsIterator = newClaimPayments.iterator();            
        
        
        // Loop through the claim payment records
        while (newClaimPaymentsIterator.hasNext()) {
          
          
          final Stg835Clp crStg835ClpA = (Stg835Clp) newClaimPaymentsIterator.next();
          vClpKey = crStg835ClpA.getClpKey();
          vClaimKey = crStg835ClpA.getClaimKey();
          
          // Get all the service lines for the given claim payment          
          Query getServiceLinesForClaimPaymentQuery = dbEMStg.createQuery("SELECT sl"
                  + "  FROM gov.va.med.domain.erepos.Stg835Svc sl "
                  + "   WHERE sl.clpKey=?1 AND sl.svcStatus=?2");
          getServiceLinesForClaimPaymentQuery.setParameter(1, vClpKey);
          getServiceLinesForClaimPaymentQuery.setParameter(2, "NEW");
          
          List serviceLinesForClaimPayment = getServiceLinesForClaimPaymentQuery.getResultList();    
          Iterator serviceLinesForClaimPaymentIterator = serviceLinesForClaimPayment.iterator();
          
          // Loop through the service lines
          while (serviceLinesForClaimPaymentIterator.hasNext()) {
            final Stg835Svc crStg835SvcA = (Stg835Svc) serviceLinesForClaimPaymentIterator.next();
            tProcCode = crStg835SvcA.getSvc01procCd();
            
            // If this service line's procedure code starts with X,
            //   it needs to be converted
            if (tProcCode.substring(0, 1).equalsIgnoreCase("X")) {
              String transactionType     = "";
              String vRevCode    = null;
              String vMedCode    = null;
              boolean convertThroughXCodeTable = false;

              // Get the claim to which this claim payment record is linked
              // TODO:  Can't this be done once per claim payment,
              //   outside of the service line loop?
                          
              Query getClaimForPaymentQuery = dbEMErepos.createQuery("select cl from gov.va.med.domain.erepos.Claim cl where cl.claimKey=?1");
              getClaimForPaymentQuery.setParameter(1, vClaimKey);
              
              List claimsForPayment = getClaimForPaymentQuery.getResultList();    
              Iterator claimsForPaymentIterator = claimsForPayment.iterator();

              if (claimsForPaymentIterator.hasNext()) {
            	  Claim claimForPayment = (Claim) claimsForPaymentIterator.next();
                  transactionType = claimForPayment.getClaimType();
              } else
              {
                // There are a small number of situations where a CLAIM_KEY in
                //   STG_835_CLP does not correspond to any in table CLAIM.
                // In such a situation, transactionType will be left blank.
                // We want to go to the XCODE conversion table.
                // Setting transactionType will skip the three following queries, and
                //   setting convertThroughXCodeTable will cause the XCODE table
                //   to be used
                transactionType = "Claim Not Found";
                convertThroughXCodeTable = true;
              }

              // For institutional claims, look at the entries in
              //   INST_SVC_LINE, matching by this service line's amount
              if ((transactionType.equalsIgnoreCase("004010X096A1")) || 
            	  (transactionType.substring(0, 10).equalsIgnoreCase("005010X223"))) {
            	
            	//TWP:REMEDIATE - ? we load 1 table at a time and trick hibernate by using transient in the config file
            	// so associated object do not get stored as null - this prevents us from querying .claim.claimKey since claim is transient.
            	  
//            	Query getInstitutionalServiceLineAmountQuery = dbEMErepos.createQuery("select svcline "
//                          + "  FROM gov.va.med.domain.erepos.InstSvcLine svcline "
//                          + "   WHERE svcline.claim.claimKey=?1 AND svcline.sv203lnItmChrg=?2");
            	  
                Query getInstitutionalServiceLineAmountQuery = dbEMErepos.createNativeQuery("select * FROM "
                        + "  INST_SVC_LINE  WHERE CLAIM_KEY=?1 AND SV203LN_ITM_CHRG=?2", InstSvcLine.class);
            	getInstitutionalServiceLineAmountQuery.setParameter(1, vClaimKey);
            	getInstitutionalServiceLineAmountQuery.setParameter(2, crStg835SvcA.getSvc02chrgAmt());
            	traceLOGGER.debug(" getInstitutionalServiceLineAmountQuery claim key:" + vClaimKey);
                List institutionalServiceLineAmounts = getInstitutionalServiceLineAmountQuery.getResultList();    
                Iterator institutionalServiceLineAmountIterator = institutionalServiceLineAmounts.iterator();

                // If there is only one institutional service
                //   for this amount, use its codes
                final int institutionalServiceLinesCount = institutionalServiceLineAmounts.size();
                if (institutionalServiceLinesCount == 1) {
                    final InstSvcLine rowInstSvcLine = (InstSvcLine) institutionalServiceLineAmountIterator.next();
                    vRevCode = rowInstSvcLine.getSv201svcLnRevCd();
                    vMedCode = rowInstSvcLine.getSv202procCd();

                // If there are more than one, convert through the XCODE table
                } else {
                  convertThroughXCodeTable = true;
                }

              // For dental claims, look at the entries in
              //   DENTAL_SVC_LINE for this service line's amount
              } else if ((transactionType.equalsIgnoreCase("004010X097A1")) || 
                	  (transactionType.substring(0, 10).equalsIgnoreCase("005010X224"))) {            	              	  
               	
              	//TWP:REMEDIATE - ? we load 1 table at a time and trick hibernate by using transient in the config file
              	// so associated object do not get stored as null - this prevents us from querying .claim.claimKey since claim is transient.
            	  
//            	  Query getDentalServiceLineAmountQuery = dbEMErepos.createQuery("SELECT ds"
//                      + "  FROM gov.va.med.domain.erepos.DentalSvcLine ds "
//                      + "   WHERE ds.claim.claimKey=?1 AND ds.sv302lnItmChrg=?2");
            	  
            	  Query getDentalServiceLineAmountQuery = dbEMErepos.createNativeQuery("SELECT * from DENTAL_SVC_LINE " +
            			"WHERE CLAIM_KEY=?1 AND SV302LN_ITM_CHRG=?2", DentalSvcLine.class);
            	  getDentalServiceLineAmountQuery.setParameter(1, vClaimKey);
            	  getDentalServiceLineAmountQuery.setParameter(2, crStg835SvcA.getSvc02chrgAmt());
            	  List dentlaServiceLineAmounts = getDentalServiceLineAmountQuery.getResultList();    
            	  Iterator dentlaServiceLineAmountIterator = dentlaServiceLineAmounts.iterator();
                  
                // If there is only one dental service for this amount, use its code
                if (dentlaServiceLineAmounts.size() == 1) {
                    final DentalSvcLine rowDentalSvcLine = (DentalSvcLine) dentlaServiceLineAmountIterator.next();
                    vMedCode = rowDentalSvcLine.getSv301adaCd();

                // If there are more than one, use the XCode table
                } else {
                    convertThroughXCodeTable = true;
                }

              // For professional claims, look at the entries in
              //   PRFSN_SVC_LINE for this service line's amount
              } else if ((transactionType.equalsIgnoreCase("004010X098A1")) || 
                	  (transactionType.substring(0, 10).equalsIgnoreCase("005010X222"))) {


//            	  //TWP:Was not a join but using hql required join - REMEDIATED for PERFORMANCE
//                  Query getInitialConversionCodeQuery = dbEMErepos.createQuery("SELECT ps"
//                          + "  FROM gov.va.med.domain.erepos.PrfsnSvcLine ps "
//                          + "   WHERE ps.claim.claimKey=?1 AND ps.sv102lineItmChrg=?2");
            	  
                  Query getInitialConversionCodeQuery = dbEMErepos.createNativeQuery("SELECT *"
                          + "  FROM PRFSN_SVC_LINE "
                          + "   WHERE CLAIM_KEY=?1 AND SV102LINE_ITM_CHRG=?2", PrfsnSvcLine.class);
                  
                  getInitialConversionCodeQuery.setParameter(1, vClaimKey);
                  getInitialConversionCodeQuery.setParameter(2, crStg835SvcA.getSvc02chrgAmt());
                  List initialXcodes = getInitialConversionCodeQuery.getResultList();    
                  Iterator initialXcodesIterator = initialXcodes.iterator();
                 
                  // If there is only one professional service for this amount, use it
                  if (initialXcodes.size() == 1) {
                      final PrfsnSvcLine rowPrfsnSvcLine = (PrfsnSvcLine) initialXcodesIterator.next();
                      vMedCode = rowPrfsnSvcLine.getSv101procCd();

                  // If there are more than one, use the XCode table
                  } else {
                      convertThroughXCodeTable = true;
                  }
                }

                // If more than one match was found, perform a lookup into
                //   our conversion table
                if (convertThroughXCodeTable) {
                    crStg835SvcA.setSvcStatus(INPROCESS_STATUS);
//                    Query getConversionCodeQuery = dbEMStg.createQuery("SELECT xc "
//                            + "  FROM gov.va.med.domain.erepos.Stg835Xcode xc "
//                            + "   WHERE xc.id.xcode=?1 AND xc.id.status=?2 ");

                    //JPA IS having a problem populating xcode object (table has no PK - object only has 1 field id) 
                    //using the query commented out above. Therefore, I changed it to use a native query - TWP 
                    
                    Query getConversionCodeQuery = dbEMStg.createNativeQuery("SELECT PROCEDURE_CODE "
                          + "  FROM STG_835_XCODE WHERE XCODE=?1 AND STATUS=?2");                  
                    getConversionCodeQuery.setParameter(1, tProcCode);
                    getConversionCodeQuery.setParameter(2, "ACTIVE");
                    List xcodes = getConversionCodeQuery.getResultList();    
                    Iterator xcodeIterator = xcodes.iterator();
                    if (xcodeIterator.hasNext()) {
                        final String crStg835Xcode = (String) xcodeIterator.next();
                        crStg835SvcA.setSvc01procCd(crStg835Xcode);
                        numConvertedThroughXCodeTable += 1;

                    // If no match was found in the conversion table,
                    //   mark the service line as an error
                    } else {
                        allSvcClear = false;
                        crStg835SvcA.setSvcStatus(ERROR_STATUS);
                        hasXCodeError = true;
                        numErrors += 1;
                    }

                // If only one match was found in the subsidiary tables
                //   for the current claim amount, use it to
                //    set the procedure code in the staging area
                } else {
                  // A Procedure Code is optional for Inst Claims but Rev Code is required on inbound.
                  if (vMedCode == null) {
                      crStg835SvcA.setSvc01procCd(vRevCode);
                      crStg835SvcA.setSvc01svcIdQual("NU");
                  } else {
                      crStg835SvcA.setSvc01procCd(vMedCode);
                  }
                  crStg835SvcA.setSvcStatus(INPROCESS_STATUS);
                  numConvertedNormally += 1;
                }

            // If this service line's procedure code does not start
            //   with X, we're done, just change its status
            } else {
                crStg835SvcA.setSvcStatus(INPROCESS_STATUS);
            }
            dbEMStg.persist(crStg835SvcA);
          } // end of loop over service lines

          // If all the service lines were processable, the claim
          //   payment record is processable
          if (allSvcClear) {
            crStg835ClpA.setClpStatus(INPROCESS_STATUS);

          // If some service line had an unconvertable procedure code,
          //   mark the claim payment as an error
          } else {
            crStg835ClpA.setClpStatus(ERROR_STATUS);

            // This means not all claim payments were processable
            allClpClear = false;

            // Reset for the next service line
            allSvcClear = true;
            hasXCodeError = true;
          }          
          dbEMStg.persist(crStg835ClpA);
        } // end of this claim payment record

        // If all the claim payments were processable, the header
        //   is processable
        if (allClpClear) {
          crStg835HdrA.setStseStatus(INPROCESS_STATUS);

        // Otherwise, the header contains an error
        } else {
          crStg835HdrA.setStseStatus(ERROR_STATUS);

          // Reset for the next claim payment
          allClpClear = true;
          hasXCodeError = true;
        }
        dbEMStg.persist(crStg835HdrA);
      }
     
      tProcCode = null;
      System.out.println("Committing the Staging transaction with the changed procedure codes at "
            + DateUtil.getCurrentTimeFormatted(DATE_FORMAT));
      dbEMStg.getTransaction().commit();
      System.out.println("Commit successful");
      
      dbEMStg.getTransaction().begin();
      // Report the number of XCodes converted
      System.out.println("XCode conversion completed");
      System.out.println("XCode conversion completed");
      System.out.println("XCodes converted normally: " + numConvertedNormally);
      System.out.println("XCodes converted normally: " + numConvertedNormally);
      System.out.println("XCodes converted through XCode lookup: "
          + numConvertedThroughXCodeTable);
      System.out.println("XCodes converted through XCode lookup: "
              + numConvertedThroughXCodeTable);
      System.out.println("XCode conversion problems: " + numErrors);
      System.out.println("XCode conversion problems: " + numErrors);


    } catch(Exception ex) {
        System.out.println("An exception occured while converting proc codes" + ex.getMessage());
      System.out.println("An exception occured while converting proc codes" + ex.getMessage());
      throw new DbTransactionEnderException(ex);
    }
  } // end method convertProcCode

  // Set instance variables back to blank strings in preparation for processing
  //   another header
  private void initializeHeaderVariables()
  {
      vSt02txnCtlNbrPadded = "";
      vSt02txnCtlNumberNoPad = "";
      vClaimKey = "";
      vDtlHeadInfoKey = "";
      vClpKey = "";
      vSvcKey = "";
      vStgClpKey = "";
      vStgSvcKey = "";
      vFacilityId = "";
      vProviderId = "";
      vProviderNpi = null;
      vPayeeName  = "";
      vProdDate   = "";
      vClaimDate  = "";
      vIntAmt = zeroBD;
      v837FileNumber = "";
      v837St02txnCtlNbr = "";
      v837SbsrId = "";
      v837BillHrchlKey = "";
      v837SbsrHrchlKey = "";
  }

    /**
     * Translates the 837 Rendering provider id qualifier into a valid
     * 835 Service Provider id qualifier.  Valid values for the 837
     * Rendering provider id qualifier are "XX", "24", and "34"; Advised
     * values for the Service provider are "XX", and "FI".  Inputs of
     * "34", and "24" will return "FI"; and "XX" will return "XX".
     *
     * @param qualifier 837 Rendering provider code qualifier.
     *
     * @return 835 service provider code qualifier. null if the input
     *    is other than "XX", "34", or "24".
     */
    public static String transRenderProvIdQual(final String qualifier) {
        String rtn = null;
        if ("XX".equals(qualifier)) {
            rtn = qualifier;
        } else if ("24".equals(qualifier) || "34".equals(qualifier)) {
            rtn = "FI";
        }
        return rtn;
    }

    /**
     * Checks whether the value in the Rendering provider Id is in the
     * valid format for the given Qualifier.  "24", and "34" qualifiers
     * should contain TINs, therefore must be 9 digits.  "XX" qualifiers
     * should contain NPIs and must be 10 digits.  Any qualifier other than
     * "24", 34", or "XX" will return false.
     *
     * @param idQualifier  the 837 Rendering provider's qualifier
     * @param id the 837 Rendering providers id
     * @return true if the idQualifier is "24" or "34" and the id is nine digits; or
     *   if the idQualifier is "XX" and the id is ten digits. false otherwise.
     */
    private static boolean validRenderingProviderId(final String idQualifier, final String id) {
        // invalid on null values
        if (idQualifier == null || id == null) {
            return false;
        }

        // 24 is the Empoyer's Identification Number and 34 is the
        // Social Security Number the Id contained should be the 9
        // digit tax id.
        if (("24".equals(idQualifier) || "34".equals(idQualifier)) && id.matches("\\d{9}")) {
            return true;
        }

        // XX is the National Provider Identifier and should be
        // a ten digit id.
        if ("XX".equals(idQualifier) && id.matches("\\d{10}")) {
            return true;
        }

        return false;
    }
}
