package gov.va.med.hac.edi.era.jpa;

import gov.va.med.domain.erepos.Stg835Counter;

import java.io.PrintWriter;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.Query;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;




public final class StagingFacade
{
  static final Logger traceLOGGER = LoggerFactory.getLogger("trace." + StagingFacade.class);

  

  // Updates the file_number column in each of several tables (by calling
  //   updateFileNumberInTables) because the file_number had an error, and will be rerun.
  //   A suffix of "_1" is appended to the original file name, or incremented
  //   if the file had already been run more than once.
  public void updateFileNumber( final String pOldFileNumber, EntityManager dbEM)
  throws SQLException
  {
    int vCnt = 0;
    String vCounter = "";
    String baseFileNumber = "";
    String newFileNumber = "";

    // Use only the part of the file number before the second underscore
    baseFileNumber = getOriginalFileNumber(pOldFileNumber);

    // Get the record from STG_835_COUNTER for this fileset
    // This table records the number of times a file has been processed
    // query the database to see if this batch was already loaded
    Query numberOfTimesFileProcessedQuery = dbEM.createQuery("SELECT s"
                        + "  FROM gov.va.med.domain.erepos.Stg835Counter s"
                        + " WHERE s.fileNumber=?1");
    numberOfTimesFileProcessedQuery.setParameter(1, baseFileNumber);
    List stg835Counters = numberOfTimesFileProcessedQuery.getResultList();    
    
    Stg835Counter crStg835Counter = null;
    if(stg835Counters.size() > 0)
    {
    	crStg835Counter = (Stg835Counter) stg835Counters.get(0);
    }
    vCounter = "";

    // If no record was found, create one
    if (crStg835Counter == null) {
      final Stg835Counter trStg835Counter = new Stg835Counter();
      vCounter = "1";
      trStg835Counter.setFileNumber(pOldFileNumber);
      trStg835Counter.setCounter(vCounter);    
      // Process the updates in a loop
//      dbEM.getTransaction().begin();
      dbEM.persist(trStg835Counter);
      // Process the updates in a loop
//      dbEM.getTransaction().commit();

    // If a record was found, increment its counter
    } else {
      vCounter = crStg835Counter.getCounter();
      vCnt = Integer.parseInt(vCounter) + 1;
      vCounter = Integer.toString(vCnt);
      crStg835Counter.setCounter(vCounter);
    }

    newFileNumber = baseFileNumber + "_" + vCounter;
    updateFileNumberInStagingTables(pOldFileNumber, newFileNumber, dbEM);
    traceLOGGER.info("Successfully Updated the  FileNumber from "
      + pOldFileNumber + " to " + newFileNumber + "for Transactions in 'ERROR' status");
    
  } // end method updateFileNumber

  // Update the file number in several of the staging tables to the given one
  private void updateFileNumberInStagingTables(
      final String pOldFileNumber,
      final String pNewFileNumber,
      EntityManager dbEM)
  throws SQLException
  {



    // In each table to be updated, select rows to update based on
    //   the old file number, stored in instance variable vFileNumber,
    //   as well as on status
    final String sHdrSel = "(select ST02TXN_CTL_NBR from STG_835_HDR where STSE_STATUS = 'ERROR' " +
            "and FILE_NUMBER = '" + pOldFileNumber + "') and FILE_NUMBER = '" + pOldFileNumber + "'";

    // Put the updates into an array, so they can be processed in a loop
    final String[] updates = {
      "Update STG_835_CLP set FILE_NUMBER = '" + pNewFileNumber
          + "' where ST02TXN_CTL_NBR in " + sHdrSel, // NOPMD
      "Update STG_835_HDR_TRN set FILE_NUMBER = '" + pNewFileNumber
          + "' where ST02TXN_CTL_NBR in " + sHdrSel,
      "Update STG_835_HDR_BPR set FILE_NUMBER = '" + pNewFileNumber
          + "' where ST02TXN_CTL_NBR in " + sHdrSel,
      "Update STG_835_PLB set FILE_NUMBER = '" + pNewFileNumber
          + "' where ST02TXN_CTL_NBR in " + sHdrSel,
      // Need to update STG_835_HDR last, since its file number is used in the
      //   subselect for all the other tables
      "Update STG_835_HDR set FILE_NUMBER = '" + pNewFileNumber
          + "' where STSE_STATUS = 'ERROR' and FILE_NUMBER = '" + pOldFileNumber + "' "
    };

    // Process the updates in a loop
//    dbEM.getTransaction().begin();
    
    // Execute the updates in a loop    
    for (int i = 0; i < updates.length; i++) {
    	Query updateStatement = dbEM.createNativeQuery(updates[i]);
    	updateStatement.executeUpdate();
    }

    // Commit the updates
//    dbEM.getTransaction().commit();
  } // end method updateFileNumberInTables

  // Update the status in three of the staging tables to INPROCESS
  public void updateStatusColumns(final String pFileNumber,
		  EntityManager dbEM)
  throws SQLException
  {
  
    // Put the updates into an array, so they can be executed in a loo
    final String [] updates = {
        "UPDATE STG_835_HDR SET STSE_STATUS = 'INPROCESS' "+
                "WHERE FILE_NUMBER = '" + pFileNumber + "' "+
                "AND STG_835_HDR.STSE_STATUS='NEW'",
        "UPDATE STG_835_CLP SET CLP_STATUS  = 'INPROCESS' "+
                "WHERE FILE_NUMBER = '" + pFileNumber + "' "+
                "AND STG_835_CLP.CLP_STATUS='NEW'",
        "UPDATE STG_835_SVC SET SVC_STATUS  = 'INPROCESS' "+
                "WHERE SVC_STATUS='NEW' "+
                "AND CLP_KEY IN (SELECT CLP_KEY FROM STG_835_CLP "+
                "WHERE FILE_NUMBER = '" + pFileNumber + "') "
    };

//    dbEM.getTransaction().begin();
    
    // Execute the updates in a loop    
    for (int i = 0; i < updates.length; i++) {
    	Query updateStatement = dbEM.createNativeQuery(updates[i]);
    	updateStatement.executeUpdate();
    }

    // Commit the updates
    //TWP: added back 6/02/2010
    traceLOGGER.info("Committing the database transaction");
    dbEM.getTransaction().commit();
    traceLOGGER.info("Commit successful");
    dbEM.getTransaction().begin();
  } // end method updateStatusColumns

  // Take a string, which might have two underscores, and return the substring
  //   up to but not including any second underscore
  // TODO: Using some other framework, or newer Java version, put together a
  //   generic version of this, and put it in a common place.  Tokenizing might work
  public static String getOriginalFileNumber(final String inFileNumber)
  {
    String returnFileNumber = inFileNumber;

    // If passed null, just return null
    if (inFileNumber == null)
    {
      return null;
    }

    // Get the position of the first underscore in the given file number
    final int firstUnderscorePosition = inFileNumber.indexOf('_');

    // Get the position of the second underscore
    // A -1 means there isn't a second one
    final int secondUnderscorePosition = inFileNumber.indexOf('_',firstUnderscorePosition + 1);

    if (secondUnderscorePosition > 0)
    {
      returnFileNumber = inFileNumber.substring(0,secondUnderscorePosition);
    }

    return returnFileNumber;
  } // end method getOriginalFileNumber
}