package gov.va.med.hac.edi.extract.feeclaimstatus.initial;

import gov.va.med.domain.erepos.Claim;
import gov.va.med.domain.erepos.DemographicInfo;
import gov.va.med.domain.erepos.EdiProcessLog;
import gov.va.med.domain.erepos.EntityAdtlId;
import gov.va.med.domain.erepos.EntityName;
import gov.va.med.domain.erepos.RejectClaim;
import gov.va.med.domain.fppsowner.PersonInfo;
import gov.va.med.hac.edi.utils.DateUtil;
import gov.va.med.hac.edi.utils.StringUtils;

import org.hibernate.ScrollMode;
import org.hibernate.ScrollableResults;
import org.hibernate.Session;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.FileOutputStream;
import java.io.PrintWriter;

import java.math.BigDecimal;

import java.text.SimpleDateFormat;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.List;
import java.util.UUID;

import javax.persistence.EntityManager;
import javax.persistence.EntityTransaction;
import javax.persistence.PersistenceException;
import javax.persistence.Query;


class ProcessEmdeonInitialClaimStatus {
    static final Logger LOGGER = LoggerFactory.getLogger(ProcessEmdeonInitialClaimStatus.class);
    private static final String PAYER_ID = "VAFEE";
    private static final String TRADING_PARTNER_ID = "VAFNH";

    //TWP I changed these types below based on the guides I was using is there anything more up to date
    //The guides were published in 2006
    private static final String PROFESSIONAL_CLAIM_TYPE = "005010X222";
    private static final String INSTITUTIONAL_CLAIM_TYPE = "005010X223";
    private static final String DENTAL_CLAIM_TYPE = "005010X224";
    private static final String INSTITUTIONAL_CLAIM_TYPE_4010 = "004010X096A1";
    private static final String PROFESSIONAL_CLAIM_TYPE_4010 = "004010X098A1";
    private static final String DENTAL_CLAIM_TYPE_4010 = "004010X097A1";
    static final String CRLF = "\r\n";
    static final String DELIMITER = "|";
    static final boolean REQUIRED = true;
    static final boolean OPTIONAL = false;
    EntityManager em;
    boolean dailyCstat;
    String claimKey = "";
    String fileNbr = "";
    String st02CtlNbr = "";
    String billPrvdId = "";
    String sbcrId = "";
    String claimType = "";
    String statusCd = "";
    String currentDate = "";
    String currentTime = "";
    Date beginDate;
    Date endDate;
    int cnt = 1;
    int rejCnt = 0;
    int accCnt = 0;

    /*
    * Called during daily CSTAT file creation
    */
    final void process(String clearinghouseId, String filePath,
        EntityManager entityManager) {
        LOGGER.debug("processing with clearinghouseId: {}  filePath: {} ",
            clearinghouseId, filePath);
        this.dailyCstat = true;
        this.em = entityManager;
        processClaimStatus(clearinghouseId, filePath);
    }

    /*
     *called during quarterly CSTAT file creation
     */
    final void process(String clearinghouseId, String filePath, Date begin,
        Date end, EntityManager entityManager) {
        LOGGER.debug("processing with clearinghouseId: {}, filePath: {}, begin date: {}, end date: {}",
            new Object[] { clearinghouseId, filePath, begin, end });
        this.dailyCstat = false;
        this.em = entityManager;
        this.beginDate = begin;
        this.endDate = end;
        processClaimStatus(clearinghouseId, filePath);
    }

    final void processClaimStatus(String chId, String filePath) {
        EntityTransaction tx = em.getTransaction();
        tx.begin();

        try {
            // DEF005231 - The counter was being reset when transitioning
            // from 12115 to 12116. This caused the record counts in one or
            // the other to be incorrect.
            this.cnt = 1;

            Calendar cal = new GregorianCalendar();
            java.util.Date date = cal.getTime();
            SimpleDateFormat transDate = new SimpleDateFormat("yyyyMMdd");
            SimpleDateFormat transTime = new SimpleDateFormat("HHmmss");
            this.currentDate = transDate.format(date);
            this.currentTime = transTime.format(date);

            /* get the list of claims that need to be reported on
             * to different queries, one for the daily CSTAT file
             * and one for the Quarterly CSTAT file.
             */
            Session session = (Session) em.getDelegate();
            String qstr = "";
            ScrollableResults claims = null;

            if (this.dailyCstat) {
                String qcntStr = "select count(*) from eClaim c " +
                    "where c.statusCode in ('BUILT', 'REJECTED') " +
                    "and c.receiverId like '1211%' ";
                Long count = (Long) em.createQuery(qcntStr).getSingleResult();

                if (count.longValue() == 0) {
                    LOGGER.info("There are no new Claims to report status on.");

                    // needed to prevent zero record files.
                    return;
                }
            }

            if (this.dailyCstat) {
                qstr = "select c from eClaim c " +
                    "where c.statusCode in ('BUILT', 'REJECTED') " +
                    "and c.receiverId like '1211%' " +
                    "order by c.dateCreated ";
                claims = session.createQuery(qstr)
                                .scroll(ScrollMode.FORWARD_ONLY);
            } else {
                qstr = "select c from eClaim c " +
                    "where c.receiverId like '1211%' " +
                    "and not c.statusCode = 'BROKEN'" +
                    "and c.dateCreated between :start and :end";
                claims = session.createQuery(qstr)
                                .setParameter("start", this.beginDate)
                                .setParameter("end", this.endDate)
                                .scroll(ScrollMode.FORWARD_ONLY);
            }

            if (claims != null) {
                /*write out the CSTAT FILE*/
                //TWP Changed Naming Convention for 5010 may require updates to scheduler etc.
                String fileName = this.currentDate + this.currentTime + "_" +
                    PAYER_ID + ".cstat";
                File csFile = new File(filePath + fileName);
                PrintWriter vCsFile = new PrintWriter(new FileOutputStream(
                            csFile));
                createHdrRec(vCsFile);
                createPrimaryDetailRec(claims, session, vCsFile);
                createTrlRec(vCsFile);
                vCsFile.flush();
                vCsFile.close();

                /*enter work done in database process log*/
                if (this.dailyCstat) {
                    insertProcessLog("BUILT", this.accCnt);
                    insertProcessLog("REJECTED", this.rejCnt);
                }

                tx.commit();
                LOGGER.info("Claim Status File is Successfully Created!!!");
            }
        } catch (Exception ex) {
            LOGGER.error("Unable to create CSTAT, errored on Claim Key: {}",
                this.claimKey);
            LOGGER.error(ex.getMessage(), ex);
            tx.rollback();
        }
    }

    private StringBuffer append(StringBuffer sb, String value) {
        sb.append(value);
        sb.append(DELIMITER);

        return sb;
    }

    final void createHdrRec(PrintWriter vCsFile) {
        StringBuffer sbHdrRec = new StringBuffer();
        sbHdrRec.append("HDR"); //Record ID  - TWP: Changed to 3 char for 5010
        delimit(sbHdrRec);
        sbHdrRec.append(this.currentDate + this.currentTime); //File Group ID TWP: No one really knows what this is
        delimit(sbHdrRec);
        sbHdrRec.append("01"); //File Group Sequence Number - I guess there is only 1 file per group?
        delimit(sbHdrRec);
        sbHdrRec.append("01"); //File Group Count - looks like we are not using the whole file group concept
        delimit(sbHdrRec);
        sbHdrRec.append(this.currentDate); //Creation Date 8 chars yyyyMMdd
        delimit(sbHdrRec);
        sbHdrRec.append(this.currentTime); //Creation Time 6 chars HHmmss
        delimit(sbHdrRec);
        sbHdrRec.append(TRADING_PARTNER_ID); //Trading Partner Id  TWP: Changed to pad to 10 from 5
        delimit(sbHdrRec);
        sbHdrRec.append("VA Fee Basis Program"); //Submitter Name  - TWP: is this correct?
        delimit(sbHdrRec);
        sbHdrRec.append("Contact Local VAMC"); //Payer Contact Name
        delimit(sbHdrRec);
        sbHdrRec.append(""); //Payer Support Telephone Number
        delimit(sbHdrRec);
        sbHdrRec.append(""); //PayerSupport Email
        delimit(sbHdrRec);

        if (this.dailyCstat) { //Load type - full or incremental
            sbHdrRec.append("I");
        } else {
            sbHdrRec.append("F");
        }

        delimit(sbHdrRec);
        sbHdrRec.append(StringUtils.rPadSpaces(this.currentDate +
                this.currentTime + "_VAFEE", 20)); //Payer Uniqie File Identifier
        delimit(sbHdrRec);
        sbHdrRec.append("CStat"); //File Type - TWP updated for 5010 but spec does not make sense max len = 4 but value must = Cstat which is 5 long?
        delimit(sbHdrRec);
        sbHdrRec.append("03"); //Version Code TWP updated from 02 to 03
        delimit(sbHdrRec);
        sbHdrRec.append("00"); //Release Code
        sbHdrRec.append(CRLF); //Record Teminator new for 5010
        vCsFile.print(sbHdrRec); //TWP Changed to print rather than println since I am explicitly adding CRLF  
        vCsFile.flush();
        sbHdrRec = null;
    }

    final void createPrimaryDetailRec(ScrollableResults claims,
        Session session, PrintWriter vCsFile) {
        int count = 0;
        int batchSize = 10;
        String seqNumber = "";

        while (claims.next()) {
            Claim claim = (Claim) claims.get(0);
            this.claimKey = claim.getClaimKey();
            this.fileNbr = claim.getFileNumber();
            this.st02CtlNbr = claim.getSt02txnCtlNbr();
            this.billPrvdId = claim.getNm109billPrvdId();
            this.sbcrId = claim.getSubscriberId();
            this.claimType = claim.getClaimType();
            this.statusCd = claim.getStatusCode();
            this.cnt++;

            StringBuffer sbDtlRec = new StringBuffer();
            sbDtlRec.append("CLM"); //Record ID - 3 long in 5010
            delimit(sbDtlRec);
            sbDtlRec.append(formatOutputString(Integer.toString(this.cnt),
                    REQUIRED, 1, 10, StringFormat.LPADZERO)); //Record Number         
            sbDtlRec.append(PAYER_ID); //Payer Id
            delimit(sbDtlRec);

            sbDtlRec.append("021"); //Maintenance Type Code 
                                    //TWP As Per converstation with dave we are always adding records
                                    //in all our incremental loads
                                    //We do not send a update or delete record - is this correct Sreedhar?

            delimit(sbDtlRec);

            //The billing provider ois identified by loop 2010AA and the Entity Identifier Code = 85
            EntityName entityNameBillingProv = getEntityName("2010AA",
                    "AND NM101ENTY_ID_CD = '85' ");
            String nm108CdQual = entityNameBillingProv.getNm108idCdQual();

            //TWP: Changed where we get the EIN/TIN from. it comes from entity additional information now
            //loop 2010AA with a qualifier of EI for Employers Identification Number or SY for
            //Social Security Number
            String qstr2 = "select ea from eEntityAdtlId ea " +
                "where ea.txnSet.id.fileNumber = :file " +
                "  and ea.txnSet.id.st02txnCtlNbr = :ctlnbr " +
                "  and ea.loopId = '2010AA' " + "  and ea.txType like '837%' " +
                "  and ea.ref01idQlfr in ('EI', 'SY') ";
            Query q2 = em.createQuery(qstr2);
            q2.setParameter("file", claim.getFileNumber());
            q2.setParameter("ctlnbr", claim.getSt02txnCtlNbr());

            List<EntityAdtlId> entityAdtls = q2.getResultList();
            Iterator<EntityAdtlId> entityAdtlsIterator = entityAdtls.iterator();

            String entityDataEINorSSNQualifier = ""; //This is the qualifier that tells us what is in ref02 
            String entityEINorSSN = "";

            if (entityAdtlsIterator.hasNext()) {
                EntityAdtlId eAID = entityAdtlsIterator.next();
                entityDataEINorSSNQualifier = eAID.getRef01idQlfr();
                entityEINorSSN = eAID.getRef02scndId();
            }

            //If we have an EIN or SSN we will populate it here
            if ((entityNameBillingProv != null) &&
                    ("24".equals(nm108CdQual) || "34".equals(nm108CdQual))) {
                sbDtlRec.append(formatOutputString(
                        entityNameBillingProv.getNm109idCd(), REQUIRED, 9, 9,
                        StringFormat.RPADSPACES)); //Billing Provider Tax Id 
            } else if ("EI".equals(entityDataEINorSSNQualifier) ||
                    "SY".equals(entityDataEINorSSNQualifier)) {
                sbDtlRec.append(formatOutputString(entityEINorSSN, REQUIRED, 9,
                        9, StringFormat.RPADSPACES)); //Billing Provider Tax Id
            } else {
                sbDtlRec.append(formatOutputString("", OPTIONAL, 9, 9,
                        StringFormat.RPADSPACES)); //Billing Provider Tax Id             
            }

            // We have either the Tax ID or the NPI so the
            // 'Billing Provider Federal Tax ID' isn't needed
            sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 50,
                    StringFormat.RPADSPACES)); //Billing Provider Assigned Payer Number  TWP Changed length from 20 to 50

            if ("XX".equals(nm108CdQual)) {
                //XX indicates we have a Centers for Medicare and Medicaid Services National Provider Identifier
                //Therefore we send the Identification Code which is an NPI
                sbDtlRec.append(formatOutputString(this.billPrvdId, OPTIONAL,
                        10, 10, StringFormat.RPADSPACES)); //Billing Provider National Provider ID
            } else {
                sbDtlRec.append(formatOutputString("", OPTIONAL, 10, 10,
                        StringFormat.RPADSPACES)); //Billing Provider National Provider ID            
            }

            sbDtlRec.append(formatOutputString(
                    entityNameBillingProv.getNm103lastNm(), OPTIONAL, 1, 60,
                    null)); //Billing Provider Last Name TWP: changed to length 60 from 35
            sbDtlRec.append(formatOutputString(
                    entityNameBillingProv.getNm104fstNm(), OPTIONAL, 1, 35, null)); //Billing Provider First Name TWP: changed to length 35 from 25                          
            sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 25,
                    StringFormat.RPADSPACES)); //Billing Provider Middle Name TWP: added this element
            sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 10,
                    StringFormat.RPADSPACES)); //Billing Provider Name Suffix TWP: added this element for 5010       

            EntityName serviceProvider = getEntityName(
                    "AND NM101ENTY_ID_CD = '82' AND CLAIM_KEY='" + claimKey +
                    "'");
            String serviceProviderTaxId = null;
            String serviceProviderPayerAssignedNumber = null;
            String serviceProviderNPI = null;
            String serviceProviderLastName = null;
            String serviceProviderFirstName = null;
            String serviceProviderQualifier = "";

            if (serviceProvider != null) {
                serviceProviderQualifier = serviceProvider.getNm108idCdQual();
            }

            if ((serviceProvider != null) &&
                    (serviceProviderQualifier != null) &&
                    !serviceProviderQualifier.trim().isEmpty()) {
                if (claimType.startsWith("004010")) {
                    if (("24".equalsIgnoreCase(serviceProviderQualifier) ||
                            "34".equalsIgnoreCase(serviceProviderQualifier)) &&
                            (serviceProvider.getNm109idCd() != null) &&
                            !serviceProvider.getNm109idCd().trim().isEmpty()) {
                        serviceProviderTaxId = serviceProvider.getNm109idCd();
                    }

                    if ("XX".equalsIgnoreCase(serviceProviderQualifier) &&
                            (serviceProvider.getNm109idCd() != null) &&
                            !serviceProvider.getNm109idCd().trim().isEmpty()) {
                        serviceProviderNPI = serviceProvider.getNm109idCd();
                    }
                }

                if (claimType.startsWith("005010") &&
                        "XX".equalsIgnoreCase(serviceProviderQualifier) &&
                        (serviceProvider.getNm109idCd() != null) &&
                        !serviceProvider.getNm109idCd().trim().isEmpty()) {
                    serviceProviderNPI = serviceProvider.getNm109idCd();
                }

                if ((serviceProviderTaxId != null) ||
                        (serviceProviderNPI != null)) {
                    if ((serviceProvider.getNm103lastNm() != null) &&
                            !serviceProvider.getNm103lastNm().trim().isEmpty()) {
                        serviceProviderLastName = serviceProvider.getNm103lastNm();
                    }

                    if ((serviceProvider.getNm104fstNm() != null) &&
                            !serviceProvider.getNm104fstNm().trim().isEmpty()) {
                        serviceProviderFirstName = serviceProvider.getNm104fstNm();
                    }
                }
            }

            sbDtlRec.append(formatOutputString(serviceProviderTaxId, false, 9,
                    9, StringFormat.RPADSPACES));
            sbDtlRec.append(formatOutputString(
                    serviceProviderPayerAssignedNumber, false, 1, 50,
                    StringFormat.RPADSPACES));
            sbDtlRec.append(formatOutputString(serviceProviderNPI, false, 10,
                    10, StringFormat.RPADSPACES));
            sbDtlRec.append(formatOutputString(serviceProviderLastName, false,
                    1, 60, StringFormat.RPADSPACES));
            sbDtlRec.append(formatOutputString(serviceProviderFirstName, false,
                    1, 35, StringFormat.RPADSPACES));

            sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 25,
                    StringFormat.RPADSPACES)); //Service Provider Middle Name
            sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 10,
                    StringFormat.RPADSPACES)); //Service Provider Name suffix 
            sbDtlRec.append(formatOutputString("", OPTIONAL, 2, 80,
                    StringFormat.RPADSPACES)); //Employer Identification Number 
            sbDtlRec.append(formatOutputString("", OPTIONAL, 2, 80,
                    StringFormat.RPADSPACES)); //Employer Name

            PersonInfo person = getPersonName(this.sbcrId);

            //TWP Seems Like the old code made the assumption that the subscriber and patient were the same
            //am I interpreting this correctly?
            if (person != null) {
                sbDtlRec.append(formatOutputString(person.getPersonId(),
                        OPTIONAL, 2, 80, StringFormat.RPADSPACES)); //Subscriber Id
                sbDtlRec.append(formatOutputString(person.getLastName(),
                        OPTIONAL, 1, 60, StringFormat.RPADSPACES)); //Subscriber Last Name            
                sbDtlRec.append(formatOutputString(person.getFirstName(),
                        OPTIONAL, 1, 35, StringFormat.RPADSPACES)); //Subscriber First Name            
                sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 25,
                        StringFormat.RPADSPACES)); //Subscriber Middle Name            
                sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 10,
                        StringFormat.RPADSPACES)); //Subscriber Name Suffix            
                sbDtlRec.append(formatOutputString("", OPTIONAL, 2, 80,
                        StringFormat.RPADSPACES)); //Patient ID - TWP looks like we did not send this in the past
                sbDtlRec.append(formatOutputString(person.getLastName(),
                        OPTIONAL, 1, 60, StringFormat.RPADSPACES)); //Patient Last Name            
                sbDtlRec.append(formatOutputString(person.getFirstName(),
                        OPTIONAL, 1, 35, StringFormat.RPADSPACES)); //Patient First Name
                sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 25,
                        StringFormat.RPADSPACES)); //Patient Middle Name            
                sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 10,
                        StringFormat.RPADSPACES)); //Patient Name Suffix
            } else {
                //IL Selects the insured or subscriber - MI selects the Member identification number
                String vQl = "AND NM101ENTY_ID_CD = 'IL' AND NM108ID_CD_QUAL = 'MI' ";
                EntityName entityNameS = getEntityName("2010BA", vQl);
                sbDtlRec.append(formatOutputString(entityNameS.getNm109idCd(),
                        OPTIONAL, 2, 80, StringFormat.RPADSPACES)); //Subscriber Id
                sbDtlRec.append(formatOutputString(
                        entityNameS.getNm103lastNm(), OPTIONAL, 1, 60,
                        StringFormat.RPADSPACES)); //Subscriber Last Name            
                sbDtlRec.append(formatOutputString(
                        entityNameS.getNm104fstNm(), OPTIONAL, 1, 35,
                        StringFormat.RPADSPACES)); //Subscriber First Name            
                sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 25,
                        StringFormat.RPADSPACES)); //Subscriber Middle Name            
                sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 10,
                        StringFormat.RPADSPACES)); //Subscriber Name Suffix            
                sbDtlRec.append(formatOutputString("", OPTIONAL, 2, 80,
                        StringFormat.RPADSPACES)); //Patient ID - TWP looks like we did not send this in the past
                sbDtlRec.append(formatOutputString(
                        entityNameS.getNm103lastNm(), OPTIONAL, 1, 60,
                        StringFormat.RPADSPACES)); //Patient Last Name            
                sbDtlRec.append(formatOutputString(
                        entityNameS.getNm104fstNm(), OPTIONAL, 1, 35,
                        StringFormat.RPADSPACES)); //Patient First Name
                sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 25,
                        StringFormat.RPADSPACES)); //Patient Middle Name            
                sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 10,
                        StringFormat.RPADSPACES)); //Patient Name Suffix                      
            }

            //TWP: Patient Date of Birth is now required - should I try to get it from PersonInfo as well???
            //This does not appear to meet the new spec
            DemographicInfo demographicInfo = getDemographicInfo();

            if (demographicInfo != null) {
                sbDtlRec.append(formatOutputString(
                        demographicInfo.getDmg02brthDt(), REQUIRED, 8, 8,
                        StringFormat.RPADSPACES)); //Patient Date of Birth

                if ((demographicInfo.getDmg03gndr() != null) &&
                        ("M".equalsIgnoreCase(demographicInfo.getDmg03gndr()) ||
                        "F".equalsIgnoreCase(demographicInfo.getDmg03gndr()))) {
                    sbDtlRec.append(formatOutputString(
                            demographicInfo.getDmg03gndr(), OPTIONAL, 1, 1,
                            StringFormat.RPADSPACES)); //Patient Gender
                } else {
                    sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 1,
                            StringFormat.RPADSPACES)); //Patient Gender
                }
            } else {
                sbDtlRec.append(formatOutputString("", REQUIRED, 8, 8,
                        StringFormat.RPADSPACES)); //Patient Date of Birth
                sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 1,
                        StringFormat.RPADSPACES)); //Patient Gender
            }

            //sbDtlRec.append(StringUtils.rPadSpaces(getRefValue("D9"), 15));  TWP removed webMDClaim Number
            //TWP What do we use instead
            sbDtlRec.append(formatOutputString(getRefValue("D9"), OPTIONAL, 1,
                    50, StringFormat.RPADSPACES)); //Emdeon Claim Number

            sbDtlRec.append(formatOutputString(claim.getClm02clmChrgAmt(),
                    REQUIRED, 1, 18, StringFormat.RPADSPACES)); //Claim Charge Amount          
            sbDtlRec.append(formatOutputString("0.00", OPTIONAL, 1, 18,
                    StringFormat.LPADZERO)); //Claim Payment Amount TWP:Changed to 18

            sbDtlRec.append(formatOutputString("", OPTIONAL, 8, 8,
                    StringFormat.RPADSPACES)); //Claim Adjudication/Payment Date
            sbDtlRec.append(formatOutputString("", OPTIONAL, 8, 8,
                    StringFormat.RPADSPACES)); //Check EFT/Date
            sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 16,
                    StringFormat.RPADSPACES)); //Check EFT Number TWP:to 16
            sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 3,
                    StringFormat.RPADSPACES)); //Bill Type

            if (isRejectedClaim()) {
                seqNumber = getSeqNbr();
                sbDtlRec.append(formatOutputString(seqNumber, REQUIRED, 1, 50,
                        StringFormat.RPADSPACES)); //Payer Claim Identification Number TWP:to 50 
            } else {
                sbDtlRec.append(formatOutputString(getClaimIndex(this.claimKey),
                        REQUIRED, 1, 50, StringFormat.RPADSPACES)); //Payer Claim Identification Number TWP:to 50
            }

            sbDtlRec.append(formatOutputString(claim.getClm01ptAcctNbr(),
                    REQUIRED, 1, 50, StringFormat.RPADSPACES)); //Patient Account Number TWP:Changed to 50                
            sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 50,
                    StringFormat.RPADSPACES)); //Pharmacy Prescription Number
            sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 50,
                    StringFormat.RPADSPACES)); //Voucher Identifier
            sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 50,
                    StringFormat.RPADSPACES)); //Application of Location System Identifier
            sbDtlRec.append(formatOutputString("", OPTIONAL, 1, 50,
                    StringFormat.RPADSPACES)); //Group Number

            sbDtlRec = fillClaimServiceDates(sbDtlRec); //Claim Service Date Start & Claim Service Date End

            vCsFile.print(sbDtlRec);
            vCsFile.print(CRLF);
            vCsFile.flush();

            //Create claim status lines for this cliam
            createStcRec(claim, seqNumber, vCsFile);

            // Properties props = System.getProperties();
            if (isAcceptedClaim()) {
                this.accCnt++;

                if (this.dailyCstat) {
                    claim.setStatusCode("277U_ACCPT_SENT");
                }
            } else if (isRejectedClaim()) {
                this.rejCnt++;

                if (this.dailyCstat) {
                    claim.setStatusCode("277U_REJ_SENT");
                }
            }

            //We flush and clear aren't we losing our status updates? TWP
            if ((++count % batchSize) == 0) {
                session.flush();
                session.clear();
            }

            LOGGER.debug("{} records written", count);
        }
    }

    final void createTrlRec(PrintWriter vCsFile) {
        StringBuffer sbTrlRec = new StringBuffer();
        sbTrlRec.append("TRLR"); //Record ID
        delimit(sbTrlRec);
        //      sbTrlRec.append(PAYER_ID);  //Trading Partner ID
        //      delimit(sbTrlRec);
        sbTrlRec.append(StringUtils.lPadZeros(Integer.toString(this.cnt - 1), 10)); //Record Count  (Note format output will delimit the string)
                                                                                    //TWP spec says record count length is 1 has to be a typo
                                                                                    //      delimit(sbTrlRec);

        vCsFile.print(sbTrlRec);
        vCsFile.print(CRLF); //Record Terminator
        sbTrlRec = null;
        ;
    }

    //TWP: bug fix rejected status had ' not _
    boolean isRejectedClaim() {
        return this.statusCd.equalsIgnoreCase("REJECTED") ||
        this.statusCd.equalsIgnoreCase("277U_REJ_SENT");
    }

    boolean isAcceptedClaim() {
        return this.statusCd.equalsIgnoreCase("BUILT") ||
        this.statusCd.equalsIgnoreCase("277U_ACCPT_SENT");
    }

    @SuppressWarnings("unchecked")
    final EntityName getEntityName(String vLoopId, String vQual) {
        String qstr = "select e from eEntityName e " +
            "where e.txnSet.id.fileNumber = :file " +
            "  and e.txnSet.id.st02txnCtlNbr = :ctlnbr " +
            "  and e.loopId =  :loopid " + vQual;
        Query q = em.createQuery(qstr);
        q.setParameter("file", this.fileNbr);
        q.setParameter("ctlnbr", this.st02CtlNbr);
        q.setParameter("loopid", vLoopId);

        EntityName en = null;
        List<EntityName> entityNames = q.getResultList();

        if ((entityNames.size() > 0) && (entityNames.get(0) != null)) {
            en = entityNames.get(0);
        } else {
            LOGGER.warn("Error in getting EntityName for fileNumber: {}, st02txnCtlNbr: {}, loopId: {}, qal: {}",
                new Object[] { this.fileNbr, this.st02CtlNbr, vLoopId, vQual });
        }

        return en;
    }

    @SuppressWarnings("unchecked")
    final EntityName getEntityName(String vQual) {
        String qstr = "select e from eEntityName e " +
            "where e.txnSet.id.fileNumber = :file " +
            "  and e.txnSet.id.st02txnCtlNbr = :ctlnbr " + vQual;
        Query q = em.createQuery(qstr);
        q.setParameter("file", this.fileNbr);
        q.setParameter("ctlnbr", this.st02CtlNbr);

        EntityName en = null;
        List<EntityName> entityNames = q.getResultList();

        if ((entityNames.size() > 0) && (entityNames.get(0) != null)) {
            en = entityNames.get(0);
        } else {
            LOGGER.debug("Did not get EntityName for fileNumber: {}, st02txnCtlNbr: {}, loopId: {}, qal: {} note:it may not be required",
                new Object[] { this.fileNbr, this.st02CtlNbr, vQual });
        }

        return en;
    }

    @SuppressWarnings("unchecked")
    final PersonInfo getPersonName(String vSsn) {
        Query q = em.createQuery(
                "select p from foPersonInfo p where p.personId = :id ");
        q.setParameter("id", vSsn);

        PersonInfo pi = null;
        List<PersonInfo> pis = q.getResultList();

        if ((pis.size() > 0) && (pis.get(0) != null)) {
            pi = (PersonInfo) pis.get(0);
        } else {
            LOGGER.debug("Unable to get PersonInfo for person Id: {}", vSsn);
        }

        return pi;
    }

    @SuppressWarnings("unchecked")
    final DemographicInfo getDemographicInfo() {
        String qstr = "select d from eDemographicInfo d " +
            "where d.txnSet.id.fileNumber = :file " +
            "  and d.txnSet.id.st02txnCtlNbr = :ctlnbr " +
            "  and d.nm109sbsrId = :sbcrid " +
            "  and d.loopId in ('2010BA', '2010CA')";
        Query q = em.createQuery(qstr);
        q.setParameter("file", this.fileNbr);
        q.setParameter("ctlnbr", this.st02CtlNbr);
        q.setParameter("sbcrid", this.sbcrId);

        DemographicInfo di = null;
        List dis = q.getResultList();

        if ((dis.size() > 0) && (dis.get(0) != null)) {
            di = (DemographicInfo) dis.get(0);
        } else {
            LOGGER.debug("Unable to get Demographic Info for file: {}, st02txnCtlNbr: {}, nm109sbsrId: {} ",
                new Object[] { this.fileNbr, this.st02CtlNbr, this.sbcrId });
        }

        return di;
    }

    @SuppressWarnings("unchecked")
    final String getRefValue(String vRefQual) {
        String vRefVal = "";
        StringBuilder qstrb = new StringBuilder(
                "select e from eEntityAdtlId e ");

        if (vRefQual.equalsIgnoreCase("EI")) {
            qstrb.append("where e.txnSet.id.fileNumber = '").append(this.fileNbr)
                 .append("' ").append("and e.txnSet.id.st02txnCtlNbr = '")
                 .append(this.st02CtlNbr).append("' ")
                 .append("and e.loopId = '2010AA' ")
                 .append("and e.ref01idQlfr = '").append(vRefQual).append("' ");
        } else {
            qstrb.append("where e.claim.claimKey = '").append(this.claimKey)
                 .append("' ").append("and e.ref01idQlfr = '").append(vRefQual)
                 .append("' ");
        }

        Query q = em.createQuery(qstrb.toString());
        List entityAdtlIds = q.getResultList();

        if ((entityAdtlIds.size() > 0) && (entityAdtlIds.get(0) != null)) {
            EntityAdtlId entityAdtlId = (EntityAdtlId) entityAdtlIds.get(0);
            vRefVal = entityAdtlId.getRef02scndId();
        } else {
            LOGGER.debug("Unable to get EntityAtlId info for filenumber: {}, st02txnCtlNbr: {}, loopId: '2010AA', ref01IdQlfr: {}",
                new Object[] { this.fileNbr, this.st02CtlNbr, vRefQual });
        }

        return vRefVal;
    }
/**
It appears that this method is never called ?
**/
    @SuppressWarnings("unchecked")
    final StringBuffer fillStatusCodesForRejectedClaim(StringBuffer sbDtlRec) {
        int i = 0;
        String vErrorCode = "";
        List<RejectClaim> rejectClaims = getReasonsClaimRejected();

        for (RejectClaim rejectClaim : rejectClaims) {
            if (i == 0) {
                vErrorCode = rejectClaim.getRejectReasonCodes().getRejectCode();
            }

            if (this.dailyCstat) {
                rejectClaim.setStatus("277U_SENT");
            }

            i++;
        }

        String[] tokens = vErrorCode.split(";");

        int vLen = tokens.length;

        for (int j = 0; j < vLen; j++) {
            sbDtlRec = fillErrorCodes(sbDtlRec, tokens[j]);
        }

        //Totally Confused we are under isRejected in the main code flow
        //so why would we then return a status that shows this claim was accepted?
        if (vErrorCode.equals("")) {
            sbDtlRec.append(formatOutputString("A2", REQUIRED, 2, 3,
                    StringFormat.RPADSPACES)); //Claim Status Category Code 1 - A2=accepted
            sbDtlRec.append(formatOutputString("20", REQUIRED, 2, 3,
                    StringFormat.RPADSPACES)); //Claim Status Code 1 - 20=the claim was accepted for processing
            sbDtlRec.append(formatOutputString("", OPTIONAL, 2, 3,
                    StringFormat.RPADSPACES)); //Entity Code 1
        }

        for (int k = 0; k < (3 - vLen); k++) {
            sbDtlRec.append(formatOutputString("", OPTIONAL, 2, 3,
                    StringFormat.RPADSPACES)); //Claim Status Category Code X
            sbDtlRec.append(formatOutputString("", OPTIONAL, 2, 3,
                    StringFormat.RPADSPACES)); //Claim Status Code X
            sbDtlRec.append(formatOutputString("", OPTIONAL, 2, 3,
                    StringFormat.RPADSPACES)); //Entity Code X
        }

        return sbDtlRec;
    }

    final StringBuffer fillErrorCodes(StringBuffer sbDtlRec, String vErrCd) {
        if (!(vErrCd.equals(""))) {
            String[] tkns = vErrCd.split(":");

            if (tkns[0] != null) {
                sbDtlRec.append(formatOutputString(tkns[0], OPTIONAL, 2, 3,
                        StringFormat.RPADSPACES)); //Claim Status Category Code
            } else {
                sbDtlRec.append(formatOutputString("", OPTIONAL, 2, 3,
                        StringFormat.RPADSPACES)); //Claim Status Category Code
            }

            if (tkns[1] != null) {
                sbDtlRec.append(formatOutputString(tkns[1], OPTIONAL, 2, 3,
                        StringFormat.RPADSPACES)); //Claim Status Code
            } else {
                sbDtlRec.append(formatOutputString("", OPTIONAL, 2, 3,
                        StringFormat.RPADSPACES)); //Claim Status Code
            }

            if (tkns.length > 2) {
                if (tkns[2] != null) {
                    sbDtlRec.append(formatOutputString(tkns[2], OPTIONAL, 2, 3,
                            StringFormat.RPADSPACES)); //Entity Code X
                } else {
                    sbDtlRec.append(formatOutputString("", OPTIONAL, 2, 3,
                            StringFormat.RPADSPACES)); //Entity Code X
                }
            } else {
                sbDtlRec.append(formatOutputString("", OPTIONAL, 2, 3,
                        StringFormat.RPADSPACES)); //Entity Code X
            }
        }

        return sbDtlRec;
    }

    final StringBuffer fillClaimServiceDates(StringBuffer sbDtlRec) {
        String vClmStartDt = "";
        String vClmEndDt = "";
        String strDtQry = "";

        if (this.claimType.equalsIgnoreCase(INSTITUTIONAL_CLAIM_TYPE) ||
                this.claimType.equalsIgnoreCase(INSTITUTIONAL_CLAIM_TYPE_4010)) {
            /*
             * DEF005887 - EAT - 20081006 Check for 'type' of date sent. It may be
             * either YYYYMMDD or YYYYMMDD-YYYYMMDD (a range) If it is a single
             * date than that is both the start date and end date.
             */
            strDtQry = "SELECT DTP03DT_FLD FROM CLAIM_DT " +
                "WHERE CLAIM_KEY = '" + this.claimKey + "' " +
                "AND TX_TYPE = '837I' AND DTP01DTTM_QUAL = '434' " +
                "ORDER BY DTP03DT_FLD DESC ";
            vClmStartDt = getDate(strDtQry);

            if (vClmStartDt.indexOf('-') == -1) {
                vClmEndDt = vClmStartDt;
            } else {
                vClmEndDt = vClmStartDt.substring(9, 17);
                vClmStartDt = vClmStartDt.substring(0, 8);
            }

            writeServiceDates(sbDtlRec, vClmStartDt, vClmEndDt);
        } else if (this.claimType.equalsIgnoreCase(DENTAL_CLAIM_TYPE) ||
                this.claimType.equalsIgnoreCase(DENTAL_CLAIM_TYPE_4010)) {
            //TWP these qualifiers may not be correct? Check them
            strDtQry = "SELECT DTP03DT_FLD FROM CLAIM_DT " +
                "WHERE CLAIM_KEY = '" + this.claimKey + "' " +
                "AND TX_TYPE = '837D' AND DTP01DTTM_QUAL = '472' " +
                "AND DENTAL_SERVICE_KEY IS NULL ";
            vClmStartDt = getDate(strDtQry);

            if (vClmStartDt.equals("")) {
                strDtQry = "SELECT MIN(TO_NUMBER(DTP03DT_FLD)) FROM CLAIM_DT " +
                    "WHERE CLAIM_KEY = '" + this.claimKey + "' " +
                    "AND TX_TYPE = '837D' AND DTP01DTTM_QUAL = '472' ";
                vClmStartDt = getDate(strDtQry);
                strDtQry = "SELECT MAX(TO_NUMBER(DTP03DT_FLD)) FROM CLAIM_DT " +
                    "WHERE CLAIM_KEY = '" + this.claimKey + "' " +
                    "AND TX_TYPE = '837D' AND DTP01DTTM_QUAL = '472' ";
                vClmEndDt = getDate(strDtQry);
            } else {
                if (vClmStartDt.length() > 8) {
                    vClmEndDt = vClmStartDt.substring(9, 17);
                    vClmStartDt = vClmStartDt.substring(0, 8);
                } else {
                    vClmEndDt = vClmStartDt;
                }
            }

            writeServiceDates(sbDtlRec, vClmStartDt, vClmEndDt);
        } else if (this.claimType.equalsIgnoreCase(PROFESSIONAL_CLAIM_TYPE) ||
                this.claimType.equalsIgnoreCase(PROFESSIONAL_CLAIM_TYPE_4010)) {
            strDtQry = "SELECT DTP03DT_FLD FROM CLAIM_DT " +
                "WHERE CLAIM_KEY = '" + this.claimKey + "' " +
                "AND TX_TYPE = '837P' AND DTP01DTTM_QUAL = '472' " +
                "AND DTP02DTTM_FMT_QUAL = 'RD8' " +
                "ORDER BY DTP03DT_FLD DESC ";
            vClmStartDt = getDate(strDtQry);

            if (vClmStartDt.equals("")) {
                strDtQry = "SELECT MIN(TO_NUMBER(DTP03DT_FLD)) FROM CLAIM_DT " +
                    "WHERE CLAIM_KEY = '" + this.claimKey + "' " +
                    "AND TX_TYPE = '837P' AND DTP01DTTM_QUAL = '472' ";
                vClmStartDt = getDate(strDtQry);
                strDtQry = "SELECT MAX(TO_NUMBER(DTP03DT_FLD)) FROM CLAIM_DT " +
                    "WHERE CLAIM_KEY = '" + this.claimKey + "' " +
                    "AND TX_TYPE = '837P' AND DTP01DTTM_QUAL = '472' ";
                vClmEndDt = getDate(strDtQry);
            } else {
                vClmEndDt = vClmStartDt.substring(9, 17);
                vClmStartDt = vClmStartDt.substring(0, 8);
            }

            writeServiceDates(sbDtlRec, vClmStartDt, vClmEndDt);
        } else {
            sbDtlRec.append(formatOutputString("17760704", REQUIRED, 8, 8,
                    StringFormat.RPADSPACES)); //Claim Service Date Start
            sbDtlRec.append(formatOutputString("17760704", REQUIRED, 8, 8,
                    StringFormat.RPADSPACES, false)); //Claim Service Date End
        }

        return sbDtlRec;
    }

    private void writeServiceDates(StringBuffer sbDtlRec, String vClmStartDt,
        String vClmEndDt) {
        if (vClmStartDt.equals("")) {
            sbDtlRec.append(formatOutputString("17760704", REQUIRED, 8, 8,
                    StringFormat.RPADSPACES)); //Claim Service Date Start
        } else {
            sbDtlRec.append(formatOutputString(vClmStartDt, REQUIRED, 8, 8,
                    StringFormat.RPADSPACES)); //Claim Service Date Start
        }

        if (vClmEndDt.equals("")) {
            sbDtlRec.append(formatOutputString("17760704", REQUIRED, 8, 8,
                    StringFormat.RPADSPACES, false)); //Claim Service Date End
        } else {
            sbDtlRec.append(formatOutputString(vClmEndDt, REQUIRED, 8, 8,
                    StringFormat.RPADSPACES, false)); //Claim Service Date End
        }
    }

    @SuppressWarnings("unchecked")
    final String getDate(String strDateQuery) {
        String vDt = "";

        Query q = em.createNativeQuery(strDateQuery);
        List results = q.getResultList();

        if ((results.size() > 0) && (results.get(0) != null)) {
            vDt = results.get(0).toString();
        }

        return vDt;
    }

    final String getClaimIndex(String ediClaimKey) {
        String retVal = "";
        LOGGER.debug("Getting Claim Index for: " + ediClaimKey);

        Query q = em.createQuery(
                "select c.claimIndex from foClaim c where c.ediClaimKey = :key ");
        q.setParameter("key", ediClaimKey);

        try {
            Long index = (Long) q.getSingleResult();
            retVal = index.toString();
            LOGGER.debug("Returning Claim Index :" + retVal);
        } catch (PersistenceException pe) {
            LOGGER.warn("Unable to get FEE Claim Index for EDI claim : {} ",
                ediClaimKey);
        }

        return retVal;
    }

    final String getSeqNbr() {
        String strSeqNbr = "SELECT E_REPOS.WEBMD_REJ_CLAIM_SEQ.NEXTVAL FROM DUAL";
        Query q = em.createNativeQuery(strSeqNbr);

        return q.getSingleResult().toString();
    }

    final void insertProcessLog(String vType, int count) {
        EdiProcessLog ediProcessLog = new EdiProcessLog();
        ediProcessLog.setEdiProcessId(UUID.randomUUID().toString().toUpperCase());
        ediProcessLog.setFileDate(DateUtil.getCurSqlDate());
        ediProcessLog.setProcessName("X12 CLAIM STATUS");
        ediProcessLog.setMessage("NUM. OF X12 CLAIMS SENT WITH CLAIM STATUS " +
            vType);
        ediProcessLog.setCounts(Long.valueOf(count));
        ediProcessLog.setClaimType("837");
        ediProcessLog.setStatus("NEW");
        em.persist(ediProcessLog);
    }

    private void delimit(StringBuffer sb) {
        sb.append(DELIMITER);
    }

    /*This is a new record added for 5010 */
    private void createStcRec(Claim claim, String seqNumber, PrintWriter vCsFile) {
        String payerClaimId = "";

        if (isRejectedClaim()) {
            createRejectStatusAndMarkSent(claim, seqNumber, vCsFile);
        } else {
            //Since the claim has been accepted we will return a status of accepted		      
            //Spec requirements
            //A2 - ACK - Accepted into the adjudication system. - Normal return
            payerClaimId = getClaimIndex(claimKey);
            writeSTLRec(claim, payerClaimId, "A2", "20", "", vCsFile);
        }
    }

    // This writes an STL record to the status file for a rejected claim.
    // One of the following statuses will be written to the file: A3, A6 & A7 records) 
    // It also updates the status of the claim so that we know we have reported the
    // status of a particular claim back to Emdeon.
    private void createRejectStatusAndMarkSent(Claim claim,
        String payerClaimId, PrintWriter vCsFile) {
        List<RejectClaim> rejectClaims = getReasonsClaimRejected();

        //TWP: What do we do if there are no reject codes in the DB?
        //I decided to send an A3 ACK - returned as unprocessable - 
        //with 0 - no furthur information avail
        if (rejectClaims.size() < 1) {
            String str = "A rejected claim does not have an entry in the REJECT_CLAIM table key:" +
                claim.getClaimKey();
            LOGGER.error(str);
            throw new RuntimeException(str);

            //writeSTLRec(claim,payerClaimId, "A3", "0", " ", vCsFile);
        } else {
            for (RejectClaim rejectClaim : rejectClaims) {
                String fullRejectReasonCode = rejectClaim.getRejectReasonCodes()
                                                         .getRejectCode();

                //Get the reject reasons and entity id from the database
                //The reject codes have a lot of information embedded in them
                //They are delimited by ;
                //Each token contains 2 or more "sub-tokens" delimited by :
                //The first two tokens in the : delimited sub-token represent a 
                //status (error) category and category status code pair  
                //If there is a third token - it is an entity code
                //statusCat:StatuscCode:EntityCode;statusCat:StatuscCode:EntityCode
                String[] individualRejectCodeGroup = fullRejectReasonCode.split(
                        ";");

                //The first two codes represent status
                for (int j = 0; j < individualRejectCodeGroup.length; j++) {
                    String[] individualComponentsOfRejectCode = individualRejectCodeGroup[j].split(
                            ":");
                    String statusCategoryCode = individualComponentsOfRejectCode[0];
                    String statusCode = individualComponentsOfRejectCode[1];
                    String entityCode = "";

                    if (individualComponentsOfRejectCode.length > 2) {
                        entityCode = individualComponentsOfRejectCode[2];
                    }

                    //Actually write the status line in the file
                    writeSTLRec(claim, payerClaimId, statusCategoryCode,
                        statusCode, entityCode, vCsFile);
                }

                //Update the status of this claim in the file.
                //TWP - Check this during testing make sure the database commit is in the proper place
                if (this.dailyCstat) {
                    rejectClaim.setStatus("277U_SENT");
                }
            }
        }
    }

    private List<RejectClaim> getReasonsClaimRejected() {
        String qstr = "";

        if (this.dailyCstat) {
            qstr = "select rc from eRejectClaim rc " +
                "where rc.ediClaimKey = :key " + "and rc.status = 'NEW' " +
                "order by rc.priority";
        } else {
            qstr = "select rc from eRejectClaim rc " +
                "where rc.ediClaimKey = :key " + "order by rc.priority";
        }

        Query q = em.createQuery(qstr);
        q.setParameter("key", this.claimKey);

        List<RejectClaim> rejectClaims = q.getResultList();

        return rejectClaims;
    }

    private void writeSTLRec(Claim claim, String payerClaimId,
        String statusCategoryCode, String statusCode, String entityCode,
        PrintWriter vCsFile) {
        StringBuffer sbHdrRec = new StringBuffer();
        sbHdrRec.append("STC"); //Record ID
        delimit(sbHdrRec);
        this.cnt++;
        sbHdrRec.append(formatOutputString(Integer.toString(this.cnt),
                REQUIRED, 1, 10, StringFormat.LPADZERO)); //Record Number - must be in sync with other records
        sbHdrRec.append(PAYER_ID); //Payer ID
        delimit(sbHdrRec);
        sbHdrRec.append(formatOutputString(payerClaimId, REQUIRED, 1, 50,
                StringFormat.RPADSPACES));

        //TWP What do we use for an identification number - we have not assigned it yet
        //for a claim that has only been accepted ???? we have a GUID
        //Should we really be running this file after the FPPS PL/SQL loading runs so we can assign it a claim index?
        sbHdrRec.append(""); //Line Item Control Number
        delimit(sbHdrRec);

        Date today = Calendar.getInstance().getTime();
        String dateFormat = "yyyyMMddHHmmss";
        sbHdrRec.append(new SimpleDateFormat(dateFormat).format(today)); //Status Information Effective Date
        delimit(sbHdrRec);

        sbHdrRec.append(statusCategoryCode); //Claim Status Category Code  
        delimit(sbHdrRec);

        sbHdrRec.append(statusCode); //Claim Status Code 
        delimit(sbHdrRec);

        sbHdrRec.append(entityCode); //Entity Code 
        delimit(sbHdrRec);

        sbHdrRec.append(""); //Data In Error - we do not have this from what I can tell so leave it blank
        delimit(sbHdrRec);
        sbHdrRec.append(""); //Emdeon Status Code - I don;t see this either
                             //	      delimit(sbHdrRec);	      

        sbHdrRec.append(CRLF); //Record Teminator new for 5010
        vCsFile.print(sbHdrRec); //TWP Changed to print rather than println since I am explicitly adding CRLF  
        vCsFile.flush();
        sbHdrRec = null;
    }

    private String formatOutputString(Object dataToFormat, boolean required,
        int minLength, int maxLength, StringFormat format) {
        return formatOutputString(dataToFormat, required, minLength, maxLength,
            format, true);
    }

    /**
     * This method should be called for each field which
     * will make the code above basically match the spec.
     * It will format the field to the specs given and delimit
     * each value returned.
     *
     * @param dataToFormat
     * @param minLenght
     * @param maxLength
     * @param format
     * @param delimit
     * @return
     */
    private String formatOutputString(Object dataToFormat, boolean required,
        int minLength, int maxLength, StringFormat format, boolean delimit) {
        //If we don't have data for a required field we just balk right now - what do you really want to happen?
        if ((required && (dataToFormat == null)) ||
                (required &&
                (dataToFormat instanceof String &&
                (((String) dataToFormat).trim().length() < 1)))) {
            throw new RuntimeException(
                "We don't have this required data see stack trace.");
        }

        //We are assuming if we send a value it must meet the spec minLength and maxLength
        //But optional fields may be left blank - i.e. a blank field will appear in the output
        //For example see ;; in the following sample line -> 
        //													;nonblankfield;;nonblankfield;
        String stringToReturn = "";

        if ((dataToFormat != null) &&
                !(dataToFormat instanceof String &&
                (((String) dataToFormat).trim().length() < 1))) {
            //If the data needs to be formatted a specific way format it
            if (format != null) {
                stringToReturn = this.formatString(dataToFormat, minLength,
                        maxLength, format);
            } else {
                stringToReturn = dataToFormat.toString();
            }

            //Check to see if the value complies with EMDEONS minimum length requirement 		  	 
            int stringLength = stringToReturn.length();

            if (stringLength < minLength) {
                throw new RuntimeException(
                    "Please specify a formatter for this data so it can be formatted to meet the minimum lenght requirement." +
                    dataToFormat);
            }

            //Check to see if the value complies with EMDEONS max length requirement - 
            //If it does not we make truncate it - this seems dangerous - TWP.	   
            if (stringLength > maxLength) {
                stringToReturn = truncate(stringToReturn, dataToFormat,
                        maxLength);
            }
        }

        if (delimit) {
            stringToReturn += DELIMITER;
        }

        return stringToReturn;
    }

    private String truncate(String formattedString, Object dataToFormat,
        int maxLength) {
        int beginIndex = formattedString.length() - maxLength;
        String stringToReturn = formattedString.substring(beginIndex);
        LOGGER.warn("Truncating data from " + formattedString + " to " +
            stringToReturn);

        return stringToReturn;
    }

    /**
     * Throws a runtime exception if it can not format the data to the specification given.
     * Will also throw an exception if null is passed in - it expects to format a string.
     *
     * @param stringToFormat
     * @param minLength
     * @param maxLength
     * @param format
     * @return
     */
    private String formatString(Object stringToFormat, int minLength,
        int maxLength, StringFormat format) {
        String stringToReturn = null;

        if ((stringToFormat != null) && stringToFormat instanceof BigDecimal) {
            stringToFormat = stringToFormat.toString();
        }

        if ((stringToFormat != null) &&
                (((String) stringToFormat).length() < minLength)) {
            if (StringFormat.RPADSPACES == format) {
                stringToReturn = StringUtils.padString((String) stringToFormat,
                        minLength, ' ', false);
            }

            if (StringFormat.LPADZERO == format) {
                stringToReturn = StringUtils.padString((String) stringToFormat,
                        minLength, '0', true);
            }
        } else {
            if ((stringToFormat != null) &&
                    (((String) stringToFormat).length() >= minLength)) {
                stringToReturn = (String) stringToFormat;
            }
        }

        if (StringFormat.LPADDECZERO2PLACES == format) {
            stringToReturn = StringUtils.lPadDecimalZeros2Places((String) stringToFormat,
                    maxLength);
        }

        if (stringToReturn == null) {
            throw new RuntimeException("Could not format the object - " +
                stringToFormat +
                " - conversion of this type of data is not yet supported.");
        }

        return stringToReturn;
    }
    private enum StringFormat {RPADSPACES,
        LPADZERO,
        LPADDECZERO2PLACES;
    }
}
