package gov.va.med.hac.edi.reconciliation;

import static gov.va.med.hac.edi.reconciliation.Helper.CLAIM_ADJUDICATIONS_CALC_METHOD_CODE;
import static gov.va.med.hac.edi.reconciliation.Helper.COMPLETED_LINE_STATUS_CODE;
import static gov.va.med.hac.edi.reconciliation.Helper.DATA_ERROR;
import static gov.va.med.hac.edi.reconciliation.Helper.DUPLICATE_LINE_ADJUDICATION;
import static gov.va.med.hac.edi.reconciliation.Helper.EPSILON;
import static gov.va.med.hac.edi.reconciliation.Helper.FAILED_TO_MATCH_AMOUNT;
import static gov.va.med.hac.edi.reconciliation.Helper.FAILED_TO_MATCH_CLAIM_ID;
import static gov.va.med.hac.edi.reconciliation.Helper.NON;
import static gov.va.med.hac.edi.reconciliation.Helper.RESOLUTION_STATUS_CODE_RESOLVE;
import gov.va.med.domain.fppsowner.CalcMethod;
import gov.va.med.domain.fppsowner.ClaimProcedure;
import gov.va.med.domain.fppsowner.Claim;
import gov.va.med.domain.fppsowner.ClaimPayment;
import gov.va.med.domain.fppsowner.ClaimPaymentType;
import gov.va.med.domain.fppsowner.ClaimProcRecon;
import gov.va.med.domain.fppsowner.ClaimProcStatus;
// import gov.va.med.domain.fppsowner.ClaimReconciliation;
import gov.va.med.domain.fppsowner.*;
import gov.va.med.domain.fppsowner.ClaimReconciliationAdjust;
import gov.va.med.domain.fppsowner.ClaimRemitRemark;
import gov.va.med.domain.fppsowner.FeeClaimPaymentLines;
import gov.va.med.domain.fppsowner.FeeClaimPayments;
import gov.va.med.domain.fppsowner.RemittanceRemark;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.EntityTransaction;
import javax.persistence.Query;

import org.apache.commons.lang.exception.ExceptionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 *
 * Class for reconciling Claim type FEE payments.
 *
 */
public class ClaimTypeReconciliation {
   static final Logger LOGGER = LoggerFactory.getLogger(OtherTypeReconciliation.class);
   EntityManager em = null;
   String username = "";
   int reconciledPayments = 0;
   int paymentErrors = 0;

   
   
   /**
    * Class constructor.
    * 
    * @param em  JPA entity manager.
    * @param username user name from database connection properties.
    */
   public ClaimTypeReconciliation(EntityManager em, String username) {
      this.em = em;
      this.username = username;
   }

   
   
   @SuppressWarnings("unchecked")
   final void reconcileClaimTypePayments() {
	   
	  boolean DEBUG = true;
	  
	
      Query q = em.createNativeQuery(
            "SELECT FEE_CLAIM_PAYMENT_ID FROM FEE_CLAIM_PAYMENTS " +
            "  WHERE RESOLUTION_STATUS_CODE =  'NEW' " +
            "  AND TRANSACTION_TYPE_CODE = 'C' ");
     
      
/**      
      Query q = em.createNativeQuery(
              "SELECT FEE_CLAIM_PAYMENT_ID FROM FEE_CLAIM_PAYMENTS " +
              "  WHERE RESOLUTION_STATUS_CODE =  'NEW' " +
              "  AND TRANSACTION_TYPE_CODE = 'C' AND BILLED_AMOUNT > 0 AND PAID_AMOUNT > 0 AND MODIFIED_BY='FPPS_OWNER'");
 **/     
      
      
      // none found - 6/25/2015
      List<String> fcpIds = q.getResultList();
      
      if(DEBUG) {
      	System.out.println("ClaimTypeReconciliation.reconcileClaimTypePayments() - feeClaimPayments available: " + fcpIds.size());
      }
      
      for (String id : fcpIds) {
    	  // get single payment
         Query q1 = em.createQuery("select f from foFeeClaimPayments f where f.feeClaimPaymentId = :id");
         q1.setParameter("id", id);
         FeeClaimPayments fcp = (FeeClaimPayments) q1.getSingleResult();
         
         // get claims for payment
         Query q2 = em.createNativeQuery("select count(1) from claims c where claim_index = :index ");
         q2.setParameter("index", fcp.getClaimIndex());
         
         BigDecimal claimCount = (BigDecimal) q2.getSingleResult();
         
         // must be 1 
         if (claimCount.intValue() == 1) {
            verifyAndReconcilePayment(id);
         } else {
            // Can't find associated claim for this payment.
            Helper.recordPaymentError(fcp.getFeeClaimPaymentId(), FAILED_TO_MATCH_CLAIM_ID, username, em);
            paymentErrors++;
         }
      }
      
      LOGGER.info("Completed Claim Type Reconciliation with {} reconciled payments and {} payment errors.",
            reconciledPayments, paymentErrors);
   }

   
   
   final void verifyAndReconcilePayment(String feeClaimPaymentid) {
	   
	  boolean DEBUG = true;
	   
	  if(DEBUG)  {
		//  System.out.println("verifyAndReconcilePayment() - feeClaimPaymentid is: " + feeClaimPaymentid);
	  }
	   
      // Claim Type adjudications are guaranteed to have exactly one payment
      // line
      Query q = em.createQuery("select f from foFeeClaimPayments f where f.feeClaimPaymentId = :id ");
      q.setParameter("id", feeClaimPaymentid);
      FeeClaimPayments fcp = (FeeClaimPayments) q.getSingleResult();
      
      // get the associated claim
      Query qclm = em.createQuery("select c from foClaim c where c.claimIndex = :index ");
      qclm.setParameter("index", fcp.getClaimIndex());
      Claim clm = (Claim) qclm.getSingleResult();
          
      
      // Calculate billedAmount by summing all claim procedure
      // billed amounts.
      // If any claim procedure has already
      // been reconciled then this claim type payment is in error.
      BigDecimal totalBilledAmount = BigDecimal.ZERO;
      Collection<Long> dupSeqErrors = new ArrayList<Long>();
      for (ClaimProcedure proc : clm.getClaimProcedures()) {
         if (proc.getClaimProcRecons().size() == 0) {
            totalBilledAmount = totalBilledAmount.add(proc.getBilledAmount());
         } else {
            dupSeqErrors.add(proc.getClaimProcedureSeq());
         }
      }
      
      // If any line on this claim has been adjudicated
      // set a claim level error.
      if (dupSeqErrors.size() != 0) {
         Helper.recordPaymentLineErrors(feeClaimPaymentid, fcp.getFeeClaimPaymentLines(), DUPLICATE_LINE_ADJUDICATION, username, em);
         paymentErrors++;
         return;
      }

      FeeClaimPaymentLines fcpl = (FeeClaimPaymentLines) fcp.getFeeClaimPaymentLines()
            .iterator().next();
      BigDecimal difference = BigDecimal.ZERO;
      if (fcpl.getAdjustmentAmount1() != null) {
         difference = totalBilledAmount.subtract(fcpl.getAdjustmentAmount1());
      }

      if (fcpl.getAdjustmentAmount2() != null) {
         difference = difference.subtract(fcpl.getAdjustmentAmount2());
      }

      if (fcp.getPaidAmount() != null) {
         difference = difference.subtract(fcp.getPaidAmount());
      }

      if (difference.abs().doubleValue() > EPSILON) {
         Helper.recordPaymentError(feeClaimPaymentid, FAILED_TO_MATCH_AMOUNT, username, em);
         paymentErrors++;
         return;
      }

      BigDecimal paidAmount = (fcp.getPaidAmount() != null) ? fcp.getPaidAmount() : BigDecimal.ZERO;
      String typeOfPayment = fcpl.getTypeOfPayment();
      if (paidAmount.doubleValue() < EPSILON) {
         // This is a null payment (paidAmount = 0.0 at the claim level).
         // Verify that typeOfPayment is NON or empty at the line level.
         // The FeeClaimPaymentLineBase.validate() sets typeOfPayment to
         // NON if the line level payment is 0.00, but the line level
         // payment amount is ignored for 'C' type claim payments.
         if ((typeOfPayment == null) || (typeOfPayment.length() == 0)) {
            // Set to NON because adjudicateClaimPaymentClaimType()
            // expects typeOfPayment to be set at the line level.
            setPaymentToNon(fcpl.getFeeClaimPaymentLineId(), em);
         } else if (!typeOfPayment.equals(NON)) {
            Helper.recordPaymentError(feeClaimPaymentid, DATA_ERROR, username, em);
            paymentErrors++;
            return;
         }

      } else {
         // paidAmount >= 0.01
         // The FeeClaimPaymentLineBase.validate() sets typeOfPayment to
         // NON if it is empty and the paidAmount at the line level is 0.00.
         // Since the line level paidAmount will normally be 0.00 for 'C'
         // type payments check for typeOfPayment == NON.
         if ((typeOfPayment == null) || (typeOfPayment.length() == 0) || (typeOfPayment.equals(NON))) {
            Helper.recordPaymentError(feeClaimPaymentid, DATA_ERROR, username, em);
            paymentErrors++;
            return;
         }
      }

      boolean success = recordClaimPayment(fcp.getFeeClaimPaymentId(), username, em);
      if (success) {
          reconciledPayments++;
      }
   }

   
   
  


@SuppressWarnings("unchecked")
   static boolean recordClaimPayment(String fcpId, String username, EntityManager entityManager) {
      boolean success = true;
      boolean DEBUG = true;
      
      
      if(DEBUG) {
    //	  System.out.println("recordClaimPayment() - fcpid is: " + fcpId);
      }
      
      java.sql.Date today = new java.sql.Date(new java.util.Date().getTime());
      EntityTransaction tx = null;
      
      
      try {
         tx = entityManager.getTransaction();
         tx.begin();
         Query q = entityManager.createQuery("select f from foFeeClaimPayments f where f.feeClaimPaymentId = :id ");
         q.setParameter("id", fcpId);
         FeeClaimPayments fcp = (FeeClaimPayments) q.getSingleResult();
         Query qclm = entityManager.createQuery("select c from foClaim c where c.claimIndex = :index ");
         qclm.setParameter("index", fcp.getClaimIndex());
         Claim claim = (Claim) qclm.getSingleResult();
         
         
         if(DEBUG) {
 //      	  System.out.println("recordClaimPayment() - claim is: " + claim.getClaimIndex());
         }
         
         
         // Claim Type adjudications are have exactly one Fee Claim Payment Line
         // so we know where the type of payment is coming from.
         FeeClaimPaymentLines feePaymentLine = (FeeClaimPaymentLines) fcp.getFeeClaimPaymentLines()
               .iterator()
               .next();
         
         // Get Claim Payment Type
         Query qcpt = entityManager.createQuery("select t from foClaimPaymentType t where t.claimPaymentTypeCode = :code ");
         String typeOfPayment = feePaymentLine.getTypeOfPayment();
         if(typeOfPayment==null)  //TDP added to default to NON payment where type_of_payment is null
        	 typeOfPayment = "NON";
         // workaround for miss naming of
         // claim_payment_type.claim_payment_type_code (CHECK vs. CHK)
// VITRIA         
         // comes in from Vitria as CHK but is in the CLAIM_PAYMENT_TYPE table
         // as CHECK.
         if ("CHK".equals(typeOfPayment)) {
            typeOfPayment = "CHECK";
         }
         qcpt.setParameter("code", typeOfPayment);
         ClaimPaymentType claimPaymentType = null;
         try {
            claimPaymentType = (ClaimPaymentType) qcpt.getSingleResult();
         } catch (Exception e) {
            LOGGER.warn("unknown claim payment type code {} ",
                  feePaymentLine.getTypeOfPayment());
         }
         
         // Create Claim Payment Line
         ClaimPayment claimPayment = new ClaimPayment();
         claimPayment.setPaymentBatchDate(fcp.getFeeInvoiceDate());
         claimPayment.setTreasuryPaymentDate(feePaymentLine.getCheckDate());
         claimPayment.setPaymentTrackingNumber(feePaymentLine.getCheckNumber());
         claimPayment.setPaidAmount(fcp.getPaidAmount());
         claimPayment.setFeeInvoiceNumber(fcp.getFeeInvoiceNumber());
         claimPayment.setClaimPaymentType(claimPaymentType);
         claimPayment.setCreatedBy(username);
         claimPayment.setDateCreated(today);
         claimPayment.setClaim(claim);
         claim.getClaimPayments().add(claimPayment);

         
if(DEBUG) {
	System.out.println("recordClaimPayment() - new ClaimPayment added to claim: " + claim.getClaimIndex());
}
         // Get validated DRG and DRG Description
         Query qdrg = entityManager.createNativeQuery(
               "select dc.drg_code, dcd.description " +
               " from fpps_owner.drg_codes dc, fpps_owner.drg_code_descriptions dcd " +
               " where dc.drg_code = :code " +
               " and dc.drg_id = dcd.drg_id " +
               " and :date between dcd.effective_date and nvl(dcd.end_date, sysdate) ");
         qdrg.setParameter("code", fcp.getDiagnosisRelatedGroupCode());
         qdrg.setParameter("date", fcp.getPeriodStartDate());
         String drgcode = null;
         String drgdesc = null;
         Collection<Object[]> rslt = qdrg.getResultList();
         for (Object[] r : rslt) {
            drgcode = (String) r[0];
            drgdesc = (String) r[1];
         }
         
         // Get calculation method
         Query qcalc = entityManager.createQuery("select c from foCalcMethod c where c.calcMethodCd = :method ");
         qcalc.setParameter("method", CLAIM_ADJUDICATIONS_CALC_METHOD_CODE);
         CalcMethod calcMethod = (CalcMethod) qcalc.getSingleResult();

         // Create the Claim Reconciliation and Claim Reconciliation Adjustment
         // records
         
         // IF NONE OF THESE RECORDS ARE CREATED THERE WILL BE NO RECORDS IN THE stg_* tables
         
         ClaimReconciliation clmRecon = new ClaimReconciliation();
         clmRecon.setCalcMethod(calcMethod);
         clmRecon.setPaidAmount(fcp.getPaidAmount());
         clmRecon.setDisbursedAmount(fcp.getDisbursedAmount());
         clmRecon.setInterestAmount(fcp.getInterestAmount());
         clmRecon.setCoveredDays(fcp.getCoveredDays());
         clmRecon.setDrgCd(drgcode);
         clmRecon.setDrgCodeDesc(drgdesc);
         clmRecon.setSent835Flag('N');
         clmRecon.setCreatedBy(username);
         clmRecon.setDateCreated(today);
         clmRecon.setClaim(claim);
         
         
         
if(DEBUG) {
	System.out.println("recordClaimPayment() - new ClaimReconciliation added to claim: " + claim.getClaimIndex());
}
           
         
         
         claim.getClaimReconciliations().add(clmRecon);

         BigDecimal adjAmount1 = feePaymentLine.getAdjustmentAmount1();
         if ((adjAmount1 != null) && (adjAmount1.doubleValue() > EPSILON)) {
            ClaimReconciliationAdjust adjust1 = new ClaimReconciliationAdjust();
            adjust1.setAdjustmentAmount(feePaymentLine.getAdjustmentAmount1());
            adjust1.setAdjustmentGroupCode(feePaymentLine.getAdjustmentGroupCode1());
            adjust1.setAdjustmentReasonCode(feePaymentLine.getAdjustmentReasonCode1());
            adjust1.setClaimIndex(claim.getClaimIndex());
            adjust1.setCreatedBy(username);
            adjust1.setDateCreated(today);
            adjust1.setClaimReconciliation(clmRecon);
            clmRecon.getClaimReconciliationAdjusts().add(adjust1);
         }

         
         
if(DEBUG) {
	System.out.println("recordClaimPayment() - new ClaimReconciliationAdjust created and attached to ClaimReconciliation: " + clmRecon.getClaimRecId());
}
           
         
         
         BigDecimal adjAmount2 = feePaymentLine.getAdjustmentAmount2();
         if ((adjAmount2 != null) && (adjAmount2.doubleValue() > EPSILON)) {
            ClaimReconciliationAdjust adjust2 = new ClaimReconciliationAdjust();
            adjust2.setAdjustmentAmount(feePaymentLine.getAdjustmentAmount2());
            adjust2.setAdjustmentGroupCode(feePaymentLine.getAdjustmentGroupCode2());
            adjust2.setAdjustmentReasonCode(feePaymentLine.getAdjustmentReasonCode2());
            adjust2.setClaimIndex(claim.getClaimIndex());
            adjust2.setCreatedBy(username);
            adjust2.setDateCreated(today);
            adjust2.setClaimReconciliation(clmRecon);
 //           clmRecon.getClaimReconciliationAdjusts().add(adjust2);
         }

         if(DEBUG) {
        		System.out.println("recordClaimPayment() - new ClaimReconciliationAdjust created and attached to ClaimReconciliation: " + clmRecon.getClaimRecId());
        	}
        	           
         
         
         // Get Claim Procedure status code of ACCEPTED
         Query qcps = entityManager.createQuery("select s from foClaimProcStatus s where s.claimProcStatusCd = :code ");
         qcps.setParameter("code", "ACCEPTED");
         ClaimProcStatus cps = (ClaimProcStatus) qcps.getSingleResult();

         // Write null records for DB.CLAIM_PROCEDURE_ADJUDICATIONS
         for (ClaimProcedure proc : claim.getClaimProcedures()) {
            ClaimProcRecon procRecon = new ClaimProcRecon();
            procRecon.setSent835Flag('N');
            procRecon.setClaimProcStatus(cps);
            procRecon.setCreatedBy(username);
            procRecon.setDateCreated(today);
            procRecon.setClaim(claim);
            claim.getClaimProcRecons().add(procRecon);
            procRecon.setClaimPayment(claimPayment);
            claimPayment.getClaimProcRecons().add(procRecon);
            procRecon.setClaimReconciliation(clmRecon);
            clmRecon.getClaimProcRecons().add(procRecon);
            procRecon.setClaimProcedure(proc);
            proc.getClaimProcRecons().add(procRecon);
            
            proc.setLineStatus(COMPLETED_LINE_STATUS_CODE);
            
            
            if(DEBUG) {
        		System.out.println("recordClaimPayment() - new ClaimProcRecon created: " + procRecon.getClaimProcRecId() );
        	}
          }

         
        	           
         
         
         
         // add Claim Remittance Remarks.
         String remittanceRemarkCode = feePaymentLine.getRemittanceRemarkCode();
         String[] rrcs = new String[0];
         
// REMITTANCE 
         if (remittanceRemarkCode != null) {
            rrcs = remittanceRemarkCode.split(",");
         }

         Query qrr = entityManager.createQuery("select rr from foRemittanceRemark rr where rr.remittanceRemarkCd = :code");

         for (String rrc : rrcs) {
            qrr.setParameter("code", rrc);
            RemittanceRemark rr = null;
            try {
               rr = (RemittanceRemark) qrr.getSingleResult();
            } catch (Exception e) {
               LOGGER.debug("unable to locate Remittance Remark with code of {} ",
                     rrc);
            }
            if (rr != null) {
               ClaimRemitRemark claimRR = new ClaimRemitRemark();
               claimRR.setRemittanceRemark(rr);
               claimRR.setDateCreated(today);
               claimRR.setCreatedBy(username);
               claimRR.setClaimReconciliation(clmRecon);
//               clmRecon.getClaimRemitRemarks().add(claimRR);
               
               if(DEBUG) {
            	  System.out.println("recordClaimPaymnent() - new ClaimRemitRemark created and added to claim reconciliation:" + clmRecon.getClaimRecId());
            		}
                     
            }
         }
         
 if(DEBUG) {
	 System.out.println("recordClaimPayment() - end of method - setting fcp.ResolutionStatusCode to RESOLUTION_STATUS_CODE_RESOLVE");
 }
         
         fcp.setResolutionStatusCode(RESOLUTION_STATUS_CODE_RESOLVE);
         
         if(DEBUG) {
        	 System.out.println("recordClaimPayment() - end of method - setting fcp.ResolutionStatusCode to RESOLUTION_STATUS_CODE_RESOLVE");
         }
          
         
         
         tx.commit();
         
         if(DEBUG) {
        	 System.out.println("recordClaimPayment() - DONE - fcp record COMMITTED");
         }        
         
         Helper.closeIfClaimComplete(claim.getClaimIndex(), username, entityManager);
         
         
      } catch (Exception e) {
         success = false;
         Throwable t = ExceptionUtils.getRootCause(e);
         LOGGER.warn("Failure in reconciling payment {} due to {}", fcpId,
               (t == null) ? e.getMessage() : t.getMessage());

         if (tx != null) {
            tx.rollback();
         }
         
         Helper.markAsFailure(fcpId, username, entityManager);
      }
      return success;
   }

   
   
   static void setPaymentToNon(String fcplId, EntityManager entityManager) {
      EntityTransaction tx = null;
      try {
         tx = entityManager.getTransaction();
         tx.begin();
         Query q = entityManager.createQuery("select f from foFeeClaimPaymentLine f where f.FeeClaimPaymentLineId = :id ");
         q.setParameter("id", fcplId);
         FeeClaimPaymentLines fcpl = (FeeClaimPaymentLines) q.getSingleResult();
         fcpl.setTypeOfPayment(NON);
         tx.commit();
      } catch (Exception e) {
         if (tx != null) {
            tx.rollback();
         }
         LOGGER.warn("Unable to set Fee Claim Payment Status to NON for Payment {} ", fcplId);
      }
   }
}
