package gov.va.med.hac.edi.reconciliation;

import gov.va.med.domain.fppsowner.AdjustmentGroup;
import gov.va.med.domain.fppsowner.AdjustmentReason;
import gov.va.med.domain.fppsowner.CalcMethod;
import gov.va.med.domain.fppsowner.Claim;
import gov.va.med.domain.fppsowner.ClaimPayment;
import gov.va.med.domain.fppsowner.ClaimPaymentType;
import gov.va.med.domain.fppsowner.ClaimProcRecAdjust;
import gov.va.med.domain.fppsowner.ClaimProcRecon;
import gov.va.med.domain.fppsowner.ClaimProcRemitRemark;
import gov.va.med.domain.fppsowner.ClaimProcStatus;
import gov.va.med.domain.fppsowner.ClaimProcedure;
import gov.va.med.domain.fppsowner.ClaimReconciliation;
import gov.va.med.domain.fppsowner.FeeClaimPaymentLines;
import gov.va.med.domain.fppsowner.FeeClaimPayments;
import gov.va.med.domain.fppsowner.RemittanceRemark;
import gov.va.med.domain.fppsowner.VaFacility;
import gov.va.med.domain.fppsowner.VaFacilityType;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedMap;
import java.util.SortedSet;
import java.util.TreeMap;
import java.util.TreeSet;

import javax.persistence.EntityManager;
import javax.persistence.EntityTransaction;
import javax.persistence.NoResultException;
import javax.persistence.PersistenceException;
import javax.persistence.Query;

import org.apache.commons.lang.exception.ExceptionUtils;
import org.hibernate.Session;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import static gov.va.med.hac.edi.reconciliation.Helper.CLAIM_ADJUDICATIONS_CALC_METHOD_CODE;
import static gov.va.med.hac.edi.reconciliation.Helper.COMPLETED_LINE_STATUS_CODE;
import static gov.va.med.hac.edi.reconciliation.Helper.DATA_ERROR;
import static gov.va.med.hac.edi.reconciliation.Helper.DUPLICATE_LINE_ADJUDICATION;
import static gov.va.med.hac.edi.reconciliation.Helper.EPSILON;
import static gov.va.med.hac.edi.reconciliation.Helper.FAILED_TO_MATCH_AMOUNT;
import static gov.va.med.hac.edi.reconciliation.Helper.FAILED_TO_MATCH_CLAIM_ID;
import static gov.va.med.hac.edi.reconciliation.Helper.FAILED_TO_MATCH_LINE_ID;
import static gov.va.med.hac.edi.reconciliation.Helper.RESOLUTION_STATUS_CODE_RESOLVE;

/**
 *
 * This class performs validation and reconciliation of Line type 
 * Claim Payments.
 *
 * called from gov.va.med.hac.edi.reconciliation.Reconcile.java 
 */
public class LineTypeReconciliation {
    EntityManager em = null;
    String user = "";
    static int reconciledCounts = 0;
    static int missmatchedAmtCounts = 0;
    static int unmatchedSeqNumCounts = 0;
    static int previouslyReconCounts = 0;
    java.sql.Date today = new java.sql.Date(new java.util.Date().getTime());
    static final Logger LOGGER = LoggerFactory.getLogger(LineTypeReconciliation.class);


   /**
    * The class constructor.
    * 
    * @param em the JPA entity manager controlling persistence.
    * @param username the user name used to make the database connection.
    */
   public LineTypeReconciliation(EntityManager em, String username) {
        this.em = em;
        this.user = username;
    }

// HERE   
   static List getAllRemittanceRemarks(FeeClaimPaymentLines feePaymentLine) {

	  boolean DEBUG = true; 
	   
	  List<String> list = new ArrayList<String>();
	  String remittanceRemarkCode_1B;
	  String remittanceRemarkCode_2A;
	  String remittanceRemarkCode_2B;
	  String remittanceRemarkCode_3A;
	  String remittanceRemarkCode_3B;
	  String remittanceRemarkCode_4A;
	  String remittanceRemarkCode_4B;
	  String remittanceRemarkCode_5A;
	  String remittanceRemarkCode_5B; 
		  
	  // this field might contain comma-separated values - hold-over from original table structure
	  String remittanceRemarkCode = feePaymentLine.getRemittanceRemarkCode();
	  if (remittanceRemarkCode != null) {
		  String[] rrcs = new String[0];
          rrcs = remittanceRemarkCode.split(",");
             
          for (String rrc : rrcs) {
			  if(!list.contains(rrc)) {
				  list.add(rrc);
			  } 
           }	// end for()
      }	// end if
	  
	  remittanceRemarkCode_1B = feePaymentLine.getRemittanceRemarkCode_1B();
	  if (remittanceRemarkCode_1B != null) {
			  if(!list.contains(remittanceRemarkCode_1B)) {
				  list.add(remittanceRemarkCode_1B);
			  }
      }	// end if
	  
	  remittanceRemarkCode_2A = feePaymentLine.getRemittanceRemarkCode_2A();
	  if (remittanceRemarkCode_2A != null) {
			  if(!list.contains(remittanceRemarkCode_2A)) {
				  list.add(remittanceRemarkCode_2A);
			  }
      }	// end if
	  
	  
	  remittanceRemarkCode_2B = feePaymentLine.getRemittanceRemarkCode_2B();
	  if (remittanceRemarkCode_2B != null) {
			  if(!list.contains(remittanceRemarkCode_2B)) {
				  list.add(remittanceRemarkCode_2B);
			  }
      }	// end if
	  
	  remittanceRemarkCode_3A = feePaymentLine.getRemittanceRemarkCode_3A();
	  if (remittanceRemarkCode_3A != null) {
			  if(!list.contains(remittanceRemarkCode_3A)) {
				  list.add(remittanceRemarkCode_3A);
			  }
      }	// end if
	  
	  remittanceRemarkCode_3B = feePaymentLine.getRemittanceRemarkCode_3B();
	  if (remittanceRemarkCode_3B != null) {
			  if(!list.contains(remittanceRemarkCode_3B)) {
				  list.add(remittanceRemarkCode_3B);
			  }
      }	// end if
	  
	  remittanceRemarkCode_4A = feePaymentLine.getRemittanceRemarkCode_4A();
	  if (remittanceRemarkCode_4A != null) {
			  if(!list.contains(remittanceRemarkCode_4A)) {
				  list.add(remittanceRemarkCode_4A);
			  }
      }	// end if
	  
	  remittanceRemarkCode_4B = feePaymentLine.getRemittanceRemarkCode_4B();
	  if (remittanceRemarkCode_4B != null) {
			  if(!list.contains(remittanceRemarkCode_4B)) {
				  list.add(remittanceRemarkCode_4B);
			  }
      }	// end if
	  
	  
	  // 5
	  remittanceRemarkCode_5A = feePaymentLine.getRemittanceRemarkCode_5A();
	  if (remittanceRemarkCode_5A != null) {
			  if(!list.contains(remittanceRemarkCode_5A)) {
				  list.add(remittanceRemarkCode_5A);
			  }
      }	// end if
	  
	  remittanceRemarkCode_5B = feePaymentLine.getRemittanceRemarkCode_5B();
	  if (remittanceRemarkCode_5B != null) {
			  if(!list.contains(remittanceRemarkCode_5B)) {
				  list.add(remittanceRemarkCode_5B);
			  }
      }	// end if
// END READ OF REMARKS	  
	  
	  return list;
   } // end getAllRemittanceRemarks()
  
  
   // this is called from:
   // enterClaimPayments()
   // reconcilePayments()
   // verifyAndReconcilePayment()
   // reconcileLineTypePayments()
   // Reconcile.main()
   @SuppressWarnings("unchecked")
   static boolean recordPayments(String fcpId, String username, EntityManager entityManager) {
	   
	  boolean DEBUG=true; 
	   
	   
if(DEBUG) {
	System.out.println("recordPayments() - FeeClaimPaymentID is: " + fcpId);
}
	   
      boolean success = true;
      java.sql.Date today = new java.sql.Date(new java.util.Date().getTime());
      EntityTransaction tx = null;
      
      
      
   // Get today as a Calendar
      Calendar x = Calendar.getInstance();
      // Subtract 1 day
      x.add(Calendar.DATE, -2);
      // Make an SQL Date out of that
      java.sql.Date yesterday = new java.sql.Date(x.getTimeInMillis());
      
       
      try {
         tx = entityManager.getTransaction();
         tx.begin();

         Query q = entityManager.createQuery("select f from foFeeClaimPayments f where f.feeClaimPaymentId = :id ");
         q.setParameter("id", fcpId);
         FeeClaimPayments fcp = (FeeClaimPayments) q.getSingleResult();

System.out.println("getting periodStartDate() - " + fcp.getPeriodStartDate().toString());   
System.out.println("getting periodEnd() - " + fcp.getPeriodEndDate().toString());
System.out.println("getting code: " + fcp.getDiagnosisRelatedGroupCode() );
         // Get validated DRG and DRG Description - BROKEN FOR SOME REASON 10/26/2015
         Query qdrg = entityManager.createNativeQuery(
               "select dc.drg_code, dcd.description " +
               " from fpps_owner.drg_codes dc, fpps_owner.drg_code_descriptions dcd " +
               " where dc.drg_code = :code " +
               " and dc.drg_id = dcd.drg_id " +
               " and :date between dcd.effective_date and nvl(dcd.end_date, sysdate) ");
         qdrg.setParameter("code", fcp.getDiagnosisRelatedGroupCode());
         qdrg.setParameter("date", fcp.getPeriodStartDate());			// defined as date in FEE_CLAIM_PAYMENT table
         
         
         String drgcode = null;
         String drgdesc = null;
         Collection<Object[]> rslt =  qdrg.getResultList();
                
         for (Object[] r : rslt) {
            drgcode = (String) r[0];
            drgdesc = (String) r[1];
         }
         System.out.println("4"); 
         // Get Claim Payment Type from claim_payment_type table (EFT/CHK/NON)
         Query qcpt = entityManager.createQuery("select t from foClaimPaymentType t where t.claimPaymentTypeCode = :code ");
         
         
      
         String typeOfPayment = getClaimPaymentType(fcp);  //gets first non-null payment type from list of payment lines
         //default return value for getClaimPaymentType() is NULL
         if(typeOfPayment==null)  //TDP added to default to NON payment where type_of_payment is null
        	 typeOfPayment="NON";
         // workaround for miss-naming of
         // claim_payment_type.claim_payment_type_code (CHECK vis. CHK)
         // comes in from Vitria as CHK but is in the CLAIM_PAYMENT_TYPE table
         // as CHECK.
         if ("CHK".equals(typeOfPayment)) {
            typeOfPayment = "CHECK";
         }
         qcpt.setParameter("code", typeOfPayment);
         ClaimPaymentType claimPaymentType = null;
         try {
            claimPaymentType = (ClaimPaymentType) qcpt.getSingleResult();
         } catch (Exception e) {
            LOGGER.warn("unknown claim payment type code {} ", getClaimPaymentType(fcp));
         }

         // get the claim calculation method
         Query qcalc = entityManager.createQuery("select c from foCalcMethod c where c.calcMethodCd = :method ");
         qcalc.setParameter("method", CLAIM_ADJUDICATIONS_CALC_METHOD_CODE);
         CalcMethod calcMethod = (CalcMethod) qcalc.getSingleResult();

         // Get the Claim
         Query qclm = entityManager.createQuery("select c from foClaim c where c.claimIndex = :index ");
         qclm.setParameter("index", fcp.getClaimIndex());
         Claim claim = (Claim) qclm.getSingleResult();

if(DEBUG) {
	System.out.println("recordPayments() - claim information is...............");
	System.out.println("recordPayments() - claim key is: " + claim.getClaimIndex() );
	System.out.println("recordPayments() - claim edi key is: " + claim.getEdiClaimKey());
	System.out.println("recordPayments() - claim edi key LENGTH is: " + claim.getEdiClaimKey().length() );
	System.out.println("recordPayments() - claim bill type cd is: " + claim.getBillTypeCd() );
	// System.out.println("recordPayments() - claim facility cd is: " + claim.getVaFacility().toString() );
	VaFacility vaFac = claim.getVaFacility();
	System.out.println("recordPayments() - vaFacility code is: " + vaFac.getVaFacilityCd() );
	System.out.println("recordPayments() - vaFacility parent facility code is: " + vaFac.getParentVaFacilityCd() );
	VaFacilityType vaFacType = vaFac.getVaFacilityType();
	System.out.println("recordPayments() - vaFacility type code is: " + vaFacType.getVaTypeCd() );
	System.out.println("recordPayments() - claim service type code is: " + claim.getServiceType().getServiceTypeCd() );
	System.out.println("\n");
//	System.out.println("recordPayments() -vaFacility type " + vaFacType.
	
}
         
         
         // Get Claim Procedure status code of ACCEPTED
         Query qcps = entityManager.createQuery("select s from foClaimProcStatus s where s.claimProcStatusCd = :code ");
         qcps.setParameter("code", "ACCEPTED");
         ClaimProcStatus cps = (ClaimProcStatus) qcps.getSingleResult();

         // Create Claim Payment Line
         
// get ClaimPayment         
         ClaimPayment claimPayment = new ClaimPayment();
         claimPayment.setPaymentBatchDate(fcp.getFeeInvoiceDate());
         claimPayment.setTreasuryPaymentDate(getCheckDate(fcp));
         claimPayment.setPaymentTrackingNumber(getCheckNumber(fcp));
         claimPayment.setPaidAmount(getTotalLinePaidAmount(fcp));
         claimPayment.setFeeInvoiceNumber(fcp.getFeeInvoiceNumber());
         claimPayment.setClaimPaymentType(claimPaymentType); //set this value using what's retrieved from fcp lines
         claimPayment.setCreatedBy(username);
         claimPayment.setDateCreated(today);
         claimPayment.setClaim(claim);
         claim.getClaimPayments().add(claimPayment);

/**         
if(DEBUG) {
	System.out.println("claim payments for claim # " + claim.getClaimIndex() + " are:\n");
	Set cpmnts = claim.getClaimPayments();

	Iterator iter = cpmnts.iterator();
	while (iter.hasNext()) {
		ClaimPayment cp = (ClaimPayment) iter.next();
		System.out.println("recordPayments() - ClaimPaymentID for claim is: " + cp.getClaimPaymentId());
		System.out.println("recordPayments() - Tracking Number for CP is: " + cp.getPaymentTrackingNumber());
	}
}	// END DEBUG
**/
         

         // Create the Claim Reconciliation and Claim Reconciliation Adjustment
         // records
         ClaimReconciliation clmRecon = new ClaimReconciliation();
         clmRecon.setCalcMethod(calcMethod);
         clmRecon.setPaidAmount(getTotalLinePaidAmount(fcp));
         clmRecon.setDisbursedAmount(fcp.getDisbursedAmount());
         clmRecon.setInterestAmount(fcp.getInterestAmount());
         clmRecon.setCoveredDays(fcp.getCoveredDays());
         clmRecon.setDrgCd(drgcode);
         clmRecon.setDrgCodeDesc(drgdesc);
         clmRecon.setSent835Flag('N');
         clmRecon.setCreatedBy(username);
         
// set to yesterday to Stage will pick up the record............  
         clmRecon.setDateCreated(yesterday);
// ORIGINAL code         
//         clmRecon.setDateCreated(today);
         
         clmRecon.setClaim(claim);
         
if(DEBUG) {
	System.out.println("recordPayments() - ClaimRecon props - date created: " + clmRecon.getDateCreated());
	System.out.println("recordPayments() - ClaimRecon props - claim ID: " + claim.getClaimIndex());
	System.out.println("recordPayments() - ClaimRecon props - created by: " + username);
}
         
         
         
         
         claim.getClaimReconciliations().add(clmRecon);

// FeeClaimPaymentLines         
         Collection<FeeClaimPaymentLines> feePaymentLines = fcp.getFeeClaimPaymentLines();
 
// loop through FeeClaimPaymentLines         
         for (FeeClaimPaymentLines feePaymentLine : feePaymentLines) {
        	 
        	 if(DEBUG) {
        		 System.out.println("recordPayments() - in FeeClaimPaymentLines loop");
        		 System.out.println("recordPayments() - fcpLineID is: " + feePaymentLine.getFeeClaimPaymentLineId());
        		 System.out.println("recordPayments() - fcpLine remittance remarks is: " + feePaymentLine.getRemittanceRemarkCode() );
           	 }
 			            Collection<Long> seqs = getSeqAsLongs(feePaymentLine.getLineItemSequence());
			            ClaimProcRecon firstProcRecon = null;

// loop through procedures			            
			            for (Long seq : seqs) {
			            	
			            	if(DEBUG) {
			           		 System.out.println("recordPayments() - in ClaimProcRecon loop");
			           	 	}
			            	
   Query qclmproc = entityManager
         .createQuery("select cp from foClaimProcedure cp where cp.claim.claimIndex = :index and cp.claimProcedureSeq = :seq");
			               qclmproc.setParameter("index", claim.getClaimIndex());
			               qclmproc.setParameter("seq", seq);
			               
				               	               
			               
			               
if(DEBUG) {	
	System.out.println("recordPayments() - index and seq are: " + claim.getClaimIndex() + ", " + seq);
}
					ClaimProcedure clmProc = new ClaimProcedure();
					
					try {
							clmProc = (ClaimProcedure) qclmproc.getSingleResult();
							// clmProc = (ClaimProcedure) qclmproc.getResultList();
					}catch(NoResultException ex) {
						System.out.println("recordPayments() - NoResultException error with single clmProc: " + ex.getMessage() + ", " + claim.getClaimIndex());
						ex.printStackTrace();
					} catch(Exception ex) {
						System.out.println("recordPayments() - error with single clmProc: " + ex.getMessage() + ", " + claim.getClaimIndex());
						ex.printStackTrace();
					}
			               
   if(DEBUG) {
   		 System.out.println("recordPayments() - in ClaimProcRecon loop - ClaimProcedure is: " + clmProc.getClaimProcId() );
   }
			               
			               
			               
			               // first time through
			               if (firstProcRecon == null) {
// CREATE NEW  Claim Process Reconciliation
			            	  // set properties  
			                  ClaimProcRecon procRecon = new ClaimProcRecon();
			                  procRecon.setClaimProcStatus(cps);
			                  procRecon.setPaidAmount(feePaymentLine.getPaidAmount());
			                  char bundledFlag = isBundled(feePaymentLine) ? 'Y' : 'N';
			                  procRecon.setBundleFlag(bundledFlag);
			                  procRecon.setSent835Flag('N');
			                  procRecon.setCreatedBy(username);
			                  procRecon.setDateCreated(today);
			                  procRecon.setClaim(claim);
			                  
			                  claim.getClaimProcRecons().add(procRecon);
			                  
			                  procRecon.setClaimPayment(claimPayment);
			                  claimPayment.getClaimProcRecons().add(procRecon);
			                  procRecon.setClaimReconciliation(clmRecon);
			                  clmRecon.getClaimProcRecons().add(procRecon);
			                  procRecon.setClaimProcedure(clmProc);
			                  clmProc.getClaimProcRecons().add(procRecon);

			                  
if(DEBUG) {
	System.out.println("recordPayments() - in ClaimProcRecon loop - - setting claimProc line status to: " + COMPLETED_LINE_STATUS_CODE);
}

			                  clmProc.setLineStatus(COMPLETED_LINE_STATUS_CODE);
			
			                  // set this as the first reconciliation.
			                  firstProcRecon = procRecon;

// ADJUSTMENT AMOUNT 1			                  
			                  BigDecimal adjAmount1 = feePaymentLine.getAdjustmentAmount1();
			                  
if(DEBUG) {
	System.out.println("recordPayments() - adjAmt1 is: " + adjAmount1);
}
			                  
			                  if ((adjAmount1 != null) && (adjAmount1.doubleValue() > EPSILON)) {
			                     ClaimProcRecAdjust cpra1 = new ClaimProcRecAdjust();
			                     // Set adjustment group code
			                     Query qag1 = entityManager
			                           .createQuery("select ag from foAdjustmentGroup ag where ag.adjustmentGroupCd = :code");
			                     qag1.setParameter("code", feePaymentLine.getAdjustmentGroupCode1());
			                     
			                     
// ADJUSTMENT GROUP 1	
// NOTE: 10/26 - ALL Groups need to have same value as the spec. only has ouput for 1. OR the multiple
// GROUP (1-5) NEED TO BE DONE AWAY WITH and only have one Adjustment_Group field to hold the global
// value for all 5-6 fields. 			                     
			                     AdjustmentGroup ag1 = null;
			                     try {
			                        ag1 = (AdjustmentGroup) qag1.getSingleResult();
			                     } catch (PersistenceException e) {
			                        LOGGER.debug("Unable to find AdjustmentGroup record for {}", feePaymentLine.getAdjustmentGroupCode1());
			                     }

			                     if(DEBUG) {
			                    		System.out.println("recordPayments() - adjGroup1 is: " + ag1.getAdjustmentGroupCd());
			                    	}			                     
			                    		                     
			                     cpra1.setAdjustmentGroup(ag1);
			                     
// GET ADJUSTMENT REASON CODE 1                 
			                     // Set adjustment reason code
			                     Query qar1 = entityManager
			                           .createQuery("select ar from foAdjustmentReason ar where ar.adjustmentReasonCd = :code");
			                     qar1.setParameter("code", feePaymentLine.getAdjustmentReasonCode1());
			                     
			                     
			                      AdjustmentReason ar1 = null;
			                     try {
			                        ar1 = (AdjustmentReason) qar1.getSingleResult();
			                     } catch (PersistenceException e) {
			                        LOGGER.debug("Unable to find Adjustment Reson record for {}", feePaymentLine.getAdjustmentReasonCode1());
			                     }
			                     if(DEBUG) {
			                    		System.out.println("recordPayments() - adjReason1 is: " + ar1.getAdjustmentReasonCd());
			                    	}		                     
			                     
			                     cpra1.setAdjustmentReason(ar1);
			                     cpra1.setAdjustmentAmount(adjAmount1);
			                     cpra1.setCreatedBy(username);
			                     cpra1.setDateCreated(today);
			                     cpra1.setClaimProcRecon(procRecon);
			                     procRecon.getClaimProcRecAdjusts().add(cpra1);
			                     
 if(DEBUG) {
	 System.out.println("recordPayments() - new adjustment added to ClaimProcRecon #: " + procRecon.getClaimProcRecId());
 }
 
			                     
			                  }

			                  
// SET ADJUSTMENT AMOUNT 2			                  
			                  BigDecimal adjAmount2 = feePaymentLine.getAdjustmentAmount2();
			                  
			                  
  if(DEBUG) {
		System.out.println("recordPayments() - adjAmt2 is: " + adjAmount2);
}
			                					                  
			                  
			                  if ((adjAmount2 != null) && (adjAmount2.doubleValue() > EPSILON)) {
			                     ClaimProcRecAdjust cpra2 = new ClaimProcRecAdjust();
			                     // Set adjustment group code
			                     Query qag2 = entityManager
			                           .createQuery("select ag from foAdjustmentGroup ag where ag.adjustmentGroupCd = :code");
			                     qag2.setParameter("code", feePaymentLine.getAdjustmentGroupCode2());
			                     
			                     
// SET ADJUSTMENT GROUP 2			                     
			                     AdjustmentGroup ag2 = null;
			                     try {
			                        ag2 = (AdjustmentGroup) qag2.getSingleResult();
			                     } catch (PersistenceException e) {
			                        LOGGER.debug("Unable to find AdjustmentGroup record for {}", feePaymentLine.getAdjustmentGroupCode2());
			                     }
			                     
if(DEBUG) {
	System.out.println("recordPayments() - agcode_2 is: " + ag2.getAdjustmentGroupCd());
}			                     
			                     
			                     
			                     cpra2.setAdjustmentGroup(ag2);
			                     
// GET ADJUSTMENT REASON CODE 2			                     
			                     Query qar2 = entityManager
			                           .createQuery("select ar from foAdjustmentReason ar where ar.adjustmentReasonCd = :code");
			                     
                    
			                     qar2.setParameter("code", feePaymentLine.getAdjustmentReasonCode2());
			                     AdjustmentReason ar2 = null;
			                     
			                     
			                     try {
			                        ar2 = (AdjustmentReason) qar2.getSingleResult();
			                     } catch (PersistenceException e) {
			                        LOGGER.debug("Unable to find Adjustement Reson record for {}", feePaymentLine.getAdjustmentReasonCode2());
			                     }
			                     
			                     
 if(DEBUG) {
		System.out.println("recordPayments() - adjReason2 is: " + ar2.getAdjustmentReasonCd());
}			                     
			                     
			                     cpra2.setAdjustmentReason(ar2);
			                     cpra2.setAdjustmentAmount(adjAmount2);
			                     cpra2.setCreatedBy(username);
			                     cpra2.setDateCreated(today);
			                     cpra2.setClaimProcRecon(procRecon);
			                     procRecon.getClaimProcRecAdjusts().add(cpra2);
			                  }
// END ADJUSTMENT CODE 2

			                  
/** INSERT ITEMS 3 AND 4 HERE **/
			                  
  /** START OF ITEMS 3 **/
  
			               // SET ADJUSTMENT AMOUNT 3			                  
			                  BigDecimal adjAmount3 = feePaymentLine.getAdjustmentAmount3();
			                  
			                  
  if(DEBUG) {
		System.out.println("recordPayments() - adjAmt3 is: " + adjAmount3);
}
			                					                  
			                  
			                  if ((adjAmount3 != null) && (adjAmount3.doubleValue() > EPSILON)) {
			                     ClaimProcRecAdjust cpra3 = new ClaimProcRecAdjust();
			                     // Set adjustment group code
			                     Query qag3 = entityManager
			                           .createQuery("select ag from foAdjustmentGroup ag where ag.adjustmentGroupCd = :code");
			                     qag3.setParameter("code", feePaymentLine.getAdjustmentGroupCode3());
			                     
			                     
// SET ADJUSTMENT GROUP 3			                     
			                     AdjustmentGroup ag3 = null;
			                     try {
			                        ag3 = (AdjustmentGroup) qag3.getSingleResult();
			                     } catch (PersistenceException e) {
			                        LOGGER.debug("Unable to find AdjustmentGroup record for {}", feePaymentLine.getAdjustmentGroupCode3());
			                     }
			                     
if(DEBUG) {
	System.out.println("recordPayments() - agcode_3 is: " + ag3.getAdjustmentGroupCd());
}			                     
			                     
			                     
			                     cpra3.setAdjustmentGroup(ag3);
		                     
			                     
			                     
// GET ADJUSTMENT REASON CODE 3			                     
			                     Query qar3 = entityManager
			                           .createQuery("select ar from foAdjustmentReason ar where ar.adjustmentReasonCd = :code");
			                     
                    
			                     qar3.setParameter("code", feePaymentLine.getAdjustmentReasonCode3());
			                     AdjustmentReason ar3 = null;
			                     
			                     
			                     try {
			                        ar3 = (AdjustmentReason) qar3.getSingleResult();
			                     } catch (PersistenceException e) {
			                        LOGGER.debug("Unable to find Adjustement Reson record for {}", feePaymentLine.getAdjustmentReasonCode3());
			                     }
			                     
			                     
 if(DEBUG) {
		System.out.println("recordPayments() - adjReason3 is: " + ar3.getAdjustmentReasonCd());
}			                     
			                     
			                     cpra3.setAdjustmentReason(ar3);
			                     cpra3.setAdjustmentAmount(adjAmount3);
			                     cpra3.setCreatedBy(username);
			                     cpra3.setDateCreated(today);
			                     cpra3.setClaimProcRecon(procRecon);
			                     procRecon.getClaimProcRecAdjusts().add(cpra3);
			                  } 
			                  
  /** END OF ITEMS 3 **/				                  
			                  
			                  

/** START OF ITEMS 4 **/

				               // SET ADJUSTMENT AMOUNT 4			                  
			                  BigDecimal adjAmount4 = feePaymentLine.getAdjustmentAmount4();
			                  
			                  
  if(DEBUG) {
		System.out.println("recordPayments() - adjAmt4 is: " + adjAmount4);
}
			                					                  
			                  
			                  if ((adjAmount4 != null) && (adjAmount4.doubleValue() > EPSILON)) {
			                     ClaimProcRecAdjust cpra4 = new ClaimProcRecAdjust();
			                     // Set adjustment group code
			                     Query qag4 = entityManager
			                           .createQuery("select ag from foAdjustmentGroup ag where ag.adjustmentGroupCd = :code");
			                     qag4.setParameter("code", feePaymentLine.getAdjustmentGroupCode4());
			                     
			                     
// SET ADJUSTMENT GROUP 4			                     
			                     AdjustmentGroup ag4 = null;
			                     try {
			                        ag4 = (AdjustmentGroup) qag4.getSingleResult();
			                     } catch (PersistenceException e) {
			                        LOGGER.debug("Unable to find AdjustmentGroup record for {}", feePaymentLine.getAdjustmentGroupCode4());
			                     }
			                     
if(DEBUG) {
	System.out.println("recordPayments() - agcode_4 is: " + ag4.getAdjustmentGroupCd());
}			                     
			                     
			                     
			                     cpra4.setAdjustmentGroup(ag4);
		                     
			                     
			                     
// GET ADJUSTMENT REASON CODE 4			                     
			                     Query qar4 = entityManager
			                           .createQuery("select ar from foAdjustmentReason ar where ar.adjustmentReasonCd = :code");
			                     
                    
			                     qar4.setParameter("code", feePaymentLine.getAdjustmentReasonCode4());
			                     AdjustmentReason ar4 = null;
			                     
			                     
			                     try {
			                        ar4 = (AdjustmentReason) qar4.getSingleResult();
			                     } catch (PersistenceException e) {
			                        LOGGER.debug("Unable to find Adjustement Reson record for {}", feePaymentLine.getAdjustmentReasonCode4());
			                     }
			                     
			                     
 if(DEBUG) {
		System.out.println("recordPayments() - adjReason4 is: " + ar4.getAdjustmentReasonCd());
}			                     
			                     
			                     cpra4.setAdjustmentReason(ar4);
			                     cpra4.setAdjustmentAmount(adjAmount4);
			                     cpra4.setCreatedBy(username);
			                     cpra4.setDateCreated(today);
			                     cpra4.setClaimProcRecon(procRecon);
			                     procRecon.getClaimProcRecAdjusts().add(cpra4);
			                  } 
			                  			                  
			                  
/** END OF ITEMS 4 **/			                  
			                  
			                  
			                  
			                  
			                  
			                  
/** ENB OF ITEMS 3 AND 4 **/			                  
 
/** START OF ITEMS 5 **/	
	
			                  
			                  BigDecimal adjAmount5 = feePaymentLine.getAdjustmentAmount5();

			                  if(DEBUG) {
			              		System.out.println("recordPayments() - adjAmt5 is: " + adjAmount5);
			                  }
			                  
			                  if ((adjAmount5 != null) && (adjAmount5.doubleValue() > EPSILON)) {
				                     ClaimProcRecAdjust cpra5 = new ClaimProcRecAdjust();
				                     // Set adjustment group code
				                     Query qag5 = entityManager
				                           .createQuery("select ag from foAdjustmentGroup ag where ag.adjustmentGroupCd = :code");
				                     qag5.setParameter("code", feePaymentLine.getAdjustmentGroupCode5());
			                  
			                  
			                     Query qar5 = entityManager
			                           .createQuery("select ar from foAdjustmentReason ar where ar.adjustmentReasonCd = :code");
			                     
                 
			                     qar5.setParameter("code", feePaymentLine.getAdjustmentReasonCode5());
			                     AdjustmentReason ar5 = null;
			                     
			                     
// get group value
			                     AdjustmentGroup ag5 = null;
			                     try {
			                        ag5 = (AdjustmentGroup) qag5.getSingleResult();
			                     } catch (PersistenceException e) {
			                        LOGGER.debug("Unable to find AdjustmentGroup 5 record for {}", feePaymentLine.getAdjustmentGroupCode2());
			                     }
			                     
if(DEBUG) {
	System.out.println("recordPayments() - ag5 is: " + ag5.getAdjustmentGroupCd());
}			                     
			                     cpra5.setAdjustmentGroup(ag5);
			                     
			                     
// get reason value			                     
			                     try {
			                        ar5 = (AdjustmentReason) qar5.getSingleResult();
			                        
if(DEBUG) {
	 System.out.println("recordPayments() - adjReason5 value is: " + ar5.getAdjustmentReasonCd());
}
			                        
			                     } catch (PersistenceException e) {
			                    	 
 if(DEBUG) {
	 System.out.println("recordPayments() - error getting CARC 5 for fcpl#: " + feePaymentLine.getFeeClaimPaymentLineId());
 }
			                    	 
			                        LOGGER.debug("Unable to find Adjustement Reason 5 record for {}", feePaymentLine.getAdjustmentReasonCode5());
			                     }
			                     
			                     cpra5.setAdjustmentReason(ar5);
			                     cpra5.setAdjustmentAmount(adjAmount5);
			                     cpra5.setCreatedBy(username);
			                     cpra5.setDateCreated(today);
			                     cpra5.setClaimProcRecon(procRecon);
			                     procRecon.getClaimProcRecAdjusts().add(cpra5); 
			                     
			                     if(DEBUG) {
		                    		 System.out.println("recordPayments() - done updating adjustments for fcpl#: " + feePaymentLine.getFeeClaimPaymentLineId());
		                    		 System.out.println("recordPayments() - done updating adjustments for ClaimProcRecon#: " + procRecon.getClaimProcRecId());

		                    	 }
			                  }	// end if amount > 0 check   
// END OF CARC 5			                  
			                  
			                  
  // existing RARCS delimited string = MA125,N14
  // NOTE: THESE ARE TIED TO THE CLAIM_PROC_RECON table   
			     // String remittanceRemarkCode = feePaymentLine.getRemittanceRemarkCode();
			     // new method to gather up all remittanceRemarkCodes, comma-delimited
			     List<String> list = getAllRemittanceRemarks(feePaymentLine);
			     
			     //             String[] rrcs = new String[0];
			                  
			     //             if (remittanceRemarkCode != null) {
			     //                rrcs = remittanceRemarkCode.split(",");
			     //             }
			                  
			                  
			                  Query qrr = entityManager.createQuery(
			                        "select rr from foRemittanceRemark rr where rr.remittanceRemarkCd = :code");
			                  
			      // LOOP THROUGH LIST AND WRITE RECORDS.....            
			                  for (String rrc : list) {
			                	  
			                	  if(DEBUG) {
			                		  System.out.println("recordPayments() - RRemark is: " + rrc);
			                	  }
			                	  
			                     qrr.setParameter("code", rrc);
			                     
			                     RemittanceRemark rr = null;
// if remittance remarks are REPEATED there will be a database error - 
// same value can't be written twice.			                    
			                     try {
			                        rr = (RemittanceRemark) qrr.getSingleResult();
			                     } catch (Exception e) {
			                        LOGGER.debug("recordPayments() - unable to locate Remittance Remark with code of {} ",  rrc);
			                     }
			                     
			                     
			                     if (rr != null) {
			                        ClaimProcRemitRemark cprr = new ClaimProcRemitRemark();
			                        cprr.setRemittanceRemark(rr);
			                        cprr.setCreatedBy(username);
			                        cprr.setDateCreated(today);
			                        
			                        cprr.setClaimProcRecon(procRecon);
			                        
			                        // this is necessary for some strange reason - rr recs. NOT BEING written otherwise - BB - 10/20/2015 
			                        entityManager.persist(cprr);
			                        
			                        procRecon.getClaimProcRemitRemarks().add(cprr);
			                        entityManager.persist(procRecon);
if(DEBUG) {
	 System.out.println("recordPayments() - ClaimProcRemitRemark " + rr.getRemittanceRemarkCd() + " added to ClaimProcRecon: " + procRecon.getClaimProcRecId());
	 System.out.println("recordPayments() - ClaimProcRemitRemark count is: " + procRecon.getClaimProcRemitRemarks().size() );
 }
			                     }	// end if
			                     
entityManager.persist(procRecon);
			                  }	// end remittanceRemarks for loop 
			                  
			                  
			                  
			                  
			                  
			               } else {
			                  ClaimProcRecon procRecon = new ClaimProcRecon();
			                  procRecon.setPaidAmount(BigDecimal.ZERO);
			                  procRecon.setBundleFlag('Y');
			                  procRecon.setClaimProcStatus(cps);
			                  procRecon.setCreatedBy(username);
			                  procRecon.setDateCreated(today);
			                  procRecon.setClaim(claim);
			                  claim.getClaimProcRecons().add(procRecon);
			                  procRecon.setClaimProcRecon(firstProcRecon);
			                  firstProcRecon.getClaimProcRecons().add(procRecon);
			                  procRecon.setClaimProcedure(clmProc);
			                  clmProc.getClaimProcRecons().add(procRecon);
			               }
			            }
         }	// end for
       
         if(DEBUG) {
        	 System.out.println("recordPayments() - end of method - setting fcp.ResolutionStatusCode to RESOLUTION_STATUS_CODE_RESOLVE");
         } 
         
         
         // FINALLY SET FEE_CLAIM_PAYMENT RECORD TO 'RESOLVE' and write the object graph to DB
         fcp.setResolutionStatusCode(RESOLUTION_STATUS_CODE_RESOLVE);  // set to 'RESOLVE'
 
         
entityManager.flush();
         
         
         // final write - block this until all of the code is tested.

tx.commit();
         
         if(DEBUG) {
        	 System.out.println("recordPayments() - end of method - DONE - transaction committed");
         } 
          
      } catch (Exception e) {
         success = false;
         Throwable t = ExceptionUtils.getRootCause(e);
         LOGGER.warn("recordPayments() - Failure in reconciling payment {} due to {}", fcpId,
               (t == null) ? e.getMessage() : t.getMessage());

         if(DEBUG) {
        	 System.out.println("recordPayments() - failed update for fcpid: " + fcpId ); 
        	 System.out.println(e.getMessage());
        	 e.printStackTrace();
         }
         
         if (tx != null) {
            tx.rollback();
         }

         Helper.markAsFailure(fcpId, username, entityManager);
      }
      return success;
   }
   
   
    @SuppressWarnings("unchecked")
final void reconcileLineTypePayments() {
	
	boolean DEBUG = true;
	
    final int batchSize = 20;
    
      /**
     	Claims with a status of ACCEPTED are candidates for migration into the FPPS system (FPPS_OWNER schema).  
     	The FeeClaimLoad.java class 
		(EDI_PROCS_5010\src\main\gov\va\med\hac\edi\reposload\fee\FeeClaimLoad.java )
		calls the FPPS_OWNER.fpps_owner_load package and the start_load_process procedure to perform the actual 
		migration of E_REPOS claims into the FPPS system.  
     */
    

    // 9,193
    Query q = em.createNativeQuery("SELECT FEE_CLAIM_PAYMENT_ID FROM FEE_CLAIM_PAYMENTS "
                                 + "  WHERE RESOLUTION_STATUS_CODE =  'NEW' " 
                                 + "  AND TRANSACTION_TYPE_CODE = 'L' "
                                 + "  AND CLAIM_INDEX='671682'	" );
    
    
                                // + "  AND CLAIM_INDEX='9668'	" );
    // add this to query to get the only record that falls all the way through
    // FeeClaimPaymentID is: 71B401D0-83C8-DE22-E040-BF0A070C6F6C
    List<String> fcpIds = null;
    
    try {
     	fcpIds = q.getResultList();
     	
     	
     	 if(DEBUG) {
          	System.out.println("LineTypeReconciliation.reconcileLinePayments() - feeClaimPayments available: " + fcpIds.size());
         }
    } catch(Exception e) {
    	System.out.println("reconcileLineTypePayments() error getting FeeClaimPayments: " + e.getMessage());
    	e.printStackTrace();
    }
     
    
    Session session = (Session) em.getDelegate();
    int count = 0;
    
    for (String id : fcpIds) {
    	// get single payment
        Query q1 = em.createQuery("select f from foFeeClaimPayments f where f.feeClaimPaymentId = :id");
        q1.setParameter("id", id);
        FeeClaimPayments fcp = (FeeClaimPayments) q1.getSingleResult();
        
        
if(DEBUG) {
	System.out.println("reconcileLineTypePayments() - claim is: " + fcp.getClaimIndex() );
	System.out.println("reconcileLineTypePayments() - fcPayment id is: " + fcp.getFeeClaimPaymentId());
	System.out.println("reconcileLineTypePayments() - fcPayment resolution status code is: " + fcp.getResolutionStatusCode());
}
        
        if (containsValidSeqs(fcp)) {
         	
        	if(DEBUG) {
        		System.out.println("reconcileLineTypePayments() - claim is: " + fcp.getClaimIndex() + " containsValidSeqs" );
        		
        	}       	
        	
            Query q2 = em.createNativeQuery("select count(1) from claims c where claim_index = :index ");
            q2.setParameter("index", fcp.getClaimIndex());
            BigDecimal claimCount = (BigDecimal) q2.getSingleResult();
            
            
            
            // if 1 item found process it.
            if (claimCount.intValue() == 1) {
            	
            	
            	if(DEBUG) {
            		System.out.println("reconcileLineTypePayments() - claim count is 1");	//is: " + fcp.getClaimIndex() + "containsValidSeqs" );
            	}       	
                verifyAndReconcilePayment(fcp.getFeeClaimPaymentId(), user, em);
            } else {
                Helper.recordPaymentError(fcp.getFeeClaimPaymentId(), FAILED_TO_MATCH_CLAIM_ID, user, em);
                LOGGER.warn("FEE Claim Payment {} failed to match Claim with index {}."
                             , fcp.getFeeClaimPaymentId(), fcp.getClaimIndex());
            }
        } else {
            Helper.recordPaymentError(fcp.getFeeClaimPaymentId(), DATA_ERROR, user, em);
            System.out.println("FEE Claim Payment ERROR{} contains invalid sequence numbers: " + fcp.getFeeClaimPaymentId());
            
            LOGGER.warn("FEE Claim Payment {} contains invalid sequence numbers", fcp.getFeeClaimPaymentId());
        }
        
        count++;
        if (count % batchSize == 0) {
           session.flush();
           session.clear();
        }

    }

    LOGGER.info("{} Payments Reconciled", reconciledCounts);
    LOGGER.info("{} Payments with Mismatched Amounts", missmatchedAmtCounts);
    LOGGER.info("{} Payments with Unmatched Sequence Numbers", unmatchedSeqNumCounts);
    LOGGER.info("{} Payments that have been Previously Reconciled", previouslyReconCounts);
     
}



	@SuppressWarnings("unchecked")
	/**
    final void reconcileLineTypePayments() {
    	
    	boolean DEBUG = true;
    	
        final int batchSize = 20;
        Query q = em.createNativeQuery("SELECT FEE_CLAIM_PAYMENT_ID FROM FEE_CLAIM_PAYMENTS "
                                     + "  WHERE RESOLUTION_STATUS_CODE =  'NEW' " 
                                     + "  AND TRANSACTION_TYPE_CODE = 'L' ");
        List<String> fcpIds = q.getResultList();
        
        if(DEBUG) {
        	System.out.println("LineTypeReconciliation.reconcileLinePayments() - feeClaimPayments available: " + fcpIds.size());
        }
        // no records found - 6/25/2015
        
        Session session = (Session) em.getDelegate();
        int count = 0;
        
        for (String id : fcpIds) {
        	// get single payment
            Query q1 = em.createQuery("select f from foFeeClaimPayments f where f.feeClaimPaymentId = :id");
            q1.setParameter("id", id);
            FeeClaimPayments fcp = (FeeClaimPayments) q1.getSingleResult();
            
            // what is 'ValidSeqs'?
            if (containsValidSeqs(fcp)) {
                Query q2 = em.createNativeQuery("select count(1) from claims c where claim_index = :index ");
                q2.setParameter("index", fcp.getClaimIndex());
                BigDecimal claimCount = (BigDecimal) q2.getSingleResult();
                
                // if 1 item found process it.
                if (claimCount.intValue() == 1) {
                    verifyAndReconcilePayment(fcp.getFeeClaimPaymentId(), user, em);
                } else {
                    Helper.recordPaymentError(fcp.getFeeClaimPaymentId(), FAILED_TO_MATCH_CLAIM_ID, user, em);
                    LOGGER.warn("FEE Claim Payment {} failed to match Claim with index {}."
                                 , fcp.getFeeClaimPaymentId(), fcp.getClaimIndex());
                }
            } else {
                Helper.recordPaymentError(fcp.getFeeClaimPaymentId(), DATA_ERROR, user, em);
                LOGGER.warn("FEE Claim Payment {} contains invalid sequence numbers", fcp.getFeeClaimPaymentId());
            }
            
            count++;
            if (count % batchSize == 0) {
               session.flush();
               session.clear();
            }

        }

        LOGGER.info("{} Payments Reconciled", reconciledCounts);
        LOGGER.info("{} Payments with Mismatched Amounts", missmatchedAmtCounts);
        LOGGER.info("{} Payments with Unmatched Sequence Numbers", unmatchedSeqNumCounts);
        LOGGER.info("{} Payments that have been Previously Reconciled", previouslyReconCounts);
    }
**/
    
    
   static void verifyAndReconcilePayment(String fcpId, String username, EntityManager entitymanager) {
       Query q = entitymanager.createQuery("select f from foFeeClaimPayments f where f.feeClaimPaymentId = :id ");
       q.setParameter("id", fcpId);
       FeeClaimPayments fcp = (FeeClaimPayments) q.getSingleResult();
       Collection<FeeClaimPaymentLines> fcpls = fcp.getFeeClaimPaymentLines();
       
       
       if (fcpls.size() > 0) {
          reconcilePayments(fcp, username, entitymanager);
       } else {
          Helper.recordPaymentError(fcpId, FAILED_TO_MATCH_LINE_ID, username, entitymanager);
          LOGGER.warn("FEE Claim payment {} did not contain any payment lines. ", fcpId);
       }
   }

   
   
   static void reconcilePayments(FeeClaimPayments fcp, String username, EntityManager entityManager) {
	   
	   boolean DEBUG = true;
	   
	   if(DEBUG) {
		   System.out.println("reconcilePayments() - fcPaymentsLine collection size is: " + fcp.getFeeClaimPaymentLines().size());
	   }
	   
	   
      // the normalpayments, bundledpayments, and unbundledpayments are a buckets for sorting 
      // the line payments.  The keys are the line sequence numbers from the fee claim payment lines.
	  // need to get 'normal' 
      SortedMap<Long, Collection<FeeClaimPaymentLines>> normalpayments = new TreeMap<Long, Collection<FeeClaimPaymentLines>>();
      List<FeeClaimPaymentLines> bundledpayments = new LinkedList<FeeClaimPaymentLines>();
      SortedMap<Long, Collection<FeeClaimPaymentLines>> unbundledpayments = new TreeMap<Long, Collection<FeeClaimPaymentLines>>();
   
      
      
      
      // put each payment line into its appropriate bucket (normal, bundled, and unbundled)
      // the bundled maps are composed if the the claim procedure sequence Id (integer)
      // and a collection of the fee payment lines associated with that procedure.
      sortFeeClaimPaymentLines(fcp, normalpayments, bundledpayments, unbundledpayments);
      
      
   // after sort sizes are................ 
      if(DEBUG) {
    	  System.out.println("reconcilePayments() - normalpayments size is: " + normalpayments.size() );
    	  System.out.println("reconcilePayments() - bundledpayments list size is: " + bundledpayments.size() );
    	  System.out.println("reconcilePayments() - unbundledpayments Map size is: " + unbundledpayments.size() );
       }
      
      // records fail sort
      if (!isValidSort(normalpayments, bundledpayments, unbundledpayments, fcp)) {
         Helper.recordPaymentError(fcp.getFeeClaimPaymentId(), DATA_ERROR, username, entityManager);
         
         
         if(DEBUG) {
        	System.out.println("Recorded invalid sequence sort for fee claim payment {}: " + fcp.getFeeClaimPaymentId() + " EXITING METHOD");
         }
         
         LOGGER.debug("Recorded invalid sequence sort for fee claim payment {}", fcp.getFeeClaimPaymentId());
         return;
      }
      
      

      
      Query q = entityManager.createQuery("select c from foClaim c where c.claimIndex = :index ");
      q.setParameter("index", fcp.getClaimIndex());
      Claim claim = (Claim) q.getSingleResult();
      
      
      if(DEBUG) {
    		System.out.println("reconcilePayments() - SINGLE claim index is: " + claim.getClaimIndex());
    		System.out.println("reconcilePayments() - SINGLE claim FACILITY is: " + claim.getVaFacility().getVaFacilityCd().toString() );
     	}      
       
      
      // results will contain the outcome of the reconciliation process.
      LineReconciliationResults results = new LineReconciliationResults();
      
      
      // processes Fee_Claim_Payment_Lines 
      reconcileNormalPayments(claim, normalpayments, results);
      
      reconcileBundledPayments(claim, bundledpayments, results);
      
      reconcileUnbundledPayments(claim, unbundledpayments, results);
      
      //remove any reconciled payments if there has been one payment line in error
      results.normalize();
      
      enterPaymentErrors(results.unmatchedSequenceNumbers, FAILED_TO_MATCH_LINE_ID, username, entityManager);
      
      enterPaymentErrors(results.previouslyReconciledPayments, DUPLICATE_LINE_ADJUDICATION, username, entityManager);
      
      enterPaymentErrors(results.missmatchedAmtPayments, FAILED_TO_MATCH_AMOUNT, username, entityManager);
      
      
      // WRITE FEE_CLAIM_PAYMENT_LINES STARTS HERE............
      enterClaimPayments(results.reconciledPayments, username, entityManager);

      reconciledCounts += results.reconciledPayments.size();
      missmatchedAmtCounts += results.missmatchedAmtPayments.size();
      unmatchedSeqNumCounts += results.unmatchedSequenceNumbers.size();
      previouslyReconCounts += results.previouslyReconciledPayments.size();

      LOGGER.debug("Reconciled Payment {}", fcp.getFeeClaimPaymentId());
      
      if(DEBUG) {
    	  System.out.println("reconcilePaments() - Reconciled Payment {}: " + fcp.getFeeClaimPaymentId());
      }
      

   }	// end reconcilePaments()

   
   
   static void reconcileNormalPayments(Claim claim
         , SortedMap<Long, Collection<FeeClaimPaymentLines>> normalpayments
         , LineReconciliationResults results) {
	   
	  boolean DEBUG = true;
			  
      Collection<ClaimProcedure> claimProcedures = claim.getClaimProcedures();
   
      if(DEBUG) {
    	  System.out.println("reconcileNormalPayments() - claimProcedures count is: " + claimProcedures.size() );
      }
      
      // if it's not in here, it's not a 'normal payment'...........
      Set<Long> seqs = normalpayments.keySet();
       
      for (Long seq : seqs) { 
     	  
         boolean matchedSeq = false;
         
         if(DEBUG) {
        	 System.out.println("reconcileNormalPayments() - seq is: " + seq.intValue() );
         }
         
         FeeClaimPaymentLines fcpl = normalpayments.get(seq).iterator().next();
         
         
         // HANDLE PROCEDURES
        
				         for (ClaimProcedure procedure : claimProcedures) {
				            long s = procedure.getClaimProcedureSeq();
				            long procID = procedure.getClaimProcId();
				            
				            
				            if(DEBUG) {
				            	System.out.println("reconcileNormalPayments() - ClaimProcedureId is: " + procID);
				            	System.out.println("reconcileNormalPayments() - ClaimProcedureSeq is: " + s);
				            	System.out.println("reconcileNormalPayments() - seq int value is: " + seq.intValue());
				            }				            
				            
				            if (s == seq.intValue()) {
						            	
						            	if(DEBUG) {
						            		System.out.println("reconcileNormalPayments() - matched sequence");
						            	}
						            	
						               matchedSeq = true;
						               
		// WHAT IS THIS CHECK?				               
						if (notPreviouslyReconciled(procedure)) {
						            	   
						                  BigDecimal paidAmount = fcpl.getPaidAmount() != null ? fcpl.getPaidAmount() : BigDecimal.ZERO;
						                  
						                  if(DEBUG) {
						                	  //  System.out.println("");
												System.out.println("reconcileNormalPayments() - NotPreviouslyReconciled - fcpl paidAmount1 is: " + paidAmount);
											}
						                  
						                  // HANDLE ADJUSTMENTS
						                  BigDecimal adjAmount1 = fcpl.getAdjustmentAmount1() != null ? fcpl.getAdjustmentAmount1() : BigDecimal.ZERO;
												if(DEBUG) {
													System.out.println("reconcileNormalPayments() - fcpl adjAmount1 is: " + adjAmount1);
												}
												
												BigDecimal adjAmount2 = fcpl.getAdjustmentAmount2() != null ? fcpl.getAdjustmentAmount2() : BigDecimal.ZERO;
												if(DEBUG) {
													System.out.println("reconcileNormalPayments() - fcpl adjAmount2 is: " + adjAmount2);
												}
												
						// ADD ITEMS 3 AND 4
								                 BigDecimal adjAmount3 = fcpl.getAdjustmentAmount3() != null ? fcpl.getAdjustmentAmount3() : BigDecimal.ZERO;
						
												
												if(DEBUG) {
													System.out.println("reconcileNormalPayments() - fcpl adjAmount3 is: " + adjAmount3);
												}						
												
												 BigDecimal adjAmount4 = fcpl.getAdjustmentAmount4() != null ? fcpl.getAdjustmentAmount4() : BigDecimal.ZERO;
						
													
												if(DEBUG) {
													System.out.println("reconcileNormalPayments() - fcpl adjAmount4 is: " + adjAmount4);
												}	
												
						// END ITEMS 3 AND 4						
												                  
												BigDecimal adjAmount5 = fcpl.getAdjustmentAmount5() != null ? fcpl.getAdjustmentAmount5() : BigDecimal.ZERO;
												if(DEBUG) {
													System.out.println("reconcileNormalPayments() - fcpl adjAmount5 is: " + adjAmount5);
												}
							                  
																		
												BigDecimal totalPaid = paidAmount.add(adjAmount1).add(adjAmount2).add(adjAmount3).add(adjAmount4).add(adjAmount5);
												                  
												if(DEBUG) {
													System.out.println("reconcileNormalPayments() - fcpl(s) totalPaid is: " + totalPaid);
												  }
												
												                  // from claim procedure table - not FCPL              
												 BigDecimal billedAmnt = procedure.getBilledAmount() != null ? procedure.getBilledAmount() : BigDecimal.ZERO;
												                  
												                  
						                  if(DEBUG) {
												System.out.println("reconcileNormalPayments() - PROCEDURE #: " + procedure.getClaimProcId() + " billedAmt is: " + billedAmnt);
										  }
						                  
						                  
						                  
						                  BigDecimal diff = billedAmnt.subtract(totalPaid).abs();
						                  
						                  
						                  if(DEBUG) {
												System.out.println("reconcileNormalPayments() - diff is: " + diff);
										  }
						                  
						                  
						                  // IF ITS LET THAN .50 reconcile it, otherwise add to errors.....
						                  if (diff.doubleValue() < EPSILON) {
						                	  
						                	 if(DEBUG) {
						                		 System.out.println("reconcileNormalPayments() adding to reconciled payments");
						                	 }
						                     results.reconciledPayments.add(fcpl.getFeeClaimPayments().getFeeClaimPaymentId());
						                     
						                     
						                  } else {
						                	  
						                	  if(DEBUG) {
							                		 System.out.println("reconcileNormalPayments() FAILED - adding to errors");
							                	 }
						                	  
						                     addToErrors(results.missmatchedAmtPayments, fcpl, results.reconciledPayments);
						                  }
						                  
						                  break;
							                  
					 } else {
        	   
        	   if(DEBUG) {
        		    System.out.println("reconcileNormalPayments() - notPreviouslyReconciled() - adding to errors");
        		    
					System.out.println("reconcileNormalPayments() - error with sequence - calling addToErrors() for fcpl # " + fcpl.getFeeClaimPaymentLineId());
        	   }	
						            	   
						                  addToErrors(results.previouslyReconciledPayments, fcpl, results.reconciledPayments);
						               }
						               
						               
						            }	// END matched sequence test
						            
						            
				         }	// END FOR
         
        
         if (!matchedSeq) {
        	 
        	
        		 if(DEBUG) {
         		    System.out.println("reconcileNormalPayments() - !matchedSeq - adding to errors" );
         	
        		 }
             addToErrors(results.unmatchedSequenceNumbers, fcpl, results.reconciledPayments);
         }	// end matchedSeq
         
      }            
      
   }	// end reconcileNormalPayments()

   
   
   static void addToErrors(Map<String, Collection<FeeClaimPaymentLines>> errors, FeeClaimPaymentLines fcpl, Collection<String> reconciled) {
      String fcpId = fcpl.getFeeClaimPayments().getFeeClaimPaymentId();
      
      
      if (errors.containsKey(fcpId)) {
         errors.get(fcpId).add(fcpl);
      } else {
         Collection<FeeClaimPaymentLines> c = new LinkedList<FeeClaimPaymentLines>();
         c.add(fcpl);
         errors.put(fcpId, c);
      }
      
      
      
   }
   
   
   
   static void reconcileBundledPayments(Claim claim
         , List<FeeClaimPaymentLines> bundledpayments
         , LineReconciliationResults results) {
      Collection<ClaimProcedure> claimProcedures = claim.getClaimProcedures();
      for (FeeClaimPaymentLines payment : bundledpayments) {
         BigDecimal paidAmount = payment.getPaidAmount() != null ? payment.getPaidAmount() : BigDecimal.ZERO;
         BigDecimal adjAmount1 = payment.getAdjustmentAmount1() != null ? payment.getAdjustmentAmount1() : BigDecimal.ZERO;
         BigDecimal adjAmount2 = payment.getAdjustmentAmount2() != null ? payment.getAdjustmentAmount2() : BigDecimal.ZERO;
         BigDecimal totalPaid = paidAmount.add(adjAmount1).add(adjAmount2);
         BigDecimal billedAmount = BigDecimal.ZERO;
         int reconciledSegs = 0;
         int previouslyReconciled = 0;
         String[] seqs = payment.getLineItemSequence().split(",");
         for (String seqStr : seqs) {
            int seqint = Integer.parseInt(seqStr);
            for (ClaimProcedure procedure : claimProcedures) {
               long s = procedure.getClaimProcedureSeq();
               if (s == seqint) {
                  if (notPreviouslyReconciled(procedure)) {
                     if (procedure.getBilledAmount() != null) {
                        billedAmount = billedAmount.add(procedure.getBilledAmount());
                        reconciledSegs++;
                     }
                  } else {
                     previouslyReconciled++;
                  }
               }
            }
         }
         
         if (reconciledSegs == seqs.length) {       
            // all bundled sequence numbers were matched to
            // lines in the claim.
            BigDecimal diff = totalPaid.subtract(billedAmount).abs();
            if (diff.longValue() < EPSILON) {
                results.reconciledPayments.add(payment.getFeeClaimPayments().getFeeClaimPaymentId());
            } else {
               addToErrors(results.missmatchedAmtPayments, payment, results.reconciledPayments);
            }
         } else if (previouslyReconciled == seqs.length) {
            addToErrors(results.previouslyReconciledPayments, payment, results.reconciledPayments);
         } else { // Unmatched sequences takes priority over previously Reconciled errors.
            addToErrors(results.unmatchedSequenceNumbers, payment, results.reconciledPayments);
         }
      }
   }

   
   /** start here **/
   static void reconcileUnbundledPayments(Claim claim
        , SortedMap<Long, Collection<FeeClaimPaymentLines>> unbundledpayments
        , LineReconciliationResults results) {
	   
	   
	   boolean DEBUG = true;
	   
	   if(DEBUG) {
		   System.out.println("reconcileUnbundledPayments():");
		   System.out.println("\tClaim is: " + claim.getClaimIndex() );
		   System.out.println("\tUnbundledpayments size is: " + unbundledpayments.size() );
		   System.out.println("\tresults size is: " + results.reconciledPayments.size() );
	   }
	   
	   
	   
      Collection<Long> seqs = unbundledpayments.keySet();
      // key: claim proc Seq , value: collection of Fee claim payments associated with the claim procedure.
      
        
      Collection<IntermResult> tmpMatched = new LinkedList<IntermResult>();
      
      
      // what is the significance of seqs???
      for (Long seq : seqs) {
    	  
    	 if(DEBUG) {
    		 System.out.println("\treconcileUnbundledPayments() - Claim is: " + claim.getClaimIndex() );
    		 System.out.println("\treconcileUnbundledPayments() - FeeClaimPaymentLine seq is: " + seq); 
   	 }
    	  
         boolean matched = false;
         boolean prevRecon = false;
         
         
         Collection<ClaimProcedure> procedures = claim.getClaimProcedures();
         
         if(DEBUG) {
        	 System.out.println("reconcileUnbundledPayments() - claimProcedures size is: " + procedures.size());
         }
         
         
         // get procedures............99 records for claim # 536866
         int count = 0;
         for (ClaimProcedure procedure : procedures) {
        	count++;
        	
        	if(DEBUG) {
        		System.out.println("reconcileUnbundledPayments() - processing procedures count is: " + count);
        	}
        	
        	
            long s = procedure.getClaimProcedureSeq();
            
        	if(DEBUG) {
        		System.out.println("reconcileUnbundledPayments() - ClaimProcedureSeq is: " + s);
        	}
            
              
            if (s == seq.intValue()) {
               matched = true;
               
  // get valid values and update DB so more records can be used.             
               if(DEBUG) {
           			System.out.println("reconcileUnbundledPayments() - s IS EQUAL seq.intValue()");
           			System.out.println("\treconcileUnbundledPayments() - Claim is: " + claim.getClaimIndex() );
          			System.out.println("\treconcileUnbundledPayments() - procedure is: " + procedure.getClaimProcId() );
           			System.out.println("\treconcileUnbundledPayments() - FeeClaimPaymentLine seq is: " + seq); 
           			// System.out.println("\treconcileUnbundledPayments() - s == " + s);
           			System.out.println("\treconcileUnbundledPayments() - seq.intValue() is: " + seq.intValue());
            			// System.out.println("\treconcileUnbundledPayments() - is: " + procedure.
           	    }
               
               
               if (notPreviouslyReconciled(procedure)) {

            	 if(DEBUG)  { 	
            	   System.out.println("reconcileUnbundledPayments() - notPreviouslyReconciled procedure: " + procedure.getClaimProcId());
            	 } 
            	   
                  IntermResult r = new IntermResult();
                  r.payments = unbundledpayments.get(seq);
                  r.procedure = procedure;
                  tmpMatched.add(r);
               } else {
            	   
                  prevRecon = true;

               }
            } else {
            	
            	if(DEBUG) {
            	//	System.out.println("reconcileUnbundledPayments() - s != seq.intValue()");
            	}
            	
            	
            }
         }
         
         
         if (matched && !prevRecon) {
        	 
        	 if(DEBUG) {
         		System.out.println("reconcileUnbundledPayments() - matched && !prevRecon");
         	  }
        	 
        	 
        	 
            for (IntermResult r : tmpMatched) {
            	
            	if(DEBUG) {
             		System.out.println("reconcileUnbundledPayments() - matched && !prevRecon loop - r value is: " + tmpMatched.toString());
             	  }
            	
            	
               BigDecimal billedAmount = r.procedure.getBilledAmount() != null ? r.procedure.getBilledAmount() : BigDecimal.ZERO; 
               
               if(DEBUG) {
            		System.out.println("reconcileUnbundledPayments() - matched && !prevRecon loop - billedAmount is: " + billedAmount);
               }
               
               
               BigDecimal paidAmount = BigDecimal.ZERO;
               BigDecimal adjAmount1 = BigDecimal.ZERO;
               BigDecimal adjAmount2 = BigDecimal.ZERO;
               BigDecimal adjAmount3 = BigDecimal.ZERO;
               BigDecimal adjAmount4 = BigDecimal.ZERO;
               BigDecimal adjAmount5 = BigDecimal.ZERO;
               
               
               for (FeeClaimPaymentLines mfcpl : r.payments) {
            	   
            	  if(DEBUG) {
               		System.out.println("reconcileUnbundledPayments() - matched && !prevRecon loop - FeeClaimPaymentLines loop - line value size is: " + r.payments.size());
                  } 
            	   
            	   
                  if (mfcpl.getPaidAmount() != null) {
                     paidAmount = paidAmount.add(mfcpl.getPaidAmount());
                  }
                  if (mfcpl.getAdjustmentAmount1() != null) {
                     adjAmount1 = adjAmount1.add(mfcpl.getAdjustmentAmount1());
                  }
                  if (mfcpl.getAdjustmentAmount2() != null) {
                     adjAmount2 = adjAmount2.add(mfcpl.getAdjustmentAmount2());
                  }
                  
                  
// HERE - ADD AMOUNTS 3 & 4                  
                  if (mfcpl.getAdjustmentAmount3() != null) {
                      adjAmount5 = adjAmount3.add(mfcpl.getAdjustmentAmount3());
                   }
                  
                   if (mfcpl.getAdjustmentAmount4() != null) {
                      adjAmount4 = adjAmount4.add(mfcpl.getAdjustmentAmount4());                   }
                  
                  
                  if (mfcpl.getAdjustmentAmount5() != null) {
                      adjAmount5 = adjAmount5.add(mfcpl.getAdjustmentAmount5());
                   }
              }	// end for
               
               
               
               BigDecimal totalPaid = paidAmount.add(adjAmount1).add(adjAmount2).add(adjAmount3).add(adjAmount4).add(adjAmount5);
               BigDecimal diff = billedAmount.subtract(totalPaid).abs();
               if (diff.longValue() < EPSILON) {
                  Collection<FeeClaimPaymentLines> c = unbundledpayments.get(seq);
                  for (FeeClaimPaymentLines fcpl : c) {
                     results.reconciledPayments.add(fcpl.getFeeClaimPayments().getFeeClaimPaymentId());
                  }               
               } else {
                  Collection<FeeClaimPaymentLines> c = unbundledpayments.get(seq);
                  for (FeeClaimPaymentLines fcpl : c) {
                      addToErrors(results.missmatchedAmtPayments, fcpl, results.reconciledPayments);
                  }                  
               }
            }
         } else if (matched && prevRecon) {
        	 if(DEBUG) {
          		System.out.println("reconcileUnbundledPayments() - matched && prevRecon");
          	  }
         	 
        	 
            Collection<FeeClaimPaymentLines> c = unbundledpayments.get(seq);
            
            for (FeeClaimPaymentLines fcpl : c) {
            	if(DEBUG) {
              		System.out.println("reconcileUnbundledPayments() - matched && prevRecon adding to errors");
              	 }
               addToErrors(results.previouslyReconciledPayments, fcpl, results.reconciledPayments);
               
            }  
            
            
            
         } else if (!matched && !prevRecon) {
        	 
        	 if(DEBUG) {
          		System.out.println("reconcileUnbundledPayments() - !matched && !prevRecon");
          	  }
         	 
        	 
            Collection<FeeClaimPaymentLines> c = unbundledpayments.get(seq);
            
            if(DEBUG) {
          		System.out.println("reconcileUnbundledPayments() - !matched && !prevRecon - unbundledpayments collection size is: " + c.size());
          	  }
            
            
            
            for (FeeClaimPaymentLines fcpl : c) {
            	
            	if(DEBUG) {
              		System.out.println("reconcileUnbundledPayments() - !matched && !prevRecon - adding to errors");
              	  }
            	
            	
               addToErrors(results.unmatchedSequenceNumbers, fcpl, results.reconciledPayments);
            }
         } 
      }
   }	

   
   
   static void enterPaymentErrors(Map<String, Collection<FeeClaimPaymentLines>> errors
                                  , String errorCode
                                  , String username
                                  , EntityManager entityManager) {
      Collection<String> fcpIds = errors.keySet();
      
      
      for (String fcpId : fcpIds) {
         Collection<FeeClaimPaymentLines> lineErrors = errors.get(fcpId);
         Helper.recordPaymentLineErrors(fcpId, lineErrors, errorCode, username, entityManager);
      }
   }
   
   
   

   
   
   static boolean notPreviouslyReconciled(ClaimProcedure clmProc) {
	   
	  boolean DEBUG = true;
	  
		  
	  // if there are reconciliations for this procedure, don't do it again........
      boolean rslt = clmProc.getClaimProcRecons().size() == 0;
      
      
      
      if(DEBUG) {
     	  if(clmProc.getClaimProcRecons().size() == 1) {
    		  long cprId = clmProc.getClaimProcRecons().iterator().next().getClaimProcRecId();
    		  System.out.println("notPreviouslyReconciled() - single claimProcRecon id is: " + cprId);
    	  }
     	  
     	  System.out.println("notPreviouslyReconciled() - ClaimProcedure is: " + clmProc.getClaimProcId());
		  System.out.println("notPreviouslyReconciled() - ClaimProcedure CREATED_BY is: " + clmProc.getCreatedBy());
		  System.out.println("notPreviouslyReconciled() - ClaimProcedure Seq is: " + clmProc.getClaimProcedureSeq());
		  System.out.println("notPreviouslyReconciled() - ClaimProcRecons are: " + clmProc.getClaimProcRecons().size());
		  System.out.println("notPreviouslyReconciled() - Previously Reconciled is: " + rslt);
      }	// END DEBUG
      
      
      LOGGER.debug("Claim procedure {} {} previously reconciled.", clmProc.getClaimProcId(), (rslt ? "was not" : "was"));
      
if(DEBUG) {
//	System.out.println("notPreviouslyReconciled - RETURNING TRUE REGARDLESS......");
	rslt=true;
}

      return rslt;
   }

   
   
   static boolean isValidSort(SortedMap<Long, Collection<FeeClaimPaymentLines>> normalpayments 
         , Collection<FeeClaimPaymentLines> bundled
         , SortedMap<Long, Collection<FeeClaimPaymentLines>> unbundledpayments
         , FeeClaimPayments fcp) {
	   
	   
	  boolean DEBUG = true;
	  
      boolean rslt = true;
      rslt = totalLineCountMatched(normalpayments, bundled, unbundledpayments, fcp) 
             & noDupBundledSeq(bundled, fcp) 
             & noDupNormalSeq(normalpayments, unbundledpayments, fcp)
             & noDupNormUnbundBundled(normalpayments, bundled, unbundledpayments, fcp);
      LOGGER.debug("Sort for fee claim payment {} {} valid", fcp.getFeeClaimPaymentId(), (rslt ? "is" : "is not"));
      
 if(DEBUG) {
	 System.out.println("isValidSort() - correct sort value is: " + rslt + " for ");
 }
	 
      
      
      
if(DEBUG)    {
//	rslt = true;
}
      
      
      return rslt;
   }	// end isValidSort()

   
   
   static boolean totalLineCountMatched(SortedMap<Long, Collection<FeeClaimPaymentLines>> normalpayments
         , Collection<FeeClaimPaymentLines> bundled
         , SortedMap<Long, Collection<FeeClaimPaymentLines>> unbundledpayments
         , FeeClaimPayments fcp) {
	   
	   
	  boolean DEBUG = true;  
	   
      boolean rslt = true;
      
      
      Collection<FeeClaimPaymentLines> fcpls = fcp.getFeeClaimPaymentLines();
      int totalLineCount = fcpls.size();
      int normalLineCount = normalpayments.size();
      int bundledLineCount = bundled.size();
      int unbundledLineCount = 0;
      
    
      
      for (Collection<FeeClaimPaymentLines> c : unbundledpayments.values()) {
         unbundledLineCount += c.size();
      }
      
      
      
     
      
      
      int sortedLineCount = normalLineCount + bundledLineCount + unbundledLineCount;
      
      
      rslt = (totalLineCount == sortedLineCount);
      
      
      if(DEBUG) {
    	  System.out.println("totalLineCountMatched() - totalLineCount: " + totalLineCount);
    	  System.out.println("totalLineCountMatched() - normalLineCount: " + normalLineCount);
    	  System.out.println("totalLineCountMatched() - bundlesLineCount: " + bundledLineCount);
    	  System.out.println("totalLineCountMatched() - unbundledLineCount: " + unbundledLineCount);
    	  System.out.println("totalLineCountMatched() - sortedLineCount: " + sortedLineCount);
    	  System.out.println("totalLineCountMatched() - result is: " + rslt);
      }
      
      
      
      if (LOGGER.isDebugEnabled()) {
         LOGGER.debug("Claim Payment Id (" + fcp.getFeeClaimPaymentId() + ") sorted line count " + (rslt ? "did" : "did not") + " match"
            + " received line count. Total Lines " + totalLineCount + ", Normal Lines " + normalLineCount
            + ", Bundled Lines " + bundledLineCount + ",  Unbundled Lines " + unbundledLineCount);
      }
      
      
      return rslt;
   }

   
   
   static boolean noDupBundledSeq(Collection<FeeClaimPaymentLines> bundled, FeeClaimPayments fcp) {
      boolean rslt = true;
      Collection<String> tmp = new LinkedList<String>();
      for (FeeClaimPaymentLines fcpl : bundled) {
         String[] seqs = fcpl.getLineItemSequence().split(",");
         for (String seq : seqs) {
            if (tmp.contains(seq)) {
               rslt = false;
               LOGGER.debug("FEE Claim Payment {}: duplicate bundled - bundled sequence {}", fcp.getFeeClaimPaymentId(), seq);
            } else {
               tmp.add(seq);
            }
         }
      }
      return rslt;
   }

   
   
   static boolean noDupNormalSeq(SortedMap<Long, Collection<FeeClaimPaymentLines>> normalpayments
         , SortedMap<Long, Collection<FeeClaimPaymentLines>> unbundledpayments
         , FeeClaimPayments fcp) {
      boolean rslt = true;
      Set<Long> normalSeqs = normalpayments.keySet();
      Set<Long> unbundledSeq = unbundledpayments.keySet();
      for (Long seq : unbundledSeq) {
         if (normalSeqs.contains(seq)) {
            rslt = false;
            LOGGER.debug("FEE claim Payment {}: duplicate normal - unbundled sequence {}", fcp.getFeeClaimPaymentId(), seq);
         }
      }
      return rslt;
   }
   
   
   
   static boolean noDupNormUnbundBundled(SortedMap<Long, Collection<FeeClaimPaymentLines>> normalpayments
         , Collection<FeeClaimPaymentLines> bundled
         , SortedMap<Long, Collection<FeeClaimPaymentLines>> unbundledpayments
         , FeeClaimPayments fcp) {
      boolean rslt = true;
      Collection<Long> normalseqs = normalpayments.keySet();
      Collection<Long> unbundledseqs = unbundledpayments.keySet();
      Collection<Long> bundledseqs = getBundledSeqs(bundled);
      for (Long seq : normalseqs) {
         if (bundledseqs.contains(seq)) {
            rslt =  false;
            LOGGER.debug("Fee Claim Payment {} Normal seq {} found in bundled sequences", fcp.getFeeClaimPaymentId(), seq);
         }
      }
      
      for (Long seq : unbundledseqs) {
         if (bundledseqs.contains(seq)) {
            rslt = false;
            LOGGER.debug("Fee Claim payment {} Unbundled seq {} found in bundled sequences", fcp.getFeeClaimPaymentId(), seq);
         }
      }
      return rslt;
   }
   
   
   
   static void sortFeeClaimPaymentLines(FeeClaimPayments fcp
         , SortedMap<Long, Collection<FeeClaimPaymentLines>> normalpayments 
         , List<FeeClaimPaymentLines> bundledpayments
         , SortedMap<Long, Collection<FeeClaimPaymentLines>> unbundledpayments) {
      Collection<FeeClaimPaymentLines> feeClaimPaymentLines = fcp.getFeeClaimPaymentLines();
      for (FeeClaimPaymentLines feeClaimPaymentLine : feeClaimPaymentLines) {
         if (isBundled(feeClaimPaymentLine)) {
            bundledpayments.add(feeClaimPaymentLine);
         } else {
            Long key = Long.parseLong(feeClaimPaymentLine.getLineItemSequence());
            if (!normalpayments.containsKey(key)) {
               Collection<FeeClaimPaymentLines> coll = new LinkedList<FeeClaimPaymentLines>();
               coll.add(feeClaimPaymentLine);
               normalpayments.put(key, coll);
            } else {
               // Multiple adjudication attempts on a normal line indicate
               // an unbundled line.
               Collection<FeeClaimPaymentLines> coll = normalpayments.remove(key);
               coll.add(feeClaimPaymentLine);
               unbundledpayments.put(key, coll);
            }
         }   
      }
   }
   
   
  // empty collection passed  
   static void enterClaimPayments(Set<String> fcpIds, String username, EntityManager entityManager) {
	   
	   
	   boolean DEBUG = true;
	   
if(DEBUG) {
	System.out.println("enterClaimPayments() - fcpIds size is: " + fcpIds.size());
}

	      for (String fcpId : fcpIds) {
	    	  
	   	 
if(DEBUG) {
		System.out.println("enterClaimPayments() - calling recordPayments()");
}
	    				    	  
	    	  
	         boolean paymentRecorded = recordPayments(fcpId, username, entityManager);
	         
 if(DEBUG) {
	System.out.println("enterClaimPayments() -  done recordPayments()");
 }	         
         
	         if (paymentRecorded) {
	            completeClaim(fcpId, username, entityManager);
	         }
	         
	         
	      }
   }	// end enterClaimPayments()
   
   
   
   static void completeClaim(String fcpId, String username, EntityManager entityManager) {
      Query qclmId = entityManager.createQuery("select fcp.claimIndex from foFeeClaimPayments fcp where fcp.feeClaimPaymentId = :id ");
      qclmId.setParameter("id", fcpId);
      
      Long clmId = (Long) qclmId.getSingleResult();
      Helper.closeIfClaimComplete(clmId, username, entityManager);
   }

   
   


   
   static BigDecimal getTotalLinePaidAmount(FeeClaimPayments fcp) {
       BigDecimal retVal = BigDecimal.ZERO;
       Collection<FeeClaimPaymentLines> payments = fcp.getFeeClaimPaymentLines();
       for (FeeClaimPaymentLines payment : payments) {
           if (payment.getPaidAmount() != null) {
               retVal = retVal.add(payment.getPaidAmount());
           }
       }
       return retVal;
   }
   
   
   
   static String getCheckNumber(FeeClaimPayments fcp) {
       String retVal = null;
       Collection<FeeClaimPaymentLines> payments = fcp.getFeeClaimPaymentLines();
       for (FeeClaimPaymentLines payment : payments) {
           if (payment.getCheckNumber() != null)  {
               retVal = payment.getCheckNumber().trim();
               break;
           }
       }
       return retVal;
   }
   
   /**
    * returns the first non-null payment type from list of claim payment lines
    * payment lines may contain more than one payment type
    * returns null by default
    */
   
   static String getClaimPaymentType(FeeClaimPayments fcp) {
       String retVal = null;
       Collection<FeeClaimPaymentLines> payments = fcp.getFeeClaimPaymentLines();
       for (FeeClaimPaymentLines payment : payments) {
           if (payment.getTypeOfPayment() != null) {
               retVal = payment.getTypeOfPayment().trim();
               break;
           }
       }
       return retVal;
   }
   
   
   
   static Date getCheckDate(FeeClaimPayments fcp) {
       Date retVal = null;
       Collection<FeeClaimPaymentLines> payments = fcp.getFeeClaimPaymentLines();
       for (FeeClaimPaymentLines payment : payments) {
           if (payment.getCheckDate() != null) {
               retVal = payment.getCheckDate();
               break;
           }
       }
       return retVal;
   }
   
   
   
   static boolean isBundled(FeeClaimPaymentLines fcpl) {
      try {
         Integer.parseInt(fcpl.getLineItemSequence());
         return false;
      } catch (NumberFormatException e) {
         // NumberFormateExceptions will in indicate a line item sequence 
         // of 1,2,... (i.e., bundled)
         return true;
      }
   }
   
   
   
   static boolean isBundled(Collection<Long> seqs) {
      return seqs.size() > 1;
   }
    
   
   
   static boolean containsValidSeqs(FeeClaimPayments fcp) {
	   
	  boolean DEBUG = true;  
	   
      boolean rslt = true;
      
      Collection<FeeClaimPaymentLines> fcpls = fcp.getFeeClaimPaymentLines();
      
      
      for (FeeClaimPaymentLines fcpl : fcpls) {
         String[] seqs = fcpl.getLineItemSequence().split(",");
         
         
if(DEBUG) {
	System.out.println("containsValidSeqs() - seqs array size is: " + seqs.length);
}
         
         
         for (String seq : seqs) {

if(DEBUG) {
	System.out.println("containsValidSeqs() - sequence is: " + seq);
}
        	 
            try {
               Integer.parseInt(seq.trim());
               
               if(DEBUG) {
            		System.out.println("containsValidSeqs() - seqs is: " + Integer.parseInt(seq.trim() ) );
            	}               
               
               
            } catch (NumberFormatException e) {
               rslt = false;
               
               if(DEBUG) {
            	   System.out.println("containsValidSeqs() - ERROR with fcpl # " + fcpl.getFeeClaimPaymentLineId() + " seq: " + seq);
               }
               
               LOGGER.debug("FEE Claim Payment Line {} contains bad sequence number {}"
                     , fcpl.getFeeClaimPaymentLineId(), seq);
            }
         }
      }
      
      return rslt;
   }  
   
   
   
   static Collection<Long> getBundledSeqs(Collection<FeeClaimPaymentLines> fcpls) {
      Collection<Long> retVal = new HashSet<Long>();
      for (FeeClaimPaymentLines fcpl : fcpls) {
         String[] seqs = fcpl.getLineItemSequence().split(",");
         for (String seq : seqs) {
            try {
               retVal.add(Long.parseLong(seq));
            } catch (NumberFormatException e) {
               LOGGER.debug("Unable to convert sequence {} to Integer", seq);
            }
         }
      }
      return retVal;
   }
   
   
   
   static SortedSet<Long> getSeqAsLongs(String[] seqs) {
      SortedSet<Long> set = new TreeSet<Long>();
      for (String seq : seqs) {
         set.add(Long.parseLong(seq));
      }
      return set;
   }

   
   
   static SortedSet<Long> getSeqAsLongs(String seqs) {
        if (seqs == null || "".equals(seqs)) {
            return new TreeSet<Long>();
        }
        SortedSet<Long> set = new TreeSet<Long>();
        String[] seqArray = seqs.split(",");
        for (String seq : seqArray) {
            set.add(Long.parseLong(seq));
        }
        return set;
   }
   
    
    
   /*
    * A temporary structure that contains buckets holding results of fee claim payment
    * reconciliation. 
    * 
    * the map key is the fee claim payment id (String) and a collection of claim procedure 
    * sequence numbers (Long).
    */
   static class LineReconciliationResults {
       Map<String, Collection<FeeClaimPaymentLines>> previouslyReconciledPayments = new TreeMap<String, Collection<FeeClaimPaymentLines>>();
       Map<String, Collection<FeeClaimPaymentLines>> missmatchedAmtPayments = new TreeMap<String, Collection<FeeClaimPaymentLines>>();
       Map<String, Collection<FeeClaimPaymentLines>> unmatchedSequenceNumbers = new TreeMap<String, Collection<FeeClaimPaymentLines>>();
       Set<String> reconciledPayments = new TreeSet<String>();

       void normalize() {
            Collection<String> prps = previouslyReconciledPayments.keySet();
            for (String key : prps) {
               reconciledPayments.remove(key);
            }
            Collection<String> maps = missmatchedAmtPayments.keySet();
            for (String key : maps) {
               reconciledPayments.remove(key);
            }
            Collection<String> usns = unmatchedSequenceNumbers.keySet();
            for (String key : usns) {
               reconciledPayments.remove(key);
            }
       }
   }
   
   
   
   static class IntermResult {
      ClaimProcedure procedure;
      Collection<FeeClaimPaymentLines> payments;
   }
}
