package gov.va.med.hac.edi.reposload.claimstatus.b1.jpa;

import gov.va.med.hac.edi.utils.*;
import gov.va.med.hac.edi.utils.jpa.DbTransactionEnderException;
import gov.va.med.hac.edi.utils.jpa.FileUtils;
import gov.va.med.hac.edi.utils.jpa.JPAUtil;

import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.*;

import javax.naming.NamingException;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.Query;


import java.io.*;
import java.math.BigDecimal;
import java.sql.Date;
import gov.va.med.domain.erepos.*;

public class ProcessClaimStatusB1
{

  final private PrintWriter vLogFile;

  private boolean exists = false;
  private String loadRec = "";
  //HR-PBM-PHASE 1-Begin
  // Declaring variable vClearinghouse
  private String vClearinghouse = "";
  //HR-PBM-PHASE 1-End

  // Constructor for this class
  public ProcessClaimStatusB1(PrintWriter inLogFile)
  throws IOException, NamingException {
    // Store the log file in an instance variable
    vLogFile = inLogFile;
  }

  protected void processClmStLoad(String fPath, String fName)
  throws DbTransactionEnderException
  {
  	EntityManager dbEM = null;
	try
    {
      
	  dbEM= JPAUtil.getEntityManager("dbhibernate.properties", "b1ClaimStatus");
      dbEM.getTransaction().begin();
      
      String vFile = fPath + fName;
	Query voNcpdpRb1ClaimStatus = dbEM
			.createQuery("SELECT claimStatus FROM gov.va.med.domain.erepos.NcpdpRb1ClaimStatus claimStatus WHERE claimStatus.fileNumber=?1");
	voNcpdpRb1ClaimStatus.setParameter(1, fName);
      
	List claimStatuses = voNcpdpRb1ClaimStatus.getResultList();
	Iterator claimStatusIterator = claimStatuses.iterator();

      if (claimStatusIterator.hasNext())
      {
        vLogFile.println("Found a Duplicate Ncpdp RB1 Claim Status File with File Name : " + fName);
        vLogFile.flush();
      }
      else
      {
        int idx = fName.indexOf(".");
        String vFileName = fName.substring(0,idx) + ".RB1";
        //HR-PBM-PHASE 1-Begin
        // Looking for prefix SXC in the file name to set the value for Clearinghouse
        if (fName.substring(0,3).equalsIgnoreCase("SXC")) {
          vClearinghouse = "752578509";
        }
        else {
          vClearinghouse = "403400000";
        }
        //HR-PBM-PHASE 1-End
        loadClaimStatusB1(FileUtils.readFile(vFile), vFileName, dbEM);
        loadRespProcess(vFileName, dbEM);
        vLogFile.println("Committing the database transaction");
        dbEM.getTransaction().commit();
        vLogFile.println("Commit successful");
        //HR-PBM-PHASE 1-Begin
        // Changed the MMI specific text to generic.
        //vLogFile.println("MMI Claim Status file : " + fName  + " has been Successfully loaded into eTrans Repository. ");
        vLogFile.println("PBM Claim Status file : " + fName  + " has been Successfully loaded into eTrans Repository. ");
        //HR-PBM-PHASE 1-End
      }
      FileUtils.deleteFile(vFile);
      //HR-PBM-PHASE 1-Begin
      // Changed the MMI specific text to generic.
      //vLogFile.println("MMI Claim Status file : " + fName  + " has been Successfully Deleted. ");
      vLogFile.println("PBM Claim Status file : " + fName  + " has been Successfully Deleted. ");
      //HR-PBM-PHASE 1-End
    }
    catch(Exception e)
    {
     
      e.printStackTrace();
      if(dbEM != null)
      {
    	  if(dbEM.getTransaction().isActive())
    	  {
    		  vLogFile.println("Rolling back the database transaction");
    		  dbEM.getTransaction().rollback();
    		  vLogFile.println("Rollback successful");
    	  }
      }
      
      throw new DbTransactionEnderException("Rolled Back the Transaction", e);
    }
  }

  private void loadClaimStatusB1( BufferedReader in, String fName, EntityManager dbEM)
  throws IOException, SQLException
  {
    int iLineCnt = 0;
    while ( (loadRec = in.readLine()) != null)
    {
      iLineCnt++;
      if (iLineCnt > 7)
      {
        NcpdpRb1ClaimStatus crNcpdpRb1ClaimStatus = new NcpdpRb1ClaimStatus();
        crNcpdpRb1ClaimStatus.setB1ClaimStatusKey(UUID.randomUUID().toString().toUpperCase());
        crNcpdpRb1ClaimStatus.setFileNumber(fName);
        crNcpdpRb1ClaimStatus.setBatchNumber(loadRec.substring(0,5));
        crNcpdpRb1ClaimStatus.setPatientName(loadRec.substring(7,25));
        crNcpdpRb1ClaimStatus.setPatientSsn(loadRec.substring(27,36));
        crNcpdpRb1ClaimStatus.setPharmacyEin(loadRec.substring(38,47));
        crNcpdpRb1ClaimStatus.setRxNumber(loadRec.substring(49,56));
        crNcpdpRb1ClaimStatus.setDateOfService(loadRec.substring(58,64));
        crNcpdpRb1ClaimStatus.setTotalBilledAmt(loadRec.substring(66,75));
        crNcpdpRb1ClaimStatus.setHacClaimNum(loadRec.substring(77,91));
        crNcpdpRb1ClaimStatus.setClaimStatus(loadRec.substring(95,97));
        crNcpdpRb1ClaimStatus.setCheckNum(loadRec.substring(99,108));
        crNcpdpRb1ClaimStatus.setTotalPaidAmt(loadRec.substring(110,119));
        crNcpdpRb1ClaimStatus.setAmtPaidVendor(loadRec.substring(121,130));
        crNcpdpRb1ClaimStatus.setAmtPaidPatient(loadRec.substring(132,141));
        crNcpdpRb1ClaimStatus.setDedAppliedAmt(loadRec.substring(143,152));
        crNcpdpRb1ClaimStatus.setRejectedAmt(loadRec.substring(154,163));
        crNcpdpRb1ClaimStatus.setPatientCostshare(loadRec.substring(165,174));
        String vErj = loadRec.substring(176,179);
        crNcpdpRb1ClaimStatus.setErrorRejCode(vErj);
        crNcpdpRb1ClaimStatus.setRejCode(loadRec.substring(181,184));
        crNcpdpRb1ClaimStatus.setNcpdpRejCd(getNcpdpCd(vErj, dbEM));
        crNcpdpRb1ClaimStatus.setHicNum(loadRec.substring(186,197));
        //HR-PBM-PHASE 1-Begin
        // Replaced the hard coded value with variable 'vClearinghouse'
        //crNcpdpRb2ReversalStatus.setClearinghouse("403400000");
        crNcpdpRb1ClaimStatus.setClearinghouse(vClearinghouse);
        //HR-PBM-PHASE 1-End
        dbEM.persist(crNcpdpRb1ClaimStatus);
      }
    }
    FileUtils.closeFile(in);
    insertProcessLog(iLineCnt, fName, dbEM);
  }

  private void loadRespProcess(String fName, EntityManager dbEM)
  {
    
    NcpdpResponseProcess crNcpdpResponseProcess = new NcpdpResponseProcess();
    NcpdpResponseProcessId id = new NcpdpResponseProcessId();
    id.setFileNumber(fName);
    id.setType("RB1");
    id.setStatus("NEW");
    id.setClearinghouse(vClearinghouse);
    crNcpdpResponseProcess.setId(id);
    dbEM.persist(crNcpdpResponseProcess);
  }

  private String getNcpdpCd(String vHacRejCd, EntityManager dbEM)
  {
    String vNcpRejCd = "";
    //int vRejCd = Integer.parseInt(vHacRejCd);

    Query hacNcpdpRejCdXwalkQuery = dbEM
	.createQuery("SELECT ncpdpCode FROM gov.va.med.domain.erepos.HacNcpdpRejCdXwalk ncpdpCode WHERE ncpdpCode.id.hacRejCd = ?1 AND ncpdpCode.id.status = ?2");
    hacNcpdpRejCdXwalkQuery.setParameter(1, vHacRejCd);
    hacNcpdpRejCdXwalkQuery.setParameter(2, "ACTIVE");

    List hacNcpdpRejCdXwalks = hacNcpdpRejCdXwalkQuery.getResultList();
    Iterator hacNcpdpRejCdXwalksIterator = hacNcpdpRejCdXwalks.iterator();

    
    if (hacNcpdpRejCdXwalksIterator.hasNext())
    {
      HacNcpdpRejCdXwalk crHacNcpdpRejCdXwalk = (HacNcpdpRejCdXwalk) hacNcpdpRejCdXwalksIterator.next();
      crHacNcpdpRejCdXwalk.getId().getHacRejCd();
      crHacNcpdpRejCdXwalk.getId().getStatus();
      vNcpRejCd = crHacNcpdpRejCdXwalk.getId().getNcpdpRejCd();
    }

    return vNcpRejCd;
  }

  private void insertProcessLog(int iLineCnt, String fName, EntityManager dbEM)
  throws SQLException
  {
    EdiProcessLog crEdiProcessLog = new EdiProcessLog();
    crEdiProcessLog.setEdiProcessId(UUID.randomUUID().toString().toUpperCase());
    crEdiProcessLog.setFileDate(new java.sql.Date(new GregorianCalendar().getTimeInMillis()));
    crEdiProcessLog.setProcessName("NCPDP RB1 CLAIM STATUS LOAD TO EDI REPOSITORY");
    crEdiProcessLog.setMessage("NUM. OF NCPDP RB1 CLAIM STATUS LOADED INTO EDI REPOSITORY FOR FILE :" + fName);
    crEdiProcessLog.setCounts(new Long(Integer.toString(iLineCnt)));
    crEdiProcessLog.setClaimType("NCPDP");
    crEdiProcessLog.setStatus("NEW");
    dbEM.persist(crEdiProcessLog);
  }

  private void closeFile(BufferedReader in)
  throws IOException
  {
    in.close();
  }



}