package gov.va.med.hac.edi.reposload.load278.jpa;

import gov.va.med.domain.prserv1.*;
import gov.va.med.hac.edi.utils.jpa.*;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;

import java.math.BigDecimal;

import java.sql.SQLException;

import java.util.*;

import javax.persistence.*;
import javax.persistence.EntityManager;


public class process278LoadNew {
    static final Logger LOGGER = LoggerFactory.getLogger(process278LoadNew.class);
    static final Logger traceLOGGER = LoggerFactory.getLogger("trace." +
            process278LoadNew.class);
    int v278Count = 0;
    private String loadRec = "";

    protected void processLoad(String vFilePath, EntityManager dbEM)
        throws DbTransactionEnderException {
        boolean exists = false;

        try {
            dbEM.setFlushMode(FlushModeType.COMMIT);

            // Put everything into the jpa session and commit it later once the
            // file has been read
            dbEM.getTransaction().begin();

            // Open the control file to find out what the file number (before
            // the files were split into
            // .DAT files)
            // was - then check to see if the data from this file has already
            // been loaded into
            // the database if it has - warn the user and skip processing the
            // data
            // otherwise process the data
            final IntchgControl voIntchgControl = new IntchgControl();
            final BufferedReader inFile = FileUtils.readFile(vFilePath +
                    "INTCHG_CONTROL.DAT");
            String vLoadRec = "";
            String v278FileName = "";

            if ((vLoadRec = inFile.readLine()) != null) {
                inFile.close();

                // TWP: need to find out how they name these files to check for
                // dups??
                v278FileName = vLoadRec.split("\\^", -1)[0];

                if (isDataAlreadyLoaded(dbEM, v278FileName)) {
                    LOGGER.warn("Found a Duplicate 278 File with File Name : " +
                        v278FileName);
                } else {
                    // If there is a TRADING_PARTNERS.DAT file located in the
                    // directory
                    // indicated by vFilePath load the date into the
                    // TRADING_PARTNERS table
                    exists = (new File(vFilePath + "TRADING_PARTNERS.DAT")).exists();

                    if (exists) {
                        loadTradingPartners(FileUtils.readFile(vFilePath +
                                "TRADING_PARTNERS.DAT"), dbEM);
                        LOGGER.debug("TradingPartners loaded");
                    }

                    clearCachedEntities(dbEM);

                    // If there is a INTCHG_CONTROL.DAT file located in the
                    // directory
                    // indicated by vFilePath load the date into the
                    // INTCHG_CONTROL table
                    exists = (new File(vFilePath + "INTCHG_CONTROL.DAT")).exists();

                    if (exists) {
                        loadIntchgControl(FileUtils.readFile(vFilePath +
                                "INTCHG_CONTROL.DAT"), dbEM);
                        LOGGER.debug("IntchgControl loaded");
                    }

                    clearCachedEntities(dbEM);

                    // If there is a FUNCTIONAL_GROUP.DAT file located in the
                    // directory
                    // indicated by vFilePath load the date into the
                    // FUNCTIONAL_GROUP table
                    exists = (new File(vFilePath + "FUNCTIONAL_GROUP.DAT")).exists();

                    if (exists) {
                        loadFunctionalGroup(FileUtils.readFile(vFilePath +
                                "FUNCTIONAL_GROUP.DAT"), dbEM);
                        LOGGER.debug("FunctionalGroup loaded");
                    }

                    clearCachedEntities(dbEM);

                    // If there is a TXN_SET.DAT file located in the directory
                    // indicated by vFilePath load the date into the TXN_SET
                    // table
                    exists = (new File(vFilePath + "TXN_SET.DAT")).exists();

                    if (exists) {
                        loadTxnSet(FileUtils.readFile(vFilePath +
                                "TXN_SET.DAT"), dbEM);
                        LOGGER.debug("TxnSet loaded");
                    }

                    clearCachedEntities(dbEM);

                    // If there is a HRCHL_LEVEL.DAT file located in the
                    // directory
                    // indicated by vFilePath load the date into the HRCHL_LEVEL
                    // table
                    exists = (new File(vFilePath + "HRCHL_LEVEL.DAT")).exists();

                    if (exists) {
                        loadHrchlLevel(FileUtils.readFile(vFilePath +
                                "HRCHL_LEVEL.DAT"), dbEM);
                        LOGGER.debug("HrchlLevel loaded");
                    }

                    clearCachedEntities(dbEM);

                    // If there is a REQUEST_VALIDATION.DAT file located in the
                    // directory
                    // indicated by vFilePath load the date into the
                    // REQUEST_VALIDATION table
                    exists = (new File(vFilePath + "REQUEST_VALIDATION.DAT")).exists();

                    if (exists) {
                        loadRequestValidation(FileUtils.readFile(vFilePath +
                                "REQUEST_VALIDATION.DAT"), dbEM);
                        LOGGER.debug("RequestValidation loaded");
                    }

                    clearCachedEntities(dbEM);

                    // If there is a TRACE_NUMBER.DAT file located in the
                    // directory
                    // indicated by vFilePath load the date into the
                    // TRACE_NUMBER table
                    exists = (new File(vFilePath + "TRACE_NUMBER.DAT")).exists();

                    if (exists) {
                        loadTraceNumber(FileUtils.readFile(vFilePath +
                                "TRACE_NUMBER.DAT"), dbEM);
                        LOGGER.debug("TraceNumber loaded");
                    }

                    clearCachedEntities(dbEM);

                    // If there is a DTP_DATE.DAT file located in the directory
                    // indicated by vFilePath load the date into the DTP_DATE
                    // table
                    exists = (new File(vFilePath + "DTP_DATE.DAT")).exists();

                    if (exists) {
                        loadDtpDate(FileUtils.readFile(vFilePath +
                                "DTP_DATE.DAT"), dbEM);
                        LOGGER.debug("DtpDate loaded");
                    }

                    clearCachedEntities(dbEM);

                    // If there is a ADTL_SUPL_INFO.DAT file located in the
                    // directory
                    // indicated by vFilePath load the date into the
                    // ADTL_SUPL_INFO table
                    exists = (new File(vFilePath + "ADTL_SUPL_INFO.DAT")).exists();

                    if (exists) {
                        loadAdtlSuplInfo(FileUtils.readFile(vFilePath +
                                "ADTL_SUPL_INFO.DAT"), dbEM);
                        LOGGER.debug("AdtlSuplInfo loaded");
                    }

                    clearCachedEntities(dbEM);

                    // If there is a CNTC_INFO.DAT file located in the directory
                    // indicated by vFilePath load the date into the CNTC_INFO
                    // table
                    exists = (new File(vFilePath + "CNTC_INFO.DAT")).exists();

                    if (exists) {
                        loadCntcInfo(FileUtils.readFile(vFilePath +
                                "CNTC_INFO.DAT"), dbEM);
                        LOGGER.debug("CntcInfo loaded");
                    }

                    clearCachedEntities(dbEM);

                    // If there is a DEMOGRAPHIC_INFO.DAT file located in the
                    // directory
                    // indicated by vFilePath load the date into the
                    // DEMOGRAPHIC_INFO table
                    exists = (new File(vFilePath + "DEMOGRAPHIC_INFO.DAT")).exists();

                    if (exists) {
                        loadDemographicInfo(FileUtils.readFile(vFilePath +
                                "DEMOGRAPHIC_INFO.DAT"), dbEM);
                        LOGGER.debug("DemographicInfo loaded");
                    }

                    clearCachedEntities(dbEM);

                    // If there is a ENTITY_ADDRESS.DAT file located in the
                    // directory
                    // indicated by vFilePath load the date into the
                    // ENTITY_ADDRESS table
                    exists = (new File(vFilePath + "ENTITY_ADDRESS.DAT")).exists();

                    if (exists) {
                        loadEntityAddress(FileUtils.readFile(vFilePath +
                                "ENTITY_ADDRESS.DAT"), dbEM);
                        LOGGER.debug("EntityAddress loaded");
                    }

                    clearCachedEntities(dbEM);

                    // If there is a ENTITY_ADTL_ID.DAT file located in the
                    // directory
                    // indicated by vFilePath load the date into the
                    // ENTITY_ADTL_ID table
                    exists = (new File(vFilePath + "ENTITY_ADTL_ID.DAT")).exists();

                    if (exists) {
                        loadEntityAdtlId(FileUtils.readFile(vFilePath +
                                "ENTITY_ADTL_ID.DAT"), dbEM);
                        LOGGER.debug("EntityAdtlId loaded");
                    }

                    clearCachedEntities(dbEM);

                    // If there is a ENTITY_NAME.DAT file located in the
                    // directory
                    // indicated by vFilePath load the date into the ENTITY_NAME
                    // table
                    exists = (new File(vFilePath + "ENTITY_NAME.DAT")).exists();

                    if (exists) {
                        loadEntityName(FileUtils.readFile(vFilePath +
                                "ENTITY_NAME.DAT"), dbEM);
                        LOGGER.debug("EntityName loaded");
                    }

                    clearCachedEntities(dbEM);

                    // If there is a HI_CODES.DAT file located in the directory
                    // indicated by vFilePath load the date into the HI_CODES
                    // table
                    exists = (new File(vFilePath + "HI_CODES.DAT")).exists();

                    if (exists) {
                        loadHiCodes(FileUtils.readFile(vFilePath +
                                "HI_CODES.DAT"), dbEM);
                        LOGGER.debug("HiCodes loaded");
                    }

                    clearCachedEntities(dbEM);

                    // If there is a AMBLN_TRNPT_INFO.DAT file located in the
                    // directory
                    // indicated by vFilePath load the date into the
                    // AMBLN_TRNPT_INFO table
                    exists = (new File(vFilePath + "AMBLN_TRNPT_INFO.DAT")).exists();

                    if (exists) {
                        loadAmblnTrnptInfo(FileUtils.readFile(vFilePath +
                                "AMBLN_TRNPT_INFO.DAT"), dbEM);
                        LOGGER.debug("AmblnTrnptInfo loaded");
                    }

                    clearCachedEntities(dbEM);

                    // If there is a SPNL_MPLT_SVC_INFO.DAT file located in the
                    // directory
                    // indicated by vFilePath load the date into the
                    // SPNL_MPLT_SVC_INFO table
                    exists = (new File(vFilePath + "SPNL_MPLT_SVC_INFO.DAT")).exists();

                    if (exists) {
                        loadSpnlMpltSvcInfo(FileUtils.readFile(vFilePath +
                                "SPNL_MPLT_SVC_INFO.DAT"), dbEM);
                        LOGGER.debug("SpnlMpltSvcInfo loaded");
                    }

                    clearCachedEntities(dbEM);

                    // If there is a HOME_OXN_THPY_CERT.DAT file located in the
                    // directory
                    // indicated by vFilePath load the date into the
                    // HOME_OXN_THPY_CERT table
                    exists = (new File(vFilePath + "HOME_OXN_THPY_CERT.DAT")).exists();

                    if (exists) {
                        loadHomeOxnThpyCert(FileUtils.readFile(vFilePath +
                                "HOME_OXN_THPY_CERT.DAT"), dbEM);
                        LOGGER.debug("HomeOxnThpyCert loaded");
                    }

                    clearCachedEntities(dbEM);

                    // If there is a HOME_HCARE_INFO.DAT file located in the
                    // directory
                    // indicated by vFilePath load the date into the
                    // HOME_HCARE_INFO table
                    exists = (new File(vFilePath + "HOME_HCARE_INFO.DAT")).exists();

                    if (exists) {
                        loadHomeHcareInfo(FileUtils.readFile(vFilePath +
                                "HOME_HCARE_INFO.DAT"), dbEM);
                        LOGGER.debug("HomeHcareInfo loaded");
                    }

                    clearCachedEntities(dbEM);

                    // If there is a INST_CLAIM_CODE.DAT file located in the
                    // directory
                    // indicated by vFilePath load the date into the
                    // INST_CLAIM_CODE table
                    exists = (new File(vFilePath + "INST_CLAIM_CODE.DAT")).exists();

                    if (exists) {
                        loadInstClaimCode(FileUtils.readFile(vFilePath +
                                "INST_CLAIM_CODE.DAT"), dbEM);
                        LOGGER.debug("InstClaimCode loaded");
                    }

                    clearCachedEntities(dbEM);

                    // If there is a HCARE_SVC_REV_INFO.DAT file located in the
                    // directory
                    // indicated by vFilePath load the date into the
                    // HCARE_SVC_REV_INFO table
                    exists = (new File(vFilePath + "HCARE_SVC_REV_INFO.DAT")).exists();

                    if (exists) {
                        loadHcareSvcRevInfo(FileUtils.readFile(vFilePath +
                                "HCARE_SVC_REV_INFO.DAT"), dbEM);
                        LOGGER.debug("HcareSvcRevInfo loaded");
                    }

                    clearCachedEntities(dbEM);

                    // If there is a HCARE_SVC_REVIEW.DAT file located in the
                    // directory
                    // indicated by vFilePath load the date into the
                    // HCARE_SVC_REVIEW table
                    exists = (new File(vFilePath + "HCARE_SVC_REVIEW.DAT")).exists();

                    if (exists) {
                        loadHcareSvcReview(FileUtils.readFile(vFilePath +
                                "HCARE_SVC_REVIEW.DAT"), dbEM);
                        LOGGER.debug("HcareSvcReview loaded");
                    }

                    clearCachedEntities(dbEM);

                    // If there is a PRV_SPCL_INFO.DAT file located in the
                    // directory
                    // indicated by vFilePath load the date into the
                    // PRV_SPCL_INFO table
                    exists = (new File(vFilePath + "PRV_SPCL_INFO.DAT")).exists();

                    if (exists) {
                        loadPrvSpclInfo(FileUtils.readFile(vFilePath +
                                "PRV_SPCL_INFO.DAT"), dbEM);
                        LOGGER.debug("PrvSpclInfo loaded");
                    }

                    clearCachedEntities(dbEM);

                    // If there is a HCARE_SERVICE_DLVR.DAT file located in the
                    // directory
                    // indicated by vFilePath load the date into the
                    // HCARE_SERVICE_DLVR table
                    exists = (new File(vFilePath + "HCARE_SERVICE_DLVR.DAT")).exists();

                    if (exists) {
                        loadHcareServiceDlvr(FileUtils.readFile(vFilePath +
                                "HCARE_SERVICE_DLVR.DAT"), dbEM);
                        LOGGER.debug("HcareServiceDlvr loaded");
                    }

                    clearCachedEntities(dbEM);

                    // If there is a INS_RELATIONSHIP.DAT file located in the
                    // directory
                    // indicated by vFilePath load the date into the
                    // INS_RELATIONSHIP table
                    exists = (new File(vFilePath + "INS_RELATIONSHIP.DAT")).exists();

                    if (exists) {
                        loadInsRelationship(FileUtils.readFile(vFilePath +
                                "INS_RELATIONSHIP.DAT"), dbEM);
                        LOGGER.debug("InsRelationship loaded");
                    }

                    clearCachedEntities(dbEM);

                    // If there is a MESSAGE_TEXT.DAT file located in the
                    // directory
                    // indicated by vFilePath load the date into the
                    // MESSAGE_TEXT table
                    exists = (new File(vFilePath + "MESSAGE_TEXT.DAT")).exists();

                    if (exists) {
                        loadMessageText(FileUtils.readFile(vFilePath +
                                "MESSAGE_TEXT.DAT"), dbEM);
                        LOGGER.debug("MessageText loaded");
                    }

                    clearCachedEntities(dbEM);

                    //NEWNEWNEWNEWNEWNEWNEWN					
                    // If there is a DENTAL_SVC_LINE.DAT file located in the
                    // directory
                    // indicated by vFilePath load the date into the
                    // DENTAL_SVC_LINE table
                    exists = (new File(vFilePath + "DENTAL_SVC_LINE.DAT")).exists();

                    if (exists) {
                        loadDentalSvcLine(FileUtils.readFile(vFilePath +
                                "DENTAL_SVC_LINE.DAT"), dbEM);
                        LOGGER.debug("DentalSvcLine loaded");
                    }

                    clearCachedEntities(dbEM);

                    // If there is a INST_SVC_LINE.DAT file located in the
                    // directory
                    // indicated by vFilePath load the date into the
                    // INST_SVC_LINE table
                    exists = (new File(vFilePath + "INST_SVC_LINE.DAT")).exists();

                    if (exists) {
                        loadInstSvcLine(FileUtils.readFile(vFilePath +
                                "INST_SVC_LINE.DAT"), dbEM);
                        LOGGER.debug("InstSvcLine loaded");
                    }

                    clearCachedEntities(dbEM);

                    // If there is a PRFSN_SVC_LINE.DAT file located in the
                    // directory
                    // indicated by vFilePath load the date into the
                    // PRFSN_SVC_LINE.DAT table
                    exists = (new File(vFilePath + "PRFSN_SVC_LINE.DAT")).exists();

                    if (exists) {
                        loadPrfsnSvcLine(FileUtils.readFile(vFilePath +
                                "PRFSN_SVC_LINE.DAT"), dbEM);
                        LOGGER.debug("PrfsnSvcLine loaded");
                    }

                    clearCachedEntities(dbEM);

                    // If there is a TOOTH_INFORMATION.DAT file located in the
                    // directory
                    // indicated by vFilePath load the date into the
                    // TOOTH_INFORMATION table
                    exists = (new File(vFilePath + "TOOTH_INFORMATION.DAT")).exists();

                    if (exists) {
                        this.loadToothInformation(FileUtils.readFile(vFilePath +
                                "TOOTH_INFORMATION.DAT"), dbEM);
                        LOGGER.debug("PrfsnSvcLine loaded");
                    }

                    clearCachedEntities(dbEM);

                    String vTransDate = getIsa13Dt(vFilePath);
                    // insertProcessLog(v278Count, vTransDate, dbEM);
                    LOGGER.info("Committing the database transaction");

                    // Commit the entire load at once
                    dbEM.getTransaction().commit();
                    LOGGER.info("Commit successful");
                }

                deleteFiles(vFilePath);
            }
        } catch (Exception e) {
            LOGGER.error("Rolling back the database transaction");

            if ((dbEM != null) && dbEM.getTransaction().isActive()) {
                dbEM.getTransaction().rollback();
                LOGGER.info("Rollback successful");
            } else {
                LOGGER.error("Transaction was not active - no rollback");
            }

            throw new DbTransactionEnderException("Rolled Back the Transaction",
                e);
        }
    }

    protected void loadAdtlSuplInfo(BufferedReader in, EntityManager em)
        throws IOException {
        while ((loadRec = in.readLine()) != null) {
            traceLOGGER.trace("AdtlSuplInfo as read:" + loadRec);

            final String[] tokens = loadRec.split("\\^", -1);
            final AdtlSuplInfo adtlSuplInfo = new AdtlSuplInfo();
            adtlSuplInfo.setPwkKey(tokens[0]);
            adtlSuplInfo.setTxType(tokens[1]);
            adtlSuplInfo.setFileNumber(tokens[2]);
            adtlSuplInfo.setSt02txnCtlNbr(tokens[3]);
            adtlSuplInfo.setLoopId(tokens[4]);
            adtlSuplInfo.setHrchlLevelKey(tokens[5]);
            adtlSuplInfo.setSeqNbr(new BigDecimal(tokens[6]));
            adtlSuplInfo.setPwk01rptTypeCd(tokens[7]);
            adtlSuplInfo.setPwk02rptTrnsmCd(tokens[8]);
            adtlSuplInfo.setPwk03rptCopyNeeded(tokens[9]);
            adtlSuplInfo.setPwd04entyIdCd(tokens[10]);
            adtlSuplInfo.setPwk05idCdQual(tokens[11]);
            adtlSuplInfo.setPwk06atmntCtlNbr(tokens[12]);
            adtlSuplInfo.setPwk07dsc(tokens[13]);
            adtlSuplInfo.setPwk09rqstCatCd(tokens[14]);

            em.persist(adtlSuplInfo);
        }

        FileUtils.closeFile(in);
        System.out.println("Created rows for AdtlSuplInfo table");
    }

    protected void loadAmblnTrnptInfo(BufferedReader in, EntityManager em)
        throws IOException {
        while ((loadRec = in.readLine()) != null) {
            traceLOGGER.trace("AmblnTrnptInfo as read:" + loadRec);

            final String[] tokens = loadRec.split("\\^", -1);
            final AmblnTrnptInfo amblnTrnptInfo = new AmblnTrnptInfo();
            amblnTrnptInfo.setAmblnTptInfoKey(tokens[0]);
            amblnTrnptInfo.setTxType(tokens[1]);
            amblnTrnptInfo.setFileNumber(tokens[2]);
            amblnTrnptInfo.setSt02txnCtlNbr(tokens[3]);
            amblnTrnptInfo.setLoopId(tokens[4]);
            amblnTrnptInfo.setHrchlLevelKey(tokens[5]);
            amblnTrnptInfo.setCr101uomCd(tokens[6]);

            if ((tokens[7] != null) && (tokens[7].trim().length() != 0)) {
                amblnTrnptInfo.setCr102ptWt(new BigDecimal(tokens[7]));
            }

            amblnTrnptInfo.setCr103amblnTrnCd(tokens[8]);
            amblnTrnptInfo.setCr104amblnTrnRsn(tokens[9]);
            amblnTrnptInfo.setCr105uomDist(tokens[10]);

            if (tokens[11] != null) {
                amblnTrnptInfo.setCr106trnspDist(new BigDecimal(tokens[11]));
            }

            amblnTrnptInfo.setCr109rtrpPrps(tokens[12]);
            amblnTrnptInfo.setCr110strchPrps(tokens[13]);

            em.persist(amblnTrnptInfo);
        }

        FileUtils.closeFile(in);
        System.out.println("Created rows for AmblnTrnptInfo table");
    }

    protected void loadCntcInfo(BufferedReader in, EntityManager em)
        throws IOException {
        while ((loadRec = in.readLine()) != null) {
            traceLOGGER.trace("CntcInfo as read:" + loadRec);

            final String[] tokens = loadRec.split("\\^", -1);
            final CntcInfo cntcInfo = new CntcInfo();
            cntcInfo.setContactInfoKey(tokens[0]);
            cntcInfo.setTxType(tokens[1]);
            cntcInfo.setFileNumber(tokens[2]);
            cntcInfo.setSt02txnCtlNbr(tokens[3]);
            cntcInfo.setLoopId(tokens[4]);
            cntcInfo.setHrchlLevelKey(tokens[5]);
            cntcInfo.setPer01cntcFctn(tokens[6]);
            cntcInfo.setPer02cntcNm(tokens[7]);
            cntcInfo.setPer03commQual1(tokens[8]);
            cntcInfo.setPer04commNbr1(tokens[9]);
            cntcInfo.setPer05commQual2(tokens[10]);
            cntcInfo.setPer06commNbr2(tokens[11]);
            cntcInfo.setPer07commQual3(tokens[12]);
            cntcInfo.setPer08commNbr3(tokens[13]);
            cntcInfo.setPer09cntcInqRef(tokens[14]);

            em.persist(cntcInfo);
        }

        FileUtils.closeFile(in);
        System.out.println("Created rows for CntcInfo table");
    }

    protected void loadDemographicInfo(BufferedReader in, EntityManager em)
        throws IOException {
        while ((loadRec = in.readLine()) != null) {
            traceLOGGER.trace("DemographicInfo as read:" + loadRec);

            final String[] tokens = loadRec.split("\\^", -1);
            final DemographicInfo demographicInfo = new DemographicInfo();
            demographicInfo.setDmgInfoKey(tokens[0]);
            demographicInfo.setTxType(tokens[1]);
            demographicInfo.setFileNumber(tokens[2]);
            demographicInfo.setSt02txnCtlNbr(tokens[3]);
            demographicInfo.setLoopId(tokens[4]);
            demographicInfo.setHrchlLevelKey(tokens[5]);
            demographicInfo.setDmg01dttmFmtQual(tokens[6]);
            demographicInfo.setDmg02brthDt(tokens[7]);
            demographicInfo.setDmg03gndr(tokens[8]);
            demographicInfo.setDmg10CdLstQualCd(tokens[9]);
            demographicInfo.setDmg11IndustryCd(tokens[10]);

            em.persist(demographicInfo);
        }

        FileUtils.closeFile(in);
        System.out.println("Created rows for DemographicInfo table");
    }

    protected void loadDentalSvcLine(BufferedReader in, EntityManager em)
        throws IOException {
        while ((loadRec = in.readLine()) != null) {
            traceLOGGER.trace("DentalSvcLine as read:" + loadRec);

            final String[] tokens = loadRec.split("\\^", -1);
            final DentalSvcLine dentalSvcLine = new DentalSvcLine();
            dentalSvcLine.setClaimKey((tokens[0]));
            dentalSvcLine.setServiceKey((tokens[1]));
            dentalSvcLine.setLx01asgnLnItm(new BigDecimal(tokens[2]));
            dentalSvcLine.setSv301adaCdQual((tokens[3]));
            dentalSvcLine.setSv301adaCd(tokens[4]);
            dentalSvcLine.setSv301procMod1((tokens[5]));
            dentalSvcLine.setSv301procMod2((tokens[6]));
            dentalSvcLine.setSv301procMod3((tokens[7]));
            dentalSvcLine.setSv301procMod4((tokens[8]));

            if ((tokens[9] != null) && (tokens[9].trim().length() != 0)) {
                dentalSvcLine.setSv302lnItmChrg(new BigDecimal(tokens[9]));
            }

            dentalSvcLine.setSv304oralDsgt1((tokens[10]));
            dentalSvcLine.setSv304oralDsgt2((tokens[11]));
            dentalSvcLine.setSv304oralDsgt3((tokens[12]));
            dentalSvcLine.setSv304oralDsgt4((tokens[13]));
            dentalSvcLine.setSv304oralDsgt5((tokens[14]));
            dentalSvcLine.setSv305plcmtCd(ConversionUtil.stringToSingleChar(
                    tokens[15]));

            if ((tokens[16] != null) && (tokens[16].trim().length() != 0)) {
                dentalSvcLine.setSv306procCnt(new BigDecimal(tokens[16]));
            }

            dentalSvcLine.setSv301desc(tokens[17]);
            dentalSvcLine.setSv301prSrId(tokens[18]);

            em.persist(dentalSvcLine);
        }

        FileUtils.closeFile(in);
        System.out.println("Created rows for DentalSvcLine table");
    }

    protected void loadDtpDate(BufferedReader in, EntityManager em)
        throws IOException {
        while ((loadRec = in.readLine()) != null) {
            traceLOGGER.trace("DtpDate as read:" + loadRec);

            final String[] tokens = loadRec.split("\\^", -1);
            final DtpDate dtpDate = new DtpDate();
            dtpDate.setDateKey(tokens[0]);
            dtpDate.setTxType(tokens[1]);
            dtpDate.setFileNumber(tokens[2]);
            dtpDate.setSt02txnCtlNbr(tokens[3]);
            dtpDate.setLoopId(tokens[4]);
            dtpDate.setHrchlLevelKey(tokens[5]);
            dtpDate.setDtp01dttmQual(tokens[6]);
            dtpDate.setDtp02dttmFmtQual(tokens[7]);
            dtpDate.setDtp03dtFld(tokens[8]);

            em.persist(dtpDate);
        }

        FileUtils.closeFile(in);
        System.out.println("Created rows for DtpDate table");
    }

    protected void loadEntityAddress(BufferedReader in, EntityManager em)
        throws IOException {
        while ((loadRec = in.readLine()) != null) {
            traceLOGGER.trace("EntityAddress as read:" + loadRec);

            final String[] tokens = loadRec.split("\\^", -1);
            final EntityAddress entityAddress = new EntityAddress();
            entityAddress.setEntityAddressKey(tokens[0]);
            entityAddress.setTxType(tokens[1]);
            entityAddress.setFileNumber(tokens[2]);
            entityAddress.setSt02txnCtlNbr(tokens[3]);
            entityAddress.setLoopId(tokens[4]);
            entityAddress.setHrchlLevelKey(tokens[5]);
            entityAddress.setN301adrLn1(tokens[6]);
            entityAddress.setN302adrLn2(tokens[7]);
            entityAddress.setN401city(tokens[8]);
            entityAddress.setN402st(tokens[9]);
            entityAddress.setN403postalCd(tokens[10]);
            entityAddress.setN404ctryCd(tokens[11]);
            entityAddress.setN405locationQual(tokens[12]);
            entityAddress.setN406locationId(tokens[13]);
            entityAddress.setN407ctrySubdvsnCd(tokens[14]);

            em.persist(entityAddress);
        }

        FileUtils.closeFile(in);
        System.out.println("Created rows for EntityAddress table");
    }

    protected void loadEntityAdtlId(BufferedReader in, EntityManager em)
        throws IOException {
        while ((loadRec = in.readLine()) != null) {
            traceLOGGER.trace("EntityAdtlId as read:" + loadRec);

            final String[] tokens = loadRec.split("\\^", -1);
            final EntityAdtlId entityAdtlId = new EntityAdtlId();
            entityAdtlId.setRefSegKey(tokens[0]);
            entityAdtlId.setTxType(tokens[1]);
            entityAdtlId.setFileNumber(tokens[2]);
            entityAdtlId.setSt02txnCtlNbr(tokens[3]);
            entityAdtlId.setSeqNbr(new BigDecimal(tokens[4]));
            entityAdtlId.setLoopId(tokens[5]);
            entityAdtlId.setHrchlLevelKey(tokens[6]);
            entityAdtlId.setRef01idQlfr(tokens[7]);
            entityAdtlId.setRef02scndId(tokens[8]);
            entityAdtlId.setRef03desc(tokens[9]);
            entityAdtlId.setRef041refIdQlfr(tokens[10]);
            entityAdtlId.setRef042othrPyrPrmryId(tokens[11]);
            entityAdtlId.setRef043refIdQlfr(tokens[12]);
            entityAdtlId.setRef044refId(tokens[13]);
            entityAdtlId.setRef045refIdQlfr(tokens[14]);
            entityAdtlId.setRef046refId(tokens[15]);

            em.persist(entityAdtlId);
        }

        FileUtils.closeFile(in);
        System.out.println("Created rows for EntityAdtlId table");
    }

    protected void loadEntityName(BufferedReader in, EntityManager em)
        throws IOException {
        while ((loadRec = in.readLine()) != null) {
            traceLOGGER.trace("EntityName as read:" + loadRec);

            final String[] tokens = loadRec.split("\\^", -1);
            final EntityName entityName = new EntityName();
            entityName.setEntityNameKey(tokens[0]);
            entityName.setTxType(tokens[1]);
            entityName.setFileNumber(tokens[2]);
            entityName.setSt02txnCtlNbr(tokens[3]);
            entityName.setLoopId(tokens[4]);
            entityName.setHrchlLevelKey(tokens[5]);
            entityName.setNm101entyIdCd(tokens[6]);
            entityName.setNm102entyQual(tokens[7]);
            entityName.setNm103lastNm(tokens[8]);
            entityName.setNm104fstNm(tokens[9]);
            entityName.setNm105midNm(tokens[10]);
            entityName.setNm106nmPfx(tokens[11]);
            entityName.setNm107nmSfx(tokens[12]);
            entityName.setNm108idCdQual(tokens[13]);
            entityName.setNm109idCd(tokens[14]);
            entityName.setNm110entyRelCd(tokens[15]);
            entityName.setNm111entyIdCd(tokens[16]);
            entityName.setNm112lastNmOrganizNm(tokens[17]);

            em.persist(entityName);
        }

        FileUtils.closeFile(in);
        System.out.println("Created rows for EntityName table");
    }

    protected void loadFunctionalGroup(BufferedReader in, EntityManager em)
        throws IOException {
        while ((loadRec = in.readLine()) != null) {
            traceLOGGER.trace("FunctionalGroup as read:" + loadRec);

            final String[] tokens = loadRec.split("\\^", -1);
            final FunctionalGroup functionalGroup = new FunctionalGroup();
            functionalGroup.setFileNumber(tokens[0]);
            functionalGroup.setTxType(tokens[1]);
            functionalGroup.setGs01fctlIdCd((tokens[2]));
            functionalGroup.setGs02appSndCd(tokens[3]);
            functionalGroup.setGs03appRcvCd(tokens[4]);
            functionalGroup.setGs04creDt(tokens[5]);
            functionalGroup.setGs05creTm(tokens[6]);
            functionalGroup.setGs06grpCtlNbr(tokens[7]);
            functionalGroup.setGs07respAgcyCd((tokens[8]));
            functionalGroup.setGs08verRlsIndId(tokens[9]);
            functionalGroup.setGe01nbrTranSets(tokens[10]);
            functionalGroup.setGe02grpCtlNbr(tokens[11]);

            em.persist(functionalGroup);
        }

        FileUtils.closeFile(in);
        System.out.println("Created rows for FunctionalGroup table");
    }

    protected void loadHcareServiceDlvr(BufferedReader in, EntityManager em)
        throws IOException {
        while ((loadRec = in.readLine()) != null) {
            traceLOGGER.trace("HcareServiceDlvr as read:" + loadRec);

            final String[] tokens = loadRec.split("\\^", -1);
            final HcareServiceDlvr hcareServiceDlvr = new HcareServiceDlvr();
            hcareServiceDlvr.setHcareSvcDlvrKey(tokens[0]);
            hcareServiceDlvr.setTxType(tokens[1]);
            hcareServiceDlvr.setFileNumber(tokens[2]);
            hcareServiceDlvr.setSt02txnCtlNbr(tokens[3]);
            hcareServiceDlvr.setLoopId(tokens[4]);
            hcareServiceDlvr.setHrchlLevelKey(tokens[5]);
            hcareServiceDlvr.setHsd01qtyQual(tokens[6]);

            if ((tokens[7] != null) && (tokens[7].trim().length() != 0)) {
                hcareServiceDlvr.setHsd02nbrOfVisits(new BigDecimal(tokens[7]));
            }

            hcareServiceDlvr.setHsd03frqPer(tokens[8]);

            if ((tokens[9] != null) && (tokens[9].trim().length() != 0)) {
                hcareServiceDlvr.setHsd04frqCnt(new BigDecimal(tokens[9]));
            }

            hcareServiceDlvr.setHsd05frqTmPer(tokens[10]);

            if ((tokens[11] != null) && (tokens[11].trim().length() != 0)) {
                hcareServiceDlvr.setHsd06nbrOfPer(new BigDecimal(tokens[11]));
            }

            hcareServiceDlvr.setHsd07calPtrnCd(tokens[12]);
            hcareServiceDlvr.setHsd08tmPtrnCd(tokens[13]);

            em.persist(hcareServiceDlvr);
        }

        FileUtils.closeFile(in);
        System.out.println("Created rows for HcareServiceDlvr table");
    }

    protected void loadHcareSvcReview(BufferedReader in, EntityManager em)
        throws IOException {
        while ((loadRec = in.readLine()) != null) {
            traceLOGGER.trace("HcareSvcReview as read:" + loadRec);

            final String[] tokens = loadRec.split("\\^", -1);
            final HcareSvcReview hcareSvcReview = new HcareSvcReview();
            hcareSvcReview.setHcareSvcReviewKey(tokens[0]);
            hcareSvcReview.setTxType(tokens[1]);
            hcareSvcReview.setFileNumber(tokens[2]);
            hcareSvcReview.setSt02txnCtlNbr(tokens[3]);
            hcareSvcReview.setLoopId(tokens[4]);
            hcareSvcReview.setHrchlLevelKey(tokens[5]);
            hcareSvcReview.setHcr01actionCd(tokens[6]);
            hcareSvcReview.setHcr02certNbr(tokens[7]);
            hcareSvcReview.setHcr03rejRsnCd(tokens[8]);
            hcareSvcReview.setHcr04RespCd(tokens[9]);

            em.persist(hcareSvcReview);
        }

        FileUtils.closeFile(in);
        System.out.println("Created rows for HcareSvcReview table");
    }

    protected void loadHcareSvcRevInfo(BufferedReader in, EntityManager em)
        throws IOException {
        while ((loadRec = in.readLine()) != null) {
            traceLOGGER.trace("HcareSvcRevInfo as read:" + loadRec);

            final String[] tokens = loadRec.split("\\^", -1);
            final HcareSvcRevInfo hcareSvcRevInfo = new HcareSvcRevInfo();
            hcareSvcRevInfo.setHcareSvcRevInfoKey(tokens[0]);
            hcareSvcRevInfo.setTxType(tokens[1]);
            hcareSvcRevInfo.setFileNumber(tokens[2]);
            hcareSvcRevInfo.setSt02txnCtlNbr(tokens[3]);
            hcareSvcRevInfo.setLoopId(tokens[4]);
            hcareSvcRevInfo.setHrchlLevelKey(tokens[5]);
            hcareSvcRevInfo.setUm01reqCatCd(tokens[6]);
            hcareSvcRevInfo.setUm02certTypeCd(tokens[7]);
            hcareSvcRevInfo.setUm03svcTypeCd(tokens[8]);
            hcareSvcRevInfo.setUm041facTypeCd(tokens[9]);
            hcareSvcRevInfo.setUm042facCdQual(tokens[10]);
            hcareSvcRevInfo.setUm043clmFrqCd(tokens[11]);
            hcareSvcRevInfo.setUm06lvlSvcCd(tokens[12]);

            em.persist(hcareSvcRevInfo);
        }

        FileUtils.closeFile(in);
        System.out.println("Created rows for HcareSvcRevInfo table");
    }

    protected void loadHiCodes(BufferedReader in, EntityManager em)
        throws IOException {
        while ((loadRec = in.readLine()) != null) {
            traceLOGGER.trace("HiCodes as read:" + loadRec);

            final String[] tokens = loadRec.split("\\^", -1);
            final HiCodes hiCodes = new HiCodes();
            final HiCodesId hiCodesId = new HiCodesId();
            hiCodesId.setHiKey(tokens[0]);
            hiCodesId.setSeqNbr(new BigDecimal(tokens[4]));

            hiCodes.setId(hiCodesId);
            hiCodes.setTxType(tokens[1]);
            hiCodes.setFileNumber(tokens[2]);
            hiCodes.setSt02txnCtlNbr(tokens[3]);

            hiCodes.setLoopId(tokens[5]);
            hiCodes.setHrchlLevelKey(tokens[6]);
            hiCodes.setHi01cdListQual((tokens[7]));
            hiCodes.setHi01procCd(tokens[8]);
            hiCodes.setHi01dttmQual((tokens[9]));
            hiCodes.setHi01procDt(tokens[10]);

            if ((tokens[11] != null) && (tokens[11].trim().length() != 0)) {
                hiCodes.setHi01amount(new BigDecimal(tokens[11]));
            }

            if ((tokens[12] != null) && (tokens[12].trim().length() != 0)) {
                hiCodes.setHi01qnty(new BigDecimal(tokens[12]));
            }

            hiCodes.setHi01versionId(tokens[13]);
            hiCodes.setHi01indstryCd(tokens[14]);
            hiCodes.setHi01cndtnResponseCd(ConversionUtil.stringToSingleChar(
                    tokens[15]));
            hiCodes.setHi02cdListQual((tokens[16]));
            hiCodes.setHi02procCd(tokens[17]);
            hiCodes.setHi02dttmQual((tokens[18]));
            hiCodes.setHi02procDt(tokens[19]);

            if ((tokens[20] != null) && (tokens[20].trim().length() != 0)) {
                hiCodes.setHi02amount(new BigDecimal(tokens[20]));
            }

            if ((tokens[21] != null) && (tokens[21].trim().length() != 0)) {
                hiCodes.setHi02qnty(new BigDecimal(tokens[21]));
            }

            hiCodes.setHi02versionId(tokens[22]);
            hiCodes.setHi02indstryCd(tokens[23]);
            hiCodes.setHi02cndtnResponseCd(ConversionUtil.stringToSingleChar(
                    tokens[24]));
            hiCodes.setHi03cdListQual((tokens[25]));
            hiCodes.setHi03procCd(tokens[26]);
            hiCodes.setHi03dttmQual((tokens[27]));
            hiCodes.setHi03procDt(tokens[28]);

            if ((tokens[29] != null) && (tokens[29].trim().length() != 0)) {
                hiCodes.setHi03amount(new BigDecimal(tokens[29]));
            }

            if ((tokens[30] != null) && (tokens[30].trim().length() != 0)) {
                hiCodes.setHi03qnty(new BigDecimal(tokens[30]));
            }

            hiCodes.setHi03versionId(tokens[31]);
            hiCodes.setHi03indstryCd(tokens[32]);
            hiCodes.setHi03cndtnResponseCd(ConversionUtil.stringToSingleChar(
                    tokens[33]));
            hiCodes.setHi04cdListQual((tokens[34]));
            hiCodes.setHi04procCd(tokens[35]);
            hiCodes.setHi04dttmQual((tokens[36]));
            hiCodes.setHi04procDt(tokens[37]);

            if ((tokens[38] != null) && (tokens[38].trim().length() != 0)) {
                hiCodes.setHi04amount(new BigDecimal(tokens[38]));
            }

            if ((tokens[39] != null) && (tokens[39].trim().length() != 0)) {
                hiCodes.setHi04qnty(new BigDecimal(tokens[39]));
            }

            hiCodes.setHi04versionId(tokens[40]);
            hiCodes.setHi04indstryCd(tokens[41]);
            hiCodes.setHi04cndtnResponseCd(ConversionUtil.stringToSingleChar(
                    tokens[42]));
            hiCodes.setHi05cdListQual((tokens[43]));
            hiCodes.setHi05procCd(tokens[44]);
            hiCodes.setHi05dttmQual((tokens[45]));
            hiCodes.setHi05procDt(tokens[46]);

            if ((tokens[47] != null) && (tokens[47].trim().length() != 0)) {
                hiCodes.setHi05amount(new BigDecimal(tokens[47]));
            }

            if ((tokens[48] != null) && (tokens[48].trim().length() != 0)) {
                hiCodes.setHi05qnty(new BigDecimal(tokens[48]));
            }

            hiCodes.setHi05versionId(tokens[49]);
            hiCodes.setHi05indstryCd(tokens[50]);
            hiCodes.setHi05cndtnResponseCd(ConversionUtil.stringToSingleChar(
                    tokens[51]));
            hiCodes.setHi06cdListQual((tokens[52]));
            hiCodes.setHi06procCd(tokens[53]);
            hiCodes.setHi06dttmQual((tokens[54]));
            hiCodes.setHi06procDt(tokens[55]);

            if ((tokens[56] != null) && (tokens[56].trim().length() != 0)) {
                hiCodes.setHi06amount(new BigDecimal(tokens[56]));
            }

            if ((tokens[57] != null) && (tokens[57].trim().length() != 0)) {
                hiCodes.setHi06qnty(new BigDecimal(tokens[57]));
            }

            hiCodes.setHi06versionId(tokens[58]);
            hiCodes.setHi06indstryCd(tokens[59]);
            hiCodes.setHi06cndtnResponseCd(ConversionUtil.stringToSingleChar(
                    tokens[60]));
            hiCodes.setHi07cdListQual((tokens[61]));
            hiCodes.setHi07procCd(tokens[62]);
            hiCodes.setHi07dttmQual((tokens[63]));
            hiCodes.setHi07procDt(tokens[64]);

            if ((tokens[65] != null) && (tokens[65].trim().length() != 0)) {
                hiCodes.setHi07amount(new BigDecimal(tokens[65]));
            }

            if ((tokens[66] != null) && (tokens[66].trim().length() != 0)) {
                hiCodes.setHi07qnty(new BigDecimal(tokens[66]));
            }

            hiCodes.setHi07versionId(tokens[67]);
            hiCodes.setHi07indstryCd(tokens[68]);
            hiCodes.setHi07cndtnResponseCd(ConversionUtil.stringToSingleChar(
                    tokens[69]));
            hiCodes.setHi08cdListQual((tokens[70]));
            hiCodes.setHi08procCd(tokens[71]);
            hiCodes.setHi08dttmQual((tokens[72]));
            hiCodes.setHi08procDt(tokens[73]);

            if ((tokens[74] != null) && (tokens[74].trim().length() != 0)) {
                hiCodes.setHi08amount(new BigDecimal(tokens[74]));
            }

            if ((tokens[75] != null) && (tokens[75].trim().length() != 0)) {
                hiCodes.setHi08qnty(new BigDecimal(tokens[75]));
            }

            hiCodes.setHi08versionId(tokens[76]);
            hiCodes.setHi08indstryCd(tokens[77]);
            hiCodes.setHi08cndtnResponseCd(ConversionUtil.stringToSingleChar(
                    tokens[78]));
            hiCodes.setHi09cdListQual((tokens[79]));
            hiCodes.setHi09procCd(tokens[80]);
            hiCodes.setHi09dttmQual((tokens[81]));
            hiCodes.setHi09procDt(tokens[82]);

            if ((tokens[83] != null) && (tokens[83].trim().length() != 0)) {
                hiCodes.setHi09amount(new BigDecimal(tokens[83]));
            }

            if ((tokens[84] != null) && (tokens[84].trim().length() != 0)) {
                hiCodes.setHi09qnty(new BigDecimal(tokens[84]));
            }

            hiCodes.setHi09versionId(tokens[85]);
            hiCodes.setHi09indstryCd(tokens[86]);
            hiCodes.setHi09cndtnResponseCd(ConversionUtil.stringToSingleChar(
                    tokens[87]));
            hiCodes.setHi10cdListQual((tokens[88]));
            hiCodes.setHi10procCd(tokens[89]);
            hiCodes.setHi10dttmQual((tokens[90]));
            hiCodes.setHi10procDt(tokens[91]);

            if ((tokens[92] != null) && (tokens[92].trim().length() != 0)) {
                hiCodes.setHi10amount(new BigDecimal(tokens[92]));
            }

            if ((tokens[93] != null) && (tokens[93].trim().length() != 0)) {
                hiCodes.setHi10qnty(new BigDecimal(tokens[93]));
            }

            hiCodes.setHi10versionId(tokens[94]);
            hiCodes.setHi10indstryCd(tokens[95]);
            hiCodes.setHi10cndtnResponseCd(ConversionUtil.stringToSingleChar(
                    tokens[96]));
            hiCodes.setHi11cdListQual((tokens[97]));
            hiCodes.setHi11procCd(tokens[98]);
            hiCodes.setHi11dttmQual((tokens[99]));
            hiCodes.setHi11procDt(tokens[100]);

            if ((tokens[101] != null) && (tokens[101].trim().length() != 0)) {
                hiCodes.setHi11amount(new BigDecimal(tokens[101]));
            }

            if ((tokens[102] != null) && (tokens[102].trim().length() != 0)) {
                hiCodes.setHi11qnty(new BigDecimal(tokens[102]));
            }

            hiCodes.setHi11versionId(tokens[103]);
            hiCodes.setHi11indstryCd(tokens[104]);
            hiCodes.setHi11cndtnResponseCd(ConversionUtil.stringToSingleChar(
                    tokens[105]));
            hiCodes.setHi12cdListQual((tokens[106]));
            hiCodes.setHi12procCd(tokens[107]);
            hiCodes.setHi12dttmQual((tokens[108]));
            hiCodes.setHi12procDt(tokens[109]);

            if ((tokens[110] != null) && (tokens[110].trim().length() != 0)) {
                hiCodes.setHi12amount(new BigDecimal(tokens[110]));
            }

            if ((tokens[111] != null) && (tokens[111].trim().length() != 0)) {
                hiCodes.setHi12qnty(new BigDecimal(tokens[111]));
            }

            hiCodes.setHi12versionId(tokens[112]);
            hiCodes.setHi12indstryCd(tokens[113]);
            hiCodes.setHi12cndtnResponseCd(ConversionUtil.stringToSingleChar(
                    tokens[114]));

            em.persist(hiCodes);
        }

        FileUtils.closeFile(in);
        System.out.println("Created rows for HiCodes table");
    }

    protected void loadHomeHcareInfo(BufferedReader in, EntityManager em)
        throws IOException {
        while ((loadRec = in.readLine()) != null) {
            traceLOGGER.trace("HomeHcareInfo as read:" + loadRec);

            final String[] tokens = loadRec.split("\\^", -1);
            final HomeHcareInfo homeHcareInfo = new HomeHcareInfo();
            homeHcareInfo.setHomeHcareInfoKey(tokens[0]);
            homeHcareInfo.setTxType(tokens[1]);
            homeHcareInfo.setFileNumber(tokens[2]);
            homeHcareInfo.setSt02txnCtlNbr(tokens[3]);
            homeHcareInfo.setSeqNbr(new BigDecimal(tokens[4]));
            homeHcareInfo.setLoopId(tokens[5]);
            homeHcareInfo.setHrchlLevelKey(tokens[6]);
            homeHcareInfo.setCr601prognosisCd(tokens[7]);
            homeHcareInfo.setCr602svcFromDt(tokens[8]);
            homeHcareInfo.setCr603dttmFmtQual(tokens[9]);
            homeHcareInfo.setCr604certPer(tokens[10]);
            homeHcareInfo.setCr605diagDt(tokens[11]);
            homeHcareInfo.setCr606sklNursFcly(tokens[12]);
            homeHcareInfo.setCr607mdcrCvgInd(tokens[13]);
            homeHcareInfo.setCr608certTypeCd(tokens[14]);
            homeHcareInfo.setCr609sgryDt(tokens[15]);
            homeHcareInfo.setCr610svcIdQual(tokens[16]);
            homeHcareInfo.setCr611srgclProcCd(tokens[17]);
            homeHcareInfo.setCr612physOrdDt(tokens[18]);
            homeHcareInfo.setCr613lastVisitDt(tokens[19]);
            homeHcareInfo.setCr614physCntcDt(tokens[20]);
            homeHcareInfo.setCr615lastAdmQual(tokens[21]);
            homeHcareInfo.setCr616lastAdmPer(tokens[22]);
            homeHcareInfo.setCr617fclyTypeCd(tokens[23]);
            homeHcareInfo.setCr618scndDiagDt1(tokens[24]);
            homeHcareInfo.setCr619scndDiagDt2(tokens[25]);
            homeHcareInfo.setCr620scndDiagDt3(tokens[26]);
            homeHcareInfo.setCr621scndDiagDt4(tokens[27]);

            em.persist(homeHcareInfo);
        }

        FileUtils.closeFile(in);
        System.out.println("Created rows for HomeHcareInfo table");
    }

    protected void loadHomeOxnThpyCert(BufferedReader in, EntityManager em)
        throws IOException {
        while ((loadRec = in.readLine()) != null) {
            traceLOGGER.trace("HomeOxnThpyCert as read:" + loadRec);

            final String[] tokens = loadRec.split("\\^", -1);
            final HomeOxnThpyCert homeOxnThpyCert = new HomeOxnThpyCert();
            homeOxnThpyCert.setHOxnThpyCertKey(tokens[0]);
            homeOxnThpyCert.setTxType(tokens[1]);
            homeOxnThpyCert.setFileNumber(tokens[2]);
            homeOxnThpyCert.setSt02txnCtlNbr(tokens[3]);
            homeOxnThpyCert.setSeqNbr(new BigDecimal(tokens[4]));
            homeOxnThpyCert.setLoopId(tokens[5]);
            homeOxnThpyCert.setHrchlLevelKey(tokens[6]);
            homeOxnThpyCert.setCr503OxyEqTyCd(tokens[7]);
            homeOxnThpyCert.setCr504OxyEqTyCd(tokens[8]);
            homeOxnThpyCert.setCr506oxyFlRt(tokens[9]);
            homeOxnThpyCert.setCr507dlyOxyUseCt(tokens[10]);
            homeOxnThpyCert.setCr508oxyUsePdHrCt(tokens[11]);
            homeOxnThpyCert.setCr509resTptOrTxtDesc(tokens[12]);
            homeOxnThpyCert.setCr516pblOxySysFlRt(tokens[13]);
            homeOxnThpyCert.setCr517oxyDlySysCd(tokens[14]);
            homeOxnThpyCert.setCr518oxyEqTyCd(tokens[15]);

            em.persist(homeOxnThpyCert);
        }

        FileUtils.closeFile(in);
        System.out.println("Created rows for HomeOxnThpyCert table");
    }

    protected void loadHrchlLevel(BufferedReader in, EntityManager em)
        throws IOException {
        while ((loadRec = in.readLine()) != null) {
            traceLOGGER.trace("HrchlLevel as read:" + loadRec);

            final String[] tokens = loadRec.split("\\^", -1);
            final HrchlLevel hrchlLevel = new HrchlLevel();
            hrchlLevel.setHrchlLevelKey(tokens[0]);
            hrchlLevel.setTxType(tokens[1]);
            hrchlLevel.setFileNumber(tokens[2]);
            hrchlLevel.setSt02txnCtlNbr(tokens[3]);
            hrchlLevel.setLoopId(tokens[4]);
            hrchlLevel.setHl01hrchlId(tokens[5]);
            hrchlLevel.setHl02hrchlPrntId(tokens[6]);
            hrchlLevel.setHl03hrchlLvlCd(tokens[7]);
            hrchlLevel.setHl04hrchlChildCd(tokens[8]);

            em.persist(hrchlLevel);
        }

        FileUtils.closeFile(in);
        System.out.println("Created rows for HrchlLevel table");
    }

    protected void loadInstClaimCode(BufferedReader in, EntityManager em)
        throws IOException {
        while ((loadRec = in.readLine()) != null) {
            traceLOGGER.trace("InstClaimCode as read:" + loadRec);

            final String[] tokens = loadRec.split("\\^", -1);
            final InstClaimCode instClaimCode = new InstClaimCode();
            instClaimCode.setInstClaimCdKey(tokens[0]);
            instClaimCode.setTxType(tokens[1]);
            instClaimCode.setFileNumber(tokens[2]);
            instClaimCode.setSt02txnCtlNbr(tokens[3]);
            instClaimCode.setLoopId(tokens[4]);
            instClaimCode.setHrchlLevelKey(tokens[5]);
            instClaimCode.setCl101admTypeCd(tokens[6]);
            instClaimCode.setCl102admSrcCd(tokens[7]);
            instClaimCode.setCl103ptStatCd(tokens[8]);

            em.persist(instClaimCode);
        }

        FileUtils.closeFile(in);
        System.out.println("Created rows for InstClaimCode table");
    }

    protected void loadInstSvcLine(BufferedReader in, EntityManager em)
        throws IOException {
        while ((loadRec = in.readLine()) != null) {
            traceLOGGER.trace("InstSvcLine as read:" + loadRec);

            final String[] tokens = loadRec.split("\\^", -1);
            final InstSvcLine instSvcLine = new InstSvcLine();
            instSvcLine.setClaimKey((tokens[0]));
            instSvcLine.setServiceKey((tokens[1]));
            instSvcLine.setLx01asgnLnNbr(new BigDecimal(tokens[2]));
            instSvcLine.setSv201svcLnRevCd(tokens[3]);
            instSvcLine.setSv202svcIdQual(tokens[4]);
            instSvcLine.setSv202procCd(tokens[5]);
            instSvcLine.setSv202procMod1((tokens[6]));
            instSvcLine.setSv202procMod2((tokens[7]));
            instSvcLine.setSv202procMod3((tokens[8]));
            instSvcLine.setSv202procMod4((tokens[9]));
            instSvcLine.setSv203lnItmChrg(new BigDecimal(tokens[10]));
            instSvcLine.setSv204uomCd((tokens[11]));
            instSvcLine.setSv205svcUnitCnt(new BigDecimal(tokens[12]));

            if ((tokens[13] != null) && (tokens[13].trim().length() != 0)) {
                instSvcLine.setSv206svcLnRt(new BigDecimal(tokens[13]));
            }

            if ((tokens[14] != null) && (tokens[14].trim().length() != 0)) {
                instSvcLine.setSv207denyChrgAmt(new BigDecimal(tokens[14]));
            }

            instSvcLine.setSv2028prdctSvcId(tokens[15]);
            instSvcLine.setSv202desc(tokens[16]);
            instSvcLine.setSv210lvlCareCd(tokens[17]);

            em.persist(instSvcLine);
        }

        FileUtils.closeFile(in);
        System.out.println("Created rows for InstSvcLine table");
    }

    protected void loadInsRelationship(BufferedReader in, EntityManager em)
        throws IOException {
        while ((loadRec = in.readLine()) != null) {
            traceLOGGER.trace("InsRelationship as read:" + loadRec);

            final String[] tokens = loadRec.split("\\^", -1);
            final InsRelationship insRelationship = new InsRelationship();
            insRelationship.setInsRelKey(tokens[0]);
            insRelationship.setTxType(tokens[1]);
            insRelationship.setFileNumber(tokens[2]);
            insRelationship.setSt02txnCtlNbr(tokens[3]);
            insRelationship.setLoopId(tokens[4]);
            insRelationship.setHrchlLevelKey(tokens[5]);
            insRelationship.setIns01insuredInd(tokens[6]);
            insRelationship.setIns02indRelCd(tokens[7]);
            insRelationship.setIns17birthSeqNbr(tokens[8]);
            insRelationship.setIns08empStCd(tokens[9]);

            em.persist(insRelationship);
        }

        FileUtils.closeFile(in);
        System.out.println("Created rows for InsRelationship table");
    }

    protected void loadIntchgControl(BufferedReader in, EntityManager em)
        throws IOException {
        while ((loadRec = in.readLine()) != null) {
            traceLOGGER.trace("IntchgControl as read:" + loadRec);

            final String[] tokens = loadRec.split("\\^", -1);
            final IntchgControl intchgControl = new IntchgControl();
            intchgControl.setFileNumber(tokens[0]);
            intchgControl.setTxType(tokens[1]);
            intchgControl.setIsa01authQual((tokens[2]));
            intchgControl.setIsa02authInfo((tokens[3]));
            intchgControl.setIsa03scrInfoQual((tokens[4]));
            intchgControl.setIsa04scrInfo((tokens[5]));
            intchgControl.setIsa05sndIdQual((tokens[6]));
            intchgControl.setIsa06sndId((tokens[7]));
            intchgControl.setIsa07rcvIdQual((tokens[8]));
            intchgControl.setIsa08rcvId((tokens[9]));
            intchgControl.setIsa09intchgDt((tokens[10]));
            intchgControl.setIsa10intchgTm((tokens[11]));
            intchgControl.setIsa11intchgStd(ConversionUtil.stringToSingleChar(
                    tokens[12]));
            intchgControl.setIsa12intchgVer((tokens[13]));
            intchgControl.setIsa13intchgNbr((tokens[14]));
            intchgControl.setIsa14ackRqst(ConversionUtil.stringToSingleChar(
                    tokens[15]));
            intchgControl.setIsa15useInd(ConversionUtil.stringToSingleChar(
                    tokens[16]));
            intchgControl.setIsa16cpntEleSep(ConversionUtil.stringToSingleChar(
                    tokens[17]));
            intchgControl.setIea01nbrInclGrp((tokens[18]));
            intchgControl.setIea02intchgNbr((tokens[19]));

            em.persist(intchgControl);
        }

        FileUtils.closeFile(in);
        System.out.println("Created rows for IntchgControl table");
    }

    protected void loadMessageText(BufferedReader in, EntityManager em)
        throws IOException {
        while ((loadRec = in.readLine()) != null) {
            traceLOGGER.trace("MessageText as read:" + loadRec);

            final String[] tokens = loadRec.split("\\^", -1);
            final MessageText messageText = new MessageText();
            messageText.setMsgTextKey(tokens[0]);
            messageText.setTxType(tokens[1]);
            messageText.setFileNumber(tokens[2]);
            messageText.setSt02txnCtlNbr(tokens[3]);
            messageText.setLoopId(tokens[4]);
            messageText.setHrchlLevelKey(tokens[5]);
            messageText.setMsg01ffromText(tokens[6]);

            em.persist(messageText);
        }

        FileUtils.closeFile(in);
        System.out.println("Created rows for MessageText table");
    }

    protected void loadPrfsnSvcLine(BufferedReader in, EntityManager em)
        throws IOException {
        while ((loadRec = in.readLine()) != null) {
            traceLOGGER.trace("PrfsnSvcLine as read:" + loadRec);

            final String[] tokens = loadRec.split("\\^", -1);
            final PrfsnSvcLine prfsnSvcLine = new PrfsnSvcLine();
            prfsnSvcLine.setClaimKey((tokens[0]));
            prfsnSvcLine.setServiceKey((tokens[1]));
            prfsnSvcLine.setLx01asgnLnItm(new BigDecimal(tokens[2]));
            prfsnSvcLine.setSv101procCdQual((tokens[3]));
            prfsnSvcLine.setSv101procCd(tokens[4]);
            prfsnSvcLine.setSv101procMod1((tokens[5]));
            prfsnSvcLine.setSv101procMod2((tokens[6]));
            prfsnSvcLine.setSv101procMod3((tokens[7]));
            prfsnSvcLine.setSv101procMod4((tokens[8]));
            prfsnSvcLine.setSv102lineItmChrg(new BigDecimal(tokens[9]));
            prfsnSvcLine.setSv103uomCd((tokens[10]));
            prfsnSvcLine.setSv104svcUnitCnt(new BigDecimal(tokens[11]));
            prfsnSvcLine.setSv105placeSvcCd((tokens[12]));
            prfsnSvcLine.setSv111epsdtInd(ConversionUtil.stringToSingleChar(
                    tokens[13]));
            prfsnSvcLine.setSv112fmlyPlnInd(ConversionUtil.stringToSingleChar(
                    tokens[14]));
            prfsnSvcLine.setSv1018prdctSvcId(tokens[15]);
            prfsnSvcLine.setSv101desc(tokens[16]);
            prfsnSvcLine.setSv120lvlCareCd(tokens[17]);

            em.persist(prfsnSvcLine);
        }

        FileUtils.closeFile(in);
        System.out.println("Created rows for PrfsnSvcLine table");
    }

    protected void loadPrvSpclInfo(BufferedReader in, EntityManager em)
        throws IOException {
        while ((loadRec = in.readLine()) != null) {
            traceLOGGER.trace("PrvSpclInfo as read:" + loadRec);

            final String[] tokens = loadRec.split("\\^", -1);
            final PrvSpclInfo prvSpclInfo = new PrvSpclInfo();
            prvSpclInfo.setPrvSpclInfoKey(tokens[0]);
            prvSpclInfo.setTxType(tokens[1]);
            prvSpclInfo.setFileNumber(tokens[2]);
            prvSpclInfo.setSt02txnCtlNbr(tokens[3]);
            prvSpclInfo.setSeqNbr(new BigDecimal(tokens[4]));
            prvSpclInfo.setLoopId(tokens[5]);
            prvSpclInfo.setHrchlLevelKey(tokens[6]);
            prvSpclInfo.setPrv01prvdTypeCd(tokens[7]);
            prvSpclInfo.setPrv02prvdSpcQual(tokens[8]);
            prvSpclInfo.setPrv03prvdSpcCd(tokens[9]);

            em.persist(prvSpclInfo);
        }

        FileUtils.closeFile(in);
        System.out.println("Created rows for PrvSpclInfo table");
    }

    protected void loadRequestValidation(BufferedReader in, EntityManager em)
        throws IOException {
        while ((loadRec = in.readLine()) != null) {
            traceLOGGER.trace("RequestValidation as read:" + loadRec);

            final String[] tokens = loadRec.split("\\^", -1);
            final RequestValidation requestValidation = new RequestValidation();
            requestValidation.setReqValidKey(tokens[0]);
            requestValidation.setTxType(tokens[1]);
            requestValidation.setFileNumber(tokens[2]);
            requestValidation.setSt02txnCtlNbr(tokens[3]);
            requestValidation.setLoopId(tokens[4]);
            requestValidation.setHrchlLevelKey(tokens[5]);
            requestValidation.setSeqNbr(tokens[6]);
            requestValidation.setAaa01validReqInd(tokens[7]);
            requestValidation.setAaa02agencyQualCd(tokens[8]);
            requestValidation.setAaa01rejRsnCd(tokens[9]);
            requestValidation.setAaa01fwpActCd(tokens[10]);

            em.persist(requestValidation);
        }

        FileUtils.closeFile(in);
        System.out.println("Created rows for RequestValidation table");
    }

    protected void loadSpnlMpltSvcInfo(BufferedReader in, EntityManager em)
        throws IOException {
        while ((loadRec = in.readLine()) != null) {
            traceLOGGER.trace("SpnlMpltSvcInfo as read:" + loadRec);

            final String[] tokens = loadRec.split("\\^", -1);
            final SpnlMpltSvcInfo spnlMpltSvcInfo = new SpnlMpltSvcInfo();
            spnlMpltSvcInfo.setSpnlMpltSvcInfoKey(tokens[0]);
            spnlMpltSvcInfo.setTxType(tokens[1]);
            spnlMpltSvcInfo.setFileNumber(tokens[2]);
            spnlMpltSvcInfo.setSt02txnCtlNbr(tokens[3]);
            spnlMpltSvcInfo.setLoopId(tokens[4]);
            spnlMpltSvcInfo.setHrchlLevelKey(tokens[5]);

            if ((tokens[6] != null) && (tokens[6].trim().length() != 0)) {
                spnlMpltSvcInfo.setSeqNbr(new BigDecimal(tokens[6]));
            }

            if ((tokens[7] != null) && (tokens[7].trim().length() != 0)) {
                spnlMpltSvcInfo.setCr201trtmSerNbr(new BigDecimal(tokens[7]));
            }

            if ((tokens[8] != null) && (tokens[8].trim().length() != 0)) {
                spnlMpltSvcInfo.setCr202trtmCnt(new BigDecimal(tokens[8]));
            }

            spnlMpltSvcInfo.setCr203slxtnLvlCd1(tokens[9]);
            spnlMpltSvcInfo.setCr204slxtnLvlCd2(tokens[10]);
            spnlMpltSvcInfo.setCr205uomCd(tokens[11]);

            if ((tokens[12] != null) && (tokens[12].trim().length() != 0)) {
                spnlMpltSvcInfo.setCr206trtmPerCnt(new BigDecimal(tokens[12]));
            }

            if ((tokens[13] != null) && (tokens[13].trim().length() != 0)) {
                spnlMpltSvcInfo.setCr207moTrtmCnt(new BigDecimal(tokens[13]));
            }

            spnlMpltSvcInfo.setCr208ptCondCd(tokens[14]);
            spnlMpltSvcInfo.setCr209cmplcnInd(tokens[15]);
            spnlMpltSvcInfo.setCr210ptCondDesc1(tokens[16]);
            spnlMpltSvcInfo.setCr211ptCondDesc2(tokens[17]);
            spnlMpltSvcInfo.setCr212xrayAvlInd(tokens[18]);

            em.persist(spnlMpltSvcInfo);
        }

        FileUtils.closeFile(in);
        System.out.println("Created rows for SpnlMpltSvcInfo table");
    }

    protected void loadToothInformation(BufferedReader in, EntityManager em)
        throws IOException {
        while ((loadRec = in.readLine()) != null) {
            traceLOGGER.trace("ToothInformation as read:" + loadRec);

            final String[] tokens = loadRec.split("\\^", -1);
            final ToothInformation toothInformation = new ToothInformation();
            final ToothInformationId toothInformationId = new ToothInformationId();
            toothInformationId.setDentalServiceKey((tokens[1]));
            toothInformationId.setSeqNbr(new BigDecimal(tokens[2]));
            toothInformation.setId(toothInformationId);
            toothInformation.setClaimKey((tokens[0]));
            toothInformation.setToo01cdQual((tokens[3]));
            toothInformation.setToo02tthNbr(tokens[4]);
            toothInformation.setToo03tthSrfcCd1((tokens[5]));
            toothInformation.setToo03tthSrfcCd2((tokens[6]));
            toothInformation.setToo03tthSrfcCd3((tokens[7]));
            toothInformation.setToo03tthSrfcCd4((tokens[8]));
            toothInformation.setToo03tthSrfcCd5((tokens[9]));
            em.persist(toothInformation);
        }

        FileUtils.closeFile(in);
        System.out.println("Created rows for ToothInformation table");
    }

    protected void loadTraceNumber(BufferedReader in, EntityManager em)
        throws IOException {
        while ((loadRec = in.readLine()) != null) {
            traceLOGGER.trace("TraceNumber as read:" + loadRec);

            final String[] tokens = loadRec.split("\\^", -1);
            final TraceNumber traceNumber = new TraceNumber();
            final TraceNumberId traceNumberId = new TraceNumberId();
            traceNumberId.setFileNumber(tokens[0]);
            traceNumberId.setSt02txnCtlNbr(tokens[2]);
            traceNumberId.setSeqNbr(new BigDecimal(tokens[4]));
            traceNumber.setId(traceNumberId);
            traceNumber.setTxType(tokens[1]);
            traceNumber.setHrchlLevelKey(tokens[3]);
            traceNumber.setTrn01trcTypeCd(tokens[5]);
            traceNumber.setTrn02svcTrcNbr(tokens[6]);
            traceNumber.setTrn03origCmpId(tokens[7]);
            traceNumber.setTrn04refId(tokens[8]);

            em.persist(traceNumber);
        }

        FileUtils.closeFile(in);
        System.out.println("Created rows for TraceNumber table");
    }

    protected void loadTradingPartners(BufferedReader in, EntityManager em)
        throws IOException {
        while ((loadRec = in.readLine()) != null) {
            traceLOGGER.trace("TradingPartners as read:" + loadRec);

            final String[] tokens = loadRec.split("\\^", -1);
            final TradingPartners tradingPartners = new TradingPartners();
            final TradingPartnersId tradingPartnersId = new TradingPartnersId();
            tradingPartnersId.setTradingPartnerId(tokens[0]);
            tradingPartnersId.setAbbrevTag(tokens[1]);
            tradingPartners.setId(tradingPartnersId);
            tradingPartners.setName(tokens[2]);
            tradingPartners.setAddressLine1(tokens[3]);
            tradingPartners.setAddressLine2(tokens[4]);
            tradingPartners.setCity(tokens[5]);
            tradingPartners.setState(tokens[6]);
            tradingPartners.setZipCode(tokens[7]);
            tradingPartners.setContactName(tokens[8]);
            tradingPartners.setContactPhone(tokens[9]);

            em.persist(tradingPartners);
        }

        FileUtils.closeFile(in);
        System.out.println("Created rows for TradingPartners table");
    }

    protected void loadTxnSet(BufferedReader in, EntityManager em)
        throws IOException {
        while ((loadRec = in.readLine()) != null) {
            traceLOGGER.trace("TxnSet as read:" + loadRec);

            final String[] tokens = loadRec.split("\\^", -1);
            final TxnSet txnSet = new TxnSet();
            final TxnSetId txnSetId = new TxnSetId();
            txnSetId.setFileNumber(tokens[0]);
            txnSetId.setSt02txnCtlNbr(tokens[3]);

            txnSet.setId(txnSetId);
            txnSet.setTxType(tokens[1]);
            txnSet.setSt01txnId(tokens[2]);

            txnSet.setSt03implmtCnvntnReg(tokens[4]);
            txnSet.setBht01hrchlStrc(tokens[5]);
            txnSet.setBht02txnPrpsCd(tokens[6]);
            txnSet.setBht03orgnTxnId(tokens[7]);
            txnSet.setBht04txnCreDt(tokens[8]);
            txnSet.setBht05txnCreTm(tokens[9]);
            txnSet.setBht06txnTypeCd(tokens[10]);
            txnSet.setSe01txnSgmtCnt(new BigDecimal(tokens[11]));
            txnSet.setSe02txnCtlNbr(tokens[12]);

            em.persist(txnSet);
        }

        FileUtils.closeFile(in);
        System.out.println("Created rows for TxnSet table");
    }

    private void clearCachedEntities(EntityManager dbEM) {
        dbEM.flush();
        dbEM.clear();
    }

    private void deleteFiles(String vFilePath) {
        FileUtils.deleteFile(vFilePath + "INTCHG_CONTROL.DAT");
        FileUtils.deleteFile(vFilePath + "TRADING_PARTNERS.DAT");
        FileUtils.deleteFile(vFilePath + "INTCHG_CONTROL.DAT");
        FileUtils.deleteFile(vFilePath + "FUNCTIONAL_GROUP.DAT");
        FileUtils.deleteFile(vFilePath + "TXN_SET.DAT");
        FileUtils.deleteFile(vFilePath + "HRCHL_LEVEL.DAT");
        FileUtils.deleteFile(vFilePath + "REQUEST_VALIDATION.DAT");
        FileUtils.deleteFile(vFilePath + "TRACE_NUMBER.DAT");
        FileUtils.deleteFile(vFilePath + "DTP_DATE.DAT");
        FileUtils.deleteFile(vFilePath + "ADTL_SUPL_INFO.DAT");
        FileUtils.deleteFile(vFilePath + "CNTC_INFO.DAT");
        FileUtils.deleteFile(vFilePath + "DEMOGRAPHIC_INFO.DAT");
        FileUtils.deleteFile(vFilePath + "ENTITY_ADDRESS.DAT");
        FileUtils.deleteFile(vFilePath + "ENTITY_ADTL_ID.DAT");
        FileUtils.deleteFile(vFilePath + "ENTITY_NAME.DAT");
        FileUtils.deleteFile(vFilePath + "HI_CODES.DAT");
        FileUtils.deleteFile(vFilePath + "AMBLN_TRNPT_INFO.DAT");
        FileUtils.deleteFile(vFilePath + "SPNL_MPLT_SVC_INFO.DAT");
        FileUtils.deleteFile(vFilePath + "HOME_OXN_THPY_CERT.DAT");
        FileUtils.deleteFile(vFilePath + "HOME_HCARE_INFO.DAT");
        FileUtils.deleteFile(vFilePath + "INST_CLAIM_CODE.DAT");
        FileUtils.deleteFile(vFilePath + "HCARE_SVC_REV_INFO.DAT");
        FileUtils.deleteFile(vFilePath + "HCARE_SVC_REVIEW.DAT");
        FileUtils.deleteFile(vFilePath + "PRV_SPCL_INFO.DAT");
        FileUtils.deleteFile(vFilePath + "HCARE_SERVICE_DLVR.DAT");
        FileUtils.deleteFile(vFilePath + "INS_RELATIONSHIP.DAT");
        FileUtils.deleteFile(vFilePath + "MESSAGE_TEXT.DAT");
        FileUtils.deleteFile(vFilePath + "DENTAL_SVC_LINE.DAT");
        FileUtils.deleteFile(vFilePath + "INST_SVC_LINE.DAT");
        FileUtils.deleteFile(vFilePath + "PRFSN_SVC_LINE.DAT");
        FileUtils.deleteFile(vFilePath + "TOOTH_INFORMATION.DAT ");
    }

    // private void insertProcessLog(String v278Cnt, String vFileDt,
    // EntityManager dbEM) throws SQLException {
    // final EdiProcessLog crEdiProcessLog = new EdiProcessLog();
    // crEdiProcessLog.setEdiProcessId(UUID.randomUUID().toString()
    // .toUpperCase());
    // crEdiProcessLog.setFileDate(DateUtil.getSqlDate(vFileDt));
    // crEdiProcessLog.setProcessName("278 LOAD");
    // crEdiProcessLog.setMessage("NUM. OF 278'S LOADED INTO EDI REPOSITORY");
    // crEdiProcessLog.setCounts(new Long(v278Cnt));
    // crEdiProcessLog.setStatus("NEW");
    // dbEM.persist(crEdiProcessLog);
    // }

    /**
     * Get the interchange control date from the file INTCHG_CONTROL.DAT located
     * in the directory indicated by vFilePath.
     *
     * @param vFilePath
     *            - the path to the file INTCHG_CONTROL.DAT file.
     * @return a string representing the date
     * @throws IOException
     */
    private String getIsa13Dt(String vFilePath) throws IOException {
        String vIsa13Dt = "";
        String lRec = "";
        final BufferedReader in = FileUtils.readFile(vFilePath +
                "INTCHG_CONTROL.DAT");

        if ((lRec = in.readLine()) != null) {
            final String[] tokens = lRec.split("\\^", -1);
            vIsa13Dt = tokens[10];
        }

        in.close();

        return vIsa13Dt;
    }

    /**
     * Returns true if this data has already been loaded - identified by the
     * fileNumber in the INTCHG_CONTROL file. TODO: Write a unit test for this.
     * Change the SQL to SELECT COUNT (*)?
     *
     * @param dbEM
     *            - the database connection / entity manager
     * @param fileNumber
     *            - the fileNumber that identifies a particular batch
     */
    private boolean isDataAlreadyLoaded(EntityManager dbEM, String fileNumber) {
        String fileNumberTruncated = fileNumber;

        if (fileNumber.length() >= 14) {
            fileNumberTruncated = fileNumber.substring(0, 13);
        }

        if (fileNumberTruncated.trim().length() == 0) {
            throw new RuntimeException("No filenumber to check against!");
        }

        // query the database to see if this batch was already loaded
        Query isBatchAlreadyLoadedQuery = dbEM.createQuery(
                "SELECT i FROM gov.va.med.domain.prserv1.IntchgControl i WHERE i.fileNumber like :fileNumber");
        isBatchAlreadyLoadedQuery.setParameter("fileNumber",
            fileNumberTruncated + "%");

        try {
            if (isBatchAlreadyLoadedQuery.getSingleResult() != null) {
                return true;
            }
        } catch (NoResultException e) {
            // Just eat this -- it means the batch has not been loaded before.
        }

        return false;
    }
}
