package gov.va.med.hac.edi.reposload.stg835;

import gov.va.med.hac.edi.utils.jpa.JPAUtil;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.GnuParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;

import javax.persistence.EntityManager;


/**
 * This class is responsible for staging 835 data from
 * FPPS_OWNER to the E_REPOS 835 Staging tables.
 *
 */
public class Stage {
    static final Logger LOGGER = LoggerFactory.getLogger(Stage.class);

    private Stage() {
    }

    /**
     * This is the entry point for the FEE 835 staging process.
     * This calls the DB stored procedure
     * <code>fpps_owner.load_835.start_835_load</code>. The
     * transaction state is maintained in the stored procedure.
     *
     * @param args no arguments needed
     */
    public static void main(String[] args) {
    	
    	boolean DEBUG = true;
    	
    	
        CommandLine line = null;
        Options opts = getCliOptions();

        try {
            CommandLineParser parser = new GnuParser();
            line = parser.parse(opts, args);

            // If the "help" option is specified, just show usage options and exit
            if (line.hasOption("h")) {
                printUsage(opts, args, false);
                System.exit(1);
            }

            // If any required command line parameter is missing, just show usage options and exit
        } catch (MissingOptionException moe) {
            printUsage(opts, args, true);
            System.exit(1);
        } catch (ParseException pe) {
            System.out.println("Problem parsing the command line");
            printUsage(opts, args, false);
            System.exit(1);
        }

        EntityManager em = null;

        try {
            System.out.println("\n\n");
            System.out.println(
                "<<<<<<<<<<<<< Beginning FEE 835 Staging <<<<<<<<<<<<<<<<<<<<");

            File configFile = null;
            String configFilePath = "";
            configFilePath = line.getOptionValue("f");

            if ((configFilePath != null) && (configFilePath.length() > 0)) {
                configFile = JPAUtil.getJpaConfigFile(configFilePath);
            } else {
                configFile = JPAUtil.getJpaConfigFile("edihibernate.properties");
                // configFile = JPAUtil.getJpaConfigFile("dbhibernate.properties");
            }

            em = JPAUtil.getEntityManager(configFile);
            
            boolean isOpen = em.isOpen();
            
            if(DEBUG) {
            	System.out.println("Stage.mainO) - em is open: " + isOpen);
            }
            
  
if(DEBUG) {            
            // CLEAR STAGING TABLES FIRST....do I have the correct privileges to do this?
            // System.out.println("Stage.main() - calling fpps_owner.load_835.trunc_fpps_stg_835_tables() ");
            // String callKill = "{call fpps_owner.load_835.trunc_fpps_stg_835_tables()}";
            // JPAUtil.runDbProcedure(em, callKill);
            
         //   System.out.println("Stage - cleared staging tables");
}            
            System.out.println("Stage.main() - calling fpps_owner.load_835_HARRIS.start_835_load() ");
            // NOTE: load_835_HARRIS selects ONLY a single record for testing
            String call = "{call fpps_owner.load_835_HARRIS.start_835_load()}";
            JPAUtil.runDbProcedure(em, call);
            
            if(DEBUG) {
            	System.out.println("SUCCESSFULLY COMPLETED FEE 835 Staging");
            }
        } catch (Throwable t) {
        	
        	if(DEBUG)  {
        		System.out.println("Stage.main() - error running stored proc: " + t.getMessage());
        		t.printStackTrace();
        	}
        	
        	
        	
            LOGGER.error(t.getMessage(), t);
        } finally {
            if ((em != null) && em.isOpen()) {
                em.close();
            }

            System.out.println(
                "<<<<<<<<<<<<< Ending FEE 835 Staging <<<<<<<<<<<<<<<<<<<<");
        }
    }

    private static void printUsage(Options opts, String[] inArgs,
        boolean missingArg) {
        final int helpwidth = 80;
        HelpFormatter fmtr = new HelpFormatter();
        System.out.println();

        if (missingArg) {
            System.out.println("Some required commandline argument was missing");
        }

        fmtr.printHelp(helpwidth, "Stage", "", opts, "", true);

        if (missingArg) {
            System.out.println(inArgs.length +
                " command line arguments were received: ");

            for (int i = 0; i < inArgs.length; i++) {
                System.out.println(inArgs[i]);
            }
        }
    }

    /**
     * Helper method to populate command line options objects.
     *
     * @return the <code>Options</code> object containing the set of command line option.
     */
    @SuppressWarnings("static-access")
    private static Options getCliOptions() {
        Options options = new Options();

        Option configfile = OptionBuilder.withArgName("jpaFile").hasArg()
                                         .withLongOpt("jpaFile")
                                         .withDescription("The JPA Configuration file, such as hibernate.properties.")
                                         .create("f");
        options.addOption(configfile);

        return options;
    }
}
