package gov.va.med.hac.edi.utils;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

// This class collects static methods that are used to manipulate strings
public class StringUtils 
{

  // Return a string containing the given string padded to the given required length
  //   with the supplied character, on the indicated side.
  // If the given string is null, return just the padding
  //   If the given string is at least as long as the required length, the
  //   beginning characters are returned, up to the required length.
  // TODO: There are utilities, such as Jakarta commons, that have methods
  //   like this.  The functionality is also available in the new
  //   java.util.Formatter class
  public static String padString(final String inStringToPad,
                                 final int inRequiredLength,
                                 final char inPadChar,
                                 final boolean leftPad)
  {
    String vReturnStr = "";
    
    // If nothing was passed in to be padded, use the empty string
    String vStringToPad;
    if (inStringToPad == null || "null".equalsIgnoreCase(inStringToPad)) {
      vStringToPad = "";
    } else {
      vStringToPad = inStringToPad;
    }
    
    // If the input string is longer than the required length, shorten it
    // This will cause the rest of the processing to have no effect
    int actualLength = vStringToPad.length();
    if (actualLength > inRequiredLength) {
      vStringToPad = inStringToPad.substring(0, inRequiredLength);
      actualLength = inRequiredLength;
    }
    
    // Loop through the # of chars that need to be added, increasing the padding
    final StringBuffer padding  = new StringBuffer("");
    final int loopTimes = inRequiredLength - actualLength;
    for (int i=0; i < loopTimes; i++) {
      padding.append(inPadChar);
    }
    
    // Put the padding on the left or right of the string, depending on a parameter
    if (leftPad) {
      vReturnStr = padding.toString() + vStringToPad;
    } else {
      vReturnStr = vStringToPad + padding.toString();      
    }

    // return the result
    return vReturnStr;
  } // end method padString
  
  // convenience method that is often used
  public static String lPadZeros(final String inStringToPad,
                                 final int inRequiredLength)
  {
    return padString(inStringToPad, inRequiredLength, '0', true);
  }

  // convenience method that is seldom used
  public static String lPadZeros(final BigDecimal inBigDecimalToPad,
                                 final int inRequiredLength)
  {
    if (inBigDecimalToPad == null) {
      return padString ("", inRequiredLength, '0', true);
    } else {
      return padString (inBigDecimalToPad.toString(), inRequiredLength, '0', true);
    }
  }
  
  
//convenience method that is often used
  public static String rPadSpaces(final byte inByteToPad,
                                 final int inRequiredLength)
  {
	if(inByteToPad == 0)
	{
		return padString("", inRequiredLength, ' ', false);
	}
    return padString(inByteToPad+"", inRequiredLength, ' ', false);
  }


  // convenience method that is often used
  public static String rPadSpaces(final char inCharToPad,
                                 final int inRequiredLength)
  {
	if(inCharToPad == 0)
	{
		return padString("", inRequiredLength, ' ', false);
	}
    return padString(inCharToPad+"", inRequiredLength, ' ', false);
  }
  
  //convenience method that is often used
  public static String rPadSpaces(final Character inCharToPad,
                                 final int inRequiredLength)
  {
	if(inCharToPad == null)
	{
		return padString("", inRequiredLength, ' ', false);
	}
    return padString(inCharToPad+"", inRequiredLength, ' ', false);
  }

  
  // convenience method that is often used
  public static String rPadSpaces(final String inStringToPad,
                                 final int inRequiredLength)
  {
    return padString(inStringToPad, inRequiredLength, ' ', false);
  }

  // convenience method that is often used
  public static String rPadSpaces(final BigDecimal inBigDecimalToPad,
                                 final int inRequiredLength)
  {
    String stringToPad;
    if (inBigDecimalToPad == null) 
    {
      stringToPad = "";
    } else {
      stringToPad = inBigDecimalToPad.toString();
    }
    
    return padString(stringToPad, inRequiredLength, ' ', false);
  }

  // convenience method that is often used
  public static String lPadZerosRPadSpaces(final String inStringToPad,
                                 final int inZerosLength, final int inTotalLength)
  {
    String paddedWithZeros = "";
    
    // If the input string is null, skip the zeros on the left
    if (inStringToPad != null) {
      paddedWithZeros = lPadZeros(inStringToPad, inZerosLength);
    } 
    
    // Next, pad on the right with spaces
    return rPadSpaces(paddedWithZeros, inTotalLength);
  }

    // TODO: This padding method seems to take a BigDecimal, assumed to represent
    //   an integral value (because it was obtained by multiplying a dollar amount
    //   by 100), removing the decimal point and any zeros after it, and padding the
    //   result with zeros on the left.
    // It should use BigDecimal.intValue or, better, intValueExact,
    //   which will throw an exception if the input is not integral
    public static String lPadBigDecimalZeros(BigDecimal padDec, int reqLen)
    {
        int actualLen = 0;
        String vPadStr = "";
        if (padDec != null) {
            vPadStr = padDec.toString();
            if (!(vPadStr.equalsIgnoreCase("0"))) {
                final int idx1 = vPadStr.lastIndexOf('.');
                if (idx1 > 0) {
                    vPadStr = vPadStr.substring(0, idx1);
                }
                actualLen = vPadStr.length();
            } else {
                vPadStr = "";
            }
        } else {
            vPadStr = "";
        }
        String zeros  = "";
        final int loopTimes = reqLen - actualLen;
        for (int i=0; i < loopTimes; i++) {
            zeros += "0";
        }
        return zeros + vPadStr;
    }
    
    // Seems to ensure that the result of padding the given string will end in ".00"
    public static String lPadDecimalZeros2Places(String padInt, int reqLen)
    {
        int idx = 0;
        String strDecimal = "";
        String padDecimal = "";
        if(padInt != null) {
            idx = padInt.lastIndexOf('.');
            if(idx > 0) {
                strDecimal = padInt.substring(idx + 1, padInt.length());
                if(strDecimal.length() == 1) {
                    padInt += "0";
                }
            } else {
                padInt += ".00";
            }
        } else {
            padInt = "0.00";
        }
        padDecimal = lPadZeros(padInt, reqLen);

        return padDecimal;
    }

  public static boolean hasChars(final String inString)
  {
    return (inString != null && inString.length() > 0); 
  }
  
  // Concatenate the given list of strings
  public static String concatenateList(ArrayList inStrings, boolean convertInt)
  {
    int listSize = inStrings.size();
    StringBuffer fullString = new StringBuffer();
    String nextString;
    for (int i = 0; i < listSize; i++) 
    {
      nextString = (String) inStrings.get(i);
      if (convertInt) {
        nextString = "" + Integer.parseInt(nextString);
      }
      fullString.append("'" + nextString + "'");
      if (i < listSize - 1) {
        fullString.append(',');
      }
    }
    
    return fullString.toString();
  }
  
  // Concatenate part of the given list of strings
  public static String concatenateList(ArrayList inStrings,
                                       boolean convertInt,
                                       int startIndex,
                                       int endIndex)
  {
    int stopIndex = endIndex;
    int lastFilledIndex = inStrings.size() - 1;
    
    // Maybe the list does not hold as many elements as we would wish
    if (endIndex > lastFilledIndex) {
      stopIndex = lastFilledIndex;
    }
    
    //int listSize = inStrings.size();
    StringBuffer fullString = new StringBuffer();
    String nextString;
    
    // Loop from the start through the end
    for (int i = startIndex; i <= stopIndex ; i++) 
    {
      nextString = (String) inStrings.get(i);
      if (convertInt) {
        nextString = "" + Integer.parseInt(nextString);
      }
      fullString.append("'" + nextString + "'");
      if (i < stopIndex) {
        fullString.append(',');
      }
    }
    
    return fullString.toString();
  }

  public static String join( Collection s, String delimiter) {
      StringBuffer buffer = new StringBuffer();
      Iterator iter = s.iterator();
      while (iter.hasNext()) {
          buffer.append(iter.next());
          if (iter.hasNext()) {
              buffer.append(delimiter);
          }
      }
      return buffer.toString();
  }

  
}